#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "khmm.h"
#include "kc.h"

#define STATE_CHANGE 100000.0			/*this is the expected changes (D value) in the transition matrix*/
#define VITHUGE  100000000000.0
#define IMPOSSIBLE 1.0e-30


void testVitTrio_CHMM (CHMM *phmm, int T, double *Of1, double *Of2, double *Om1, double *Om2, double *Oo1, double *Oo2, double *pfb, int *snpdist, double *plogproba, double *trio_lrr_sd, int osex, int chrx_flag)
{
	int 	t;		//time index
	int	*q;		// q array stores the most likely state sequence
	double	**delta;	// delta matrix stores the total probability for up to a certain state at a certain time point
	int	**psi;		// psi matrix stores the most likely transitted state (without emission yet) at each time point
 
 	int snp_count = 0;	//count of SNP markers
 	int cn_count = 0;	//count of CN markers

        int	maxvalind;
        double	maxval, val;
	double	**biot;		//emission probability
	double	**A1;		//distance-dependent state transition matrix
	double	*triopi;	//trio initialization probability

	int ifat, imot, ioff, ifmo, dn;			//index for a trio
	int sfat, smot, soff;				//state (index+1 or index+2) for a trio
	int pifat, pimot, pioff, pifmo, pdn;		//previous index for father, mother and offspring, respectively
	double cim[5][5][5];				//cim=CNV inheritance matrix (from CN=0 to CN=4, totaling 125 cells in the matrix)
	double a = 0.04*0.04;				//alpha is used to model the relative probability of having very different copy number in 2 chromosomes, compared to having approximate copy number in 2 chromosomes
	double e1 = 1.5e-6;				//from normal to denovo (24 events for 28 offspring: 24/28/550000)
	double e2 = 0.08;				//from denovo to normal (expected denovo region=12.5 SNPs)
	int trioN = 125*1+25*4+5*4+4;			//maximum CN index for a trio
	CHMM hmm1;					//set up a new HMM data structure (to prevent the situation where we mistakenly modify original model)
	CHMM *phmm1 = &hmm1;				//use a pointer to simplify the following code
	
	double chit[5][5][5][5][5][5];
	if (chrx_flag) {
		if (osex == 1) {
			calculateCHIT_male_x (a, chit);
		} else if (osex == 2) {
			calculateCHIT_female_x (a, chit);
		} else {
			nrerror ("please specify sex (1 for male, 2 for female) for chrX CNV calls");
		}
	} else {
		calculateCHIT (a, chit);			//initilize the CNV haplotype inheritance table
	}
	
	for (pifat=0; pifat<=4; pifat++) {		//normalize the matrix so that they sum up to 1 (previous CHIT is not normalized)
		for (ifat=0; ifat<=4; ifat++) {
			for (pimot=0; pimot<=4; pimot++) {
				for (imot=0; imot<=4; imot++) {
					for (pioff=0; pioff<=4; pioff++) {
						val = 0;
						for (ioff=0; ioff<=4; ioff++) {
							val += chit[pifat][ifat][pimot][imot][pioff][ioff];
						}
						if (val != 0) {
							for (ioff=0; ioff<=4; ioff++) {
								chit[pifat][ifat][pimot][imot][pioff][ioff] /= val;
							}
						}
						//fprintf (stderr, "NOTICE: CHIT table for %i %i %i %i %i is %f\n", pifat, ifat, pimot, imot, pioff, val);
					}
				}
			}
		}
	}
	

	/* initialize matrices */
	q = ivector(1,T);
	delta = dmatrix(1, T, 0, trioN);
	psi = imatrix(1, T, 0, trioN);
	triopi = dvector(0, trioN);
	A1 = dmatrix (1, phmm->N, 1, phmm->N);		//transition matrix prototype
	biot = dmatrix(0, trioN, 1, T);			//assign memory for the biot matrix
	
	//InitCHMM (phmm1, phmm->N, phmm->M);		//initialize the copy of original HMM model
	CopyCHMM (phmm, phmm1);				//copy original HMM model to its copy

	/* initialize CNV inheritance matrix */
	if (chrx_flag) {
		if (osex == 1) {	//male chrx
			cim[0][0][0]=1; cim[0][0][1]=0; cim[0][0][2]=0; cim[0][0][3]=0; cim[0][0][4]=0; cim[0][1][0]=0.5; cim[0][1][1]=0.5; cim[0][1][2]=0; cim[0][1][3]=0; cim[0][1][4]=0; cim[0][2][0]=0.5*a; cim[0][2][1]=1-a; cim[0][2][2]=0.5*a; cim[0][2][3]=0; cim[0][2][4]=0; cim[0][3][0]=0.5*a; cim[0][3][1]=0.5*(1-a); cim[0][3][2]=0.5*(1-a); cim[0][3][3]=0.5*a; cim[0][3][4]=0; cim[0][4][0]=0; cim[0][4][1]=0.25; cim[0][4][2]=0.5; cim[0][4][3]=0.25; cim[0][4][4]=0; cim[1][0][0]=1; cim[1][0][1]=0; cim[1][0][2]=0; cim[1][0][3]=0; cim[1][0][4]=0; cim[1][1][0]=0.5; cim[1][1][1]=0.5; cim[1][1][2]=0; cim[1][1][3]=0; cim[1][1][4]=0; cim[1][2][0]=0.5*a; cim[1][2][1]=1-a; cim[1][2][2]=0.5*a; cim[1][2][3]=0; cim[1][2][4]=0; cim[1][3][0]=0.5*a; cim[1][3][1]=0.5*(1-a); cim[1][3][2]=0.5*(1-a); cim[1][3][3]=0.5*a; cim[1][3][4]=0; cim[1][4][0]=0; cim[1][4][1]=0.25; cim[1][4][2]=0.5; cim[1][4][3]=0.25; cim[1][4][4]=0; cim[2][0][0]=1; cim[2][0][1]=0; cim[2][0][2]=0; cim[2][0][3]=0; cim[2][0][4]=0; cim[2][1][0]=0.5; cim[2][1][1]=0.5; cim[2][1][2]=0; cim[2][1][3]=0; cim[2][1][4]=0; cim[2][2][0]=0.5*a; cim[2][2][1]=1-a; cim[2][2][2]=0.5*a; cim[2][2][3]=0; cim[2][2][4]=0; cim[2][3][0]=0.5*a; cim[2][3][1]=0.5*(1-a); cim[2][3][2]=0.5*(1-a); cim[2][3][3]=0.5*a; cim[2][3][4]=0; cim[2][4][0]=0; cim[2][4][1]=0.25; cim[2][4][2]=0.5; cim[2][4][3]=0.25; cim[2][4][4]=0; cim[3][0][0]=1; cim[3][0][1]=0; cim[3][0][2]=0; cim[3][0][3]=0; cim[3][0][4]=0; cim[3][1][0]=0.5; cim[3][1][1]=0.5; cim[3][1][2]=0; cim[3][1][3]=0; cim[3][1][4]=0; cim[3][2][0]=0.5*a; cim[3][2][1]=1-a; cim[3][2][2]=0.5*a; cim[3][2][3]=0; cim[3][2][4]=0; cim[3][3][0]=0.5*a; cim[3][3][1]=0.5*(1-a); cim[3][3][2]=0.5*(1-a); cim[3][3][3]=0.5*a; cim[3][3][4]=0; cim[3][4][0]=0; cim[3][4][1]=0.25; cim[3][4][2]=0.5; cim[3][4][3]=0.25; cim[3][4][4]=0; cim[4][0][0]=1; cim[4][0][1]=0; cim[4][0][2]=0; cim[4][0][3]=0; cim[4][0][4]=0; cim[4][1][0]=0.5; cim[4][1][1]=0.5; cim[4][1][2]=0; cim[4][1][3]=0; cim[4][1][4]=0; cim[4][2][0]=0.5*a; cim[4][2][1]=1-a; cim[4][2][2]=0.5*a; cim[4][2][3]=0; cim[4][2][4]=0; cim[4][3][0]=0.5*a; cim[4][3][1]=0.5*(1-a); cim[4][3][2]=0.5*(1-a); cim[4][3][3]=0.5*a; cim[4][3][4]=0; cim[4][4][0]=0; cim[4][4][1]=0.25; cim[4][4][2]=0.5; cim[4][4][3]=0.25; cim[4][4][4]=0;
		} else if (osex == 2) {	//female chrx
			cim[0][0][0]=1; cim[0][0][1]=0; cim[0][0][2]=0; cim[0][0][3]=0; cim[0][0][4]=0; cim[0][1][0]=0.5; cim[0][1][1]=0.5; cim[0][1][2]=0; cim[0][1][3]=0; cim[0][1][4]=0; cim[0][2][0]=0.5*a; cim[0][2][1]=1-a; cim[0][2][2]=0.5*a; cim[0][2][3]=0; cim[0][2][4]=0; cim[0][3][0]=0.5*a; cim[0][3][1]=0.5*(1-a); cim[0][3][2]=0.5*(1-a); cim[0][3][3]=0.5*a; cim[0][3][4]=0; cim[0][4][0]=0; cim[0][4][1]=0.25; cim[0][4][2]=0.5; cim[0][4][3]=0.25; cim[0][4][4]=0; cim[1][0][0]=0; cim[1][0][1]=1; cim[1][0][2]=0; cim[1][0][3]=0; cim[1][0][4]=0; cim[1][1][0]=0; cim[1][1][1]=0.5; cim[1][1][2]=0.5; cim[1][1][3]=0; cim[1][1][4]=0; cim[1][2][0]=0; cim[1][2][1]=0.5*a; cim[1][2][2]=1-a; cim[1][2][3]=0.5*a; cim[1][2][4]=0; cim[1][3][0]=0; cim[1][3][1]=0.5*a; cim[1][3][2]=0.5*(1-a); cim[1][3][3]=0.5*(1-a); cim[1][3][4]=0.5*a; cim[1][4][0]=0; cim[1][4][1]=0; cim[1][4][2]=0.25; cim[1][4][3]=0.5; cim[1][4][4]=0.25; cim[2][0][0]=0; cim[2][0][1]=0; cim[2][0][2]=1; cim[2][0][3]=0; cim[2][0][4]=0; cim[2][1][0]=0; cim[2][1][1]=0; cim[2][1][2]=0.5; cim[2][1][3]=0.5; cim[2][1][4]=0; cim[2][2][0]=0; cim[2][2][1]=0; cim[2][2][2]=0.5*a; cim[2][2][3]=1-a; cim[2][2][4]=0.5*a; cim[2][3][0]=0; cim[2][3][1]=0; cim[2][3][2]=0.5*a; cim[2][3][3]=0.5*(1-a); cim[2][3][4]=0.5; cim[2][4][0]=0; cim[2][4][1]=0; cim[2][4][2]=0; cim[2][4][3]=0.25; cim[2][4][4]=0.75; cim[3][0][0]=0; cim[3][0][1]=0; cim[3][0][2]=0; cim[3][0][3]=1; cim[3][0][4]=0; cim[3][1][0]=0; cim[3][1][1]=0; cim[3][1][2]=0; cim[3][1][3]=0.5; cim[3][1][4]=0.5; cim[3][2][0]=0; cim[3][2][1]=0; cim[3][2][2]=0; cim[3][2][3]=0.5*a; cim[3][2][4]=1-0.5*a; cim[3][3][0]=0; cim[3][3][1]=0; cim[3][3][2]=0; cim[3][3][3]=0.5*a; cim[3][3][4]=1-0.5*a; cim[3][4][0]=0; cim[3][4][1]=0; cim[3][4][2]=0; cim[3][4][3]=0; cim[3][4][4]=1; cim[4][0][0]=0; cim[4][0][1]=0; cim[4][0][2]=0; cim[4][0][3]=0; cim[4][0][4]=1; cim[4][1][0]=0; cim[4][1][1]=0; cim[4][1][2]=0; cim[4][1][3]=0; cim[4][1][4]=1; cim[4][2][0]=0; cim[4][2][1]=0; cim[4][2][2]=0; cim[4][2][3]=0; cim[4][2][4]=1; cim[4][3][0]=0; cim[4][3][1]=0; cim[4][3][2]=0; cim[4][3][3]=0; cim[4][3][4]=1; cim[4][4][0]=0; cim[4][4][1]=0; cim[4][4][2]=0; cim[4][4][3]=0; cim[4][4][4]=1;
		} else {
			nrerror ("please specify sex (1 for male, 2 for female) for chrX CNV calls");
		}
	} else {	//autosome CNV inheritance matrix
		cim[0][0][0]=1; cim[0][0][1]=0; cim[0][0][2]=0; cim[0][0][3]=0; cim[0][0][4]=0; cim[0][1][0]=0.5; cim[0][1][1]=0.5; cim[0][1][2]=0; cim[0][1][3]=0; cim[0][1][4]=0; cim[0][2][0]=0.5*a; cim[0][2][1]=1-a; cim[0][2][2]=0.5*a; cim[0][2][3]=0; cim[0][2][4]=0; cim[0][3][0]=0.5*a; cim[0][3][1]=0.5*(1-a); cim[0][3][2]=0.5*(1-a); cim[0][3][3]=0.5*a; cim[0][3][4]=0; cim[0][4][0]=0; cim[0][4][1]=0.25; cim[0][4][2]=0.5; cim[0][4][3]=0.25; cim[0][4][4]=0; cim[1][0][0]=0.5; cim[1][0][1]=0.5; cim[1][0][2]=0; cim[1][0][3]=0; cim[1][0][4]=0; cim[1][1][0]=0.25; cim[1][1][1]=0.5; cim[1][1][2]=0.25; cim[1][1][3]=0; cim[1][1][4]=0; cim[1][2][0]=0.25*a; cim[1][2][1]=0.5-0.25*a; cim[1][2][2]=0.5-0.25*a; cim[1][2][3]=0.25*a; cim[1][2][4]=0; cim[1][3][0]=0.25*a; cim[1][3][1]=0.25; cim[1][3][2]=0.5*(1-a); cim[1][3][3]=0.25; cim[1][3][4]=0.25*a; cim[1][4][0]=0; cim[1][4][1]=0.125; cim[1][4][2]=0.375; cim[1][4][3]=0.375; cim[1][4][4]=0.125; cim[2][0][0]=0.5*a; cim[2][0][1]=1-a; cim[2][0][2]=0.5*a; cim[2][0][3]=0; cim[2][0][4]=0; cim[2][1][0]=0.25*a; cim[2][1][1]=0.5-0.25*a; cim[2][1][2]=0.5-0.25*a; cim[2][1][3]=0.25*a; cim[2][1][4]=0; cim[2][2][0]=0.25*a*a; cim[2][2][1]=a*(1-a); cim[2][2][2]=(1-a)*(1-a)+0.5*a*a; cim[2][2][3]=a*(1-a); cim[2][2][4]=0.25*a*a; cim[2][3][0]=0.25*a*a; cim[2][3][1]=0.75*a*(1-a); cim[2][3][2]=0.5*(1-a)*(1-a)+0.25*a*(1-a)+0.25*a*a; cim[2][3][3]=0.5*(1-a)*(1-a)+0.25*a*(1-a)+0.25*a*a; cim[2][3][4]=0.75*a*(1-a)+0.25*a*a; cim[2][4][0]=0; cim[2][4][1]=0.125*a; cim[2][4][2]=0.25; cim[2][4][3]=0.5-0.25*a; cim[2][4][4]=0.25+0.125*a; cim[3][0][0]=0.5*a; cim[3][0][1]=0.5*(1-a); cim[3][0][2]=0.5*(1-a); cim[3][0][3]=0.5*a; cim[3][0][4]=0; cim[3][1][0]=0.25*a; cim[3][1][1]=0.25; cim[3][1][2]=0.5*(1-a); cim[3][1][3]=0.25; cim[3][1][4]=0.25*a; cim[3][2][0]=0.25*a*a; cim[3][2][1]=0.75*a*(1-a); cim[3][2][2]=0.5*(1-a)*(1-a)+0.25*a*(1-a)+0.25*a*a; cim[3][2][3]=0.5*(1-a)*(1-a)+0.25*a*(1-a)+0.25*a*a; cim[3][2][4]=0.75*a*(1-a)+0.25*a*a; cim[3][3][0]=0.25*a*a; cim[3][3][1]=0.5*a*(1-a); cim[3][3][2]=0.5*a*(1-a)+0.25*(1-a)*(1-a); cim[3][3][3]=0.5*(1-a)*(1-a)+0.5*a*a; cim[3][3][4]=0.25*(1-a)*(1-a)+a*(1-a)+0.25*a*a; cim[3][4][0]=0; cim[3][4][1]=0.125*a; cim[3][4][2]=0.125*(1+a); cim[3][4][3]=0.125*(3-2*a); cim[3][4][4]=0.5; cim[4][0][0]=0; cim[4][0][1]=0.25; cim[4][0][2]=0.5; cim[4][0][3]=0.25; cim[4][0][4]=0; cim[4][1][0]=0; cim[4][1][1]=0.125; cim[4][1][2]=0.375; cim[4][1][3]=0.375; cim[4][1][4]=0.125; cim[4][2][0]=0; cim[4][2][1]=0.125*a; cim[4][2][2]=0.25; cim[4][2][3]=0.5-0.25*a; cim[4][2][4]=0.25+0.125*a; cim[4][3][0]=0; cim[4][3][1]=0.125*a; cim[4][3][2]=0.125*(1+a); cim[4][3][3]=0.125*(3-2*a); cim[4][3][4]=0.5; cim[4][4][0]=0.0000; cim[4][4][1]=0.0000; cim[4][4][2]=0.0625; cim[4][4][3]=0.2500; cim[4][4][4]=0.6875;
	}

	//fprintf (stderr, "NOTICE: Beginning Viterbi algorithm for trio-CNV detection\n");
	
	/* calculate emission probability for all states and all time points */
	adjustBSD (phmm1, trio_lrr_sd[0]);
	for (dn = 0; dn <= 1; dn++) {
		for (ifat = 0; ifat <= 4; ifat++) {
			for (imot = 0; imot <= 4; imot++) {
				for (ioff = 0; ioff <= 4; ioff++) {
					ifmo = dn*125+ifat*25+imot*5+ioff;			//combined index of trio
					sfat = ifat+1; if (sfat>=4) sfat++;
					for (t = 1; t <= T; t++) {
						if (Of2[t] > 1) {				//normally BAF>=0 and BAF<=1; we use BAF=1 to indicate a Non-Polymorphic marker
							if (!phmm1->NP_flag) nrerror ("FATAL ERROR: non-polymorphic (NP) probe detected in input data but HMM model does not contain parameters for them\n");
							biot[ifmo][t]  = b1iot (sfat, phmm1->B3_mean, phmm1->B3_sd, phmm1->B3_uf, Of1[t]);
							if (ifat+imot+ioff==0) cn_count++;
						} else {
							biot[ifmo][t]  = b1iot (sfat, phmm1->B1_mean, phmm1->B1_sd, phmm1->B1_uf, Of1[t]);
							biot[ifmo][t] += b2iot (sfat, phmm1->B2_mean, phmm1->B2_sd, phmm1->B2_uf, pfb[t], Of2[t]);
							if (ifat+imot+ioff==0) snp_count++;
						}
					}
				}
			}
		}
	}
	adjustBSD (phmm1, trio_lrr_sd[1]);
	for (dn = 0; dn <= 1; dn++) {
		for (ifat = 0; ifat <= 4; ifat++) {
			for (imot = 0; imot <= 4; imot++) {
				for (ioff = 0; ioff <= 4; ioff++) {
					ifmo = dn*125+ifat*25+imot*5+ioff;			//combined index of trio
					smot = imot+1; if (smot>=4) smot++;
					for (t = 1; t <= T; t++) {
						if (Om2[t] > 1) {				//normally BAF>=0 and BAF<=1; we use BAF=1 to indicate a Non-Polymorphic marker
							if (!phmm1->NP_flag) nrerror ("FATAL ERROR: non-polymorphic (NP) probe detected in input data but HMM model does not contain parameters for them\n");
							biot[ifmo][t] += b1iot (smot, phmm1->B3_mean, phmm1->B3_sd, phmm1->B3_uf, Om1[t]);
						} else {
							biot[ifmo][t] += b1iot (smot, phmm1->B1_mean, phmm1->B1_sd, phmm1->B1_uf, Om1[t]);
							biot[ifmo][t] += b2iot (smot, phmm1->B2_mean, phmm1->B2_sd, phmm1->B2_uf, pfb[t], Om2[t]);
						}
					}
				}
			}
		}
	}
	adjustBSD (phmm1, trio_lrr_sd[2]);
	for (dn = 0; dn <= 1; dn++) {
		for (ifat = 0; ifat <= 4; ifat++) {
			for (imot = 0; imot <= 4; imot++) {
				for (ioff = 0; ioff <= 4; ioff++) {
					ifmo = dn*125+ifat*25+imot*5+ioff;			//combined index of trio
					soff = ioff+1; if (soff>=4) soff++;
					for (t = 1; t <= T; t++) {
						if (Oo2[t] > 1) {				//normally BAF>=0 and BAF<=1; we use BAF=1 to indicate a Non-Polymorphic marker
							if (!phmm1->NP_flag) nrerror ("FATAL ERROR: non-polymorphic (NP) probe detected in input data but HMM model does not contain parameters for them\n");
							biot[ifmo][t] += b1iot (soff, phmm1->B3_mean, phmm1->B3_sd, phmm1->B3_uf, Oo1[t]);
						} else {
							biot[ifmo][t] += b1iot (soff, phmm1->B1_mean, phmm1->B1_sd, phmm1->B1_uf, Oo1[t]);
							biot[ifmo][t] += b2iot (soff, phmm1->B2_mean, phmm1->B2_sd, phmm1->B2_uf, pfb[t], Oo2[t]);
						}
					}
				}
			}
		}
	}
	if (cn_count) printf ("NOTICE: Encounterd %i SNP probes and %i NP probes\n", snp_count, cn_count);
	//fprintf (stderr, "NOTICE: Finished building emission probability matrix for %i markers\n", T);

	/* 0. calculate initial PI values (state initiation log probability) */
	for (dn = 0; dn <= 1; dn++) {
		for (ifat = 0; ifat <= 4; ifat++) {		//copy number range from 0 to 4
			for (imot = 0; imot <= 4; imot++) {
				for (ioff = 0; ioff <= 4; ioff++) {
					ifmo = dn*125+ifat*25+imot*5+ioff;
					sfat = ifat+1; smot = imot+1; soff = ioff+1;
					if (sfat>=4) sfat++; if (smot>=4) smot++; if (soff>=4) soff++;
					if (chrx_flag) {
						if (sfat < 6) {
							triopi[ifmo] = log (phmm->pi[sfat+1]) + log (phmm->pi[smot]);
						} else {
							triopi[ifmo] = log (phmm->pi[sfat]) + log (phmm->pi[smot]);
						}
					} else {
						triopi[ifmo] = log (phmm->pi[sfat]) + log (phmm->pi[smot]);
					}
					if (dn==0) {
						if (cim[ifat][imot][ioff] == 0) {
							triopi[ifmo] += log (1-e1) + log (IMPOSSIBLE);
						} else {
							triopi[ifmo] += log (1-e1) + log (cim[ifat][imot][ioff]);		//inherited CNV
						}
					} else {
						triopi[ifmo] += log (e1) + log(1.0/5.0);		//denovo CNV
					}
				}
			}
		}
	}
	
        /* 1. Initialization  */
	for (dn = 0; dn <= 1; dn++) {
		for (ifat = 0; ifat <= 4; ifat++) {
			for (imot = 0; imot <= 4; imot++) {
				for (ioff = 0; ioff <= 4; ioff++) {
					ifmo = dn*125+ifat*25+imot*5+ioff;
					delta[1][ifmo] = triopi[ifmo] + biot[ifmo][1];
					psi[1][ifmo] = 0;
					//printf ("t=1 dn=%i ifat=%i imot=%i ioff=%i triopi=%f delta=%f psi=%i\n", dn, ifat, imot, ioff, triopi[ifmo], delta[1][ifmo], psi[1][ifmo]);
				}
			}
		}
	}

        /* 2. Recursion */
        for (t = 2; t <= T; t++) {
		convertHMMTransition (phmm, A1, snpdist[t-1]);			//t-1 is used because the current val is calculated from previous values

		for (dn = 0; dn <= 1; dn++) {
			for (ifat = 0; ifat <= 4; ifat++) {
				for (imot = 0; imot <= 4; imot++) {
					for (ioff = 0; ioff <= 4; ioff++) {
						ifmo = dn*125+ifat*25+imot*5+ioff;
						maxval = -VITHUGE;
						maxvalind = 1;
						for (pdn = 0; pdn <= 1; pdn++) {
							for (pifat = 0; pifat <= 4; pifat++) {
								for (pimot = 0; pimot <= 4; pimot++) {
									for (pioff = 0; pioff <= 4; pioff++) {
										pifmo = pdn*125+pifat*25+pimot*5+pioff;
										val = delta[t-1][pifmo];
										val += log (trioTransition (cim[ifat][imot][ioff], chit[pifat][ifat][pimot][imot][pioff][ioff], A1, e1, e2, snpdist[t-1], pdn, pifat, pimot, pioff, dn, ifat, imot, ioff));

										if (val > maxval) {
											maxval = val;
											maxvalind = pifmo;
										}
									}
								}
							}
						}
						delta[t][ifmo] = maxval + biot[ifmo][t];
						psi[t][ifmo] = maxvalind;
						//if (t==4) printf ("t=4 dn=%i ifat=%i imot=%i ioff=%i val=%f delta=%f psi=%i\n", dn, ifat, imot, ioff, val, delta[t][ifmo], psi[t][ifmo]);
					}
				}
			}
		}
		if (t % 1000 == 0) fprintf(stderr, "NOTICE: Finished recursion cycle %i in Viterbi algorithm\n", t);
	}

        /* 3. Termination */
        *plogproba = -VITHUGE;
        q[T] = 0;
        for (dn = 0; dn <= 1; dn++) {
	        for (ifat = 0; ifat <= 4; ifat++) {
	        	for (imot = 0; imot <= 4; imot++) {
	        		for (ioff = 0; ioff <= 4; ioff++) {
	        			ifmo = dn*125+ifat*25+imot*5+ioff;
	        			if (delta[T][ifmo] > *plogproba) {
	        				*plogproba = delta[T][ifmo];
	        				q[T] = ifmo;
					}
				}
			}
		}
	}

	/* 4. Path (state sequence) backtracking */
	for (t = T-1; t >= 1; t--)
		q[t] = psi[t+1][q[t+1]];

	for (t=1; t<=T; t++) snpdist[t]=q[t];			// assign the mostly likely state value to PFB array, which helds symbol value

	free_dmatrix(biot, 0, trioN, 1, T);
	free_dmatrix(A1, 1, phmm->N, 1, phmm->N);
	free_ivector(q, 1, T);
	free_imatrix(psi, 1, T, 0, trioN);
	free_dmatrix(delta, 1, T, 0, trioN);
	free_dvector(triopi, 0, trioN);
	FreeCHMM (phmm1);
}

double trioTransition (double cimcell, double chit, double **A1, double e1, double e2, int dist, int pdn, int pifat, int pimot, int pioff, int dn, int ifat, int imot, int ioff)
{
	int sfat, smot, soff;			//state for father, mother and offspring (used in A1 transition matrix)
	int psfat, psmot, psoff;		//previous state for father, mother and offspring
	//note: e1 is from normal to denovo, e2 is from denovo to normal
		
	sfat = ifat+1; smot = imot+1; soff = ioff+1;
	if (sfat >= 4) sfat++; if (smot >= 4) smot++; if (soff >= 4) soff++;
	psfat = pifat+1; psmot = pimot+1; psoff = pioff+1;
	if (psfat >= 4) psfat++; if (psmot >= 4) psmot++; if (psoff >= 4) psoff++;
	
	/* the DN transition matrix is a 2X2 matrix as specified below:
	DN	0	1
	0	1-e1	e1
	1	e2	1-e2
	For simplicity, we do not use distance-dependent adjustment
	*/
	
	if (pdn == 1) {
		if (dn == 1) {
			return A1[psfat][sfat] * A1[psmot][smot] * A1[psoff][soff] * (1 - e2);		//same denovo region, consider transition for offspring only
		} else {
			if (cimcell == 0) {
				return IMPOSSIBLE;
			} else {
				return A1[psfat][sfat] * A1[psmot][smot] * cimcell * e2;		//from denovo to inherited
			}
		}
	} else if (pdn == 0) {
		if (dn == 1) {
			return A1[psfat][sfat] * A1[psmot][smot] * (1.0/5.0) * e1;			//from inherited to denovo
		} else {
			if (chit == 0) {
				return IMPOSSIBLE;
			}
			
			return A1[psfat][sfat] * A1[psmot][smot] * chit * (1 - e1);			//same inheritance region. need to consider haplotype inheritance
		}
	}
	return 0.0;
}

void calculateCHIT (double a, double M[5][5][5][5][5][5])
{
	int f1, f2, m1, m2, o1, o2;
	for (f1=0; f1<=4; f1++) {
		for (f2=0; f2<=4; f2++) {
			for (m1=0; m1<=4; m1++) {
				for (m2=0; m2<=4; m2++) {
					for (o1=0; o1<=4; o1++) {
						for (o2=0; o2<=4; o2++) {
							M[f1][f2][m1][m2][o1][o2] = 0;
						}
					}
				}
			}
		}
	}
	M[0][0][0][0][0][0]=1*1/4.0+1*1/4.0+1*1/4.0+1*1/4.0;
	M[0][0][0][1][0][0]=1*1/4.0+1*1/4.0;
	M[0][0][0][1][0][1]=1*1/4.0+1*1/4.0;
	M[0][0][0][2][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][0][2][0][1]=1*(1-a)/4.0+1*(1-a)/4.0+1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][0][2][0][2]=1*a/4.0+1*a/4.0;
	M[0][0][0][3][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][0][3][0][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][0][3][0][2]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][0][3][0][3]=1*a/4.0+1*a/4.0;
	M[0][0][0][4][0][1]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][0][4][0][2]=1*0.5/4.0+1*0.5/4.0+1*0.5/4.0+1*0.5/4.0;
	M[0][0][0][4][0][3]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][1][0][0][0]=1*1/4.0+1*1/4.0;
	M[0][0][1][0][1][0]=1*1/4.0+1*1/4.0;
	M[0][0][1][1][0][0]=1*1/4.0+1*1/4.0;
	M[0][0][1][1][1][1]=1*1/4.0+1*1/4.0;
	M[0][0][1][2][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][1][2][0][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][1][2][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][1][2][1][2]=1*a/4.0+1*a/4.0;
	M[0][0][1][3][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][1][3][0][2]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][1][3][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][1][3][1][3]=1*a/4.0+1*a/4.0;
	M[0][0][1][4][0][2]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][1][4][0][3]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][1][4][1][1]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][1][4][1][2]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][2][0][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][2][0][1][0]=1*(1-a)/4.0+1*(1-a)/4.0+1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][2][0][2][0]=1*a/4.0+1*a/4.0;
	M[0][0][2][1][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][2][1][1][0]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][2][1][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][2][1][2][1]=1*a/4.0+1*a/4.0;
	M[0][0][2][2][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][2][2][1][1]=1*(1-a)/4.0+1*(1-a)/4.0+1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][2][2][2][2]=1*a/4.0+1*a/4.0;
	M[0][0][2][3][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][2][3][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][2][3][1][2]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][2][3][2][3]=1*a/4.0+1*a/4.0;
	M[0][0][2][4][0][2]=1*a/4.0+1*a/4.0;
	M[0][0][2][4][1][1]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[0][0][2][4][1][2]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[0][0][2][4][1][3]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[0][0][2][4][2][2]=1*a/4.0+1*a/4.0;
	M[0][0][3][0][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][3][0][1][0]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][3][0][2][0]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][3][0][3][0]=1*a/4.0+1*a/4.0;
	M[0][0][3][1][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][3][1][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][3][1][2][0]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][3][1][3][1]=1*a/4.0+1*a/4.0;
	M[0][0][3][2][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][3][2][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][3][2][2][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][3][2][3][2]=1*a/4.0+1*a/4.0;
	M[0][0][3][3][0][0]=1*a/4.0+1*a/4.0;
	M[0][0][3][3][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][3][3][2][2]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][0][3][3][3][3]=1*a/4.0+1*a/4.0;
	M[0][0][3][4][1][1]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][3][4][1][2]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][3][4][2][2]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][3][4][2][3]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][0][1][0]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][0][2][0]=1*0.5/4.0+1*0.5/4.0+1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][0][3][0]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][1][1][1]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][1][2][0]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][1][2][1]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][1][3][0]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][2][1][1]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[0][0][4][2][2][0]=1*a/4.0+1*a/4.0;
	M[0][0][4][2][2][1]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[0][0][4][2][2][2]=1*a/4.0+1*a/4.0;
	M[0][0][4][2][3][1]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[0][0][4][3][1][1]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][3][2][1]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][3][2][2]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][3][3][2]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][4][1][1]=1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][4][2][2]=1*0.5/4.0+1*0.5/4.0+1*0.5/4.0+1*0.5/4.0;
	M[0][0][4][4][3][3]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][0][0][0][0]=1*1/4.0+1*1/4.0;
	M[0][1][0][0][0][1]=1*1/4.0+1*1/4.0;
	M[0][1][0][1][0][0]=1*1/4.0;
	M[0][1][0][1][0][1]=1*1/4.0+1*1/4.0;
	M[0][1][0][1][0][2]=1*1/4.0;
	M[0][1][0][2][0][0]=1*a/4.0;
	M[0][1][0][2][0][1]=1*a/4.0+1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][1][0][2][0][2]=1*a/4.0+1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][1][0][2][0][3]=1*a/4.0;
	M[0][1][0][3][0][0]=1*a/4.0;
	M[0][1][0][3][0][1]=1*a/4.0+1*(1-a)/4.0;
	M[0][1][0][3][0][2]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][1][0][3][0][3]=1*a/4.0+1*(1-a)/4.0;
	M[0][1][0][3][0][4]=1*a/4.0;
	M[0][1][0][4][0][1]=1*0.5/4.0;
	M[0][1][0][4][0][2]=1*0.5/4.0+1*0.5/4.0+1*0.5/4.0;
	M[0][1][0][4][0][3]=1*0.5/4.0+1*0.5/4.0+1*0.5/4.0;
	M[0][1][0][4][0][4]=1*0.5/4.0;
	M[0][1][1][0][0][0]=1*1/4.0;
	M[0][1][1][0][0][1]=1*1/4.0;
	M[0][1][1][0][1][0]=1*1/4.0;
	M[0][1][1][0][1][1]=1*1/4.0;
	M[0][1][1][1][0][0]=1*1/4.0;
	M[0][1][1][1][0][1]=1*1/4.0;
	M[0][1][1][1][1][1]=1*1/4.0;
	M[0][1][1][1][1][2]=1*1/4.0;
	M[0][1][1][2][0][0]=1*a/4.0;
	M[0][1][1][2][0][1]=1*a/4.0+1*(1-a)/4.0;
	M[0][1][1][2][0][2]=1*(1-a)/4.0;
	M[0][1][1][2][1][1]=1*(1-a)/4.0;
	M[0][1][1][2][1][2]=1*a/4.0+1*(1-a)/4.0;
	M[0][1][1][2][1][3]=1*a/4.0;
	M[0][1][1][3][0][0]=1*a/4.0;
	M[0][1][1][3][0][1]=1*a/4.0;
	M[0][1][1][3][0][2]=1*(1-a)/4.0;
	M[0][1][1][3][0][3]=1*(1-a)/4.0;
	M[0][1][1][3][1][1]=1*(1-a)/4.0;
	M[0][1][1][3][1][2]=1*(1-a)/4.0;
	M[0][1][1][3][1][3]=1*a/4.0;
	M[0][1][1][3][1][4]=1*a/4.0;
	M[0][1][1][4][0][2]=1*0.5/4.0;
	M[0][1][1][4][0][3]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][1][4][0][4]=1*0.5/4.0;
	M[0][1][1][4][1][1]=1*0.5/4.0;
	M[0][1][1][4][1][2]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][1][4][1][3]=1*0.5/4.0;
	M[0][1][2][0][0][0]=1*a/4.0;
	M[0][1][2][0][0][1]=1*a/4.0;
	M[0][1][2][0][1][0]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][1][2][0][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][1][2][0][2][0]=1*a/4.0;
	M[0][1][2][0][2][1]=1*a/4.0;
	M[0][1][2][1][0][0]=1*a/4.0;
	M[0][1][2][1][0][1]=1*a/4.0;
	M[0][1][2][1][1][0]=1*(1-a)/4.0;
	M[0][1][2][1][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][1][2][1][1][2]=1*(1-a)/4.0;
	M[0][1][2][1][2][1]=1*a/4.0;
	M[0][1][2][1][2][2]=1*a/4.0;
	M[0][1][2][2][0][0]=1*a/4.0;
	M[0][1][2][2][0][1]=1*a/4.0;
	M[0][1][2][2][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][1][2][2][1][2]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][1][2][2][2][2]=1*a/4.0;
	M[0][1][2][2][2][3]=1*a/4.0;
	M[0][1][2][3][0][0]=1*a/4.0;
	M[0][1][2][3][0][1]=1*a/4.0;
	M[0][1][2][3][1][1]=1*(1-a)/4.0;
	M[0][1][2][3][1][2]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[0][1][2][3][1][3]=1*(1-a)/4.0;
	M[0][1][2][3][2][3]=1*a/4.0;
	M[0][1][2][3][2][4]=1*a/4.0;
	M[0][1][2][4][0][2]=1*a/4.0;
	M[0][1][2][4][0][3]=1*a/4.0;
	M[0][1][2][4][1][1]=1*(0.5-0.5*a)/4.0;
	M[0][1][2][4][1][2]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[0][1][2][4][1][3]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[0][1][2][4][1][4]=1*(0.5-0.5*a)/4.0;
	M[0][1][2][4][2][2]=1*a/4.0;
	M[0][1][2][4][2][3]=1*a/4.0;
	M[0][1][3][0][0][0]=1*a/4.0;
	M[0][1][3][0][0][1]=1*a/4.0;
	M[0][1][3][0][1][0]=1*(1-a)/4.0;
	M[0][1][3][0][1][1]=1*(1-a)/4.0;
	M[0][1][3][0][2][0]=1*(1-a)/4.0;
	M[0][1][3][0][2][1]=1*(1-a)/4.0;
	M[0][1][3][0][3][0]=1*a/4.0;
	M[0][1][3][0][3][1]=1*a/4.0;
	M[0][1][3][1][0][0]=1*a/4.0;
	M[0][1][3][1][0][1]=1*a/4.0;
	M[0][1][3][1][1][1]=1*(1-a)/4.0;
	M[0][1][3][1][1][2]=1*(1-a)/4.0;
	M[0][1][3][1][2][0]=1*(1-a)/4.0;
	M[0][1][3][1][2][1]=1*(1-a)/4.0;
	M[0][1][3][1][3][1]=1*a/4.0;
	M[0][1][3][1][3][2]=1*a/4.0;
	M[0][1][3][2][0][0]=1*a/4.0;
	M[0][1][3][2][0][1]=1*a/4.0;
	M[0][1][3][2][1][1]=1*(1-a)/4.0;
	M[0][1][3][2][1][2]=1*(1-a)/4.0;
	M[0][1][3][2][2][1]=1*(1-a)/4.0;
	M[0][1][3][2][2][2]=1*(1-a)/4.0;
	M[0][1][3][2][3][2]=1*a/4.0;
	M[0][1][3][2][3][3]=1*a/4.0;
	M[0][1][3][3][0][0]=1*a/4.0;
	M[0][1][3][3][0][1]=1*a/4.0;
	M[0][1][3][3][1][1]=1*(1-a)/4.0;
	M[0][1][3][3][1][2]=1*(1-a)/4.0;
	M[0][1][3][3][2][2]=1*(1-a)/4.0;
	M[0][1][3][3][2][3]=1*(1-a)/4.0;
	M[0][1][3][3][3][3]=1*a/4.0;
	M[0][1][3][3][3][4]=1*a/4.0;
	M[0][1][3][4][1][1]=1*0.5/4.0;
	M[0][1][3][4][1][2]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][3][4][1][3]=1*0.5/4.0;
	M[0][1][3][4][2][2]=1*0.5/4.0;
	M[0][1][3][4][2][3]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][3][4][2][4]=1*0.5/4.0;
	M[0][1][4][0][1][0]=1*0.5/4.0;
	M[0][1][4][0][1][1]=1*0.5/4.0;
	M[0][1][4][0][2][0]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][4][0][2][1]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][4][0][3][0]=1*0.5/4.0;
	M[0][1][4][0][3][1]=1*0.5/4.0;
	M[0][1][4][1][1][1]=1*0.5/4.0;
	M[0][1][4][1][1][2]=1*0.5/4.0;
	M[0][1][4][1][2][0]=1*0.5/4.0;
	M[0][1][4][1][2][1]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][4][1][2][2]=1*0.5/4.0;
	M[0][1][4][1][3][0]=1*0.5/4.0;
	M[0][1][4][1][3][1]=1*0.5/4.0;
	M[0][1][4][2][1][1]=1*(0.5-0.5*a)/4.0;
	M[0][1][4][2][1][2]=1*(0.5-0.5*a)/4.0;
	M[0][1][4][2][2][0]=1*a/4.0;
	M[0][1][4][2][2][1]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*a/4.0;
	M[0][1][4][2][2][2]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*a/4.0;
	M[0][1][4][2][2][3]=1*a/4.0;
	M[0][1][4][2][3][1]=1*(0.5-0.5*a)/4.0;
	M[0][1][4][2][3][2]=1*(0.5-0.5*a)/4.0;
	M[0][1][4][3][1][1]=1*0.5/4.0;
	M[0][1][4][3][1][2]=1*0.5/4.0;
	M[0][1][4][3][2][1]=1*0.5/4.0;
	M[0][1][4][3][2][2]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][4][3][2][3]=1*0.5/4.0;
	M[0][1][4][3][3][2]=1*0.5/4.0;
	M[0][1][4][3][3][3]=1*0.5/4.0;
	M[0][1][4][4][1][1]=1*0.5/4.0;
	M[0][1][4][4][1][2]=1*0.5/4.0;
	M[0][1][4][4][2][2]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][4][4][2][3]=1*0.5/4.0+1*0.5/4.0;
	M[0][1][4][4][3][3]=1*0.5/4.0;
	M[0][1][4][4][3][4]=1*0.5/4.0;
	M[0][2][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[0][2][0][0][0][1]=(1-a)*1/4.0+(1-a)*1/4.0+(1-a)*1/4.0+(1-a)*1/4.0;
	M[0][2][0][0][0][2]=a*1/4.0+a*1/4.0;
	M[0][2][0][1][0][0]=a*1/4.0;
	M[0][2][0][1][0][1]=a*1/4.0+(1-a)*1/4.0+(1-a)*1/4.0;
	M[0][2][0][1][0][2]=a*1/4.0+(1-a)*1/4.0+(1-a)*1/4.0;
	M[0][2][0][1][0][3]=a*1/4.0;
	M[0][2][0][2][0][0]=a*a/4.0;
	M[0][2][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][0][2][0][2]=a*a/4.0+a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][0][2][0][3]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][0][2][0][4]=a*a/4.0;
	M[0][2][0][3][0][0]=a*a/4.0;
	M[0][2][0][3][0][1]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][0][3][0][2]=a*a/4.0+a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][0][3][0][3]=a*a/4.0+a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][0][3][0][4]=a*a/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][0][4][0][1]=a*0.5/4.0;
	M[0][2][0][4][0][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][0][4][0][3]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][0][4][0][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][1][0][0][0]=a*1/4.0;
	M[0][2][1][0][0][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[0][2][1][0][0][2]=a*1/4.0;
	M[0][2][1][0][1][0]=a*1/4.0;
	M[0][2][1][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[0][2][1][0][1][2]=a*1/4.0;
	M[0][2][1][1][0][0]=a*1/4.0;
	M[0][2][1][1][0][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[0][2][1][1][0][2]=a*1/4.0;
	M[0][2][1][1][1][1]=a*1/4.0;
	M[0][2][1][1][1][2]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[0][2][1][1][1][3]=a*1/4.0;
	M[0][2][1][2][0][0]=a*a/4.0;
	M[0][2][1][2][0][1]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][1][2][0][2]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][1][2][0][3]=a*(1-a)/4.0;
	M[0][2][1][2][1][1]=a*(1-a)/4.0;
	M[0][2][1][2][1][2]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][1][2][1][3]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][1][2][1][4]=a*a/4.0;
	M[0][2][1][3][0][0]=a*a/4.0;
	M[0][2][1][3][0][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][1][3][0][2]=a*a/4.0+a*(1-a)/4.0;
	M[0][2][1][3][0][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][1][3][0][4]=a*(1-a)/4.0;
	M[0][2][1][3][1][1]=a*(1-a)/4.0;
	M[0][2][1][3][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][1][3][1][3]=a*a/4.0+a*(1-a)/4.0;
	M[0][2][1][3][1][4]=a*a/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][1][4][0][2]=a*0.5/4.0;
	M[0][2][1][4][0][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][1][4][0][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][1][4][1][1]=a*0.5/4.0;
	M[0][2][1][4][1][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][1][4][1][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][1][4][1][4]=a*0.5/4.0;
	M[0][2][2][0][0][0]=a*a/4.0;
	M[0][2][2][0][0][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][2][0][0][2]=a*a/4.0;
	M[0][2][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[0][2][2][0][1][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][2][0][1][2]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[0][2][2][0][2][0]=a*a/4.0;
	M[0][2][2][0][2][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][2][0][2][2]=a*a/4.0;
	M[0][2][2][1][0][0]=a*a/4.0;
	M[0][2][2][1][0][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][2][1][0][2]=a*a/4.0;
	M[0][2][2][1][1][0]=a*(1-a)/4.0;
	M[0][2][2][1][1][1]=a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][2][1][1][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][2][1][1][3]=a*(1-a)/4.0;
	M[0][2][2][1][2][1]=a*a/4.0;
	M[0][2][2][1][2][2]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][2][1][2][3]=a*a/4.0;
	M[0][2][2][2][0][0]=a*a/4.0;
	M[0][2][2][2][0][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][2][2][0][2]=a*a/4.0;
	M[0][2][2][2][1][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[0][2][2][2][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][2][2][1][3]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[0][2][2][2][2][2]=a*a/4.0;
	M[0][2][2][2][2][3]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][2][2][2][4]=a*a/4.0;
	M[0][2][2][3][0][0]=a*a/4.0;
	M[0][2][2][3][0][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][2][3][0][2]=a*a/4.0;
	M[0][2][2][3][1][1]=a*(1-a)/4.0;
	M[0][2][2][3][1][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][2][3][1][3]=a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][2][3][1][4]=a*(1-a)/4.0;
	M[0][2][2][3][2][3]=a*a/4.0;
	M[0][2][2][3][2][4]=a*a/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][2][4][0][2]=a*a/4.0;
	M[0][2][2][4][0][3]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][2][4][0][4]=a*a/4.0;
	M[0][2][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[0][2][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[0][2][2][4][1][3]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[0][2][2][4][1][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[0][2][2][4][2][2]=a*a/4.0;
	M[0][2][2][4][2][3]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][2][4][2][4]=a*a/4.0;
	M[0][2][3][0][0][0]=a*a/4.0;
	M[0][2][3][0][0][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][3][0][0][2]=a*a/4.0;
	M[0][2][3][0][1][0]=a*(1-a)/4.0;
	M[0][2][3][0][1][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][3][0][1][2]=a*(1-a)/4.0;
	M[0][2][3][0][2][0]=a*(1-a)/4.0;
	M[0][2][3][0][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][3][0][2][2]=a*(1-a)/4.0;
	M[0][2][3][0][3][0]=a*a/4.0;
	M[0][2][3][0][3][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][3][0][3][2]=a*a/4.0;
	M[0][2][3][1][0][0]=a*a/4.0;
	M[0][2][3][1][0][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][3][1][0][2]=a*a/4.0;
	M[0][2][3][1][1][1]=a*(1-a)/4.0;
	M[0][2][3][1][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][3][1][1][3]=a*(1-a)/4.0;
	M[0][2][3][1][2][0]=a*(1-a)/4.0;
	M[0][2][3][1][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][3][1][2][2]=a*(1-a)/4.0;
	M[0][2][3][1][3][1]=a*a/4.0;
	M[0][2][3][1][3][2]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][3][1][3][3]=a*a/4.0;
	M[0][2][3][2][0][0]=a*a/4.0;
	M[0][2][3][2][0][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][3][2][0][2]=a*a/4.0;
	M[0][2][3][2][1][1]=a*(1-a)/4.0;
	M[0][2][3][2][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][3][2][1][3]=a*(1-a)/4.0;
	M[0][2][3][2][2][1]=a*(1-a)/4.0;
	M[0][2][3][2][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][3][2][2][3]=a*(1-a)/4.0;
	M[0][2][3][2][3][2]=a*a/4.0;
	M[0][2][3][2][3][3]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][3][2][3][4]=a*a/4.0;
	M[0][2][3][3][0][0]=a*a/4.0;
	M[0][2][3][3][0][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][3][3][0][2]=a*a/4.0;
	M[0][2][3][3][1][1]=a*(1-a)/4.0;
	M[0][2][3][3][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][3][3][1][3]=a*(1-a)/4.0;
	M[0][2][3][3][2][2]=a*(1-a)/4.0;
	M[0][2][3][3][2][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][2][3][3][2][4]=a*(1-a)/4.0;
	M[0][2][3][3][3][3]=a*a/4.0;
	M[0][2][3][3][3][4]=a*a/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][3][4][1][1]=a*0.5/4.0;
	M[0][2][3][4][1][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][3][4][1][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][3][4][1][4]=a*0.5/4.0;
	M[0][2][3][4][2][2]=a*0.5/4.0;
	M[0][2][3][4][2][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][3][4][2][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][0][1][0]=a*0.5/4.0;
	M[0][2][4][0][1][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][0][1][2]=a*0.5/4.0;
	M[0][2][4][0][2][0]=a*0.5/4.0+a*0.5/4.0;
	M[0][2][4][0][2][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][0][2][2]=a*0.5/4.0+a*0.5/4.0;
	M[0][2][4][0][3][0]=a*0.5/4.0;
	M[0][2][4][0][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][0][3][2]=a*0.5/4.0;
	M[0][2][4][1][1][1]=a*0.5/4.0;
	M[0][2][4][1][1][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][1][1][3]=a*0.5/4.0;
	M[0][2][4][1][2][0]=a*0.5/4.0;
	M[0][2][4][1][2][1]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][1][2][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][1][2][3]=a*0.5/4.0;
	M[0][2][4][1][3][0]=a*0.5/4.0;
	M[0][2][4][1][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][1][3][2]=a*0.5/4.0;
	M[0][2][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[0][2][4][2][1][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[0][2][4][2][1][3]=a*(0.5-0.5*a)/4.0;
	M[0][2][4][2][2][0]=a*a/4.0;
	M[0][2][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][4][2][2][2]=a*a/4.0+a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[0][2][4][2][2][3]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][2][4][2][2][4]=a*a/4.0;
	M[0][2][4][2][3][1]=a*(0.5-0.5*a)/4.0;
	M[0][2][4][2][3][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[0][2][4][2][3][3]=a*(0.5-0.5*a)/4.0;
	M[0][2][4][3][1][1]=a*0.5/4.0;
	M[0][2][4][3][1][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][3][1][3]=a*0.5/4.0;
	M[0][2][4][3][2][1]=a*0.5/4.0;
	M[0][2][4][3][2][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][3][2][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][3][2][4]=a*0.5/4.0;
	M[0][2][4][3][3][2]=a*0.5/4.0;
	M[0][2][4][3][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][3][3][4]=a*0.5/4.0;
	M[0][2][4][4][1][1]=a*0.5/4.0;
	M[0][2][4][4][1][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][4][1][3]=a*0.5/4.0;
	M[0][2][4][4][2][2]=a*0.5/4.0+a*0.5/4.0;
	M[0][2][4][4][2][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][2][4][4][2][4]=a*0.5/4.0+a*0.5/4.0;
	M[0][2][4][4][3][3]=a*0.5/4.0;
	M[0][2][4][4][3][4]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[0][3][0][0][0][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[0][3][0][0][0][2]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[0][3][0][0][0][3]=a*1/4.0+a*1/4.0;
	M[0][3][0][1][0][0]=a*1/4.0;
	M[0][3][0][1][0][1]=a*1/4.0+(1-a)*1/4.0;
	M[0][3][0][1][0][2]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[0][3][0][1][0][3]=a*1/4.0+(1-a)*1/4.0;
	M[0][3][0][1][0][4]=a*1/4.0;
	M[0][3][0][2][0][0]=a*a/4.0;
	M[0][3][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[0][3][0][2][0][2]=a*a/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][0][2][0][3]=a*a/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][0][2][0][4]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[0][3][0][3][0][0]=a*a/4.0;
	M[0][3][0][3][0][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[0][3][0][3][0][2]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[0][3][0][3][0][3]=a*a/4.0+a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][0][3][0][4]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[0][3][0][4][0][1]=a*0.5/4.0;
	M[0][3][0][4][0][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][0][4][0][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][0][4][0][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][1][0][0][0]=a*1/4.0;
	M[0][3][1][0][0][1]=(1-a)*1/4.0;
	M[0][3][1][0][0][2]=(1-a)*1/4.0;
	M[0][3][1][0][0][3]=a*1/4.0;
	M[0][3][1][0][1][0]=a*1/4.0;
	M[0][3][1][0][1][1]=(1-a)*1/4.0;
	M[0][3][1][0][1][2]=(1-a)*1/4.0;
	M[0][3][1][0][1][3]=a*1/4.0;
	M[0][3][1][1][0][0]=a*1/4.0;
	M[0][3][1][1][0][1]=(1-a)*1/4.0;
	M[0][3][1][1][0][2]=(1-a)*1/4.0;
	M[0][3][1][1][0][3]=a*1/4.0;
	M[0][3][1][1][1][1]=a*1/4.0;
	M[0][3][1][1][1][2]=(1-a)*1/4.0;
	M[0][3][1][1][1][3]=(1-a)*1/4.0;
	M[0][3][1][1][1][4]=a*1/4.0;
	M[0][3][1][2][0][0]=a*a/4.0;
	M[0][3][1][2][0][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[0][3][1][2][0][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[0][3][1][2][0][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[0][3][1][2][0][4]=a*(1-a)/4.0;
	M[0][3][1][2][1][1]=a*(1-a)/4.0;
	M[0][3][1][2][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[0][3][1][2][1][3]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[0][3][1][2][1][4]=a*a/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[0][3][1][3][0][0]=a*a/4.0;
	M[0][3][1][3][0][1]=(1-a)*a/4.0;
	M[0][3][1][3][0][2]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[0][3][1][3][0][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[0][3][1][3][0][4]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][1][3][1][1]=a*(1-a)/4.0;
	M[0][3][1][3][1][2]=(1-a)*(1-a)/4.0;
	M[0][3][1][3][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[0][3][1][3][1][4]=a*a/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][3][1][4][0][2]=a*0.5/4.0;
	M[0][3][1][4][0][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][1][4][0][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][1][4][1][1]=a*0.5/4.0;
	M[0][3][1][4][1][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][1][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][1][4][1][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][2][0][0][0]=a*a/4.0;
	M[0][3][2][0][0][1]=(1-a)*a/4.0;
	M[0][3][2][0][0][2]=(1-a)*a/4.0;
	M[0][3][2][0][0][3]=a*a/4.0;
	M[0][3][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[0][3][2][0][1][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][2][0][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][2][0][1][3]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[0][3][2][0][2][0]=a*a/4.0;
	M[0][3][2][0][2][1]=(1-a)*a/4.0;
	M[0][3][2][0][2][2]=(1-a)*a/4.0;
	M[0][3][2][0][2][3]=a*a/4.0;
	M[0][3][2][1][0][0]=a*a/4.0;
	M[0][3][2][1][0][1]=(1-a)*a/4.0;
	M[0][3][2][1][0][2]=(1-a)*a/4.0;
	M[0][3][2][1][0][3]=a*a/4.0;
	M[0][3][2][1][1][0]=a*(1-a)/4.0;
	M[0][3][2][1][1][1]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][2][1][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][2][1][1][3]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][2][1][1][4]=a*(1-a)/4.0;
	M[0][3][2][1][2][1]=a*a/4.0;
	M[0][3][2][1][2][2]=(1-a)*a/4.0;
	M[0][3][2][1][2][3]=(1-a)*a/4.0;
	M[0][3][2][1][2][4]=a*a/4.0;
	M[0][3][2][2][0][0]=a*a/4.0;
	M[0][3][2][2][0][1]=(1-a)*a/4.0;
	M[0][3][2][2][0][2]=(1-a)*a/4.0;
	M[0][3][2][2][0][3]=a*a/4.0;
	M[0][3][2][2][1][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[0][3][2][2][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][2][2][1][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][2][2][1][4]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[0][3][2][2][2][2]=a*a/4.0;
	M[0][3][2][2][2][3]=(1-a)*a/4.0;
	M[0][3][2][2][2][4]=a*a/4.0+(1-a)*a/4.0;
	M[0][3][2][3][0][0]=a*a/4.0;
	M[0][3][2][3][0][1]=(1-a)*a/4.0;
	M[0][3][2][3][0][2]=(1-a)*a/4.0;
	M[0][3][2][3][0][3]=a*a/4.0;
	M[0][3][2][3][1][1]=a*(1-a)/4.0;
	M[0][3][2][3][1][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][2][3][1][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][2][3][1][4]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][2][3][2][3]=a*a/4.0;
	M[0][3][2][3][2][4]=a*a/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][3][2][4][0][2]=a*a/4.0;
	M[0][3][2][4][0][3]=(1-a)*a/4.0;
	M[0][3][2][4][0][4]=a*a/4.0+(1-a)*a/4.0;
	M[0][3][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[0][3][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[0][3][2][4][1][3]=a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[0][3][2][4][1][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[0][3][2][4][2][2]=a*a/4.0;
	M[0][3][2][4][2][3]=(1-a)*a/4.0;
	M[0][3][2][4][2][4]=a*a/4.0+(1-a)*a/4.0;
	M[0][3][3][0][0][0]=a*a/4.0;
	M[0][3][3][0][0][1]=(1-a)*a/4.0;
	M[0][3][3][0][0][2]=(1-a)*a/4.0;
	M[0][3][3][0][0][3]=a*a/4.0;
	M[0][3][3][0][1][0]=a*(1-a)/4.0;
	M[0][3][3][0][1][1]=(1-a)*(1-a)/4.0;
	M[0][3][3][0][1][2]=(1-a)*(1-a)/4.0;
	M[0][3][3][0][1][3]=a*(1-a)/4.0;
	M[0][3][3][0][2][0]=a*(1-a)/4.0;
	M[0][3][3][0][2][1]=(1-a)*(1-a)/4.0;
	M[0][3][3][0][2][2]=(1-a)*(1-a)/4.0;
	M[0][3][3][0][2][3]=a*(1-a)/4.0;
	M[0][3][3][0][3][0]=a*a/4.0;
	M[0][3][3][0][3][1]=(1-a)*a/4.0;
	M[0][3][3][0][3][2]=(1-a)*a/4.0;
	M[0][3][3][0][3][3]=a*a/4.0;
	M[0][3][3][1][0][0]=a*a/4.0;
	M[0][3][3][1][0][1]=(1-a)*a/4.0;
	M[0][3][3][1][0][2]=(1-a)*a/4.0;
	M[0][3][3][1][0][3]=a*a/4.0;
	M[0][3][3][1][1][1]=a*(1-a)/4.0;
	M[0][3][3][1][1][2]=(1-a)*(1-a)/4.0;
	M[0][3][3][1][1][3]=(1-a)*(1-a)/4.0;
	M[0][3][3][1][1][4]=a*(1-a)/4.0;
	M[0][3][3][1][2][0]=a*(1-a)/4.0;
	M[0][3][3][1][2][1]=(1-a)*(1-a)/4.0;
	M[0][3][3][1][2][2]=(1-a)*(1-a)/4.0;
	M[0][3][3][1][2][3]=a*(1-a)/4.0;
	M[0][3][3][1][3][1]=a*a/4.0;
	M[0][3][3][1][3][2]=(1-a)*a/4.0;
	M[0][3][3][1][3][3]=(1-a)*a/4.0;
	M[0][3][3][1][3][4]=a*a/4.0;
	M[0][3][3][2][0][0]=a*a/4.0;
	M[0][3][3][2][0][1]=(1-a)*a/4.0;
	M[0][3][3][2][0][2]=(1-a)*a/4.0;
	M[0][3][3][2][0][3]=a*a/4.0;
	M[0][3][3][2][1][1]=a*(1-a)/4.0;
	M[0][3][3][2][1][2]=(1-a)*(1-a)/4.0;
	M[0][3][3][2][1][3]=(1-a)*(1-a)/4.0;
	M[0][3][3][2][1][4]=a*(1-a)/4.0;
	M[0][3][3][2][2][1]=a*(1-a)/4.0;
	M[0][3][3][2][2][2]=(1-a)*(1-a)/4.0;
	M[0][3][3][2][2][3]=(1-a)*(1-a)/4.0;
	M[0][3][3][2][2][4]=a*(1-a)/4.0;
	M[0][3][3][2][3][2]=a*a/4.0;
	M[0][3][3][2][3][3]=(1-a)*a/4.0;
	M[0][3][3][2][3][4]=a*a/4.0+(1-a)*a/4.0;
	M[0][3][3][3][0][0]=a*a/4.0;
	M[0][3][3][3][0][1]=(1-a)*a/4.0;
	M[0][3][3][3][0][2]=(1-a)*a/4.0;
	M[0][3][3][3][0][3]=a*a/4.0;
	M[0][3][3][3][1][1]=a*(1-a)/4.0;
	M[0][3][3][3][1][2]=(1-a)*(1-a)/4.0;
	M[0][3][3][3][1][3]=(1-a)*(1-a)/4.0;
	M[0][3][3][3][1][4]=a*(1-a)/4.0;
	M[0][3][3][3][2][2]=a*(1-a)/4.0;
	M[0][3][3][3][2][3]=(1-a)*(1-a)/4.0;
	M[0][3][3][3][2][4]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[0][3][3][3][3][3]=a*a/4.0;
	M[0][3][3][3][3][4]=a*a/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[0][3][3][4][1][1]=a*0.5/4.0;
	M[0][3][3][4][1][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][3][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][3][4][1][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][3][4][2][2]=a*0.5/4.0;
	M[0][3][3][4][2][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][3][4][2][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][0][1][0]=a*0.5/4.0;
	M[0][3][4][0][1][1]=(1-a)*0.5/4.0;
	M[0][3][4][0][1][2]=(1-a)*0.5/4.0;
	M[0][3][4][0][1][3]=a*0.5/4.0;
	M[0][3][4][0][2][0]=a*0.5/4.0+a*0.5/4.0;
	M[0][3][4][0][2][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][0][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][0][2][3]=a*0.5/4.0+a*0.5/4.0;
	M[0][3][4][0][3][0]=a*0.5/4.0;
	M[0][3][4][0][3][1]=(1-a)*0.5/4.0;
	M[0][3][4][0][3][2]=(1-a)*0.5/4.0;
	M[0][3][4][0][3][3]=a*0.5/4.0;
	M[0][3][4][1][1][1]=a*0.5/4.0;
	M[0][3][4][1][1][2]=(1-a)*0.5/4.0;
	M[0][3][4][1][1][3]=(1-a)*0.5/4.0;
	M[0][3][4][1][1][4]=a*0.5/4.0;
	M[0][3][4][1][2][0]=a*0.5/4.0;
	M[0][3][4][1][2][1]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][1][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][1][2][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][1][2][4]=a*0.5/4.0;
	M[0][3][4][1][3][0]=a*0.5/4.0;
	M[0][3][4][1][3][1]=(1-a)*0.5/4.0;
	M[0][3][4][1][3][2]=(1-a)*0.5/4.0;
	M[0][3][4][1][3][3]=a*0.5/4.0;
	M[0][3][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[0][3][4][2][1][2]=(1-a)*(0.5-0.5*a)/4.0;
	M[0][3][4][2][1][3]=(1-a)*(0.5-0.5*a)/4.0;
	M[0][3][4][2][1][4]=a*(0.5-0.5*a)/4.0;
	M[0][3][4][2][2][0]=a*a/4.0;
	M[0][3][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[0][3][4][2][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[0][3][4][2][2][3]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[0][3][4][2][2][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*a/4.0+(1-a)*a/4.0;
	M[0][3][4][2][3][1]=a*(0.5-0.5*a)/4.0;
	M[0][3][4][2][3][2]=(1-a)*(0.5-0.5*a)/4.0;
	M[0][3][4][2][3][3]=(1-a)*(0.5-0.5*a)/4.0;
	M[0][3][4][2][3][4]=a*(0.5-0.5*a)/4.0;
	M[0][3][4][3][1][1]=a*0.5/4.0;
	M[0][3][4][3][1][2]=(1-a)*0.5/4.0;
	M[0][3][4][3][1][3]=(1-a)*0.5/4.0;
	M[0][3][4][3][1][4]=a*0.5/4.0;
	M[0][3][4][3][2][1]=a*0.5/4.0;
	M[0][3][4][3][2][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][3][2][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][3][2][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][3][3][2]=a*0.5/4.0;
	M[0][3][4][3][3][3]=(1-a)*0.5/4.0;
	M[0][3][4][3][3][4]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][4][1][1]=a*0.5/4.0;
	M[0][3][4][4][1][2]=(1-a)*0.5/4.0;
	M[0][3][4][4][1][3]=(1-a)*0.5/4.0;
	M[0][3][4][4][1][4]=a*0.5/4.0;
	M[0][3][4][4][2][2]=a*0.5/4.0+a*0.5/4.0;
	M[0][3][4][4][2][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][4][2][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][3][4][4][3][3]=a*0.5/4.0;
	M[0][3][4][4][3][4]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[0][4][0][0][0][1]=0.5*1/4.0+0.5*1/4.0;
	M[0][4][0][0][0][2]=0.5*1/4.0+0.5*1/4.0+0.5*1/4.0+0.5*1/4.0;
	M[0][4][0][0][0][3]=0.5*1/4.0+0.5*1/4.0;
	M[0][4][0][1][0][1]=0.5*1/4.0;
	M[0][4][0][1][0][2]=0.5*1/4.0+0.5*1/4.0+0.5*1/4.0;
	M[0][4][0][1][0][3]=0.5*1/4.0+0.5*1/4.0+0.5*1/4.0;
	M[0][4][0][1][0][4]=0.5*1/4.0;
	M[0][4][0][2][0][1]=0.5*a/4.0;
	M[0][4][0][2][0][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][0][2][0][3]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][0][2][0][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][0][3][0][1]=0.5*a/4.0;
	M[0][4][0][3][0][2]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][0][3][0][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][0][3][0][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][0][4][0][2]=0.5*0.5/4.0;
	M[0][4][0][4][0][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][0][4][0][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][1][0][0][1]=0.5*1/4.0;
	M[0][4][1][0][0][2]=0.5*1/4.0+0.5*1/4.0;
	M[0][4][1][0][0][3]=0.5*1/4.0;
	M[0][4][1][0][1][1]=0.5*1/4.0;
	M[0][4][1][0][1][2]=0.5*1/4.0+0.5*1/4.0;
	M[0][4][1][0][1][3]=0.5*1/4.0;
	M[0][4][1][1][0][1]=0.5*1/4.0;
	M[0][4][1][1][0][2]=0.5*1/4.0+0.5*1/4.0;
	M[0][4][1][1][0][3]=0.5*1/4.0;
	M[0][4][1][1][1][2]=0.5*1/4.0;
	M[0][4][1][1][1][3]=0.5*1/4.0+0.5*1/4.0;
	M[0][4][1][1][1][4]=0.5*1/4.0;
	M[0][4][1][2][0][1]=0.5*a/4.0;
	M[0][4][1][2][0][2]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][1][2][0][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][1][2][0][4]=0.5*(1-a)/4.0;
	M[0][4][1][2][1][2]=0.5*(1-a)/4.0;
	M[0][4][1][2][1][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][1][2][1][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][1][3][0][1]=0.5*a/4.0;
	M[0][4][1][3][0][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][1][3][0][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[0][4][1][3][0][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][1][3][1][2]=0.5*(1-a)/4.0;
	M[0][4][1][3][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][1][3][1][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][1][4][0][3]=0.5*0.5/4.0;
	M[0][4][1][4][0][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][1][4][1][2]=0.5*0.5/4.0;
	M[0][4][1][4][1][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][1][4][1][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][2][0][0][1]=0.5*a/4.0;
	M[0][4][2][0][0][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][2][0][0][3]=0.5*a/4.0;
	M[0][4][2][0][1][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][2][0][1][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][2][0][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][2][0][2][1]=0.5*a/4.0;
	M[0][4][2][0][2][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][2][0][2][3]=0.5*a/4.0;
	M[0][4][2][1][0][1]=0.5*a/4.0;
	M[0][4][2][1][0][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][2][1][0][3]=0.5*a/4.0;
	M[0][4][2][1][1][1]=0.5*(1-a)/4.0;
	M[0][4][2][1][1][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][2][1][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][2][1][1][4]=0.5*(1-a)/4.0;
	M[0][4][2][1][2][2]=0.5*a/4.0;
	M[0][4][2][1][2][3]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][2][1][2][4]=0.5*a/4.0;
	M[0][4][2][2][0][1]=0.5*a/4.0;
	M[0][4][2][2][0][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][2][2][0][3]=0.5*a/4.0;
	M[0][4][2][2][1][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][2][2][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][2][2][1][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][2][2][2][3]=0.5*a/4.0;
	M[0][4][2][2][2][4]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][2][3][0][1]=0.5*a/4.0;
	M[0][4][2][3][0][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][2][3][0][3]=0.5*a/4.0;
	M[0][4][2][3][1][2]=0.5*(1-a)/4.0;
	M[0][4][2][3][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][2][3][1][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][2][3][2][4]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][2][4][0][3]=0.5*a/4.0;
	M[0][4][2][4][0][4]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][2][4][1][2]=0.5*(0.5-0.5*a)/4.0;
	M[0][4][2][4][1][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[0][4][2][4][1][4]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[0][4][2][4][2][3]=0.5*a/4.0;
	M[0][4][2][4][2][4]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][3][0][0][1]=0.5*a/4.0;
	M[0][4][3][0][0][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][3][0][0][3]=0.5*a/4.0;
	M[0][4][3][0][1][1]=0.5*(1-a)/4.0;
	M[0][4][3][0][1][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][3][0][1][3]=0.5*(1-a)/4.0;
	M[0][4][3][0][2][1]=0.5*(1-a)/4.0;
	M[0][4][3][0][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][3][0][2][3]=0.5*(1-a)/4.0;
	M[0][4][3][0][3][1]=0.5*a/4.0;
	M[0][4][3][0][3][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][3][0][3][3]=0.5*a/4.0;
	M[0][4][3][1][0][1]=0.5*a/4.0;
	M[0][4][3][1][0][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][3][1][0][3]=0.5*a/4.0;
	M[0][4][3][1][1][2]=0.5*(1-a)/4.0;
	M[0][4][3][1][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][3][1][1][4]=0.5*(1-a)/4.0;
	M[0][4][3][1][2][1]=0.5*(1-a)/4.0;
	M[0][4][3][1][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][3][1][2][3]=0.5*(1-a)/4.0;
	M[0][4][3][1][3][2]=0.5*a/4.0;
	M[0][4][3][1][3][3]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][3][1][3][4]=0.5*a/4.0;
	M[0][4][3][2][0][1]=0.5*a/4.0;
	M[0][4][3][2][0][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][3][2][0][3]=0.5*a/4.0;
	M[0][4][3][2][1][2]=0.5*(1-a)/4.0;
	M[0][4][3][2][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][3][2][1][4]=0.5*(1-a)/4.0;
	M[0][4][3][2][2][2]=0.5*(1-a)/4.0;
	M[0][4][3][2][2][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][3][2][2][4]=0.5*(1-a)/4.0;
	M[0][4][3][2][3][3]=0.5*a/4.0;
	M[0][4][3][2][3][4]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][3][3][0][1]=0.5*a/4.0;
	M[0][4][3][3][0][2]=0.5*a/4.0+0.5*a/4.0;
	M[0][4][3][3][0][3]=0.5*a/4.0;
	M[0][4][3][3][1][2]=0.5*(1-a)/4.0;
	M[0][4][3][3][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][3][3][1][4]=0.5*(1-a)/4.0;
	M[0][4][3][3][2][3]=0.5*(1-a)/4.0;
	M[0][4][3][3][2][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[0][4][3][3][3][4]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][3][4][1][2]=0.5*0.5/4.0;
	M[0][4][3][4][1][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][3][4][1][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][3][4][2][3]=0.5*0.5/4.0;
	M[0][4][3][4][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][0][1][1]=0.5*0.5/4.0;
	M[0][4][4][0][1][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][0][1][3]=0.5*0.5/4.0;
	M[0][4][4][0][2][1]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][0][2][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][0][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][0][3][1]=0.5*0.5/4.0;
	M[0][4][4][0][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][0][3][3]=0.5*0.5/4.0;
	M[0][4][4][1][1][2]=0.5*0.5/4.0;
	M[0][4][4][1][1][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][1][1][4]=0.5*0.5/4.0;
	M[0][4][4][1][2][1]=0.5*0.5/4.0;
	M[0][4][4][1][2][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][1][2][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][1][2][4]=0.5*0.5/4.0;
	M[0][4][4][1][3][1]=0.5*0.5/4.0;
	M[0][4][4][1][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][1][3][3]=0.5*0.5/4.0;
	M[0][4][4][2][1][2]=0.5*(0.5-0.5*a)/4.0;
	M[0][4][4][2][1][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[0][4][4][2][1][4]=0.5*(0.5-0.5*a)/4.0;
	M[0][4][4][2][2][1]=0.5*a/4.0;
	M[0][4][4][2][2][2]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][4][2][2][3]=0.5*a/4.0+0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[0][4][4][2][2][4]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[0][4][4][2][3][2]=0.5*(0.5-0.5*a)/4.0;
	M[0][4][4][2][3][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[0][4][4][2][3][4]=0.5*(0.5-0.5*a)/4.0;
	M[0][4][4][3][1][2]=0.5*0.5/4.0;
	M[0][4][4][3][1][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][3][1][4]=0.5*0.5/4.0;
	M[0][4][4][3][2][2]=0.5*0.5/4.0;
	M[0][4][4][3][2][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][3][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][3][3][3]=0.5*0.5/4.0;
	M[0][4][4][3][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][4][1][2]=0.5*0.5/4.0;
	M[0][4][4][4][1][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][4][1][4]=0.5*0.5/4.0;
	M[0][4][4][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][4][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[0][4][4][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][0][0][0][0][0]=1*1/4.0+1*1/4.0;
	M[1][0][0][0][1][0]=1*1/4.0+1*1/4.0;
	M[1][0][0][1][0][0]=1*1/4.0;
	M[1][0][0][1][0][1]=1*1/4.0;
	M[1][0][0][1][1][0]=1*1/4.0;
	M[1][0][0][1][1][1]=1*1/4.0;
	M[1][0][0][2][0][0]=1*a/4.0;
	M[1][0][0][2][0][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][0][0][2][0][2]=1*a/4.0;
	M[1][0][0][2][1][0]=1*a/4.0;
	M[1][0][0][2][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][0][0][2][1][2]=1*a/4.0;
	M[1][0][0][3][0][0]=1*a/4.0;
	M[1][0][0][3][0][1]=1*(1-a)/4.0;
	M[1][0][0][3][0][2]=1*(1-a)/4.0;
	M[1][0][0][3][0][3]=1*a/4.0;
	M[1][0][0][3][1][0]=1*a/4.0;
	M[1][0][0][3][1][1]=1*(1-a)/4.0;
	M[1][0][0][3][1][2]=1*(1-a)/4.0;
	M[1][0][0][3][1][3]=1*a/4.0;
	M[1][0][0][4][0][1]=1*0.5/4.0;
	M[1][0][0][4][0][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][0][0][4][0][3]=1*0.5/4.0;
	M[1][0][0][4][1][1]=1*0.5/4.0;
	M[1][0][0][4][1][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][0][0][4][1][3]=1*0.5/4.0;
	M[1][0][1][0][0][0]=1*1/4.0;
	M[1][0][1][0][1][0]=1*1/4.0+1*1/4.0;
	M[1][0][1][0][2][0]=1*1/4.0;
	M[1][0][1][1][0][0]=1*1/4.0;
	M[1][0][1][1][1][0]=1*1/4.0;
	M[1][0][1][1][1][1]=1*1/4.0;
	M[1][0][1][1][2][1]=1*1/4.0;
	M[1][0][1][2][0][0]=1*a/4.0;
	M[1][0][1][2][0][1]=1*(1-a)/4.0;
	M[1][0][1][2][1][0]=1*a/4.0;
	M[1][0][1][2][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][0][1][2][1][2]=1*a/4.0;
	M[1][0][1][2][2][1]=1*(1-a)/4.0;
	M[1][0][1][2][2][2]=1*a/4.0;
	M[1][0][1][3][0][0]=1*a/4.0;
	M[1][0][1][3][0][2]=1*(1-a)/4.0;
	M[1][0][1][3][1][0]=1*a/4.0;
	M[1][0][1][3][1][1]=1*(1-a)/4.0;
	M[1][0][1][3][1][2]=1*(1-a)/4.0;
	M[1][0][1][3][1][3]=1*a/4.0;
	M[1][0][1][3][2][1]=1*(1-a)/4.0;
	M[1][0][1][3][2][3]=1*a/4.0;
	M[1][0][1][4][0][2]=1*0.5/4.0;
	M[1][0][1][4][0][3]=1*0.5/4.0;
	M[1][0][1][4][1][1]=1*0.5/4.0;
	M[1][0][1][4][1][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][0][1][4][1][3]=1*0.5/4.0;
	M[1][0][1][4][2][1]=1*0.5/4.0;
	M[1][0][1][4][2][2]=1*0.5/4.0;
	M[1][0][2][0][0][0]=1*a/4.0;
	M[1][0][2][0][1][0]=1*a/4.0+1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][0][2][0][2][0]=1*a/4.0+1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][0][2][0][3][0]=1*a/4.0;
	M[1][0][2][1][0][0]=1*a/4.0;
	M[1][0][2][1][1][0]=1*a/4.0+1*(1-a)/4.0;
	M[1][0][2][1][1][1]=1*(1-a)/4.0;
	M[1][0][2][1][2][0]=1*(1-a)/4.0;
	M[1][0][2][1][2][1]=1*a/4.0+1*(1-a)/4.0;
	M[1][0][2][1][3][1]=1*a/4.0;
	M[1][0][2][2][0][0]=1*a/4.0;
	M[1][0][2][2][1][0]=1*a/4.0;
	M[1][0][2][2][1][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][0][2][2][2][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][0][2][2][2][2]=1*a/4.0;
	M[1][0][2][2][3][2]=1*a/4.0;
	M[1][0][2][3][0][0]=1*a/4.0;
	M[1][0][2][3][1][0]=1*a/4.0;
	M[1][0][2][3][1][1]=1*(1-a)/4.0;
	M[1][0][2][3][1][2]=1*(1-a)/4.0;
	M[1][0][2][3][2][1]=1*(1-a)/4.0;
	M[1][0][2][3][2][2]=1*(1-a)/4.0;
	M[1][0][2][3][2][3]=1*a/4.0;
	M[1][0][2][3][3][3]=1*a/4.0;
	M[1][0][2][4][0][2]=1*a/4.0;
	M[1][0][2][4][1][1]=1*(0.5-0.5*a)/4.0;
	M[1][0][2][4][1][2]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*a/4.0;
	M[1][0][2][4][1][3]=1*(0.5-0.5*a)/4.0;
	M[1][0][2][4][2][1]=1*(0.5-0.5*a)/4.0;
	M[1][0][2][4][2][2]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*a/4.0;
	M[1][0][2][4][2][3]=1*(0.5-0.5*a)/4.0;
	M[1][0][2][4][3][2]=1*a/4.0;
	M[1][0][3][0][0][0]=1*a/4.0;
	M[1][0][3][0][1][0]=1*a/4.0+1*(1-a)/4.0;
	M[1][0][3][0][2][0]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][0][3][0][3][0]=1*a/4.0+1*(1-a)/4.0;
	M[1][0][3][0][4][0]=1*a/4.0;
	M[1][0][3][1][0][0]=1*a/4.0;
	M[1][0][3][1][1][0]=1*a/4.0;
	M[1][0][3][1][1][1]=1*(1-a)/4.0;
	M[1][0][3][1][2][0]=1*(1-a)/4.0;
	M[1][0][3][1][2][1]=1*(1-a)/4.0;
	M[1][0][3][1][3][0]=1*(1-a)/4.0;
	M[1][0][3][1][3][1]=1*a/4.0;
	M[1][0][3][1][4][1]=1*a/4.0;
	M[1][0][3][2][0][0]=1*a/4.0;
	M[1][0][3][2][1][0]=1*a/4.0;
	M[1][0][3][2][1][1]=1*(1-a)/4.0;
	M[1][0][3][2][2][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][0][3][2][3][1]=1*(1-a)/4.0;
	M[1][0][3][2][3][2]=1*a/4.0;
	M[1][0][3][2][4][2]=1*a/4.0;
	M[1][0][3][3][0][0]=1*a/4.0;
	M[1][0][3][3][1][0]=1*a/4.0;
	M[1][0][3][3][1][1]=1*(1-a)/4.0;
	M[1][0][3][3][2][1]=1*(1-a)/4.0;
	M[1][0][3][3][2][2]=1*(1-a)/4.0;
	M[1][0][3][3][3][2]=1*(1-a)/4.0;
	M[1][0][3][3][3][3]=1*a/4.0;
	M[1][0][3][3][4][3]=1*a/4.0;
	M[1][0][3][4][1][1]=1*0.5/4.0;
	M[1][0][3][4][1][2]=1*0.5/4.0;
	M[1][0][3][4][2][1]=1*0.5/4.0;
	M[1][0][3][4][2][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][0][3][4][2][3]=1*0.5/4.0;
	M[1][0][3][4][3][2]=1*0.5/4.0;
	M[1][0][3][4][3][3]=1*0.5/4.0;
	M[1][0][4][0][1][0]=1*0.5/4.0;
	M[1][0][4][0][2][0]=1*0.5/4.0+1*0.5/4.0+1*0.5/4.0;
	M[1][0][4][0][3][0]=1*0.5/4.0+1*0.5/4.0+1*0.5/4.0;
	M[1][0][4][0][4][0]=1*0.5/4.0;
	M[1][0][4][1][1][1]=1*0.5/4.0;
	M[1][0][4][1][2][0]=1*0.5/4.0;
	M[1][0][4][1][2][1]=1*0.5/4.0+1*0.5/4.0;
	M[1][0][4][1][3][0]=1*0.5/4.0+1*0.5/4.0;
	M[1][0][4][1][3][1]=1*0.5/4.0;
	M[1][0][4][1][4][0]=1*0.5/4.0;
	M[1][0][4][2][1][1]=1*(0.5-0.5*a)/4.0;
	M[1][0][4][2][2][0]=1*a/4.0;
	M[1][0][4][2][2][1]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[1][0][4][2][2][2]=1*a/4.0;
	M[1][0][4][2][3][0]=1*a/4.0;
	M[1][0][4][2][3][1]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[1][0][4][2][3][2]=1*a/4.0;
	M[1][0][4][2][4][1]=1*(0.5-0.5*a)/4.0;
	M[1][0][4][3][1][1]=1*0.5/4.0;
	M[1][0][4][3][2][1]=1*0.5/4.0+1*0.5/4.0;
	M[1][0][4][3][2][2]=1*0.5/4.0;
	M[1][0][4][3][3][1]=1*0.5/4.0;
	M[1][0][4][3][3][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][0][4][3][4][2]=1*0.5/4.0;
	M[1][0][4][4][1][1]=1*0.5/4.0;
	M[1][0][4][4][2][1]=1*0.5/4.0;
	M[1][0][4][4][2][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][0][4][4][3][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][0][4][4][3][3]=1*0.5/4.0;
	M[1][0][4][4][4][3]=1*0.5/4.0;
	M[1][1][0][0][0][0]=1*1/4.0+1*1/4.0;
	M[1][1][0][0][1][1]=1*1/4.0+1*1/4.0;
	M[1][1][0][1][0][0]=1*1/4.0;
	M[1][1][0][1][0][1]=1*1/4.0;
	M[1][1][0][1][1][1]=1*1/4.0;
	M[1][1][0][1][1][2]=1*1/4.0;
	M[1][1][0][2][0][0]=1*a/4.0;
	M[1][1][0][2][0][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][1][0][2][0][2]=1*a/4.0;
	M[1][1][0][2][1][1]=1*a/4.0;
	M[1][1][0][2][1][2]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][1][0][2][1][3]=1*a/4.0;
	M[1][1][0][3][0][0]=1*a/4.0;
	M[1][1][0][3][0][1]=1*(1-a)/4.0;
	M[1][1][0][3][0][2]=1*(1-a)/4.0;
	M[1][1][0][3][0][3]=1*a/4.0;
	M[1][1][0][3][1][1]=1*a/4.0;
	M[1][1][0][3][1][2]=1*(1-a)/4.0;
	M[1][1][0][3][1][3]=1*(1-a)/4.0;
	M[1][1][0][3][1][4]=1*a/4.0;
	M[1][1][0][4][0][1]=1*0.5/4.0;
	M[1][1][0][4][0][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][1][0][4][0][3]=1*0.5/4.0;
	M[1][1][0][4][1][2]=1*0.5/4.0;
	M[1][1][0][4][1][3]=1*0.5/4.0+1*0.5/4.0;
	M[1][1][0][4][1][4]=1*0.5/4.0;
	M[1][1][1][0][0][0]=1*1/4.0;
	M[1][1][1][0][1][0]=1*1/4.0;
	M[1][1][1][0][1][1]=1*1/4.0;
	M[1][1][1][0][2][1]=1*1/4.0;
	M[1][1][1][1][0][0]=1*1/4.0;
	M[1][1][1][1][1][1]=1*1/4.0+1*1/4.0;
	M[1][1][1][1][2][2]=1*1/4.0;
	M[1][1][1][2][0][0]=1*a/4.0;
	M[1][1][1][2][0][1]=1*(1-a)/4.0;
	M[1][1][1][2][1][1]=1*a/4.0+1*(1-a)/4.0;
	M[1][1][1][2][1][2]=1*a/4.0+1*(1-a)/4.0;
	M[1][1][1][2][2][2]=1*(1-a)/4.0;
	M[1][1][1][2][2][3]=1*a/4.0;
	M[1][1][1][3][0][0]=1*a/4.0;
	M[1][1][1][3][0][2]=1*(1-a)/4.0;
	M[1][1][1][3][1][1]=1*a/4.0+1*(1-a)/4.0;
	M[1][1][1][3][1][3]=1*a/4.0+1*(1-a)/4.0;
	M[1][1][1][3][2][2]=1*(1-a)/4.0;
	M[1][1][1][3][2][4]=1*a/4.0;
	M[1][1][1][4][0][2]=1*0.5/4.0;
	M[1][1][1][4][0][3]=1*0.5/4.0;
	M[1][1][1][4][1][1]=1*0.5/4.0;
	M[1][1][1][4][1][2]=1*0.5/4.0;
	M[1][1][1][4][1][3]=1*0.5/4.0;
	M[1][1][1][4][1][4]=1*0.5/4.0;
	M[1][1][1][4][2][2]=1*0.5/4.0;
	M[1][1][1][4][2][3]=1*0.5/4.0;
	M[1][1][2][0][0][0]=1*a/4.0;
	M[1][1][2][0][1][0]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][1][2][0][1][1]=1*a/4.0;
	M[1][1][2][0][2][0]=1*a/4.0;
	M[1][1][2][0][2][1]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][1][2][0][3][1]=1*a/4.0;
	M[1][1][2][1][0][0]=1*a/4.0;
	M[1][1][2][1][1][0]=1*(1-a)/4.0;
	M[1][1][2][1][1][1]=1*a/4.0+1*(1-a)/4.0;
	M[1][1][2][1][2][1]=1*a/4.0+1*(1-a)/4.0;
	M[1][1][2][1][2][2]=1*(1-a)/4.0;
	M[1][1][2][1][3][2]=1*a/4.0;
	M[1][1][2][2][0][0]=1*a/4.0;
	M[1][1][2][2][1][1]=1*(1-a)/4.0+1*(1-a)/4.0+1*a/4.0;
	M[1][1][2][2][2][2]=1*(1-a)/4.0+1*(1-a)/4.0+1*a/4.0;
	M[1][1][2][2][3][3]=1*a/4.0;
	M[1][1][2][3][0][0]=1*a/4.0;
	M[1][1][2][3][1][1]=1*(1-a)/4.0+1*a/4.0;
	M[1][1][2][3][1][2]=1*(1-a)/4.0;
	M[1][1][2][3][2][2]=1*(1-a)/4.0;
	M[1][1][2][3][2][3]=1*(1-a)/4.0+1*a/4.0;
	M[1][1][2][3][3][4]=1*a/4.0;
	M[1][1][2][4][0][2]=1*a/4.0;
	M[1][1][2][4][1][1]=1*(0.5-0.5*a)/4.0;
	M[1][1][2][4][1][2]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[1][1][2][4][1][3]=1*(0.5-0.5*a)/4.0+1*a/4.0;
	M[1][1][2][4][2][2]=1*(0.5-0.5*a)/4.0+1*a/4.0;
	M[1][1][2][4][2][3]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[1][1][2][4][2][4]=1*(0.5-0.5*a)/4.0;
	M[1][1][2][4][3][3]=1*a/4.0;
	M[1][1][3][0][0][0]=1*a/4.0;
	M[1][1][3][0][1][0]=1*(1-a)/4.0;
	M[1][1][3][0][1][1]=1*a/4.0;
	M[1][1][3][0][2][0]=1*(1-a)/4.0;
	M[1][1][3][0][2][1]=1*(1-a)/4.0;
	M[1][1][3][0][3][0]=1*a/4.0;
	M[1][1][3][0][3][1]=1*(1-a)/4.0;
	M[1][1][3][0][4][1]=1*a/4.0;
	M[1][1][3][1][0][0]=1*a/4.0;
	M[1][1][3][1][1][1]=1*a/4.0+1*(1-a)/4.0;
	M[1][1][3][1][2][0]=1*(1-a)/4.0;
	M[1][1][3][1][2][2]=1*(1-a)/4.0;
	M[1][1][3][1][3][1]=1*a/4.0+1*(1-a)/4.0;
	M[1][1][3][1][4][2]=1*a/4.0;
	M[1][1][3][2][0][0]=1*a/4.0;
	M[1][1][3][2][1][1]=1*(1-a)/4.0+1*a/4.0;
	M[1][1][3][2][2][1]=1*(1-a)/4.0;
	M[1][1][3][2][2][2]=1*(1-a)/4.0;
	M[1][1][3][2][3][2]=1*(1-a)/4.0+1*a/4.0;
	M[1][1][3][2][4][3]=1*a/4.0;
	M[1][1][3][3][0][0]=1*a/4.0;
	M[1][1][3][3][1][1]=1*a/4.0+1*(1-a)/4.0;
	M[1][1][3][3][2][2]=1*(1-a)/4.0+1*(1-a)/4.0;
	M[1][1][3][3][3][3]=1*a/4.0+1*(1-a)/4.0;
	M[1][1][3][3][4][4]=1*a/4.0;
	M[1][1][3][4][1][1]=1*0.5/4.0;
	M[1][1][3][4][1][2]=1*0.5/4.0;
	M[1][1][3][4][2][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][1][3][4][2][3]=1*0.5/4.0+1*0.5/4.0;
	M[1][1][3][4][3][3]=1*0.5/4.0;
	M[1][1][3][4][3][4]=1*0.5/4.0;
	M[1][1][4][0][1][0]=1*0.5/4.0;
	M[1][1][4][0][2][0]=1*0.5/4.0+1*0.5/4.0;
	M[1][1][4][0][2][1]=1*0.5/4.0;
	M[1][1][4][0][3][0]=1*0.5/4.0;
	M[1][1][4][0][3][1]=1*0.5/4.0+1*0.5/4.0;
	M[1][1][4][0][4][1]=1*0.5/4.0;
	M[1][1][4][1][1][1]=1*0.5/4.0;
	M[1][1][4][1][2][0]=1*0.5/4.0;
	M[1][1][4][1][2][1]=1*0.5/4.0;
	M[1][1][4][1][2][2]=1*0.5/4.0;
	M[1][1][4][1][3][0]=1*0.5/4.0;
	M[1][1][4][1][3][1]=1*0.5/4.0;
	M[1][1][4][1][3][2]=1*0.5/4.0;
	M[1][1][4][1][4][1]=1*0.5/4.0;
	M[1][1][4][2][1][1]=1*(0.5-0.5*a)/4.0;
	M[1][1][4][2][2][0]=1*a/4.0;
	M[1][1][4][2][2][1]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[1][1][4][2][2][2]=1*(0.5-0.5*a)/4.0+1*a/4.0;
	M[1][1][4][2][3][1]=1*(0.5-0.5*a)/4.0+1*a/4.0;
	M[1][1][4][2][3][2]=1*(0.5-0.5*a)/4.0+1*(0.5-0.5*a)/4.0;
	M[1][1][4][2][3][3]=1*a/4.0;
	M[1][1][4][2][4][2]=1*(0.5-0.5*a)/4.0;
	M[1][1][4][3][1][1]=1*0.5/4.0;
	M[1][1][4][3][2][1]=1*0.5/4.0;
	M[1][1][4][3][2][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][1][4][3][3][2]=1*0.5/4.0+1*0.5/4.0;
	M[1][1][4][3][3][3]=1*0.5/4.0;
	M[1][1][4][3][4][3]=1*0.5/4.0;
	M[1][1][4][4][1][1]=1*0.5/4.0;
	M[1][1][4][4][2][2]=1*0.5/4.0+1*0.5/4.0+1*0.5/4.0;
	M[1][1][4][4][3][3]=1*0.5/4.0+1*0.5/4.0+1*0.5/4.0;
	M[1][1][4][4][4][4]=1*0.5/4.0;
	M[1][2][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[1][2][0][0][0][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[1][2][0][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[1][2][0][0][1][2]=a*1/4.0+a*1/4.0;
	M[1][2][0][1][0][0]=a*1/4.0;
	M[1][2][0][1][0][1]=a*1/4.0+(1-a)*1/4.0;
	M[1][2][0][1][0][2]=(1-a)*1/4.0;
	M[1][2][0][1][1][1]=(1-a)*1/4.0;
	M[1][2][0][1][1][2]=a*1/4.0+(1-a)*1/4.0;
	M[1][2][0][1][1][3]=a*1/4.0;
	M[1][2][0][2][0][0]=a*a/4.0;
	M[1][2][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][0][2][0][2]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][0][2][0][3]=(1-a)*a/4.0;
	M[1][2][0][2][1][1]=(1-a)*a/4.0;
	M[1][2][0][2][1][2]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][0][2][1][3]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][0][2][1][4]=a*a/4.0;
	M[1][2][0][3][0][0]=a*a/4.0;
	M[1][2][0][3][0][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][0][3][0][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][0][3][0][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][0][3][0][4]=(1-a)*a/4.0;
	M[1][2][0][3][1][1]=(1-a)*a/4.0;
	M[1][2][0][3][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][0][3][1][3]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][0][3][1][4]=a*a/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][0][4][0][1]=a*0.5/4.0;
	M[1][2][0][4][0][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][0][4][0][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][0][4][0][4]=(1-a)*0.5/4.0;
	M[1][2][0][4][1][2]=(1-a)*0.5/4.0;
	M[1][2][0][4][1][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][0][4][1][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][1][0][0][0]=a*1/4.0;
	M[1][2][1][0][0][1]=(1-a)*1/4.0;
	M[1][2][1][0][1][0]=a*1/4.0;
	M[1][2][1][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[1][2][1][0][1][2]=a*1/4.0;
	M[1][2][1][0][2][1]=(1-a)*1/4.0;
	M[1][2][1][0][2][2]=a*1/4.0;
	M[1][2][1][1][0][0]=a*1/4.0;
	M[1][2][1][1][0][1]=(1-a)*1/4.0;
	M[1][2][1][1][1][1]=a*1/4.0+(1-a)*1/4.0;
	M[1][2][1][1][1][2]=a*1/4.0+(1-a)*1/4.0;
	M[1][2][1][1][2][2]=(1-a)*1/4.0;
	M[1][2][1][1][2][3]=a*1/4.0;
	M[1][2][1][2][0][0]=a*a/4.0;
	M[1][2][1][2][0][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][1][2][0][2]=(1-a)*(1-a)/4.0;
	M[1][2][1][2][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][1][2][1][2]=a*a/4.0+a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][1][2][1][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][1][2][2][2]=(1-a)*(1-a)/4.0;
	M[1][2][1][2][2][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][1][2][2][4]=a*a/4.0;
	M[1][2][1][3][0][0]=a*a/4.0;
	M[1][2][1][3][0][1]=(1-a)*a/4.0;
	M[1][2][1][3][0][2]=a*(1-a)/4.0;
	M[1][2][1][3][0][3]=(1-a)*(1-a)/4.0;
	M[1][2][1][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][1][3][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][1][3][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][1][3][1][4]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][1][3][2][2]=(1-a)*(1-a)/4.0;
	M[1][2][1][3][2][3]=a*(1-a)/4.0;
	M[1][2][1][3][2][4]=a*a/4.0+(1-a)*a/4.0;
	M[1][2][1][4][0][2]=a*0.5/4.0;
	M[1][2][1][4][0][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][1][4][0][4]=(1-a)*0.5/4.0;
	M[1][2][1][4][1][1]=a*0.5/4.0;
	M[1][2][1][4][1][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][1][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][1][4][1][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][1][4][2][2]=(1-a)*0.5/4.0;
	M[1][2][1][4][2][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][1][4][2][4]=a*0.5/4.0;
	M[1][2][2][0][0][0]=a*a/4.0;
	M[1][2][2][0][0][1]=(1-a)*a/4.0;
	M[1][2][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[1][2][2][0][1][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][2][0][1][2]=a*a/4.0;
	M[1][2][2][0][2][0]=a*a/4.0;
	M[1][2][2][0][2][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][2][0][2][2]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[1][2][2][0][3][1]=(1-a)*a/4.0;
	M[1][2][2][0][3][2]=a*a/4.0;
	M[1][2][2][1][0][0]=a*a/4.0;
	M[1][2][2][1][0][1]=(1-a)*a/4.0;
	M[1][2][2][1][1][0]=a*(1-a)/4.0;
	M[1][2][2][1][1][1]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][2][1][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][2][1][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][2][1][2][2]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][2][1][2][3]=a*(1-a)/4.0;
	M[1][2][2][1][3][2]=(1-a)*a/4.0;
	M[1][2][2][1][3][3]=a*a/4.0;
	M[1][2][2][2][0][0]=a*a/4.0;
	M[1][2][2][2][0][1]=(1-a)*a/4.0;
	M[1][2][2][2][1][1]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][2][2][1][2]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][2][2][2][2]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][2][2][2][3]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][2][2][3][3]=(1-a)*a/4.0;
	M[1][2][2][2][3][4]=a*a/4.0;
	M[1][2][2][3][0][0]=a*a/4.0;
	M[1][2][2][3][0][1]=(1-a)*a/4.0;
	M[1][2][2][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][2][3][1][2]=a*(1-a)/4.0+a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][2][3][1][3]=(1-a)*(1-a)/4.0;
	M[1][2][2][3][2][2]=(1-a)*(1-a)/4.0;
	M[1][2][2][3][2][3]=a*(1-a)/4.0+a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][2][3][2][4]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][2][3][3][4]=a*a/4.0+(1-a)*a/4.0;
	M[1][2][2][4][0][2]=a*a/4.0;
	M[1][2][2][4][0][3]=(1-a)*a/4.0;
	M[1][2][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[1][2][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][2][2][4][1][3]=a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][2][2][4][1][4]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][2][2][4][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][2][2][4][2][3]=a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][2][2][4][2][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][2][2][4][3][3]=(1-a)*a/4.0;
	M[1][2][2][4][3][4]=a*a/4.0;
	M[1][2][3][0][0][0]=a*a/4.0;
	M[1][2][3][0][0][1]=(1-a)*a/4.0;
	M[1][2][3][0][1][0]=a*(1-a)/4.0;
	M[1][2][3][0][1][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][0][1][2]=a*a/4.0;
	M[1][2][3][0][2][0]=a*(1-a)/4.0;
	M[1][2][3][0][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][0][2][2]=a*(1-a)/4.0;
	M[1][2][3][0][3][0]=a*a/4.0;
	M[1][2][3][0][3][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][0][3][2]=a*(1-a)/4.0;
	M[1][2][3][0][4][1]=(1-a)*a/4.0;
	M[1][2][3][0][4][2]=a*a/4.0;
	M[1][2][3][1][0][0]=a*a/4.0;
	M[1][2][3][1][0][1]=(1-a)*a/4.0;
	M[1][2][3][1][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][3][1][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][1][2][0]=a*(1-a)/4.0;
	M[1][2][3][1][2][1]=(1-a)*(1-a)/4.0;
	M[1][2][3][1][2][2]=(1-a)*(1-a)/4.0;
	M[1][2][3][1][2][3]=a*(1-a)/4.0;
	M[1][2][3][1][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][1][3][2]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][3][1][4][2]=(1-a)*a/4.0;
	M[1][2][3][1][4][3]=a*a/4.0;
	M[1][2][3][2][0][0]=a*a/4.0;
	M[1][2][3][2][0][1]=(1-a)*a/4.0;
	M[1][2][3][2][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][3][2][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][2][2][1]=a*(1-a)/4.0;
	M[1][2][3][2][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][2][2][3]=a*(1-a)/4.0;
	M[1][2][3][2][3][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][2][3][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][3][2][4][3]=(1-a)*a/4.0;
	M[1][2][3][2][4][4]=a*a/4.0;
	M[1][2][3][3][0][0]=a*a/4.0;
	M[1][2][3][3][0][1]=(1-a)*a/4.0;
	M[1][2][3][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][3][3][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][3][2][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][3][2][3]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][3][3][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][2][3][3][3][4]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][2][3][3][4][4]=a*a/4.0+(1-a)*a/4.0;
	M[1][2][3][4][1][1]=a*0.5/4.0;
	M[1][2][3][4][1][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][3][4][1][3]=(1-a)*0.5/4.0;
	M[1][2][3][4][2][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][3][4][2][3]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][3][4][2][4]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][3][4][3][3]=(1-a)*0.5/4.0;
	M[1][2][3][4][3][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][0][1][0]=a*0.5/4.0;
	M[1][2][4][0][1][1]=(1-a)*0.5/4.0;
	M[1][2][4][0][2][0]=a*0.5/4.0+a*0.5/4.0;
	M[1][2][4][0][2][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][0][2][2]=a*0.5/4.0;
	M[1][2][4][0][3][0]=a*0.5/4.0;
	M[1][2][4][0][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][0][3][2]=a*0.5/4.0+a*0.5/4.0;
	M[1][2][4][0][4][1]=(1-a)*0.5/4.0;
	M[1][2][4][0][4][2]=a*0.5/4.0;
	M[1][2][4][1][1][1]=a*0.5/4.0;
	M[1][2][4][1][1][2]=(1-a)*0.5/4.0;
	M[1][2][4][1][2][0]=a*0.5/4.0;
	M[1][2][4][1][2][1]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][1][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][1][2][3]=a*0.5/4.0;
	M[1][2][4][1][3][0]=a*0.5/4.0;
	M[1][2][4][1][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][1][3][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][1][3][3]=a*0.5/4.0;
	M[1][2][4][1][4][1]=(1-a)*0.5/4.0;
	M[1][2][4][1][4][2]=a*0.5/4.0;
	M[1][2][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[1][2][4][2][1][2]=(1-a)*(0.5-0.5*a)/4.0;
	M[1][2][4][2][2][0]=a*a/4.0;
	M[1][2][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][2][4][2][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][2][4][2][2][3]=a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][2][4][2][3][1]=a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][2][4][2][3][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][2][4][2][3][3]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][2][4][2][3][4]=a*a/4.0;
	M[1][2][4][2][4][2]=(1-a)*(0.5-0.5*a)/4.0;
	M[1][2][4][2][4][3]=a*(0.5-0.5*a)/4.0;
	M[1][2][4][3][1][1]=a*0.5/4.0;
	M[1][2][4][3][1][2]=(1-a)*0.5/4.0;
	M[1][2][4][3][2][1]=a*0.5/4.0;
	M[1][2][4][3][2][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][3][2][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][3][3][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][3][3][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][3][3][4]=a*0.5/4.0;
	M[1][2][4][3][4][3]=(1-a)*0.5/4.0;
	M[1][2][4][3][4][4]=a*0.5/4.0;
	M[1][2][4][4][1][1]=a*0.5/4.0;
	M[1][2][4][4][1][2]=(1-a)*0.5/4.0;
	M[1][2][4][4][2][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][4][2][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][4][3][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][4][3][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][2][4][4][4][4]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[1][3][0][0][0][2]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[1][3][0][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[1][3][0][0][1][3]=a*1/4.0+a*1/4.0;
	M[1][3][0][1][0][0]=a*1/4.0;
	M[1][3][0][1][0][1]=a*1/4.0;
	M[1][3][0][1][0][2]=(1-a)*1/4.0;
	M[1][3][0][1][0][3]=(1-a)*1/4.0;
	M[1][3][0][1][1][1]=(1-a)*1/4.0;
	M[1][3][0][1][1][2]=(1-a)*1/4.0;
	M[1][3][0][1][1][3]=a*1/4.0;
	M[1][3][0][1][1][4]=a*1/4.0;
	M[1][3][0][2][0][0]=a*a/4.0;
	M[1][3][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[1][3][0][2][0][2]=a*a/4.0+(1-a)*a/4.0;
	M[1][3][0][2][0][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][3][0][2][0][4]=(1-a)*a/4.0;
	M[1][3][0][2][1][1]=(1-a)*a/4.0;
	M[1][3][0][2][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][3][0][2][1][3]=a*a/4.0+(1-a)*a/4.0;
	M[1][3][0][2][1][4]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[1][3][0][3][0][0]=a*a/4.0;
	M[1][3][0][3][0][1]=a*(1-a)/4.0;
	M[1][3][0][3][0][2]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][0][3][0][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][0][3][0][4]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][0][3][1][1]=(1-a)*a/4.0;
	M[1][3][0][3][1][2]=(1-a)*(1-a)/4.0;
	M[1][3][0][3][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][0][3][1][4]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][0][4][0][1]=a*0.5/4.0;
	M[1][3][0][4][0][2]=a*0.5/4.0+a*0.5/4.0;
	M[1][3][0][4][0][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][0][4][0][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][0][4][1][2]=(1-a)*0.5/4.0;
	M[1][3][0][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][0][4][1][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][1][0][0][0]=a*1/4.0;
	M[1][3][1][0][0][2]=(1-a)*1/4.0;
	M[1][3][1][0][1][0]=a*1/4.0;
	M[1][3][1][0][1][1]=(1-a)*1/4.0;
	M[1][3][1][0][1][2]=(1-a)*1/4.0;
	M[1][3][1][0][1][3]=a*1/4.0;
	M[1][3][1][0][2][1]=(1-a)*1/4.0;
	M[1][3][1][0][2][3]=a*1/4.0;
	M[1][3][1][1][0][0]=a*1/4.0;
	M[1][3][1][1][0][2]=(1-a)*1/4.0;
	M[1][3][1][1][1][1]=a*1/4.0+(1-a)*1/4.0;
	M[1][3][1][1][1][3]=a*1/4.0+(1-a)*1/4.0;
	M[1][3][1][1][2][2]=(1-a)*1/4.0;
	M[1][3][1][1][2][4]=a*1/4.0;
	M[1][3][1][2][0][0]=a*a/4.0;
	M[1][3][1][2][0][1]=a*(1-a)/4.0;
	M[1][3][1][2][0][2]=(1-a)*a/4.0;
	M[1][3][1][2][0][3]=(1-a)*(1-a)/4.0;
	M[1][3][1][2][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][1][2][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][1][2][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][1][2][1][4]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][1][2][2][2]=(1-a)*(1-a)/4.0;
	M[1][3][1][2][2][3]=(1-a)*a/4.0;
	M[1][3][1][2][2][4]=a*a/4.0+a*(1-a)/4.0;
	M[1][3][1][3][0][0]=a*a/4.0;
	M[1][3][1][3][0][2]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][1][3][0][4]=(1-a)*(1-a)/4.0;
	M[1][3][1][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][1][3][1][3]=a*a/4.0+a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][3][1][3][1][4]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][1][3][2][2]=(1-a)*(1-a)/4.0;
	M[1][3][1][3][2][4]=a*a/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][1][4][0][2]=a*0.5/4.0;
	M[1][3][1][4][0][3]=a*0.5/4.0;
	M[1][3][1][4][0][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][1][4][1][1]=a*0.5/4.0;
	M[1][3][1][4][1][2]=a*0.5/4.0;
	M[1][3][1][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][1][4][1][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][1][4][2][2]=(1-a)*0.5/4.0;
	M[1][3][1][4][2][3]=(1-a)*0.5/4.0;
	M[1][3][1][4][2][4]=a*0.5/4.0+a*0.5/4.0;
	M[1][3][2][0][0][0]=a*a/4.0;
	M[1][3][2][0][0][2]=(1-a)*a/4.0;
	M[1][3][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[1][3][2][0][1][1]=(1-a)*a/4.0;
	M[1][3][2][0][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][3][2][0][1][3]=a*a/4.0;
	M[1][3][2][0][2][0]=a*a/4.0;
	M[1][3][2][0][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][3][2][0][2][2]=(1-a)*a/4.0;
	M[1][3][2][0][2][3]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[1][3][2][0][3][1]=(1-a)*a/4.0;
	M[1][3][2][0][3][3]=a*a/4.0;
	M[1][3][2][1][0][0]=a*a/4.0;
	M[1][3][2][1][0][2]=(1-a)*a/4.0;
	M[1][3][2][1][1][0]=a*(1-a)/4.0;
	M[1][3][2][1][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][2][1][1][2]=(1-a)*(1-a)/4.0;
	M[1][3][2][1][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][2][1][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][2][1][2][2]=(1-a)*(1-a)/4.0;
	M[1][3][2][1][2][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][2][1][2][4]=a*(1-a)/4.0;
	M[1][3][2][1][3][2]=(1-a)*a/4.0;
	M[1][3][2][1][3][4]=a*a/4.0;
	M[1][3][2][2][0][0]=a*a/4.0;
	M[1][3][2][2][0][2]=(1-a)*a/4.0;
	M[1][3][2][2][1][1]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][2][2][1][3]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][3][2][2][2][2]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][3][2][2][2][4]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][2][2][3][3]=(1-a)*a/4.0;
	M[1][3][2][2][3][4]=a*a/4.0;
	M[1][3][2][3][0][0]=a*a/4.0;
	M[1][3][2][3][0][2]=(1-a)*a/4.0;
	M[1][3][2][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][2][3][1][2]=a*(1-a)/4.0;
	M[1][3][2][3][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][2][3][1][4]=(1-a)*(1-a)/4.0;
	M[1][3][2][3][2][2]=(1-a)*(1-a)/4.0;
	M[1][3][2][3][2][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][2][3][2][4]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][2][3][3][4]=a*a/4.0+(1-a)*a/4.0;
	M[1][3][2][4][0][2]=a*a/4.0;
	M[1][3][2][4][0][4]=(1-a)*a/4.0;
	M[1][3][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[1][3][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[1][3][2][4][1][3]=a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][3][2][4][1][4]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][3][2][4][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][3][2][4][2][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][3][2][4][2][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][3][2][4][3][3]=(1-a)*a/4.0;
	M[1][3][2][4][3][4]=a*a/4.0;
	M[1][3][3][0][0][0]=a*a/4.0;
	M[1][3][3][0][0][2]=(1-a)*a/4.0;
	M[1][3][3][0][1][0]=a*(1-a)/4.0;
	M[1][3][3][0][1][1]=(1-a)*a/4.0;
	M[1][3][3][0][1][2]=(1-a)*(1-a)/4.0;
	M[1][3][3][0][1][3]=a*a/4.0;
	M[1][3][3][0][2][0]=a*(1-a)/4.0;
	M[1][3][3][0][2][1]=(1-a)*(1-a)/4.0;
	M[1][3][3][0][2][2]=(1-a)*(1-a)/4.0;
	M[1][3][3][0][2][3]=a*(1-a)/4.0;
	M[1][3][3][0][3][0]=a*a/4.0;
	M[1][3][3][0][3][1]=(1-a)*(1-a)/4.0;
	M[1][3][3][0][3][2]=(1-a)*a/4.0;
	M[1][3][3][0][3][3]=a*(1-a)/4.0;
	M[1][3][3][0][4][1]=(1-a)*a/4.0;
	M[1][3][3][0][4][3]=a*a/4.0;
	M[1][3][3][1][0][0]=a*a/4.0;
	M[1][3][3][1][0][2]=(1-a)*a/4.0;
	M[1][3][3][1][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][3][1][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][3][1][2][0]=a*(1-a)/4.0;
	M[1][3][3][1][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][3][3][1][2][4]=a*(1-a)/4.0;
	M[1][3][3][1][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][3][1][3][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][3][1][4][2]=(1-a)*a/4.0;
	M[1][3][3][1][4][4]=a*a/4.0;
	M[1][3][3][2][0][0]=a*a/4.0;
	M[1][3][3][2][0][2]=(1-a)*a/4.0;
	M[1][3][3][2][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][3][2][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][3][2][2][1]=a*(1-a)/4.0;
	M[1][3][3][2][2][2]=(1-a)*(1-a)/4.0;
	M[1][3][3][2][2][3]=(1-a)*(1-a)/4.0;
	M[1][3][3][2][2][4]=a*(1-a)/4.0;
	M[1][3][3][2][3][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][3][2][3][4]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][3][2][4][3]=(1-a)*a/4.0;
	M[1][3][3][2][4][4]=a*a/4.0;
	M[1][3][3][3][0][0]=a*a/4.0;
	M[1][3][3][3][0][2]=(1-a)*a/4.0;
	M[1][3][3][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][3][3][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][3][3][2][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][3][3][3][2][4]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[1][3][3][3][3][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[1][3][3][3][3][4]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[1][3][3][3][4][4]=a*a/4.0+(1-a)*a/4.0;
	M[1][3][3][4][1][1]=a*0.5/4.0;
	M[1][3][3][4][1][2]=a*0.5/4.0;
	M[1][3][3][4][1][3]=(1-a)*0.5/4.0;
	M[1][3][3][4][1][4]=(1-a)*0.5/4.0;
	M[1][3][3][4][2][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][3][4][2][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][3][4][2][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][3][4][3][3]=(1-a)*0.5/4.0;
	M[1][3][3][4][3][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][0][1][0]=a*0.5/4.0;
	M[1][3][4][0][1][2]=(1-a)*0.5/4.0;
	M[1][3][4][0][2][0]=a*0.5/4.0+a*0.5/4.0;
	M[1][3][4][0][2][1]=(1-a)*0.5/4.0;
	M[1][3][4][0][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][0][2][3]=a*0.5/4.0;
	M[1][3][4][0][3][0]=a*0.5/4.0;
	M[1][3][4][0][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][0][3][2]=(1-a)*0.5/4.0;
	M[1][3][4][0][3][3]=a*0.5/4.0+a*0.5/4.0;
	M[1][3][4][0][4][1]=(1-a)*0.5/4.0;
	M[1][3][4][0][4][3]=a*0.5/4.0;
	M[1][3][4][1][1][1]=a*0.5/4.0;
	M[1][3][4][1][1][3]=(1-a)*0.5/4.0;
	M[1][3][4][1][2][0]=a*0.5/4.0;
	M[1][3][4][1][2][1]=a*0.5/4.0;
	M[1][3][4][1][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][1][2][3]=(1-a)*0.5/4.0;
	M[1][3][4][1][2][4]=a*0.5/4.0;
	M[1][3][4][1][3][0]=a*0.5/4.0;
	M[1][3][4][1][3][1]=(1-a)*0.5/4.0;
	M[1][3][4][1][3][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][1][3][3]=a*0.5/4.0;
	M[1][3][4][1][3][4]=a*0.5/4.0;
	M[1][3][4][1][4][1]=(1-a)*0.5/4.0;
	M[1][3][4][1][4][3]=a*0.5/4.0;
	M[1][3][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[1][3][4][2][1][3]=(1-a)*(0.5-0.5*a)/4.0;
	M[1][3][4][2][2][0]=a*a/4.0;
	M[1][3][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[1][3][4][2][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][3][4][2][2][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][3][4][2][2][4]=a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][3][4][2][3][1]=a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][3][4][2][3][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[1][3][4][2][3][3]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[1][3][4][2][3][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*a/4.0;
	M[1][3][4][2][4][2]=(1-a)*(0.5-0.5*a)/4.0;
	M[1][3][4][2][4][4]=a*(0.5-0.5*a)/4.0;
	M[1][3][4][3][1][1]=a*0.5/4.0;
	M[1][3][4][3][1][3]=(1-a)*0.5/4.0;
	M[1][3][4][3][2][1]=a*0.5/4.0;
	M[1][3][4][3][2][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][3][2][3]=(1-a)*0.5/4.0;
	M[1][3][4][3][2][4]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][3][3][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][3][3][3]=(1-a)*0.5/4.0;
	M[1][3][4][3][3][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][3][4][3]=(1-a)*0.5/4.0;
	M[1][3][4][3][4][4]=a*0.5/4.0;
	M[1][3][4][4][1][1]=a*0.5/4.0;
	M[1][3][4][4][1][3]=(1-a)*0.5/4.0;
	M[1][3][4][4][2][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][4][2][4]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][4][3][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][4][3][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][3][4][4][4][4]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[1][4][0][0][0][2]=0.5*1/4.0+0.5*1/4.0;
	M[1][4][0][0][0][3]=0.5*1/4.0+0.5*1/4.0;
	M[1][4][0][0][1][1]=0.5*1/4.0+0.5*1/4.0;
	M[1][4][0][0][1][2]=0.5*1/4.0+0.5*1/4.0;
	M[1][4][0][1][0][2]=0.5*1/4.0;
	M[1][4][0][1][0][3]=0.5*1/4.0+0.5*1/4.0;
	M[1][4][0][1][0][4]=0.5*1/4.0;
	M[1][4][0][1][1][1]=0.5*1/4.0;
	M[1][4][0][1][1][2]=0.5*1/4.0+0.5*1/4.0;
	M[1][4][0][1][1][3]=0.5*1/4.0;
	M[1][4][0][2][0][2]=0.5*a/4.0;
	M[1][4][0][2][0][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][0][2][0][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[1][4][0][2][1][1]=0.5*a/4.0;
	M[1][4][0][2][1][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[1][4][0][2][1][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][0][2][1][4]=0.5*a/4.0;
	M[1][4][0][3][0][2]=0.5*a/4.0;
	M[1][4][0][3][0][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][0][3][0][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][0][3][1][1]=0.5*a/4.0;
	M[1][4][0][3][1][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[1][4][0][3][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][0][3][1][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][0][4][0][3]=0.5*0.5/4.0;
	M[1][4][0][4][0][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][0][4][1][2]=0.5*0.5/4.0;
	M[1][4][0][4][1][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][0][4][1][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][1][0][0][2]=0.5*1/4.0;
	M[1][4][1][0][0][3]=0.5*1/4.0;
	M[1][4][1][0][1][1]=0.5*1/4.0;
	M[1][4][1][0][1][2]=0.5*1/4.0+0.5*1/4.0;
	M[1][4][1][0][1][3]=0.5*1/4.0;
	M[1][4][1][0][2][1]=0.5*1/4.0;
	M[1][4][1][0][2][2]=0.5*1/4.0;
	M[1][4][1][1][0][2]=0.5*1/4.0;
	M[1][4][1][1][0][3]=0.5*1/4.0;
	M[1][4][1][1][1][1]=0.5*1/4.0;
	M[1][4][1][1][1][2]=0.5*1/4.0;
	M[1][4][1][1][1][3]=0.5*1/4.0;
	M[1][4][1][1][1][4]=0.5*1/4.0;
	M[1][4][1][1][2][2]=0.5*1/4.0;
	M[1][4][1][1][2][3]=0.5*1/4.0;
	M[1][4][1][2][0][2]=0.5*a/4.0;
	M[1][4][1][2][0][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][1][2][0][4]=0.5*(1-a)/4.0;
	M[1][4][1][2][1][1]=0.5*a/4.0;
	M[1][4][1][2][1][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[1][4][1][2][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][1][2][1][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[1][4][1][2][2][2]=0.5*(1-a)/4.0;
	M[1][4][1][2][2][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][1][2][2][4]=0.5*a/4.0;
	M[1][4][1][3][0][2]=0.5*a/4.0;
	M[1][4][1][3][0][3]=0.5*a/4.0;
	M[1][4][1][3][0][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][1][3][1][1]=0.5*a/4.0;
	M[1][4][1][3][1][2]=0.5*a/4.0;
	M[1][4][1][3][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][1][3][1][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][1][3][2][2]=0.5*(1-a)/4.0;
	M[1][4][1][3][2][3]=0.5*(1-a)/4.0;
	M[1][4][1][3][2][4]=0.5*a/4.0+0.5*a/4.0;
	M[1][4][1][4][0][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][1][4][1][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][1][4][1][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][1][4][2][2]=0.5*0.5/4.0;
	M[1][4][1][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][1][4][2][4]=0.5*0.5/4.0;
	M[1][4][2][0][0][2]=0.5*a/4.0;
	M[1][4][2][0][0][3]=0.5*a/4.0;
	M[1][4][2][0][1][1]=0.5*a/4.0;
	M[1][4][2][0][1][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][0][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][0][2][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][0][2][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][0][2][3]=0.5*a/4.0;
	M[1][4][2][0][3][1]=0.5*a/4.0;
	M[1][4][2][0][3][2]=0.5*a/4.0;
	M[1][4][2][1][0][2]=0.5*a/4.0;
	M[1][4][2][1][0][3]=0.5*a/4.0;
	M[1][4][2][1][1][1]=0.5*a/4.0;
	M[1][4][2][1][1][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][2][1][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][1][1][4]=0.5*(1-a)/4.0;
	M[1][4][2][1][2][1]=0.5*(1-a)/4.0;
	M[1][4][2][1][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][1][2][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][2][1][2][4]=0.5*a/4.0;
	M[1][4][2][1][3][2]=0.5*a/4.0;
	M[1][4][2][1][3][3]=0.5*a/4.0;
	M[1][4][2][2][0][2]=0.5*a/4.0;
	M[1][4][2][2][0][3]=0.5*a/4.0;
	M[1][4][2][2][1][1]=0.5*a/4.0;
	M[1][4][2][2][1][2]=0.5*a/4.0;
	M[1][4][2][2][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][2][1][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][2][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][2][2][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][2][2][4]=0.5*a/4.0+0.5*a/4.0;
	M[1][4][2][2][3][3]=0.5*a/4.0;
	M[1][4][2][2][3][4]=0.5*a/4.0;
	M[1][4][2][3][0][2]=0.5*a/4.0;
	M[1][4][2][3][0][3]=0.5*a/4.0;
	M[1][4][2][3][1][1]=0.5*a/4.0;
	M[1][4][2][3][1][2]=0.5*a/4.0;
	M[1][4][2][3][1][3]=0.5*(1-a)/4.0;
	M[1][4][2][3][1][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][3][2][2]=0.5*(1-a)/4.0;
	M[1][4][2][3][2][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][2][3][2][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[1][4][2][3][3][4]=0.5*a/4.0+0.5*a/4.0;
	M[1][4][2][4][0][4]=0.5*a/4.0+0.5*a/4.0;
	M[1][4][2][4][1][3]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0;
	M[1][4][2][4][1][4]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[1][4][2][4][2][2]=0.5*(0.5-0.5*a)/4.0;
	M[1][4][2][4][2][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[1][4][2][4][2][4]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[1][4][2][4][3][3]=0.5*a/4.0;
	M[1][4][2][4][3][4]=0.5*a/4.0;
	M[1][4][3][0][0][2]=0.5*a/4.0;
	M[1][4][3][0][0][3]=0.5*a/4.0;
	M[1][4][3][0][1][1]=0.5*a/4.0;
	M[1][4][3][0][1][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][3][0][1][3]=0.5*(1-a)/4.0;
	M[1][4][3][0][2][1]=0.5*(1-a)/4.0;
	M[1][4][3][0][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][3][0][2][3]=0.5*(1-a)/4.0;
	M[1][4][3][0][3][1]=0.5*(1-a)/4.0;
	M[1][4][3][0][3][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][3][0][3][3]=0.5*a/4.0;
	M[1][4][3][0][4][1]=0.5*a/4.0;
	M[1][4][3][0][4][2]=0.5*a/4.0;
	M[1][4][3][1][0][2]=0.5*a/4.0;
	M[1][4][3][1][0][3]=0.5*a/4.0;
	M[1][4][3][1][1][1]=0.5*a/4.0;
	M[1][4][3][1][1][2]=0.5*a/4.0;
	M[1][4][3][1][1][3]=0.5*(1-a)/4.0;
	M[1][4][3][1][1][4]=0.5*(1-a)/4.0;
	M[1][4][3][1][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][3][1][2][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][3][1][3][1]=0.5*(1-a)/4.0;
	M[1][4][3][1][3][2]=0.5*(1-a)/4.0;
	M[1][4][3][1][3][3]=0.5*a/4.0;
	M[1][4][3][1][3][4]=0.5*a/4.0;
	M[1][4][3][1][4][2]=0.5*a/4.0;
	M[1][4][3][1][4][3]=0.5*a/4.0;
	M[1][4][3][2][0][2]=0.5*a/4.0;
	M[1][4][3][2][0][3]=0.5*a/4.0;
	M[1][4][3][2][1][1]=0.5*a/4.0;
	M[1][4][3][2][1][2]=0.5*a/4.0;
	M[1][4][3][2][1][3]=0.5*(1-a)/4.0;
	M[1][4][3][2][1][4]=0.5*(1-a)/4.0;
	M[1][4][3][2][2][2]=0.5*(1-a)/4.0;
	M[1][4][3][2][2][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][3][2][2][4]=0.5*(1-a)/4.0;
	M[1][4][3][2][3][2]=0.5*(1-a)/4.0;
	M[1][4][3][2][3][3]=0.5*(1-a)/4.0;
	M[1][4][3][2][3][4]=0.5*a/4.0+0.5*a/4.0;
	M[1][4][3][2][4][3]=0.5*a/4.0;
	M[1][4][3][2][4][4]=0.5*a/4.0;
	M[1][4][3][3][0][2]=0.5*a/4.0;
	M[1][4][3][3][0][3]=0.5*a/4.0;
	M[1][4][3][3][1][1]=0.5*a/4.0;
	M[1][4][3][3][1][2]=0.5*a/4.0;
	M[1][4][3][3][1][3]=0.5*(1-a)/4.0;
	M[1][4][3][3][1][4]=0.5*(1-a)/4.0;
	M[1][4][3][3][2][2]=0.5*(1-a)/4.0;
	M[1][4][3][3][2][3]=0.5*(1-a)/4.0;
	M[1][4][3][3][2][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[1][4][3][3][3][3]=0.5*(1-a)/4.0;
	M[1][4][3][3][3][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[1][4][3][3][4][4]=0.5*a/4.0+0.5*a/4.0;
	M[1][4][3][4][1][3]=0.5*0.5/4.0;
	M[1][4][3][4][1][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][3][4][2][2]=0.5*0.5/4.0;
	M[1][4][3][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][3][4][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][3][4][3][3]=0.5*0.5/4.0;
	M[1][4][3][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][0][1][2]=0.5*0.5/4.0;
	M[1][4][4][0][1][3]=0.5*0.5/4.0;
	M[1][4][4][0][2][1]=0.5*0.5/4.0;
	M[1][4][4][0][2][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][0][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][0][3][1]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][0][3][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][0][3][3]=0.5*0.5/4.0;
	M[1][4][4][0][4][1]=0.5*0.5/4.0;
	M[1][4][4][0][4][2]=0.5*0.5/4.0;
	M[1][4][4][1][1][3]=0.5*0.5/4.0;
	M[1][4][4][1][1][4]=0.5*0.5/4.0;
	M[1][4][4][1][2][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][1][2][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][1][2][4]=0.5*0.5/4.0;
	M[1][4][4][1][3][1]=0.5*0.5/4.0;
	M[1][4][4][1][3][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][1][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][1][4][1]=0.5*0.5/4.0;
	M[1][4][4][1][4][2]=0.5*0.5/4.0;
	M[1][4][4][2][1][3]=0.5*(0.5-0.5*a)/4.0;
	M[1][4][4][2][1][4]=0.5*(0.5-0.5*a)/4.0;
	M[1][4][4][2][2][2]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[1][4][4][2][2][3]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[1][4][4][2][2][4]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[1][4][4][2][3][1]=0.5*a/4.0;
	M[1][4][4][2][3][2]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[1][4][4][2][3][3]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[1][4][4][2][3][4]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[1][4][4][2][4][2]=0.5*(0.5-0.5*a)/4.0;
	M[1][4][4][2][4][3]=0.5*(0.5-0.5*a)/4.0;
	M[1][4][4][3][1][3]=0.5*0.5/4.0;
	M[1][4][4][3][1][4]=0.5*0.5/4.0;
	M[1][4][4][3][2][2]=0.5*0.5/4.0;
	M[1][4][4][3][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][3][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][3][3][2]=0.5*0.5/4.0;
	M[1][4][4][3][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][3][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][3][4][3]=0.5*0.5/4.0;
	M[1][4][4][3][4][4]=0.5*0.5/4.0;
	M[1][4][4][4][1][3]=0.5*0.5/4.0;
	M[1][4][4][4][1][4]=0.5*0.5/4.0;
	M[1][4][4][4][2][2]=0.5*0.5/4.0;
	M[1][4][4][4][2][3]=0.5*0.5/4.0;
	M[1][4][4][4][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[1][4][4][4][4][4]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[2][0][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[2][0][0][0][1][0]=(1-a)*1/4.0+(1-a)*1/4.0+(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][0][0][0][2][0]=a*1/4.0+a*1/4.0;
	M[2][0][0][1][0][0]=a*1/4.0;
	M[2][0][0][1][0][1]=a*1/4.0;
	M[2][0][0][1][1][0]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][0][0][1][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][0][0][1][2][0]=a*1/4.0;
	M[2][0][0][1][2][1]=a*1/4.0;
	M[2][0][0][2][0][0]=a*a/4.0;
	M[2][0][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][0][0][2][0][2]=a*a/4.0;
	M[2][0][0][2][1][0]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][0][2][1][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][0][2][1][2]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][0][2][2][0]=a*a/4.0;
	M[2][0][0][2][2][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][0][0][2][2][2]=a*a/4.0;
	M[2][0][0][3][0][0]=a*a/4.0;
	M[2][0][0][3][0][1]=a*(1-a)/4.0;
	M[2][0][0][3][0][2]=a*(1-a)/4.0;
	M[2][0][0][3][0][3]=a*a/4.0;
	M[2][0][0][3][1][0]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][0][3][1][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][0][3][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][0][3][1][3]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][0][3][2][0]=a*a/4.0;
	M[2][0][0][3][2][1]=a*(1-a)/4.0;
	M[2][0][0][3][2][2]=a*(1-a)/4.0;
	M[2][0][0][3][2][3]=a*a/4.0;
	M[2][0][0][4][0][1]=a*0.5/4.0;
	M[2][0][0][4][0][2]=a*0.5/4.0+a*0.5/4.0;
	M[2][0][0][4][0][3]=a*0.5/4.0;
	M[2][0][0][4][1][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][0][4][1][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][0][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][0][4][2][1]=a*0.5/4.0;
	M[2][0][0][4][2][2]=a*0.5/4.0+a*0.5/4.0;
	M[2][0][0][4][2][3]=a*0.5/4.0;
	M[2][0][1][0][0][0]=a*1/4.0;
	M[2][0][1][0][1][0]=a*1/4.0+(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][0][1][0][2][0]=a*1/4.0+(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][0][1][0][3][0]=a*1/4.0;
	M[2][0][1][1][0][0]=a*1/4.0;
	M[2][0][1][1][1][0]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][0][1][1][1][1]=a*1/4.0;
	M[2][0][1][1][2][0]=a*1/4.0;
	M[2][0][1][1][2][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][0][1][1][3][1]=a*1/4.0;
	M[2][0][1][2][0][0]=a*a/4.0;
	M[2][0][1][2][0][1]=a*(1-a)/4.0;
	M[2][0][1][2][1][0]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][1][2][1][1]=a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][1][2][1][2]=a*a/4.0;
	M[2][0][1][2][2][0]=a*a/4.0;
	M[2][0][1][2][2][1]=a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][1][2][2][2]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][1][2][3][1]=a*(1-a)/4.0;
	M[2][0][1][2][3][2]=a*a/4.0;
	M[2][0][1][3][0][0]=a*a/4.0;
	M[2][0][1][3][0][2]=a*(1-a)/4.0;
	M[2][0][1][3][1][0]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][1][3][1][1]=a*(1-a)/4.0;
	M[2][0][1][3][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][1][3][1][3]=a*a/4.0;
	M[2][0][1][3][2][0]=a*a/4.0;
	M[2][0][1][3][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][1][3][2][2]=a*(1-a)/4.0;
	M[2][0][1][3][2][3]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][1][3][3][1]=a*(1-a)/4.0;
	M[2][0][1][3][3][3]=a*a/4.0;
	M[2][0][1][4][0][2]=a*0.5/4.0;
	M[2][0][1][4][0][3]=a*0.5/4.0;
	M[2][0][1][4][1][1]=a*0.5/4.0;
	M[2][0][1][4][1][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][1][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][1][4][2][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][1][4][2][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][1][4][2][3]=a*0.5/4.0;
	M[2][0][1][4][3][1]=a*0.5/4.0;
	M[2][0][1][4][3][2]=a*0.5/4.0;
	M[2][0][2][0][0][0]=a*a/4.0;
	M[2][0][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][2][0][2][0]=a*a/4.0+a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][2][0][3][0]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][2][0][4][0]=a*a/4.0;
	M[2][0][2][1][0][0]=a*a/4.0;
	M[2][0][2][1][1][0]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][2][1][1][1]=a*(1-a)/4.0;
	M[2][0][2][1][2][0]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][2][1][2][1]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][2][1][3][0]=a*(1-a)/4.0;
	M[2][0][2][1][3][1]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][2][1][4][1]=a*a/4.0;
	M[2][0][2][2][0][0]=a*a/4.0;
	M[2][0][2][2][1][0]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][2][2][1][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][0][2][2][2][0]=a*a/4.0;
	M[2][0][2][2][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][2][2][2][2]=a*a/4.0;
	M[2][0][2][2][3][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][0][2][2][3][2]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][2][2][4][2]=a*a/4.0;
	M[2][0][2][3][0][0]=a*a/4.0;
	M[2][0][2][3][1][0]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][2][3][1][1]=a*(1-a)/4.0;
	M[2][0][2][3][1][2]=a*(1-a)/4.0;
	M[2][0][2][3][2][0]=a*a/4.0;
	M[2][0][2][3][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][2][3][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][2][3][2][3]=a*a/4.0;
	M[2][0][2][3][3][1]=a*(1-a)/4.0;
	M[2][0][2][3][3][2]=a*(1-a)/4.0;
	M[2][0][2][3][3][3]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][2][3][4][3]=a*a/4.0;
	M[2][0][2][4][0][2]=a*a/4.0;
	M[2][0][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[2][0][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][2][4][1][3]=a*(0.5-0.5*a)/4.0;
	M[2][0][2][4][2][1]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][0][2][4][2][2]=a*a/4.0+a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][0][2][4][2][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][0][2][4][3][1]=a*(0.5-0.5*a)/4.0;
	M[2][0][2][4][3][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][2][4][3][3]=a*(0.5-0.5*a)/4.0;
	M[2][0][2][4][4][2]=a*a/4.0;
	M[2][0][3][0][0][0]=a*a/4.0;
	M[2][0][3][0][1][0]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][3][0][2][0]=a*a/4.0+a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][3][0][3][0]=a*a/4.0+a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][3][0][4][0]=a*a/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][3][1][0][0]=a*a/4.0;
	M[2][0][3][1][1][0]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][3][1][1][1]=a*(1-a)/4.0;
	M[2][0][3][1][2][0]=a*a/4.0+a*(1-a)/4.0;
	M[2][0][3][1][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][3][1][3][0]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][3][1][3][1]=a*a/4.0+a*(1-a)/4.0;
	M[2][0][3][1][4][0]=a*(1-a)/4.0;
	M[2][0][3][1][4][1]=a*a/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][3][2][0][0]=a*a/4.0;
	M[2][0][3][2][1][0]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][3][2][1][1]=a*(1-a)/4.0;
	M[2][0][3][2][2][0]=a*a/4.0;
	M[2][0][3][2][2][1]=a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][3][2][3][1]=a*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][3][2][3][2]=a*a/4.0;
	M[2][0][3][2][4][1]=a*(1-a)/4.0;
	M[2][0][3][2][4][2]=a*a/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][3][3][0][0]=a*a/4.0;
	M[2][0][3][3][1][0]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][3][3][1][1]=a*(1-a)/4.0;
	M[2][0][3][3][2][0]=a*a/4.0;
	M[2][0][3][3][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][3][3][2][2]=a*(1-a)/4.0;
	M[2][0][3][3][3][1]=a*(1-a)/4.0;
	M[2][0][3][3][3][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][0][3][3][3][3]=a*a/4.0;
	M[2][0][3][3][4][2]=a*(1-a)/4.0;
	M[2][0][3][3][4][3]=a*a/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][3][4][1][1]=a*0.5/4.0;
	M[2][0][3][4][1][2]=a*0.5/4.0;
	M[2][0][3][4][2][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][3][4][2][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][3][4][2][3]=a*0.5/4.0;
	M[2][0][3][4][3][1]=a*0.5/4.0;
	M[2][0][3][4][3][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][3][4][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][3][4][4][2]=a*0.5/4.0;
	M[2][0][3][4][4][3]=a*0.5/4.0;
	M[2][0][4][0][1][0]=a*0.5/4.0;
	M[2][0][4][0][2][0]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][0][3][0]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][0][4][0]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][1][1][1]=a*0.5/4.0;
	M[2][0][4][1][2][0]=a*0.5/4.0;
	M[2][0][4][1][2][1]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][1][3][0]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][1][3][1]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][1][4][0]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][1][4][1]=a*0.5/4.0;
	M[2][0][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[2][0][4][2][2][0]=a*a/4.0;
	M[2][0][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][0][4][2][2][2]=a*a/4.0;
	M[2][0][4][2][3][0]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][4][2][3][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][0][4][2][3][2]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][0][4][2][4][0]=a*a/4.0;
	M[2][0][4][2][4][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][0][4][2][4][2]=a*a/4.0;
	M[2][0][4][3][1][1]=a*0.5/4.0;
	M[2][0][4][3][2][1]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][3][2][2]=a*0.5/4.0;
	M[2][0][4][3][3][1]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][3][3][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][3][4][1]=a*0.5/4.0;
	M[2][0][4][3][4][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][4][1][1]=a*0.5/4.0;
	M[2][0][4][4][2][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][4][2][2]=a*0.5/4.0+a*0.5/4.0;
	M[2][0][4][4][3][1]=a*0.5/4.0;
	M[2][0][4][4][3][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][0][4][4][3][3]=a*0.5/4.0;
	M[2][0][4][4][4][2]=a*0.5/4.0+a*0.5/4.0;
	M[2][0][4][4][4][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[2][1][0][0][1][0]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][1][0][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][1][0][0][2][1]=a*1/4.0+a*1/4.0;
	M[2][1][0][1][0][0]=a*1/4.0;
	M[2][1][0][1][0][1]=a*1/4.0;
	M[2][1][0][1][1][0]=(1-a)*1/4.0;
	M[2][1][0][1][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][1][0][1][1][2]=(1-a)*1/4.0;
	M[2][1][0][1][2][1]=a*1/4.0;
	M[2][1][0][1][2][2]=a*1/4.0;
	M[2][1][0][2][0][0]=a*a/4.0;
	M[2][1][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][1][0][2][0][2]=a*a/4.0;
	M[2][1][0][2][1][0]=(1-a)*a/4.0;
	M[2][1][0][2][1][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][0][2][1][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][0][2][1][3]=(1-a)*a/4.0;
	M[2][1][0][2][2][1]=a*a/4.0;
	M[2][1][0][2][2][2]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][1][0][2][2][3]=a*a/4.0;
	M[2][1][0][3][0][0]=a*a/4.0;
	M[2][1][0][3][0][1]=a*(1-a)/4.0;
	M[2][1][0][3][0][2]=a*(1-a)/4.0;
	M[2][1][0][3][0][3]=a*a/4.0;
	M[2][1][0][3][1][0]=(1-a)*a/4.0;
	M[2][1][0][3][1][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][0][3][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][0][3][1][3]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][0][3][1][4]=(1-a)*a/4.0;
	M[2][1][0][3][2][1]=a*a/4.0;
	M[2][1][0][3][2][2]=a*(1-a)/4.0;
	M[2][1][0][3][2][3]=a*(1-a)/4.0;
	M[2][1][0][3][2][4]=a*a/4.0;
	M[2][1][0][4][0][1]=a*0.5/4.0;
	M[2][1][0][4][0][2]=a*0.5/4.0+a*0.5/4.0;
	M[2][1][0][4][0][3]=a*0.5/4.0;
	M[2][1][0][4][1][1]=(1-a)*0.5/4.0;
	M[2][1][0][4][1][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][0][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][0][4][1][4]=(1-a)*0.5/4.0;
	M[2][1][0][4][2][2]=a*0.5/4.0;
	M[2][1][0][4][2][3]=a*0.5/4.0+a*0.5/4.0;
	M[2][1][0][4][2][4]=a*0.5/4.0;
	M[2][1][1][0][0][0]=a*1/4.0;
	M[2][1][1][0][1][0]=a*1/4.0+(1-a)*1/4.0;
	M[2][1][1][0][1][1]=(1-a)*1/4.0;
	M[2][1][1][0][2][0]=(1-a)*1/4.0;
	M[2][1][1][0][2][1]=a*1/4.0+(1-a)*1/4.0;
	M[2][1][1][0][3][1]=a*1/4.0;
	M[2][1][1][1][0][0]=a*1/4.0;
	M[2][1][1][1][1][0]=(1-a)*1/4.0;
	M[2][1][1][1][1][1]=a*1/4.0+(1-a)*1/4.0;
	M[2][1][1][1][2][1]=a*1/4.0+(1-a)*1/4.0;
	M[2][1][1][1][2][2]=(1-a)*1/4.0;
	M[2][1][1][1][3][2]=a*1/4.0;
	M[2][1][1][2][0][0]=a*a/4.0;
	M[2][1][1][2][0][1]=a*(1-a)/4.0;
	M[2][1][1][2][1][0]=(1-a)*a/4.0;
	M[2][1][1][2][1][1]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][1][2][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][1][2][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][1][2][2][2]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][1][2][2][3]=(1-a)*a/4.0;
	M[2][1][1][2][3][2]=a*(1-a)/4.0;
	M[2][1][1][2][3][3]=a*a/4.0;
	M[2][1][1][3][0][0]=a*a/4.0;
	M[2][1][1][3][0][2]=a*(1-a)/4.0;
	M[2][1][1][3][1][0]=(1-a)*a/4.0;
	M[2][1][1][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][1][3][1][2]=(1-a)*(1-a)/4.0;
	M[2][1][1][3][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][1][3][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][1][3][2][2]=(1-a)*(1-a)/4.0;
	M[2][1][1][3][2][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][1][3][2][4]=(1-a)*a/4.0;
	M[2][1][1][3][3][2]=a*(1-a)/4.0;
	M[2][1][1][3][3][4]=a*a/4.0;
	M[2][1][1][4][0][2]=a*0.5/4.0;
	M[2][1][1][4][0][3]=a*0.5/4.0;
	M[2][1][1][4][1][1]=a*0.5/4.0;
	M[2][1][1][4][1][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][1][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][1][4][1][4]=(1-a)*0.5/4.0;
	M[2][1][1][4][2][1]=(1-a)*0.5/4.0;
	M[2][1][1][4][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][1][4][2][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][1][4][2][4]=a*0.5/4.0;
	M[2][1][1][4][3][2]=a*0.5/4.0;
	M[2][1][1][4][3][3]=a*0.5/4.0;
	M[2][1][2][0][0][0]=a*a/4.0;
	M[2][1][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][2][0][1][1]=(1-a)*a/4.0;
	M[2][1][2][0][2][0]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][2][0][2][1]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][2][0][3][0]=(1-a)*a/4.0;
	M[2][1][2][0][3][1]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][2][0][4][1]=a*a/4.0;
	M[2][1][2][1][0][0]=a*a/4.0;
	M[2][1][2][1][1][0]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][2][1][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][2][1][2][0]=(1-a)*(1-a)/4.0;
	M[2][1][2][1][2][1]=a*a/4.0+a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][2][1][2][2]=(1-a)*(1-a)/4.0;
	M[2][1][2][1][3][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][2][1][3][2]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][2][1][4][2]=a*a/4.0;
	M[2][1][2][2][0][0]=a*a/4.0;
	M[2][1][2][2][1][0]=(1-a)*a/4.0;
	M[2][1][2][2][1][1]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][2][2][2][1]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][2][2][2][2]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][2][2][3][2]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][2][2][3][3]=(1-a)*a/4.0;
	M[2][1][2][2][4][3]=a*a/4.0;
	M[2][1][2][3][0][0]=a*a/4.0;
	M[2][1][2][3][1][0]=(1-a)*a/4.0;
	M[2][1][2][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][2][3][1][2]=a*(1-a)/4.0;
	M[2][1][2][3][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][2][3][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][2][3][2][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][2][3][3][2]=a*(1-a)/4.0;
	M[2][1][2][3][3][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][2][3][3][4]=(1-a)*a/4.0;
	M[2][1][2][3][4][4]=a*a/4.0;
	M[2][1][2][4][0][2]=a*a/4.0;
	M[2][1][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[2][1][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[2][1][2][4][1][3]=a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[2][1][2][4][2][1]=(1-a)*(0.5-0.5*a)/4.0;
	M[2][1][2][4][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][1][2][4][2][3]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][1][2][4][2][4]=(1-a)*(0.5-0.5*a)/4.0;
	M[2][1][2][4][3][2]=a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[2][1][2][4][3][3]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[2][1][2][4][3][4]=a*(0.5-0.5*a)/4.0;
	M[2][1][2][4][4][3]=a*a/4.0;
	M[2][1][3][0][0][0]=a*a/4.0;
	M[2][1][3][0][1][0]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][3][0][1][1]=(1-a)*a/4.0;
	M[2][1][3][0][2][0]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][0][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][0][3][0]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][0][3][1]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][0][4][0]=(1-a)*a/4.0;
	M[2][1][3][0][4][1]=a*a/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][3][1][0][0]=a*a/4.0;
	M[2][1][3][1][1][0]=(1-a)*a/4.0;
	M[2][1][3][1][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][3][1][2][0]=a*(1-a)/4.0;
	M[2][1][3][1][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][1][2][2]=(1-a)*(1-a)/4.0;
	M[2][1][3][1][3][0]=(1-a)*(1-a)/4.0;
	M[2][1][3][1][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][1][3][2]=a*(1-a)/4.0;
	M[2][1][3][1][4][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][3][1][4][2]=a*a/4.0+(1-a)*a/4.0;
	M[2][1][3][2][0][0]=a*a/4.0;
	M[2][1][3][2][1][0]=(1-a)*a/4.0;
	M[2][1][3][2][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][3][2][2][1]=a*(1-a)/4.0+a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][2][2][2]=(1-a)*(1-a)/4.0;
	M[2][1][3][2][3][1]=(1-a)*(1-a)/4.0;
	M[2][1][3][2][3][2]=a*(1-a)/4.0+a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][2][4][2]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][3][2][4][3]=a*a/4.0+(1-a)*a/4.0;
	M[2][1][3][3][0][0]=a*a/4.0;
	M[2][1][3][3][1][0]=(1-a)*a/4.0;
	M[2][1][3][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][3][3][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][3][2][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][3][3][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][3][3][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[2][1][3][3][4][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[2][1][3][3][4][4]=a*a/4.0+(1-a)*a/4.0;
	M[2][1][3][4][1][1]=a*0.5/4.0;
	M[2][1][3][4][1][2]=a*0.5/4.0;
	M[2][1][3][4][2][1]=(1-a)*0.5/4.0;
	M[2][1][3][4][2][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][3][4][2][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][3][4][3][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][3][4][3][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][3][4][3][4]=(1-a)*0.5/4.0;
	M[2][1][3][4][4][3]=a*0.5/4.0;
	M[2][1][3][4][4][4]=a*0.5/4.0;
	M[2][1][4][0][1][0]=a*0.5/4.0;
	M[2][1][4][0][2][0]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][0][2][1]=(1-a)*0.5/4.0;
	M[2][1][4][0][3][0]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][0][3][1]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][0][4][0]=(1-a)*0.5/4.0;
	M[2][1][4][0][4][1]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][1][1][1]=a*0.5/4.0;
	M[2][1][4][1][2][0]=a*0.5/4.0;
	M[2][1][4][1][2][1]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][1][2][2]=(1-a)*0.5/4.0;
	M[2][1][4][1][3][0]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][1][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][1][3][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][1][4][0]=(1-a)*0.5/4.0;
	M[2][1][4][1][4][1]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][1][4][2]=a*0.5/4.0;
	M[2][1][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[2][1][4][2][2][0]=a*a/4.0;
	M[2][1][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][1][4][2][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][1][4][2][3][0]=(1-a)*a/4.0;
	M[2][1][4][2][3][1]=a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[2][1][4][2][3][2]=a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[2][1][4][2][3][3]=(1-a)*a/4.0;
	M[2][1][4][2][4][1]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][1][4][2][4][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][1][4][2][4][3]=a*a/4.0;
	M[2][1][4][3][1][1]=a*0.5/4.0;
	M[2][1][4][3][2][1]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][3][2][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][3][3][1]=(1-a)*0.5/4.0;
	M[2][1][4][3][3][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][3][3][3]=(1-a)*0.5/4.0;
	M[2][1][4][3][4][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][3][4][3]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][4][1][1]=a*0.5/4.0;
	M[2][1][4][4][2][1]=(1-a)*0.5/4.0;
	M[2][1][4][4][2][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][4][3][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][4][3][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][4][4][3]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][1][4][4][4][4]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[2][2][0][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0+(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][2][0][0][2][2]=a*1/4.0+a*1/4.0;
	M[2][2][0][1][0][0]=a*1/4.0;
	M[2][2][0][1][0][1]=a*1/4.0;
	M[2][2][0][1][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][2][0][1][1][2]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][2][0][1][2][2]=a*1/4.0;
	M[2][2][0][1][2][3]=a*1/4.0;
	M[2][2][0][2][0][0]=a*a/4.0;
	M[2][2][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][2][0][2][0][2]=a*a/4.0;
	M[2][2][0][2][1][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][2][0][2][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][2][0][2][1][3]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][2][0][2][2][2]=a*a/4.0;
	M[2][2][0][2][2][3]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][2][0][2][2][4]=a*a/4.0;
	M[2][2][0][3][0][0]=a*a/4.0;
	M[2][2][0][3][0][1]=a*(1-a)/4.0;
	M[2][2][0][3][0][2]=a*(1-a)/4.0;
	M[2][2][0][3][0][3]=a*a/4.0;
	M[2][2][0][3][1][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][2][0][3][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][2][0][3][1][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][2][0][3][1][4]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][2][0][3][2][2]=a*a/4.0;
	M[2][2][0][3][2][3]=a*(1-a)/4.0;
	M[2][2][0][3][2][4]=a*a/4.0+a*(1-a)/4.0;
	M[2][2][0][4][0][1]=a*0.5/4.0;
	M[2][2][0][4][0][2]=a*0.5/4.0+a*0.5/4.0;
	M[2][2][0][4][0][3]=a*0.5/4.0;
	M[2][2][0][4][1][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][0][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][0][4][1][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][0][4][2][3]=a*0.5/4.0;
	M[2][2][0][4][2][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][2][1][0][0][0]=a*1/4.0;
	M[2][2][1][0][1][0]=a*1/4.0;
	M[2][2][1][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][2][1][0][2][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][2][1][0][2][2]=a*1/4.0;
	M[2][2][1][0][3][2]=a*1/4.0;
	M[2][2][1][1][0][0]=a*1/4.0;
	M[2][2][1][1][1][1]=(1-a)*1/4.0+(1-a)*1/4.0+a*1/4.0;
	M[2][2][1][1][2][2]=(1-a)*1/4.0+(1-a)*1/4.0+a*1/4.0;
	M[2][2][1][1][3][3]=a*1/4.0;
	M[2][2][1][2][0][0]=a*a/4.0;
	M[2][2][1][2][0][1]=a*(1-a)/4.0;
	M[2][2][1][2][1][1]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][1][2][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][1][2][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][1][2][2][3]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][1][2][3][3]=a*(1-a)/4.0;
	M[2][2][1][2][3][4]=a*a/4.0;
	M[2][2][1][3][0][0]=a*a/4.0;
	M[2][2][1][3][0][2]=a*(1-a)/4.0;
	M[2][2][1][3][1][1]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][1][3][1][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][1][3][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][1][3][2][4]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][1][3][3][3]=a*(1-a)/4.0;
	M[2][2][1][3][3][4]=a*a/4.0;
	M[2][2][1][4][0][2]=a*0.5/4.0;
	M[2][2][1][4][0][3]=a*0.5/4.0;
	M[2][2][1][4][1][1]=a*0.5/4.0;
	M[2][2][1][4][1][2]=a*0.5/4.0;
	M[2][2][1][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][1][4][1][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][1][4][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][1][4][2][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][1][4][2][4]=a*0.5/4.0+a*0.5/4.0;
	M[2][2][1][4][3][3]=a*0.5/4.0;
	M[2][2][1][4][3][4]=a*0.5/4.0;
	M[2][2][2][0][0][0]=a*a/4.0;
	M[2][2][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][2][2][0][1][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][2][2][0][2][0]=a*a/4.0;
	M[2][2][2][0][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][2][2][0][2][2]=a*a/4.0;
	M[2][2][2][0][3][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][2][2][0][3][2]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][2][2][0][4][2]=a*a/4.0;
	M[2][2][2][1][0][0]=a*a/4.0;
	M[2][2][2][1][1][0]=a*(1-a)/4.0;
	M[2][2][2][1][1][1]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][2][1][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][2][1][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][2][1][3][2]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][2][1][3][3]=a*(1-a)/4.0;
	M[2][2][2][1][4][3]=a*a/4.0;
	M[2][2][2][2][0][0]=a*a/4.0;
	M[2][2][2][2][1][1]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][2][2][2][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0+a*a/4.0;
	M[2][2][2][2][3][3]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][2][2][2][4][4]=a*a/4.0;
	M[2][2][2][3][0][0]=a*a/4.0;
	M[2][2][2][3][1][1]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][2][3][1][2]=a*(1-a)/4.0;
	M[2][2][2][3][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][2][3][2][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][2][3][3][3]=a*(1-a)/4.0;
	M[2][2][2][3][3][4]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][2][3][4][4]=a*a/4.0;
	M[2][2][2][4][0][2]=a*a/4.0;
	M[2][2][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[2][2][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][2][2][4][1][3]=(1-a)*a/4.0+(1-a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[2][2][2][4][2][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][2][2][4][2][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][2][2][4][2][4]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][2][2][4][3][3]=(1-a)*a/4.0+(1-a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[2][2][2][4][3][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][2][2][4][4][4]=a*a/4.0;
	M[2][2][3][0][0][0]=a*a/4.0;
	M[2][2][3][0][1][0]=a*(1-a)/4.0;
	M[2][2][3][0][1][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][2][3][0][2][0]=a*(1-a)/4.0;
	M[2][2][3][0][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][2][3][0][2][2]=a*a/4.0;
	M[2][2][3][0][3][0]=a*a/4.0;
	M[2][2][3][0][3][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][2][3][0][3][2]=a*(1-a)/4.0;
	M[2][2][3][0][4][1]=(1-a)*a/4.0+(1-a)*a/4.0;
	M[2][2][3][0][4][2]=a*a/4.0+a*(1-a)/4.0;
	M[2][2][3][1][0][0]=a*a/4.0;
	M[2][2][3][1][1][1]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][3][1][2][0]=a*(1-a)/4.0;
	M[2][2][3][1][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][3][1][3][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][3][1][3][3]=a*(1-a)/4.0;
	M[2][2][3][1][4][2]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][3][1][4][3]=a*a/4.0;
	M[2][2][3][2][0][0]=a*a/4.0;
	M[2][2][3][2][1][1]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][3][2][2][1]=a*(1-a)/4.0;
	M[2][2][3][2][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][3][2][3][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][2][3][2][3][3]=a*(1-a)/4.0;
	M[2][2][3][2][4][3]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][3][2][4][4]=a*a/4.0;
	M[2][2][3][3][0][0]=a*a/4.0;
	M[2][2][3][3][1][1]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][2][3][3][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0+a*(1-a)/4.0;
	M[2][2][3][3][3][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0+a*(1-a)/4.0;
	M[2][2][3][3][4][4]=(1-a)*a/4.0+(1-a)*a/4.0+a*a/4.0+a*(1-a)/4.0;
	M[2][2][3][4][1][1]=a*0.5/4.0;
	M[2][2][3][4][1][2]=a*0.5/4.0;
	M[2][2][3][4][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][2][3][4][2][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][2][3][4][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][2][3][4][3][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][2][3][4][4][4]=a*0.5/4.0+a*0.5/4.0;
	M[2][2][4][0][1][0]=a*0.5/4.0;
	M[2][2][4][0][2][0]=a*0.5/4.0+a*0.5/4.0;
	M[2][2][4][0][2][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][4][0][3][0]=a*0.5/4.0;
	M[2][2][4][0][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][4][0][3][2]=a*0.5/4.0;
	M[2][2][4][0][4][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][4][0][4][2]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][2][4][1][1][1]=a*0.5/4.0;
	M[2][2][4][1][2][0]=a*0.5/4.0;
	M[2][2][4][1][2][1]=a*0.5/4.0;
	M[2][2][4][1][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][4][1][3][0]=a*0.5/4.0;
	M[2][2][4][1][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][4][1][3][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][4][1][3][3]=a*0.5/4.0;
	M[2][2][4][1][4][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][2][4][1][4][2]=a*0.5/4.0+a*0.5/4.0;
	M[2][2][4][1][4][3]=a*0.5/4.0;
	M[2][2][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[2][2][4][2][2][0]=a*a/4.0;
	M[2][2][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][2][4][2][2][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][2][4][2][3][1]=(1-a)*a/4.0+(1-a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[2][2][4][2][3][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][2][4][2][3][3]=(1-a)*a/4.0+(1-a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[2][2][4][2][4][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][2][4][2][4][3]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][2][4][2][4][4]=a*a/4.0;
	M[2][2][4][3][1][1]=a*0.5/4.0;
	M[2][2][4][3][2][1]=a*0.5/4.0;
	M[2][2][4][3][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][2][4][3][3][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][2][4][3][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][2][4][3][4][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][2][4][3][4][4]=a*0.5/4.0+a*0.5/4.0;
	M[2][2][4][4][1][1]=a*0.5/4.0;
	M[2][2][4][4][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][2][4][4][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][2][4][4][4][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][3][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[2][3][0][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][3][0][0][1][2]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][3][0][0][2][3]=a*1/4.0+a*1/4.0;
	M[2][3][0][1][0][0]=a*1/4.0;
	M[2][3][0][1][0][1]=a*1/4.0;
	M[2][3][0][1][1][1]=(1-a)*1/4.0;
	M[2][3][0][1][1][2]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[2][3][0][1][1][3]=(1-a)*1/4.0;
	M[2][3][0][1][2][3]=a*1/4.0;
	M[2][3][0][1][2][4]=a*1/4.0;
	M[2][3][0][2][0][0]=a*a/4.0;
	M[2][3][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][3][0][2][0][2]=a*a/4.0;
	M[2][3][0][2][1][1]=(1-a)*a/4.0;
	M[2][3][0][2][1][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][3][0][2][1][3]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][3][0][2][1][4]=(1-a)*a/4.0;
	M[2][3][0][2][2][3]=a*a/4.0;
	M[2][3][0][2][2][4]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][3][0][3][0][0]=a*a/4.0;
	M[2][3][0][3][0][1]=a*(1-a)/4.0;
	M[2][3][0][3][0][2]=a*(1-a)/4.0;
	M[2][3][0][3][0][3]=a*a/4.0;
	M[2][3][0][3][1][1]=(1-a)*a/4.0;
	M[2][3][0][3][1][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[2][3][0][3][1][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][3][0][3][1][4]=(1-a)*a/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[2][3][0][3][2][3]=a*a/4.0;
	M[2][3][0][3][2][4]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][3][0][4][0][1]=a*0.5/4.0;
	M[2][3][0][4][0][2]=a*0.5/4.0+a*0.5/4.0;
	M[2][3][0][4][0][3]=a*0.5/4.0;
	M[2][3][0][4][1][2]=(1-a)*0.5/4.0;
	M[2][3][0][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][3][0][4][1][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][3][0][4][2][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][3][1][0][0][0]=a*1/4.0;
	M[2][3][1][0][1][0]=a*1/4.0;
	M[2][3][1][0][1][1]=(1-a)*1/4.0;
	M[2][3][1][0][1][2]=(1-a)*1/4.0;
	M[2][3][1][0][2][1]=(1-a)*1/4.0;
	M[2][3][1][0][2][2]=(1-a)*1/4.0;
	M[2][3][1][0][2][3]=a*1/4.0;
	M[2][3][1][0][3][3]=a*1/4.0;
	M[2][3][1][1][0][0]=a*1/4.0;
	M[2][3][1][1][1][1]=(1-a)*1/4.0+a*1/4.0;
	M[2][3][1][1][1][2]=(1-a)*1/4.0;
	M[2][3][1][1][2][2]=(1-a)*1/4.0;
	M[2][3][1][1][2][3]=(1-a)*1/4.0+a*1/4.0;
	M[2][3][1][1][3][4]=a*1/4.0;
	M[2][3][1][2][0][0]=a*a/4.0;
	M[2][3][1][2][0][1]=a*(1-a)/4.0;
	M[2][3][1][2][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][1][2][1][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][1][2][1][3]=(1-a)*(1-a)/4.0;
	M[2][3][1][2][2][2]=(1-a)*(1-a)/4.0;
	M[2][3][1][2][2][3]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][1][2][2][4]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][1][2][3][4]=a*a/4.0+a*(1-a)/4.0;
	M[2][3][1][3][0][0]=a*a/4.0;
	M[2][3][1][3][0][2]=a*(1-a)/4.0;
	M[2][3][1][3][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][1][3][1][2]=(1-a)*a/4.0;
	M[2][3][1][3][1][3]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][1][3][1][4]=(1-a)*(1-a)/4.0;
	M[2][3][1][3][2][2]=(1-a)*(1-a)/4.0;
	M[2][3][1][3][2][3]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][1][3][2][4]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][1][3][3][4]=a*a/4.0+a*(1-a)/4.0;
	M[2][3][1][4][0][2]=a*0.5/4.0;
	M[2][3][1][4][0][3]=a*0.5/4.0;
	M[2][3][1][4][1][1]=a*0.5/4.0;
	M[2][3][1][4][1][2]=a*0.5/4.0;
	M[2][3][1][4][1][3]=(1-a)*0.5/4.0;
	M[2][3][1][4][1][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][3][1][4][2][2]=(1-a)*0.5/4.0;
	M[2][3][1][4][2][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][3][1][4][2][4]=(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][3][1][4][3][4]=a*0.5/4.0+a*0.5/4.0;
	M[2][3][2][0][0][0]=a*a/4.0;
	M[2][3][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][3][2][0][1][1]=(1-a)*a/4.0;
	M[2][3][2][0][1][2]=(1-a)*a/4.0;
	M[2][3][2][0][2][0]=a*a/4.0;
	M[2][3][2][0][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][3][2][0][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][3][2][0][2][3]=a*a/4.0;
	M[2][3][2][0][3][1]=(1-a)*a/4.0;
	M[2][3][2][0][3][2]=(1-a)*a/4.0;
	M[2][3][2][0][3][3]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][3][2][0][4][3]=a*a/4.0;
	M[2][3][2][1][0][0]=a*a/4.0;
	M[2][3][2][1][1][0]=a*(1-a)/4.0;
	M[2][3][2][1][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][2][1][1][2]=(1-a)*a/4.0;
	M[2][3][2][1][2][1]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][2][1][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[2][3][2][1][2][3]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][2][1][3][2]=(1-a)*a/4.0;
	M[2][3][2][1][3][3]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][2][1][3][4]=a*(1-a)/4.0;
	M[2][3][2][1][4][4]=a*a/4.0;
	M[2][3][2][2][0][0]=a*a/4.0;
	M[2][3][2][2][1][1]=(1-a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][3][2][2][1][2]=(1-a)*a/4.0;
	M[2][3][2][2][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][2][2][2][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][2][2][3][3]=(1-a)*a/4.0;
	M[2][3][2][2][3][4]=(1-a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][3][2][2][4][4]=a*a/4.0;
	M[2][3][2][3][0][0]=a*a/4.0;
	M[2][3][2][3][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][2][3][1][2]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][2][3][2][2]=(1-a)*(1-a)/4.0;
	M[2][3][2][3][2][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0+a*a/4.0;
	M[2][3][2][3][2][4]=(1-a)*(1-a)/4.0;
	M[2][3][2][3][3][4]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][3][2][3][4][4]=a*a/4.0;
	M[2][3][2][4][0][2]=a*a/4.0;
	M[2][3][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[2][3][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][3][2][4][1][3]=(1-a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[2][3][2][4][1][4]=(1-a)*a/4.0;
	M[2][3][2][4][2][2]=(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][3][2][4][2][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[2][3][2][4][2][4]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][3][2][4][3][3]=(1-a)*a/4.0;
	M[2][3][2][4][3][4]=(1-a)*a/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][3][2][4][4][4]=a*a/4.0;
	M[2][3][3][0][0][0]=a*a/4.0;
	M[2][3][3][0][1][0]=a*(1-a)/4.0;
	M[2][3][3][0][1][1]=(1-a)*a/4.0;
	M[2][3][3][0][1][2]=(1-a)*a/4.0;
	M[2][3][3][0][2][0]=a*(1-a)/4.0;
	M[2][3][3][0][2][1]=(1-a)*(1-a)/4.0;
	M[2][3][3][0][2][2]=(1-a)*(1-a)/4.0;
	M[2][3][3][0][2][3]=a*a/4.0;
	M[2][3][3][0][3][0]=a*a/4.0;
	M[2][3][3][0][3][1]=(1-a)*(1-a)/4.0;
	M[2][3][3][0][3][2]=(1-a)*(1-a)/4.0;
	M[2][3][3][0][3][3]=a*(1-a)/4.0;
	M[2][3][3][0][4][1]=(1-a)*a/4.0;
	M[2][3][3][0][4][2]=(1-a)*a/4.0;
	M[2][3][3][0][4][3]=a*a/4.0+a*(1-a)/4.0;
	M[2][3][3][1][0][0]=a*a/4.0;
	M[2][3][3][1][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][3][1][1][2]=(1-a)*a/4.0;
	M[2][3][3][1][2][0]=a*(1-a)/4.0;
	M[2][3][3][1][2][2]=(1-a)*(1-a)/4.0;
	M[2][3][3][1][2][3]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][3][1][3][1]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][3][1][3][2]=(1-a)*(1-a)/4.0;
	M[2][3][3][1][3][4]=a*(1-a)/4.0;
	M[2][3][3][1][4][2]=(1-a)*a/4.0;
	M[2][3][3][1][4][3]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][3][1][4][4]=a*a/4.0;
	M[2][3][3][2][0][0]=a*a/4.0;
	M[2][3][3][2][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][3][2][1][2]=(1-a)*a/4.0;
	M[2][3][3][2][2][1]=a*(1-a)/4.0;
	M[2][3][3][2][2][2]=(1-a)*(1-a)/4.0;
	M[2][3][3][2][2][3]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][3][2][3][2]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][3][2][3][3]=(1-a)*(1-a)/4.0;
	M[2][3][3][2][3][4]=a*(1-a)/4.0;
	M[2][3][3][2][4][3]=(1-a)*a/4.0;
	M[2][3][3][2][4][4]=(1-a)*a/4.0+a*(1-a)/4.0+a*a/4.0;
	M[2][3][3][3][0][0]=a*a/4.0;
	M[2][3][3][3][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[2][3][3][3][1][2]=(1-a)*a/4.0;
	M[2][3][3][3][2][2]=(1-a)*(1-a)/4.0+a*(1-a)/4.0;
	M[2][3][3][3][2][3]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][3][3][3][3]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[2][3][3][3][3][4]=(1-a)*(1-a)/4.0+a*(1-a)/4.0;
	M[2][3][3][3][4][4]=(1-a)*a/4.0+(1-a)*a/4.0+a*a/4.0+a*(1-a)/4.0;
	M[2][3][3][4][1][1]=a*0.5/4.0;
	M[2][3][3][4][1][2]=a*0.5/4.0;
	M[2][3][3][4][2][2]=(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][3][3][4][2][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][3][3][4][2][4]=(1-a)*0.5/4.0;
	M[2][3][3][4][3][3]=(1-a)*0.5/4.0;
	M[2][3][3][4][3][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][3][3][4][4][4]=a*0.5/4.0+a*0.5/4.0;
	M[2][3][4][0][1][0]=a*0.5/4.0;
	M[2][3][4][0][2][0]=a*0.5/4.0+a*0.5/4.0;
	M[2][3][4][0][2][1]=(1-a)*0.5/4.0;
	M[2][3][4][0][2][2]=(1-a)*0.5/4.0;
	M[2][3][4][0][3][0]=a*0.5/4.0;
	M[2][3][4][0][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][3][4][0][3][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][3][4][0][3][3]=a*0.5/4.0;
	M[2][3][4][0][4][1]=(1-a)*0.5/4.0;
	M[2][3][4][0][4][2]=(1-a)*0.5/4.0;
	M[2][3][4][0][4][3]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][3][4][1][1][1]=a*0.5/4.0;
	M[2][3][4][1][2][0]=a*0.5/4.0;
	M[2][3][4][1][2][1]=a*0.5/4.0;
	M[2][3][4][1][2][2]=(1-a)*0.5/4.0;
	M[2][3][4][1][2][3]=(1-a)*0.5/4.0;
	M[2][3][4][1][3][0]=a*0.5/4.0;
	M[2][3][4][1][3][1]=(1-a)*0.5/4.0;
	M[2][3][4][1][3][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][3][4][1][3][3]=(1-a)*0.5/4.0;
	M[2][3][4][1][3][4]=a*0.5/4.0;
	M[2][3][4][1][4][1]=(1-a)*0.5/4.0;
	M[2][3][4][1][4][2]=(1-a)*0.5/4.0;
	M[2][3][4][1][4][3]=a*0.5/4.0+a*0.5/4.0;
	M[2][3][4][1][4][4]=a*0.5/4.0;
	M[2][3][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[2][3][4][2][2][0]=a*a/4.0;
	M[2][3][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][3][4][2][2][2]=(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][3][4][2][2][3]=(1-a)*(0.5-0.5*a)/4.0;
	M[2][3][4][2][3][1]=(1-a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[2][3][4][2][3][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[2][3][4][2][3][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[2][3][4][2][3][4]=(1-a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[2][3][4][2][4][2]=(1-a)*(0.5-0.5*a)/4.0;
	M[2][3][4][2][4][3]=(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][3][4][2][4][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][3][4][3][1][1]=a*0.5/4.0;
	M[2][3][4][3][2][1]=a*0.5/4.0;
	M[2][3][4][3][2][2]=(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][3][4][3][2][3]=(1-a)*0.5/4.0;
	M[2][3][4][3][3][2]=(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][3][4][3][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[2][3][4][3][3][4]=(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][3][4][3][4][3]=(1-a)*0.5/4.0;
	M[2][3][4][3][4][4]=(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][3][4][4][1][1]=a*0.5/4.0;
	M[2][3][4][4][2][2]=(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][3][4][4][2][3]=(1-a)*0.5/4.0;
	M[2][3][4][4][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][3][4][4][3][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[2][3][4][4][4][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][0][0][0][2]=a*1/4.0+a*1/4.0;
	M[2][4][0][0][1][1]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[2][4][0][0][1][2]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[2][4][0][0][1][3]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[2][4][0][0][2][2]=a*1/4.0+a*1/4.0;
	M[2][4][0][1][0][2]=a*1/4.0;
	M[2][4][0][1][0][3]=a*1/4.0;
	M[2][4][0][1][1][1]=(0.5-0.5*a)*1/4.0;
	M[2][4][0][1][1][2]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[2][4][0][1][1][3]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[2][4][0][1][1][4]=(0.5-0.5*a)*1/4.0;
	M[2][4][0][1][2][2]=a*1/4.0;
	M[2][4][0][1][2][3]=a*1/4.0;
	M[2][4][0][2][0][2]=a*a/4.0;
	M[2][4][0][2][0][3]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][0][2][0][4]=a*a/4.0;
	M[2][4][0][2][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][0][2][1][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][0][2][1][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][0][2][1][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][0][2][2][2]=a*a/4.0;
	M[2][4][0][2][2][3]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][0][2][2][4]=a*a/4.0;
	M[2][4][0][3][0][2]=a*a/4.0;
	M[2][4][0][3][0][3]=a*(1-a)/4.0;
	M[2][4][0][3][0][4]=a*a/4.0+a*(1-a)/4.0;
	M[2][4][0][3][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][0][3][1][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][0][3][1][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][0][3][1][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][0][3][2][2]=a*a/4.0;
	M[2][4][0][3][2][3]=a*(1-a)/4.0;
	M[2][4][0][3][2][4]=a*a/4.0+a*(1-a)/4.0;
	M[2][4][0][4][0][3]=a*0.5/4.0;
	M[2][4][0][4][0][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][0][4][1][2]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][0][4][1][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[2][4][0][4][1][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[2][4][0][4][2][3]=a*0.5/4.0;
	M[2][4][0][4][2][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][1][0][0][2]=a*1/4.0;
	M[2][4][1][0][1][1]=(0.5-0.5*a)*1/4.0;
	M[2][4][1][0][1][2]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+a*1/4.0;
	M[2][4][1][0][1][3]=(0.5-0.5*a)*1/4.0;
	M[2][4][1][0][2][1]=(0.5-0.5*a)*1/4.0;
	M[2][4][1][0][2][2]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+a*1/4.0;
	M[2][4][1][0][2][3]=(0.5-0.5*a)*1/4.0;
	M[2][4][1][0][3][2]=a*1/4.0;
	M[2][4][1][1][0][2]=a*1/4.0;
	M[2][4][1][1][1][1]=(0.5-0.5*a)*1/4.0;
	M[2][4][1][1][1][2]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[2][4][1][1][1][3]=(0.5-0.5*a)*1/4.0+a*1/4.0;
	M[2][4][1][1][2][2]=(0.5-0.5*a)*1/4.0+a*1/4.0;
	M[2][4][1][1][2][3]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[2][4][1][1][2][4]=(0.5-0.5*a)*1/4.0;
	M[2][4][1][1][3][3]=a*1/4.0;
	M[2][4][1][2][0][2]=a*a/4.0;
	M[2][4][1][2][0][3]=a*(1-a)/4.0;
	M[2][4][1][2][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][1][2][1][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][1][2][1][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][1][2][1][4]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][1][2][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][1][2][2][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][1][2][2][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][1][2][3][3]=a*(1-a)/4.0;
	M[2][4][1][2][3][4]=a*a/4.0;
	M[2][4][1][3][0][2]=a*a/4.0;
	M[2][4][1][3][0][4]=a*(1-a)/4.0;
	M[2][4][1][3][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][1][3][1][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][1][3][1][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][1][3][1][4]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][1][3][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][1][3][2][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][1][3][2][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][1][3][3][3]=a*(1-a)/4.0;
	M[2][4][1][3][3][4]=a*a/4.0;
	M[2][4][1][4][0][4]=a*0.5/4.0+a*0.5/4.0;
	M[2][4][1][4][1][3]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][1][4][1][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][1][4][2][2]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][1][4][2][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[2][4][1][4][2][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][1][4][3][3]=a*0.5/4.0;
	M[2][4][1][4][3][4]=a*0.5/4.0;
	M[2][4][2][0][0][2]=a*a/4.0;
	M[2][4][2][0][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][2][0][1][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][2][0][1][3]=(0.5-0.5*a)*a/4.0;
	M[2][4][2][0][2][1]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][2][0][2][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0+a*a/4.0;
	M[2][4][2][0][2][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][2][0][3][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][2][0][3][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][2][0][3][3]=(0.5-0.5*a)*a/4.0;
	M[2][4][2][0][4][2]=a*a/4.0;
	M[2][4][2][1][0][2]=a*a/4.0;
	M[2][4][2][1][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][2][1][1][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][2][1][1][3]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][2][1][2][1]=(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][2][1][2][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][2][1][2][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][2][1][2][4]=(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][2][1][3][2]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][2][1][3][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][2][1][3][4]=(0.5-0.5*a)*a/4.0;
	M[2][4][2][1][4][3]=a*a/4.0;
	M[2][4][2][2][0][2]=a*a/4.0;
	M[2][4][2][2][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][2][2][1][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][2][2][1][3]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][2][2][2][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][2][2][2][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][2][2][2][4]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][2][2][3][3]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][2][2][3][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][2][2][4][4]=a*a/4.0;
	M[2][4][2][3][0][2]=a*a/4.0;
	M[2][4][2][3][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][2][3][1][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][2][3][1][3]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][2][3][1][4]=a*(1-a)/4.0;
	M[2][4][2][3][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][2][3][2][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][2][3][2][4]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][2][3][3][3]=a*(1-a)/4.0;
	M[2][4][2][3][3][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][2][3][4][4]=a*a/4.0;
	M[2][4][2][4][0][4]=a*a/4.0;
	M[2][4][2][4][1][3]=(0.5-0.5*a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[2][4][2][4][1][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][4][2][4][2][2]=(0.5-0.5*a)*(0.5-0.5*a)/4.0;
	M[2][4][2][4][2][3]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0;
	M[2][4][2][4][2][4]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*a/4.0+a*a/4.0;
	M[2][4][2][4][3][3]=(0.5-0.5*a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[2][4][2][4][3][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][4][2][4][4][4]=a*a/4.0;
	M[2][4][3][0][0][2]=a*a/4.0;
	M[2][4][3][0][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][3][0][1][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][3][0][1][3]=(0.5-0.5*a)*a/4.0;
	M[2][4][3][0][2][1]=(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][0][2][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0+a*(1-a)/4.0;
	M[2][4][3][0][2][3]=(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][0][3][1]=(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][0][3][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0+a*(1-a)/4.0;
	M[2][4][3][0][3][3]=(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][0][4][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][3][0][4][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*a/4.0+a*(1-a)/4.0;
	M[2][4][3][0][4][3]=(0.5-0.5*a)*a/4.0;
	M[2][4][3][1][0][2]=a*a/4.0;
	M[2][4][3][1][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][3][1][1][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][3][1][1][3]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][3][1][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0+a*(1-a)/4.0;
	M[2][4][3][1][2][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][1][2][4]=(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][1][3][1]=(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][1][3][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][1][3][3]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0+a*(1-a)/4.0;
	M[2][4][3][1][4][2]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][3][1][4][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*a/4.0;
	M[2][4][3][1][4][4]=(0.5-0.5*a)*a/4.0;
	M[2][4][3][2][0][2]=a*a/4.0;
	M[2][4][3][2][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][3][2][1][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][3][2][1][3]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][3][2][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][3][2][2][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][3][2][2][4]=(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][2][3][2]=(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][2][3][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][3][2][3][4]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][3][2][4][3]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][3][2][4][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*a/4.0;
	M[2][4][3][3][0][2]=a*a/4.0;
	M[2][4][3][3][1][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][3][3][1][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][3][3][1][3]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[2][4][3][3][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][3][3][2][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[2][4][3][3][2][4]=(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][3][3][3][3]=(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[2][4][3][3][3][4]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[2][4][3][3][4][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*a/4.0+a*(1-a)/4.0;
	M[2][4][3][4][1][3]=a*0.5/4.0;
	M[2][4][3][4][1][4]=a*0.5/4.0;
	M[2][4][3][4][2][2]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][3][4][2][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[2][4][3][4][2][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][3][4][3][3]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][3][4][3][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][3][4][4][4]=a*0.5/4.0+a*0.5/4.0;
	M[2][4][4][0][1][2]=a*0.5/4.0;
	M[2][4][4][0][2][1]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][0][2][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][4][0][2][3]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][0][3][1]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][0][3][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][4][0][3][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][0][4][1]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][0][4][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][4][0][4][3]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][1][1][3]=a*0.5/4.0;
	M[2][4][4][1][2][2]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][1][2][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][1][2][4]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][1][3][1]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][1][3][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][1][3][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][1][3][4]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][1][4][1]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][1][4][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][4][1][4][3]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][2][1][3]=a*(0.5-0.5*a)/4.0;
	M[2][4][4][2][2][2]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][4][4][2][2][3]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][4][4][2][2][4]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][4][4][2][3][1]=(0.5-0.5*a)*a/4.0;
	M[2][4][4][2][3][2]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][4][2][3][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][4][4][2][3][4]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[2][4][4][2][4][2]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][4][4][2][4][3]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[2][4][4][2][4][4]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[2][4][4][3][1][3]=a*0.5/4.0;
	M[2][4][4][3][2][2]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][3][2][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][3][2][4]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][3][3][2]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][3][3][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][3][3][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][3][4][3]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][3][4][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][4][4][1][3]=a*0.5/4.0;
	M[2][4][4][4][2][2]=(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][4][2][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[2][4][4][4][2][4]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[2][4][4][4][3][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][4][3][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[2][4][4][4][4][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[3][0][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[3][0][0][0][1][0]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][0][0][0][2][0]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][0][0][0][3][0]=a*1/4.0+a*1/4.0;
	M[3][0][0][1][0][0]=a*1/4.0;
	M[3][0][0][1][0][1]=a*1/4.0;
	M[3][0][0][1][1][0]=(1-a)*1/4.0;
	M[3][0][0][1][1][1]=(1-a)*1/4.0;
	M[3][0][0][1][2][0]=(1-a)*1/4.0;
	M[3][0][0][1][2][1]=(1-a)*1/4.0;
	M[3][0][0][1][3][0]=a*1/4.0;
	M[3][0][0][1][3][1]=a*1/4.0;
	M[3][0][0][2][0][0]=a*a/4.0;
	M[3][0][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][0][0][2][0][2]=a*a/4.0;
	M[3][0][0][2][1][0]=(1-a)*a/4.0;
	M[3][0][0][2][1][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][0][2][1][2]=(1-a)*a/4.0;
	M[3][0][0][2][2][0]=(1-a)*a/4.0;
	M[3][0][0][2][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][0][2][2][2]=(1-a)*a/4.0;
	M[3][0][0][2][3][0]=a*a/4.0;
	M[3][0][0][2][3][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][0][0][2][3][2]=a*a/4.0;
	M[3][0][0][3][0][0]=a*a/4.0;
	M[3][0][0][3][0][1]=a*(1-a)/4.0;
	M[3][0][0][3][0][2]=a*(1-a)/4.0;
	M[3][0][0][3][0][3]=a*a/4.0;
	M[3][0][0][3][1][0]=(1-a)*a/4.0;
	M[3][0][0][3][1][1]=(1-a)*(1-a)/4.0;
	M[3][0][0][3][1][2]=(1-a)*(1-a)/4.0;
	M[3][0][0][3][1][3]=(1-a)*a/4.0;
	M[3][0][0][3][2][0]=(1-a)*a/4.0;
	M[3][0][0][3][2][1]=(1-a)*(1-a)/4.0;
	M[3][0][0][3][2][2]=(1-a)*(1-a)/4.0;
	M[3][0][0][3][2][3]=(1-a)*a/4.0;
	M[3][0][0][3][3][0]=a*a/4.0;
	M[3][0][0][3][3][1]=a*(1-a)/4.0;
	M[3][0][0][3][3][2]=a*(1-a)/4.0;
	M[3][0][0][3][3][3]=a*a/4.0;
	M[3][0][0][4][0][1]=a*0.5/4.0;
	M[3][0][0][4][0][2]=a*0.5/4.0+a*0.5/4.0;
	M[3][0][0][4][0][3]=a*0.5/4.0;
	M[3][0][0][4][1][1]=(1-a)*0.5/4.0;
	M[3][0][0][4][1][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][0][4][1][3]=(1-a)*0.5/4.0;
	M[3][0][0][4][2][1]=(1-a)*0.5/4.0;
	M[3][0][0][4][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][0][4][2][3]=(1-a)*0.5/4.0;
	M[3][0][0][4][3][1]=a*0.5/4.0;
	M[3][0][0][4][3][2]=a*0.5/4.0+a*0.5/4.0;
	M[3][0][0][4][3][3]=a*0.5/4.0;
	M[3][0][1][0][0][0]=a*1/4.0;
	M[3][0][1][0][1][0]=a*1/4.0+(1-a)*1/4.0;
	M[3][0][1][0][2][0]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][0][1][0][3][0]=a*1/4.0+(1-a)*1/4.0;
	M[3][0][1][0][4][0]=a*1/4.0;
	M[3][0][1][1][0][0]=a*1/4.0;
	M[3][0][1][1][1][0]=(1-a)*1/4.0;
	M[3][0][1][1][1][1]=a*1/4.0;
	M[3][0][1][1][2][0]=(1-a)*1/4.0;
	M[3][0][1][1][2][1]=(1-a)*1/4.0;
	M[3][0][1][1][3][0]=a*1/4.0;
	M[3][0][1][1][3][1]=(1-a)*1/4.0;
	M[3][0][1][1][4][1]=a*1/4.0;
	M[3][0][1][2][0][0]=a*a/4.0;
	M[3][0][1][2][0][1]=a*(1-a)/4.0;
	M[3][0][1][2][1][0]=(1-a)*a/4.0;
	M[3][0][1][2][1][1]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][1][2][1][2]=a*a/4.0;
	M[3][0][1][2][2][0]=(1-a)*a/4.0;
	M[3][0][1][2][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][1][2][2][2]=(1-a)*a/4.0;
	M[3][0][1][2][3][0]=a*a/4.0;
	M[3][0][1][2][3][1]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][1][2][3][2]=(1-a)*a/4.0;
	M[3][0][1][2][4][1]=a*(1-a)/4.0;
	M[3][0][1][2][4][2]=a*a/4.0;
	M[3][0][1][3][0][0]=a*a/4.0;
	M[3][0][1][3][0][2]=a*(1-a)/4.0;
	M[3][0][1][3][1][0]=(1-a)*a/4.0;
	M[3][0][1][3][1][1]=a*(1-a)/4.0;
	M[3][0][1][3][1][2]=(1-a)*(1-a)/4.0;
	M[3][0][1][3][1][3]=a*a/4.0;
	M[3][0][1][3][2][0]=(1-a)*a/4.0;
	M[3][0][1][3][2][1]=(1-a)*(1-a)/4.0;
	M[3][0][1][3][2][2]=(1-a)*(1-a)/4.0;
	M[3][0][1][3][2][3]=(1-a)*a/4.0;
	M[3][0][1][3][3][0]=a*a/4.0;
	M[3][0][1][3][3][1]=(1-a)*(1-a)/4.0;
	M[3][0][1][3][3][2]=a*(1-a)/4.0;
	M[3][0][1][3][3][3]=(1-a)*a/4.0;
	M[3][0][1][3][4][1]=a*(1-a)/4.0;
	M[3][0][1][3][4][3]=a*a/4.0;
	M[3][0][1][4][0][2]=a*0.5/4.0;
	M[3][0][1][4][0][3]=a*0.5/4.0;
	M[3][0][1][4][1][1]=a*0.5/4.0;
	M[3][0][1][4][1][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][1][4][1][3]=(1-a)*0.5/4.0;
	M[3][0][1][4][2][1]=(1-a)*0.5/4.0;
	M[3][0][1][4][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][1][4][2][3]=(1-a)*0.5/4.0;
	M[3][0][1][4][3][1]=(1-a)*0.5/4.0;
	M[3][0][1][4][3][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][1][4][3][3]=a*0.5/4.0;
	M[3][0][1][4][4][1]=a*0.5/4.0;
	M[3][0][1][4][4][2]=a*0.5/4.0;
	M[3][0][2][0][0][0]=a*a/4.0;
	M[3][0][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][0][2][0][2][0]=a*a/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][2][0][3][0]=a*a/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][2][0][4][0]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][0][2][1][0][0]=a*a/4.0;
	M[3][0][2][1][1][0]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][0][2][1][1][1]=a*(1-a)/4.0;
	M[3][0][2][1][2][0]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][0][2][1][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][0][2][1][3][0]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][0][2][1][3][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][0][2][1][4][0]=a*(1-a)/4.0;
	M[3][0][2][1][4][1]=a*a/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][0][2][2][0][0]=a*a/4.0;
	M[3][0][2][2][1][0]=(1-a)*a/4.0;
	M[3][0][2][2][1][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][0][2][2][2][0]=(1-a)*a/4.0;
	M[3][0][2][2][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][2][2][2][2]=a*a/4.0;
	M[3][0][2][2][3][0]=a*a/4.0;
	M[3][0][2][2][3][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][2][2][3][2]=(1-a)*a/4.0;
	M[3][0][2][2][4][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][0][2][2][4][2]=a*a/4.0+(1-a)*a/4.0;
	M[3][0][2][3][0][0]=a*a/4.0;
	M[3][0][2][3][1][0]=(1-a)*a/4.0;
	M[3][0][2][3][1][1]=a*(1-a)/4.0;
	M[3][0][2][3][1][2]=a*(1-a)/4.0;
	M[3][0][2][3][2][0]=(1-a)*a/4.0;
	M[3][0][2][3][2][1]=(1-a)*(1-a)/4.0;
	M[3][0][2][3][2][2]=(1-a)*(1-a)/4.0;
	M[3][0][2][3][2][3]=a*a/4.0;
	M[3][0][2][3][3][0]=a*a/4.0;
	M[3][0][2][3][3][1]=(1-a)*(1-a)/4.0;
	M[3][0][2][3][3][2]=(1-a)*(1-a)/4.0;
	M[3][0][2][3][3][3]=(1-a)*a/4.0;
	M[3][0][2][3][4][1]=a*(1-a)/4.0;
	M[3][0][2][3][4][2]=a*(1-a)/4.0;
	M[3][0][2][3][4][3]=a*a/4.0+(1-a)*a/4.0;
	M[3][0][2][4][0][2]=a*a/4.0;
	M[3][0][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[3][0][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][0][2][4][1][3]=a*(0.5-0.5*a)/4.0;
	M[3][0][2][4][2][1]=(1-a)*(0.5-0.5*a)/4.0;
	M[3][0][2][4][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][0][2][4][2][3]=(1-a)*(0.5-0.5*a)/4.0;
	M[3][0][2][4][3][1]=(1-a)*(0.5-0.5*a)/4.0;
	M[3][0][2][4][3][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][0][2][4][3][3]=(1-a)*(0.5-0.5*a)/4.0;
	M[3][0][2][4][4][1]=a*(0.5-0.5*a)/4.0;
	M[3][0][2][4][4][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*a/4.0+(1-a)*a/4.0;
	M[3][0][2][4][4][3]=a*(0.5-0.5*a)/4.0;
	M[3][0][3][0][0][0]=a*a/4.0;
	M[3][0][3][0][1][0]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][0][3][0][2][0]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][0][3][0][3][0]=a*a/4.0+a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][3][0][4][0]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][0][3][1][0][0]=a*a/4.0;
	M[3][0][3][1][1][0]=(1-a)*a/4.0;
	M[3][0][3][1][1][1]=a*(1-a)/4.0;
	M[3][0][3][1][2][0]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][0][3][1][2][1]=(1-a)*(1-a)/4.0;
	M[3][0][3][1][3][0]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][0][3][1][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][0][3][1][4][0]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][3][1][4][1]=a*a/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[3][0][3][2][0][0]=a*a/4.0;
	M[3][0][3][2][1][0]=(1-a)*a/4.0;
	M[3][0][3][2][1][1]=a*(1-a)/4.0;
	M[3][0][3][2][2][0]=(1-a)*a/4.0;
	M[3][0][3][2][2][1]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][3][2][3][0]=a*a/4.0;
	M[3][0][3][2][3][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][3][2][3][2]=a*a/4.0;
	M[3][0][3][2][4][1]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][3][2][4][2]=a*a/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[3][0][3][3][0][0]=a*a/4.0;
	M[3][0][3][3][1][0]=(1-a)*a/4.0;
	M[3][0][3][3][1][1]=a*(1-a)/4.0;
	M[3][0][3][3][2][0]=(1-a)*a/4.0;
	M[3][0][3][3][2][1]=(1-a)*(1-a)/4.0;
	M[3][0][3][3][2][2]=a*(1-a)/4.0;
	M[3][0][3][3][3][0]=a*a/4.0;
	M[3][0][3][3][3][1]=(1-a)*(1-a)/4.0;
	M[3][0][3][3][3][2]=(1-a)*(1-a)/4.0;
	M[3][0][3][3][3][3]=a*a/4.0;
	M[3][0][3][3][4][1]=a*(1-a)/4.0;
	M[3][0][3][3][4][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][0][3][3][4][3]=a*a/4.0+(1-a)*a/4.0+(1-a)*a/4.0;
	M[3][0][3][4][1][1]=a*0.5/4.0;
	M[3][0][3][4][1][2]=a*0.5/4.0;
	M[3][0][3][4][2][1]=(1-a)*0.5/4.0;
	M[3][0][3][4][2][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][3][4][2][3]=a*0.5/4.0;
	M[3][0][3][4][3][1]=(1-a)*0.5/4.0;
	M[3][0][3][4][3][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][3][4][3][3]=(1-a)*0.5/4.0;
	M[3][0][3][4][4][1]=a*0.5/4.0;
	M[3][0][3][4][4][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][3][4][4][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][0][1][0]=a*0.5/4.0;
	M[3][0][4][0][2][0]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][0][3][0]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][0][4][0]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][1][1][1]=a*0.5/4.0;
	M[3][0][4][1][2][0]=a*0.5/4.0;
	M[3][0][4][1][2][1]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][1][3][0]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][1][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][1][4][0]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][1][4][1]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[3][0][4][2][2][0]=a*a/4.0;
	M[3][0][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][0][4][2][2][2]=a*a/4.0;
	M[3][0][4][2][3][0]=(1-a)*a/4.0;
	M[3][0][4][2][3][1]=a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][0][4][2][3][2]=(1-a)*a/4.0;
	M[3][0][4][2][4][0]=a*a/4.0+(1-a)*a/4.0;
	M[3][0][4][2][4][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][0][4][2][4][2]=a*a/4.0+(1-a)*a/4.0;
	M[3][0][4][3][1][1]=a*0.5/4.0;
	M[3][0][4][3][2][1]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][3][2][2]=a*0.5/4.0;
	M[3][0][4][3][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][3][3][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][3][4][1]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][3][4][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][4][1][1]=a*0.5/4.0;
	M[3][0][4][4][2][1]=(1-a)*0.5/4.0;
	M[3][0][4][4][2][2]=a*0.5/4.0+a*0.5/4.0;
	M[3][0][4][4][3][1]=(1-a)*0.5/4.0;
	M[3][0][4][4][3][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][4][3][3]=a*0.5/4.0;
	M[3][0][4][4][4][1]=a*0.5/4.0;
	M[3][0][4][4][4][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][0][4][4][4][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[3][1][0][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][1][0][0][2][0]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][1][0][0][3][1]=a*1/4.0+a*1/4.0;
	M[3][1][0][1][0][0]=a*1/4.0;
	M[3][1][0][1][0][1]=a*1/4.0;
	M[3][1][0][1][1][1]=(1-a)*1/4.0;
	M[3][1][0][1][1][2]=(1-a)*1/4.0;
	M[3][1][0][1][2][0]=(1-a)*1/4.0;
	M[3][1][0][1][2][1]=(1-a)*1/4.0;
	M[3][1][0][1][3][1]=a*1/4.0;
	M[3][1][0][1][3][2]=a*1/4.0;
	M[3][1][0][2][0][0]=a*a/4.0;
	M[3][1][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][1][0][2][0][2]=a*a/4.0;
	M[3][1][0][2][1][1]=(1-a)*a/4.0;
	M[3][1][0][2][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][1][0][2][1][3]=(1-a)*a/4.0;
	M[3][1][0][2][2][0]=(1-a)*a/4.0;
	M[3][1][0][2][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][1][0][2][2][2]=(1-a)*a/4.0;
	M[3][1][0][2][3][1]=a*a/4.0;
	M[3][1][0][2][3][2]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][1][0][2][3][3]=a*a/4.0;
	M[3][1][0][3][0][0]=a*a/4.0;
	M[3][1][0][3][0][1]=a*(1-a)/4.0;
	M[3][1][0][3][0][2]=a*(1-a)/4.0;
	M[3][1][0][3][0][3]=a*a/4.0;
	M[3][1][0][3][1][1]=(1-a)*a/4.0;
	M[3][1][0][3][1][2]=(1-a)*(1-a)/4.0;
	M[3][1][0][3][1][3]=(1-a)*(1-a)/4.0;
	M[3][1][0][3][1][4]=(1-a)*a/4.0;
	M[3][1][0][3][2][0]=(1-a)*a/4.0;
	M[3][1][0][3][2][1]=(1-a)*(1-a)/4.0;
	M[3][1][0][3][2][2]=(1-a)*(1-a)/4.0;
	M[3][1][0][3][2][3]=(1-a)*a/4.0;
	M[3][1][0][3][3][1]=a*a/4.0;
	M[3][1][0][3][3][2]=a*(1-a)/4.0;
	M[3][1][0][3][3][3]=a*(1-a)/4.0;
	M[3][1][0][3][3][4]=a*a/4.0;
	M[3][1][0][4][0][1]=a*0.5/4.0;
	M[3][1][0][4][0][2]=a*0.5/4.0+a*0.5/4.0;
	M[3][1][0][4][0][3]=a*0.5/4.0;
	M[3][1][0][4][1][2]=(1-a)*0.5/4.0;
	M[3][1][0][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][0][4][1][4]=(1-a)*0.5/4.0;
	M[3][1][0][4][2][1]=(1-a)*0.5/4.0;
	M[3][1][0][4][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][0][4][2][3]=(1-a)*0.5/4.0;
	M[3][1][0][4][3][2]=a*0.5/4.0;
	M[3][1][0][4][3][3]=a*0.5/4.0+a*0.5/4.0;
	M[3][1][0][4][3][4]=a*0.5/4.0;
	M[3][1][1][0][0][0]=a*1/4.0;
	M[3][1][1][0][1][0]=a*1/4.0;
	M[3][1][1][0][1][1]=(1-a)*1/4.0;
	M[3][1][1][0][2][0]=(1-a)*1/4.0;
	M[3][1][1][0][2][1]=(1-a)*1/4.0;
	M[3][1][1][0][3][0]=(1-a)*1/4.0;
	M[3][1][1][0][3][1]=a*1/4.0;
	M[3][1][1][0][4][1]=a*1/4.0;
	M[3][1][1][1][0][0]=a*1/4.0;
	M[3][1][1][1][1][1]=a*1/4.0+(1-a)*1/4.0;
	M[3][1][1][1][2][0]=(1-a)*1/4.0;
	M[3][1][1][1][2][2]=(1-a)*1/4.0;
	M[3][1][1][1][3][1]=a*1/4.0+(1-a)*1/4.0;
	M[3][1][1][1][4][2]=a*1/4.0;
	M[3][1][1][2][0][0]=a*a/4.0;
	M[3][1][1][2][0][1]=a*(1-a)/4.0;
	M[3][1][1][2][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][1][2][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][1][2][2][0]=(1-a)*a/4.0;
	M[3][1][1][2][2][1]=(1-a)*(1-a)/4.0;
	M[3][1][1][2][2][2]=(1-a)*(1-a)/4.0;
	M[3][1][1][2][2][3]=(1-a)*a/4.0;
	M[3][1][1][2][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][1][2][3][2]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][1][2][4][2]=a*(1-a)/4.0;
	M[3][1][1][2][4][3]=a*a/4.0;
	M[3][1][1][3][0][0]=a*a/4.0;
	M[3][1][1][3][0][2]=a*(1-a)/4.0;
	M[3][1][1][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][1][3][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][1][3][2][0]=(1-a)*a/4.0;
	M[3][1][1][3][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][1][1][3][2][4]=(1-a)*a/4.0;
	M[3][1][1][3][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][1][3][3][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][1][3][4][2]=a*(1-a)/4.0;
	M[3][1][1][3][4][4]=a*a/4.0;
	M[3][1][1][4][0][2]=a*0.5/4.0;
	M[3][1][1][4][0][3]=a*0.5/4.0;
	M[3][1][1][4][1][1]=a*0.5/4.0;
	M[3][1][1][4][1][2]=a*0.5/4.0;
	M[3][1][1][4][1][3]=(1-a)*0.5/4.0;
	M[3][1][1][4][1][4]=(1-a)*0.5/4.0;
	M[3][1][1][4][2][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][1][4][2][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][1][4][3][1]=(1-a)*0.5/4.0;
	M[3][1][1][4][3][2]=(1-a)*0.5/4.0;
	M[3][1][1][4][3][3]=a*0.5/4.0;
	M[3][1][1][4][3][4]=a*0.5/4.0;
	M[3][1][1][4][4][2]=a*0.5/4.0;
	M[3][1][1][4][4][3]=a*0.5/4.0;
	M[3][1][2][0][0][0]=a*a/4.0;
	M[3][1][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][1][2][0][1][1]=(1-a)*a/4.0;
	M[3][1][2][0][2][0]=a*a/4.0+(1-a)*a/4.0;
	M[3][1][2][0][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][1][2][0][3][0]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][1][2][0][3][1]=a*a/4.0+(1-a)*a/4.0;
	M[3][1][2][0][4][0]=(1-a)*a/4.0;
	M[3][1][2][0][4][1]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][1][2][1][0][0]=a*a/4.0;
	M[3][1][2][1][1][0]=a*(1-a)/4.0;
	M[3][1][2][1][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][2][1][2][0]=(1-a)*a/4.0;
	M[3][1][2][1][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][2][1][2][2]=(1-a)*(1-a)/4.0;
	M[3][1][2][1][3][0]=(1-a)*(1-a)/4.0;
	M[3][1][2][1][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][2][1][3][2]=(1-a)*a/4.0;
	M[3][1][2][1][4][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][2][1][4][2]=a*a/4.0+a*(1-a)/4.0;
	M[3][1][2][2][0][0]=a*a/4.0;
	M[3][1][2][2][1][1]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][2][2][2][0]=(1-a)*a/4.0;
	M[3][1][2][2][2][2]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][1][2][2][3][1]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][1][2][2][3][3]=(1-a)*a/4.0;
	M[3][1][2][2][4][2]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][2][2][4][3]=a*a/4.0;
	M[3][1][2][3][0][0]=a*a/4.0;
	M[3][1][2][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][2][3][1][2]=a*(1-a)/4.0;
	M[3][1][2][3][2][0]=(1-a)*a/4.0;
	M[3][1][2][3][2][2]=(1-a)*(1-a)/4.0;
	M[3][1][2][3][2][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][2][3][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][2][3][3][2]=(1-a)*(1-a)/4.0;
	M[3][1][2][3][3][4]=(1-a)*a/4.0;
	M[3][1][2][3][4][2]=a*(1-a)/4.0;
	M[3][1][2][3][4][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][2][3][4][4]=a*a/4.0;
	M[3][1][2][4][0][2]=a*a/4.0;
	M[3][1][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[3][1][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[3][1][2][4][1][3]=a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][1][2][4][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][1][2][4][2][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][1][2][4][2][4]=(1-a)*(0.5-0.5*a)/4.0;
	M[3][1][2][4][3][1]=(1-a)*(0.5-0.5*a)/4.0;
	M[3][1][2][4][3][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][1][2][4][3][3]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][1][2][4][4][2]=a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][1][2][4][4][3]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*a/4.0;
	M[3][1][2][4][4][4]=a*(0.5-0.5*a)/4.0;
	M[3][1][3][0][0][0]=a*a/4.0;
	M[3][1][3][0][1][0]=a*(1-a)/4.0;
	M[3][1][3][0][1][1]=(1-a)*a/4.0;
	M[3][1][3][0][2][0]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][3][0][2][1]=(1-a)*(1-a)/4.0;
	M[3][1][3][0][3][0]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][3][0][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][3][0][4][0]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][3][0][4][1]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][3][1][0][0]=a*a/4.0;
	M[3][1][3][1][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][3][1][2][0]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][3][1][2][2]=(1-a)*(1-a)/4.0;
	M[3][1][3][1][3][1]=a*a/4.0+a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][1][3][1][4][0]=(1-a)*(1-a)/4.0;
	M[3][1][3][1][4][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][3][1][4][2]=a*a/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][3][2][0][0]=a*a/4.0;
	M[3][1][3][2][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][3][2][2][0]=(1-a)*a/4.0;
	M[3][1][3][2][2][1]=a*(1-a)/4.0;
	M[3][1][3][2][2][2]=(1-a)*(1-a)/4.0;
	M[3][1][3][2][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][3][2][3][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][3][2][4][1]=(1-a)*(1-a)/4.0;
	M[3][1][3][2][4][2]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][3][2][4][3]=a*a/4.0+(1-a)*a/4.0;
	M[3][1][3][3][0][0]=a*a/4.0;
	M[3][1][3][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][3][3][2][0]=(1-a)*a/4.0;
	M[3][1][3][3][2][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][1][3][3][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][3][3][3][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][1][3][3][4][2]=a*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][1][3][3][4][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][1][3][3][4][4]=a*a/4.0+(1-a)*a/4.0;
	M[3][1][3][4][1][1]=a*0.5/4.0;
	M[3][1][3][4][1][2]=a*0.5/4.0;
	M[3][1][3][4][2][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][3][4][2][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][3][4][3][1]=(1-a)*0.5/4.0;
	M[3][1][3][4][3][2]=(1-a)*0.5/4.0;
	M[3][1][3][4][3][3]=(1-a)*0.5/4.0;
	M[3][1][3][4][3][4]=(1-a)*0.5/4.0;
	M[3][1][3][4][4][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][3][4][4][3]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][3][4][4][4]=a*0.5/4.0;
	M[3][1][4][0][1][0]=a*0.5/4.0;
	M[3][1][4][0][2][0]=a*0.5/4.0+a*0.5/4.0;
	M[3][1][4][0][2][1]=(1-a)*0.5/4.0;
	M[3][1][4][0][3][0]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][0][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][0][4][0]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][0][4][1]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][1][1][1]=a*0.5/4.0;
	M[3][1][4][1][2][0]=a*0.5/4.0;
	M[3][1][4][1][2][1]=a*0.5/4.0;
	M[3][1][4][1][2][2]=(1-a)*0.5/4.0;
	M[3][1][4][1][3][0]=a*0.5/4.0;
	M[3][1][4][1][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][1][3][2]=(1-a)*0.5/4.0;
	M[3][1][4][1][4][0]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][1][4][1]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][1][4][2]=a*0.5/4.0+a*0.5/4.0;
	M[3][1][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[3][1][4][2][2][0]=a*a/4.0;
	M[3][1][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[3][1][4][2][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][1][4][2][3][1]=a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][1][4][2][3][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][1][4][2][3][3]=(1-a)*a/4.0;
	M[3][1][4][2][4][0]=(1-a)*a/4.0;
	M[3][1][4][2][4][1]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][1][4][2][4][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][1][4][2][4][3]=a*a/4.0;
	M[3][1][4][3][1][1]=a*0.5/4.0;
	M[3][1][4][3][2][1]=a*0.5/4.0;
	M[3][1][4][3][2][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][3][3][1]=(1-a)*0.5/4.0;
	M[3][1][4][3][3][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][3][3][3]=(1-a)*0.5/4.0;
	M[3][1][4][3][4][1]=(1-a)*0.5/4.0;
	M[3][1][4][3][4][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][3][4][3]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][4][1][1]=a*0.5/4.0;
	M[3][1][4][4][2][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][4][3][1]=(1-a)*0.5/4.0;
	M[3][1][4][4][3][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][4][4][2]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][4][4][3]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][1][4][4][4][4]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][2][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[3][2][0][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][2][0][0][2][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][2][0][0][3][2]=a*1/4.0+a*1/4.0;
	M[3][2][0][1][0][0]=a*1/4.0;
	M[3][2][0][1][0][1]=a*1/4.0;
	M[3][2][0][1][1][1]=(1-a)*1/4.0;
	M[3][2][0][1][1][2]=(1-a)*1/4.0;
	M[3][2][0][1][2][1]=(1-a)*1/4.0;
	M[3][2][0][1][2][2]=(1-a)*1/4.0;
	M[3][2][0][1][3][2]=a*1/4.0;
	M[3][2][0][1][3][3]=a*1/4.0;
	M[3][2][0][2][0][0]=a*a/4.0;
	M[3][2][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][2][0][2][0][2]=a*a/4.0;
	M[3][2][0][2][1][1]=(1-a)*a/4.0;
	M[3][2][0][2][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][2][0][2][1][3]=(1-a)*a/4.0;
	M[3][2][0][2][2][1]=(1-a)*a/4.0;
	M[3][2][0][2][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][2][0][2][2][3]=(1-a)*a/4.0;
	M[3][2][0][2][3][2]=a*a/4.0;
	M[3][2][0][2][3][3]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][2][0][2][3][4]=a*a/4.0;
	M[3][2][0][3][0][0]=a*a/4.0;
	M[3][2][0][3][0][1]=a*(1-a)/4.0;
	M[3][2][0][3][0][2]=a*(1-a)/4.0;
	M[3][2][0][3][0][3]=a*a/4.0;
	M[3][2][0][3][1][1]=(1-a)*a/4.0;
	M[3][2][0][3][1][2]=(1-a)*(1-a)/4.0;
	M[3][2][0][3][1][3]=(1-a)*(1-a)/4.0;
	M[3][2][0][3][1][4]=(1-a)*a/4.0;
	M[3][2][0][3][2][1]=(1-a)*a/4.0;
	M[3][2][0][3][2][2]=(1-a)*(1-a)/4.0;
	M[3][2][0][3][2][3]=(1-a)*(1-a)/4.0;
	M[3][2][0][3][2][4]=(1-a)*a/4.0;
	M[3][2][0][3][3][2]=a*a/4.0;
	M[3][2][0][3][3][3]=a*(1-a)/4.0;
	M[3][2][0][3][3][4]=a*a/4.0+a*(1-a)/4.0;
	M[3][2][0][4][0][1]=a*0.5/4.0;
	M[3][2][0][4][0][2]=a*0.5/4.0+a*0.5/4.0;
	M[3][2][0][4][0][3]=a*0.5/4.0;
	M[3][2][0][4][1][2]=(1-a)*0.5/4.0;
	M[3][2][0][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][2][0][4][1][4]=(1-a)*0.5/4.0;
	M[3][2][0][4][2][2]=(1-a)*0.5/4.0;
	M[3][2][0][4][2][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][2][0][4][2][4]=(1-a)*0.5/4.0;
	M[3][2][0][4][3][3]=a*0.5/4.0;
	M[3][2][0][4][3][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[3][2][1][0][0][0]=a*1/4.0;
	M[3][2][1][0][1][0]=a*1/4.0;
	M[3][2][1][0][1][1]=(1-a)*1/4.0;
	M[3][2][1][0][2][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][2][1][0][3][1]=(1-a)*1/4.0;
	M[3][2][1][0][3][2]=a*1/4.0;
	M[3][2][1][0][4][2]=a*1/4.0;
	M[3][2][1][1][0][0]=a*1/4.0;
	M[3][2][1][1][1][1]=(1-a)*1/4.0+a*1/4.0;
	M[3][2][1][1][2][1]=(1-a)*1/4.0;
	M[3][2][1][1][2][2]=(1-a)*1/4.0;
	M[3][2][1][1][3][2]=(1-a)*1/4.0+a*1/4.0;
	M[3][2][1][1][4][3]=a*1/4.0;
	M[3][2][1][2][0][0]=a*a/4.0;
	M[3][2][1][2][0][1]=a*(1-a)/4.0;
	M[3][2][1][2][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][1][2][1][2]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][1][2][2][1]=(1-a)*a/4.0;
	M[3][2][1][2][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][2][1][2][2][3]=(1-a)*a/4.0;
	M[3][2][1][2][3][2]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][1][2][3][3]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][1][2][4][3]=a*(1-a)/4.0;
	M[3][2][1][2][4][4]=a*a/4.0;
	M[3][2][1][3][0][0]=a*a/4.0;
	M[3][2][1][3][0][2]=a*(1-a)/4.0;
	M[3][2][1][3][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][1][3][1][3]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][1][3][2][1]=(1-a)*a/4.0;
	M[3][2][1][3][2][2]=(1-a)*(1-a)/4.0;
	M[3][2][1][3][2][3]=(1-a)*(1-a)/4.0;
	M[3][2][1][3][2][4]=(1-a)*a/4.0;
	M[3][2][1][3][3][2]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][1][3][3][4]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][1][3][4][3]=a*(1-a)/4.0;
	M[3][2][1][3][4][4]=a*a/4.0;
	M[3][2][1][4][0][2]=a*0.5/4.0;
	M[3][2][1][4][0][3]=a*0.5/4.0;
	M[3][2][1][4][1][1]=a*0.5/4.0;
	M[3][2][1][4][1][2]=a*0.5/4.0;
	M[3][2][1][4][1][3]=(1-a)*0.5/4.0;
	M[3][2][1][4][1][4]=(1-a)*0.5/4.0;
	M[3][2][1][4][2][2]=(1-a)*0.5/4.0;
	M[3][2][1][4][2][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][2][1][4][2][4]=(1-a)*0.5/4.0;
	M[3][2][1][4][3][2]=(1-a)*0.5/4.0;
	M[3][2][1][4][3][3]=(1-a)*0.5/4.0;
	M[3][2][1][4][3][4]=a*0.5/4.0+a*0.5/4.0;
	M[3][2][1][4][4][3]=a*0.5/4.0;
	M[3][2][1][4][4][4]=a*0.5/4.0;
	M[3][2][2][0][0][0]=a*a/4.0;
	M[3][2][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][2][2][0][1][1]=(1-a)*a/4.0;
	M[3][2][2][0][2][0]=a*a/4.0;
	M[3][2][2][0][2][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][2][2][0][3][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][2][2][0][3][2]=a*a/4.0;
	M[3][2][2][0][4][1]=(1-a)*a/4.0;
	M[3][2][2][0][4][2]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][2][2][1][0][0]=a*a/4.0;
	M[3][2][2][1][1][0]=a*(1-a)/4.0;
	M[3][2][2][1][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][2][1][2][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][2][1][2][2]=(1-a)*(1-a)/4.0;
	M[3][2][2][1][3][1]=(1-a)*(1-a)/4.0;
	M[3][2][2][1][3][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][2][1][4][2]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][2][1][4][3]=a*a/4.0+a*(1-a)/4.0;
	M[3][2][2][2][0][0]=a*a/4.0;
	M[3][2][2][2][1][1]=(1-a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][2][2][2][2][1]=(1-a)*a/4.0;
	M[3][2][2][2][2][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][2][2][3][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][2][2][3][3]=(1-a)*a/4.0;
	M[3][2][2][2][4][3]=(1-a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][2][2][2][4][4]=a*a/4.0;
	M[3][2][2][3][0][0]=a*a/4.0;
	M[3][2][2][3][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][2][3][1][2]=a*(1-a)/4.0;
	M[3][2][2][3][2][1]=(1-a)*a/4.0;
	M[3][2][2][3][2][2]=(1-a)*(1-a)/4.0;
	M[3][2][2][3][2][3]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][2][3][3][2]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][2][3][3][3]=(1-a)*(1-a)/4.0;
	M[3][2][2][3][3][4]=(1-a)*a/4.0;
	M[3][2][2][3][4][3]=a*(1-a)/4.0;
	M[3][2][2][3][4][4]=(1-a)*a/4.0+a*(1-a)/4.0+a*a/4.0;
	M[3][2][2][4][0][2]=a*a/4.0;
	M[3][2][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[3][2][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[3][2][2][4][1][3]=(1-a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[3][2][2][4][2][2]=(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[3][2][2][4][2][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][2][2][4][2][4]=(1-a)*(0.5-0.5*a)/4.0;
	M[3][2][2][4][3][2]=(1-a)*(0.5-0.5*a)/4.0;
	M[3][2][2][4][3][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][2][2][4][3][4]=(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[3][2][2][4][4][3]=(1-a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[3][2][2][4][4][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*a/4.0;
	M[3][2][3][0][0][0]=a*a/4.0;
	M[3][2][3][0][1][0]=a*(1-a)/4.0;
	M[3][2][3][0][1][1]=(1-a)*a/4.0;
	M[3][2][3][0][2][0]=a*(1-a)/4.0;
	M[3][2][3][0][2][1]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][2][3][0][3][0]=a*a/4.0;
	M[3][2][3][0][3][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][2][3][0][3][2]=a*a/4.0;
	M[3][2][3][0][4][1]=(1-a)*a/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][2][3][0][4][2]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][2][3][1][0][0]=a*a/4.0;
	M[3][2][3][1][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][3][1][2][0]=a*(1-a)/4.0;
	M[3][2][3][1][2][1]=(1-a)*a/4.0;
	M[3][2][3][1][2][2]=(1-a)*(1-a)/4.0;
	M[3][2][3][1][3][1]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][3][1][3][2]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][3][1][4][1]=(1-a)*(1-a)/4.0;
	M[3][2][3][1][4][2]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][3][1][4][3]=a*a/4.0+a*(1-a)/4.0;
	M[3][2][3][2][0][0]=a*a/4.0;
	M[3][2][3][2][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][3][2][2][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][3][2][2][2]=(1-a)*(1-a)/4.0;
	M[3][2][3][2][3][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+a*a/4.0+a*a/4.0;
	M[3][2][3][2][4][2]=(1-a)*(1-a)/4.0;
	M[3][2][3][2][4][3]=(1-a)*a/4.0+(1-a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][2][3][2][4][4]=a*a/4.0;
	M[3][2][3][3][0][0]=a*a/4.0;
	M[3][2][3][3][1][1]=(1-a)*a/4.0+a*(1-a)/4.0;
	M[3][2][3][3][2][1]=(1-a)*a/4.0;
	M[3][2][3][3][2][2]=(1-a)*(1-a)/4.0+a*(1-a)/4.0;
	M[3][2][3][3][3][2]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][3][3][3][3]=(1-a)*(1-a)/4.0+a*a/4.0;
	M[3][2][3][3][4][3]=(1-a)*(1-a)/4.0+a*(1-a)/4.0;
	M[3][2][3][3][4][4]=(1-a)*a/4.0+(1-a)*a/4.0+a*a/4.0+a*(1-a)/4.0;
	M[3][2][3][4][1][1]=a*0.5/4.0;
	M[3][2][3][4][1][2]=a*0.5/4.0;
	M[3][2][3][4][2][2]=(1-a)*0.5/4.0+a*0.5/4.0;
	M[3][2][3][4][2][3]=(1-a)*0.5/4.0+a*0.5/4.0;
	M[3][2][3][4][3][2]=(1-a)*0.5/4.0;
	M[3][2][3][4][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][2][3][4][3][4]=(1-a)*0.5/4.0;
	M[3][2][3][4][4][3]=(1-a)*0.5/4.0+a*0.5/4.0;
	M[3][2][3][4][4][4]=(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[3][2][4][0][1][0]=a*0.5/4.0;
	M[3][2][4][0][2][0]=a*0.5/4.0+a*0.5/4.0;
	M[3][2][4][0][2][1]=(1-a)*0.5/4.0;
	M[3][2][4][0][3][0]=a*0.5/4.0;
	M[3][2][4][0][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][2][4][0][4][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][2][4][0][4][2]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[3][2][4][1][1][1]=a*0.5/4.0;
	M[3][2][4][1][2][0]=a*0.5/4.0;
	M[3][2][4][1][2][1]=a*0.5/4.0;
	M[3][2][4][1][2][2]=(1-a)*0.5/4.0;
	M[3][2][4][1][3][0]=a*0.5/4.0;
	M[3][2][4][1][3][1]=(1-a)*0.5/4.0;
	M[3][2][4][1][3][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][2][4][1][4][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][2][4][1][4][2]=(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[3][2][4][1][4][3]=a*0.5/4.0+a*0.5/4.0;
	M[3][2][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[3][2][4][2][2][0]=a*a/4.0;
	M[3][2][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[3][2][4][2][2][2]=(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[3][2][4][2][3][1]=(1-a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[3][2][4][2][3][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][2][4][2][3][3]=(1-a)*a/4.0;
	M[3][2][4][2][4][1]=(1-a)*a/4.0;
	M[3][2][4][2][4][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[3][2][4][2][4][3]=(1-a)*a/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[3][2][4][2][4][4]=a*a/4.0;
	M[3][2][4][3][1][1]=a*0.5/4.0;
	M[3][2][4][3][2][1]=a*0.5/4.0;
	M[3][2][4][3][2][2]=(1-a)*0.5/4.0+a*0.5/4.0;
	M[3][2][4][3][3][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[3][2][4][3][3][3]=(1-a)*0.5/4.0;
	M[3][2][4][3][4][2]=(1-a)*0.5/4.0;
	M[3][2][4][3][4][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[3][2][4][3][4][4]=a*0.5/4.0+a*0.5/4.0;
	M[3][2][4][4][1][1]=a*0.5/4.0;
	M[3][2][4][4][2][2]=(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[3][2][4][4][3][2]=(1-a)*0.5/4.0;
	M[3][2][4][4][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[3][2][4][4][4][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0;
	M[3][2][4][4][4][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[3][3][0][0][0][0]=a*1/4.0+a*1/4.0;
	M[3][3][0][0][1][1]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][3][0][0][2][2]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][3][0][0][3][3]=a*1/4.0+a*1/4.0;
	M[3][3][0][1][0][0]=a*1/4.0;
	M[3][3][0][1][0][1]=a*1/4.0;
	M[3][3][0][1][1][1]=(1-a)*1/4.0;
	M[3][3][0][1][1][2]=(1-a)*1/4.0;
	M[3][3][0][1][2][2]=(1-a)*1/4.0;
	M[3][3][0][1][2][3]=(1-a)*1/4.0;
	M[3][3][0][1][3][3]=a*1/4.0;
	M[3][3][0][1][3][4]=a*1/4.0;
	M[3][3][0][2][0][0]=a*a/4.0;
	M[3][3][0][2][0][1]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][3][0][2][0][2]=a*a/4.0;
	M[3][3][0][2][1][1]=(1-a)*a/4.0;
	M[3][3][0][2][1][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][3][0][2][1][3]=(1-a)*a/4.0;
	M[3][3][0][2][2][2]=(1-a)*a/4.0;
	M[3][3][0][2][2][3]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][3][0][2][2][4]=(1-a)*a/4.0;
	M[3][3][0][2][3][3]=a*a/4.0;
	M[3][3][0][2][3][4]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][3][0][3][0][0]=a*a/4.0;
	M[3][3][0][3][0][1]=a*(1-a)/4.0;
	M[3][3][0][3][0][2]=a*(1-a)/4.0;
	M[3][3][0][3][0][3]=a*a/4.0;
	M[3][3][0][3][1][1]=(1-a)*a/4.0;
	M[3][3][0][3][1][2]=(1-a)*(1-a)/4.0;
	M[3][3][0][3][1][3]=(1-a)*(1-a)/4.0;
	M[3][3][0][3][1][4]=(1-a)*a/4.0;
	M[3][3][0][3][2][2]=(1-a)*a/4.0;
	M[3][3][0][3][2][3]=(1-a)*(1-a)/4.0;
	M[3][3][0][3][2][4]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][0][3][3][3]=a*a/4.0;
	M[3][3][0][3][3][4]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][3][0][4][0][1]=a*0.5/4.0;
	M[3][3][0][4][0][2]=a*0.5/4.0+a*0.5/4.0;
	M[3][3][0][4][0][3]=a*0.5/4.0;
	M[3][3][0][4][1][2]=(1-a)*0.5/4.0;
	M[3][3][0][4][1][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][0][4][1][4]=(1-a)*0.5/4.0;
	M[3][3][0][4][2][3]=(1-a)*0.5/4.0;
	M[3][3][0][4][2][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][0][4][3][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[3][3][1][0][0][0]=a*1/4.0;
	M[3][3][1][0][1][0]=a*1/4.0;
	M[3][3][1][0][1][1]=(1-a)*1/4.0;
	M[3][3][1][0][2][1]=(1-a)*1/4.0;
	M[3][3][1][0][2][2]=(1-a)*1/4.0;
	M[3][3][1][0][3][2]=(1-a)*1/4.0;
	M[3][3][1][0][3][3]=a*1/4.0;
	M[3][3][1][0][4][3]=a*1/4.0;
	M[3][3][1][1][0][0]=a*1/4.0;
	M[3][3][1][1][1][1]=a*1/4.0+(1-a)*1/4.0;
	M[3][3][1][1][2][2]=(1-a)*1/4.0+(1-a)*1/4.0;
	M[3][3][1][1][3][3]=a*1/4.0+(1-a)*1/4.0;
	M[3][3][1][1][4][4]=a*1/4.0;
	M[3][3][1][2][0][0]=a*a/4.0;
	M[3][3][1][2][0][1]=a*(1-a)/4.0;
	M[3][3][1][2][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][1][2][1][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][1][2][2][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][1][2][2][3]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][1][2][3][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][1][2][3][4]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][1][2][4][4]=a*a/4.0+a*(1-a)/4.0;
	M[3][3][1][3][0][0]=a*a/4.0;
	M[3][3][1][3][0][2]=a*(1-a)/4.0;
	M[3][3][1][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][1][3][1][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][1][3][2][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][1][3][2][4]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][1][3][3][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][1][3][3][4]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][1][3][4][4]=a*a/4.0+a*(1-a)/4.0;
	M[3][3][1][4][0][2]=a*0.5/4.0;
	M[3][3][1][4][0][3]=a*0.5/4.0;
	M[3][3][1][4][1][1]=a*0.5/4.0;
	M[3][3][1][4][1][2]=a*0.5/4.0;
	M[3][3][1][4][1][3]=(1-a)*0.5/4.0;
	M[3][3][1][4][1][4]=(1-a)*0.5/4.0;
	M[3][3][1][4][2][2]=(1-a)*0.5/4.0;
	M[3][3][1][4][2][3]=(1-a)*0.5/4.0;
	M[3][3][1][4][2][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][1][4][3][3]=(1-a)*0.5/4.0;
	M[3][3][1][4][3][4]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][1][4][4][4]=a*0.5/4.0+a*0.5/4.0;
	M[3][3][2][0][0][0]=a*a/4.0;
	M[3][3][2][0][1][0]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][3][2][0][1][1]=(1-a)*a/4.0;
	M[3][3][2][0][2][0]=a*a/4.0;
	M[3][3][2][0][2][1]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][3][2][0][2][2]=(1-a)*a/4.0;
	M[3][3][2][0][3][1]=(1-a)*a/4.0;
	M[3][3][2][0][3][2]=(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][3][2][0][3][3]=a*a/4.0;
	M[3][3][2][0][4][2]=(1-a)*a/4.0;
	M[3][3][2][0][4][3]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][3][2][1][0][0]=a*a/4.0;
	M[3][3][2][1][1][0]=a*(1-a)/4.0;
	M[3][3][2][1][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][2][1][2][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][2][1][2][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][2][1][3][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][2][1][3][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][2][1][4][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][2][1][4][4]=a*a/4.0+a*(1-a)/4.0;
	M[3][3][2][2][0][0]=a*a/4.0;
	M[3][3][2][2][1][1]=a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][2][2][2][2]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][2][2][3][3]=a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][2][2][4][4]=a*(1-a)/4.0+a*(1-a)/4.0+a*a/4.0+(1-a)*a/4.0;
	M[3][3][2][3][0][0]=a*a/4.0;
	M[3][3][2][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][2][3][1][2]=a*(1-a)/4.0;
	M[3][3][2][3][2][2]=(1-a)*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][2][3][2][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][2][3][3][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][2][3][3][4]=(1-a)*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][2][3][4][4]=a*(1-a)/4.0+a*(1-a)/4.0+a*a/4.0+(1-a)*a/4.0;
	M[3][3][2][4][0][2]=a*a/4.0;
	M[3][3][2][4][1][1]=a*(0.5-0.5*a)/4.0;
	M[3][3][2][4][1][2]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[3][3][2][4][1][3]=a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][3][2][4][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][3][2][4][2][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][3][2][4][2][4]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][3][2][4][3][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][3][2][4][3][4]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][3][2][4][4][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*a/4.0+(1-a)*a/4.0;
	M[3][3][3][0][0][0]=a*a/4.0;
	M[3][3][3][0][1][0]=a*(1-a)/4.0;
	M[3][3][3][0][1][1]=(1-a)*a/4.0;
	M[3][3][3][0][2][0]=a*(1-a)/4.0;
	M[3][3][3][0][2][1]=(1-a)*(1-a)/4.0;
	M[3][3][3][0][2][2]=(1-a)*a/4.0;
	M[3][3][3][0][3][0]=a*a/4.0;
	M[3][3][3][0][3][1]=(1-a)*(1-a)/4.0;
	M[3][3][3][0][3][2]=(1-a)*(1-a)/4.0;
	M[3][3][3][0][3][3]=a*a/4.0;
	M[3][3][3][0][4][1]=(1-a)*a/4.0;
	M[3][3][3][0][4][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][3][0][4][3]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[3][3][3][1][0][0]=a*a/4.0;
	M[3][3][3][1][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][3][1][2][0]=a*(1-a)/4.0;
	M[3][3][3][1][2][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][3][1][3][1]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][3][1][3][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][3][1][4][2]=(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][3][1][4][3]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][3][1][4][4]=a*a/4.0+a*(1-a)/4.0;
	M[3][3][3][2][0][0]=a*a/4.0;
	M[3][3][3][2][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][3][2][2][1]=a*(1-a)/4.0;
	M[3][3][3][2][2][2]=(1-a)*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][3][2][3][2]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][3][2][3][3]=a*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][3][2][4][3]=(1-a)*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][3][2][4][4]=a*(1-a)/4.0+a*(1-a)/4.0+a*a/4.0+(1-a)*a/4.0;
	M[3][3][3][3][0][0]=a*a/4.0;
	M[3][3][3][3][1][1]=a*(1-a)/4.0+(1-a)*a/4.0;
	M[3][3][3][3][2][2]=a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][3][3][3][3]=a*a/4.0+a*a/4.0+(1-a)*(1-a)/4.0+(1-a)*(1-a)/4.0;
	M[3][3][3][3][4][4]=a*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0+(1-a)*a/4.0+(1-a)*a/4.0+(1-a)*(1-a)/4.0;
	M[3][3][3][4][1][1]=a*0.5/4.0;
	M[3][3][3][4][1][2]=a*0.5/4.0;
	M[3][3][3][4][2][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][3][4][2][3]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][3][4][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][3][4][3][4]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][3][4][4][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][0][1][0]=a*0.5/4.0;
	M[3][3][4][0][2][0]=a*0.5/4.0+a*0.5/4.0;
	M[3][3][4][0][2][1]=(1-a)*0.5/4.0;
	M[3][3][4][0][3][0]=a*0.5/4.0;
	M[3][3][4][0][3][1]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][0][3][2]=(1-a)*0.5/4.0;
	M[3][3][4][0][4][1]=(1-a)*0.5/4.0;
	M[3][3][4][0][4][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][0][4][3]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[3][3][4][1][1][1]=a*0.5/4.0;
	M[3][3][4][1][2][0]=a*0.5/4.0;
	M[3][3][4][1][2][1]=a*0.5/4.0;
	M[3][3][4][1][2][2]=(1-a)*0.5/4.0;
	M[3][3][4][1][3][0]=a*0.5/4.0;
	M[3][3][4][1][3][1]=(1-a)*0.5/4.0;
	M[3][3][4][1][3][2]=(1-a)*0.5/4.0;
	M[3][3][4][1][3][3]=(1-a)*0.5/4.0;
	M[3][3][4][1][4][1]=(1-a)*0.5/4.0;
	M[3][3][4][1][4][2]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][1][4][3]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][1][4][4]=a*0.5/4.0+a*0.5/4.0;
	M[3][3][4][2][1][1]=a*(0.5-0.5*a)/4.0;
	M[3][3][4][2][2][0]=a*a/4.0;
	M[3][3][4][2][2][1]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[3][3][4][2][2][2]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][3][4][2][3][1]=a*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][3][4][2][3][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][3][4][2][3][3]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][3][4][2][4][2]=(1-a)*(0.5-0.5*a)/4.0+(1-a)*a/4.0;
	M[3][3][4][2][4][3]=a*a/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0+(1-a)*(0.5-0.5*a)/4.0;
	M[3][3][4][2][4][4]=a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*a/4.0+(1-a)*a/4.0;
	M[3][3][4][3][1][1]=a*0.5/4.0;
	M[3][3][4][3][2][1]=a*0.5/4.0;
	M[3][3][4][3][2][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][3][3][2]=a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][3][3][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][3][4][3]=(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][3][4][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][4][1][1]=a*0.5/4.0;
	M[3][3][4][4][2][2]=a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][4][3][3]=a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][3][4][4][4][4]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0+(1-a)*0.5/4.0;
	M[3][4][0][0][1][1]=0.5*1/4.0+0.5*1/4.0;
	M[3][4][0][0][1][2]=0.5*1/4.0+0.5*1/4.0;
	M[3][4][0][0][2][2]=0.5*1/4.0+0.5*1/4.0;
	M[3][4][0][0][2][3]=0.5*1/4.0+0.5*1/4.0;
	M[3][4][0][1][1][1]=0.5*1/4.0;
	M[3][4][0][1][1][2]=0.5*1/4.0+0.5*1/4.0;
	M[3][4][0][1][1][3]=0.5*1/4.0;
	M[3][4][0][1][2][2]=0.5*1/4.0;
	M[3][4][0][1][2][3]=0.5*1/4.0+0.5*1/4.0;
	M[3][4][0][1][2][4]=0.5*1/4.0;
	M[3][4][0][2][1][1]=0.5*a/4.0;
	M[3][4][0][2][1][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][0][2][1][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][0][2][1][4]=0.5*a/4.0;
	M[3][4][0][2][2][2]=0.5*a/4.0;
	M[3][4][0][2][2][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][0][2][2][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][0][3][1][1]=0.5*a/4.0;
	M[3][4][0][3][1][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][0][3][1][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][0][3][1][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][0][3][2][2]=0.5*a/4.0;
	M[3][4][0][3][2][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][0][3][2][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][0][4][1][2]=0.5*0.5/4.0;
	M[3][4][0][4][1][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][0][4][1][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][0][4][2][3]=0.5*0.5/4.0;
	M[3][4][0][4][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][1][0][1][1]=0.5*1/4.0;
	M[3][4][1][0][1][2]=0.5*1/4.0;
	M[3][4][1][0][2][1]=0.5*1/4.0;
	M[3][4][1][0][2][2]=0.5*1/4.0+0.5*1/4.0;
	M[3][4][1][0][2][3]=0.5*1/4.0;
	M[3][4][1][0][3][2]=0.5*1/4.0;
	M[3][4][1][0][3][3]=0.5*1/4.0;
	M[3][4][1][1][1][1]=0.5*1/4.0;
	M[3][4][1][1][1][2]=0.5*1/4.0;
	M[3][4][1][1][2][2]=0.5*1/4.0+0.5*1/4.0;
	M[3][4][1][1][2][3]=0.5*1/4.0+0.5*1/4.0;
	M[3][4][1][1][3][3]=0.5*1/4.0;
	M[3][4][1][1][3][4]=0.5*1/4.0;
	M[3][4][1][2][1][1]=0.5*a/4.0;
	M[3][4][1][2][1][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][1][2][1][3]=0.5*(1-a)/4.0;
	M[3][4][1][2][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][1][2][2][3]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][1][2][2][4]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][1][2][3][3]=0.5*(1-a)/4.0;
	M[3][4][1][2][3][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][1][3][1][1]=0.5*a/4.0;
	M[3][4][1][3][1][2]=0.5*a/4.0;
	M[3][4][1][3][1][3]=0.5*(1-a)/4.0;
	M[3][4][1][3][1][4]=0.5*(1-a)/4.0;
	M[3][4][1][3][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][1][3][2][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][1][3][2][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][1][3][3][3]=0.5*(1-a)/4.0;
	M[3][4][1][3][3][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][1][4][1][3]=0.5*0.5/4.0;
	M[3][4][1][4][1][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][1][4][2][2]=0.5*0.5/4.0;
	M[3][4][1][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][1][4][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][1][4][3][3]=0.5*0.5/4.0;
	M[3][4][1][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][2][0][1][1]=0.5*a/4.0;
	M[3][4][2][0][1][2]=0.5*a/4.0;
	M[3][4][2][0][2][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][2][0][2][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][2][0][2][3]=0.5*a/4.0;
	M[3][4][2][0][3][1]=0.5*a/4.0;
	M[3][4][2][0][3][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][2][0][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][2][0][4][2]=0.5*a/4.0;
	M[3][4][2][0][4][3]=0.5*a/4.0;
	M[3][4][2][1][1][1]=0.5*a/4.0;
	M[3][4][2][1][1][2]=0.5*a/4.0;
	M[3][4][2][1][2][1]=0.5*(1-a)/4.0;
	M[3][4][2][1][2][2]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][2][1][2][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][2][1][3][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][2][1][3][3]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][2][1][3][4]=0.5*(1-a)/4.0;
	M[3][4][2][1][4][3]=0.5*a/4.0;
	M[3][4][2][1][4][4]=0.5*a/4.0;
	M[3][4][2][2][1][1]=0.5*a/4.0;
	M[3][4][2][2][1][2]=0.5*a/4.0;
	M[3][4][2][2][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][2][2][2][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][2][2][3][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][2][2][3][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][2][2][4][4]=0.5*a/4.0+0.5*a/4.0;
	M[3][4][2][3][1][1]=0.5*a/4.0;
	M[3][4][2][3][1][2]=0.5*a/4.0;
	M[3][4][2][3][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][2][3][2][3]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][2][3][2][4]=0.5*(1-a)/4.0;
	M[3][4][2][3][3][3]=0.5*(1-a)/4.0;
	M[3][4][2][3][3][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][2][3][4][4]=0.5*a/4.0+0.5*a/4.0;
	M[3][4][2][4][1][3]=0.5*a/4.0;
	M[3][4][2][4][1][4]=0.5*a/4.0;
	M[3][4][2][4][2][2]=0.5*(0.5-0.5*a)/4.0;
	M[3][4][2][4][2][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[3][4][2][4][2][4]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[3][4][2][4][3][3]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0;
	M[3][4][2][4][3][4]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[3][4][2][4][4][4]=0.5*a/4.0+0.5*a/4.0;
	M[3][4][3][0][1][1]=0.5*a/4.0;
	M[3][4][3][0][1][2]=0.5*a/4.0;
	M[3][4][3][0][2][1]=0.5*(1-a)/4.0;
	M[3][4][3][0][2][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][3][0][2][3]=0.5*a/4.0;
	M[3][4][3][0][3][1]=0.5*(1-a)/4.0;
	M[3][4][3][0][3][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][3][0][3][3]=0.5*(1-a)/4.0;
	M[3][4][3][0][4][1]=0.5*a/4.0;
	M[3][4][3][0][4][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][3][0][4][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][3][1][1][1]=0.5*a/4.0;
	M[3][4][3][1][1][2]=0.5*a/4.0;
	M[3][4][3][1][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][3][1][2][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][3][1][3][1]=0.5*(1-a)/4.0;
	M[3][4][3][1][3][2]=0.5*(1-a)/4.0;
	M[3][4][3][1][3][3]=0.5*(1-a)/4.0;
	M[3][4][3][1][3][4]=0.5*(1-a)/4.0;
	M[3][4][3][1][4][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][3][1][4][3]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[3][4][3][1][4][4]=0.5*a/4.0;
	M[3][4][3][2][1][1]=0.5*a/4.0;
	M[3][4][3][2][1][2]=0.5*a/4.0;
	M[3][4][3][2][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][3][2][2][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][3][2][3][2]=0.5*(1-a)/4.0;
	M[3][4][3][2][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][3][2][3][4]=0.5*(1-a)/4.0;
	M[3][4][3][2][4][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][3][2][4][4]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[3][4][3][3][1][1]=0.5*a/4.0;
	M[3][4][3][3][1][2]=0.5*a/4.0;
	M[3][4][3][3][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[3][4][3][3][2][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][3][3][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][3][3][3][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[3][4][3][3][4][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[3][4][3][4][2][2]=0.5*0.5/4.0;
	M[3][4][3][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][3][4][2][4]=0.5*0.5/4.0;
	M[3][4][3][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][3][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][3][4][4][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][0][2][1]=0.5*0.5/4.0;
	M[3][4][4][0][2][2]=0.5*0.5/4.0;
	M[3][4][4][0][3][1]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][0][3][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][0][3][3]=0.5*0.5/4.0;
	M[3][4][4][0][4][1]=0.5*0.5/4.0;
	M[3][4][4][0][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][0][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][1][2][2]=0.5*0.5/4.0;
	M[3][4][4][1][2][3]=0.5*0.5/4.0;
	M[3][4][4][1][3][1]=0.5*0.5/4.0;
	M[3][4][4][1][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][1][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][1][3][4]=0.5*0.5/4.0;
	M[3][4][4][1][4][1]=0.5*0.5/4.0;
	M[3][4][4][1][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][1][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][1][4][4]=0.5*0.5/4.0;
	M[3][4][4][2][2][2]=0.5*(0.5-0.5*a)/4.0;
	M[3][4][4][2][2][3]=0.5*(0.5-0.5*a)/4.0;
	M[3][4][4][2][3][1]=0.5*a/4.0;
	M[3][4][4][2][3][2]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[3][4][4][2][3][3]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[3][4][4][2][3][4]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[3][4][4][2][4][2]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[3][4][4][2][4][3]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[3][4][4][2][4][4]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[3][4][4][3][2][2]=0.5*0.5/4.0;
	M[3][4][4][3][2][3]=0.5*0.5/4.0;
	M[3][4][4][3][3][2]=0.5*0.5/4.0;
	M[3][4][4][3][3][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][3][3][4]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][3][4][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][3][4][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][4][2][2]=0.5*0.5/4.0;
	M[3][4][4][4][2][3]=0.5*0.5/4.0;
	M[3][4][4][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[3][4][4][4][4][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][0][0][1][0]=0.5*1/4.0+0.5*1/4.0;
	M[4][0][0][0][2][0]=0.5*1/4.0+0.5*1/4.0+0.5*1/4.0+0.5*1/4.0;
	M[4][0][0][0][3][0]=0.5*1/4.0+0.5*1/4.0;
	M[4][0][0][1][1][0]=0.5*1/4.0;
	M[4][0][0][1][1][1]=0.5*1/4.0;
	M[4][0][0][1][2][0]=0.5*1/4.0+0.5*1/4.0;
	M[4][0][0][1][2][1]=0.5*1/4.0+0.5*1/4.0;
	M[4][0][0][1][3][0]=0.5*1/4.0;
	M[4][0][0][1][3][1]=0.5*1/4.0;
	M[4][0][0][2][1][0]=0.5*a/4.0;
	M[4][0][0][2][1][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][0][2][1][2]=0.5*a/4.0;
	M[4][0][0][2][2][0]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][0][2][2][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][0][2][2][2]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][0][2][3][0]=0.5*a/4.0;
	M[4][0][0][2][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][0][2][3][2]=0.5*a/4.0;
	M[4][0][0][3][1][0]=0.5*a/4.0;
	M[4][0][0][3][1][1]=0.5*(1-a)/4.0;
	M[4][0][0][3][1][2]=0.5*(1-a)/4.0;
	M[4][0][0][3][1][3]=0.5*a/4.0;
	M[4][0][0][3][2][0]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][0][3][2][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][0][3][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][0][3][2][3]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][0][3][3][0]=0.5*a/4.0;
	M[4][0][0][3][3][1]=0.5*(1-a)/4.0;
	M[4][0][0][3][3][2]=0.5*(1-a)/4.0;
	M[4][0][0][3][3][3]=0.5*a/4.0;
	M[4][0][0][4][1][1]=0.5*0.5/4.0;
	M[4][0][0][4][1][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][0][4][1][3]=0.5*0.5/4.0;
	M[4][0][0][4][2][1]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][0][4][2][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][0][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][0][4][3][1]=0.5*0.5/4.0;
	M[4][0][0][4][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][0][4][3][3]=0.5*0.5/4.0;
	M[4][0][1][0][1][0]=0.5*1/4.0;
	M[4][0][1][0][2][0]=0.5*1/4.0+0.5*1/4.0+0.5*1/4.0;
	M[4][0][1][0][3][0]=0.5*1/4.0+0.5*1/4.0+0.5*1/4.0;
	M[4][0][1][0][4][0]=0.5*1/4.0;
	M[4][0][1][1][1][0]=0.5*1/4.0;
	M[4][0][1][1][2][0]=0.5*1/4.0+0.5*1/4.0;
	M[4][0][1][1][2][1]=0.5*1/4.0;
	M[4][0][1][1][3][0]=0.5*1/4.0;
	M[4][0][1][1][3][1]=0.5*1/4.0+0.5*1/4.0;
	M[4][0][1][1][4][1]=0.5*1/4.0;
	M[4][0][1][2][1][0]=0.5*a/4.0;
	M[4][0][1][2][1][1]=0.5*(1-a)/4.0;
	M[4][0][1][2][2][0]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][1][2][2][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][1][2][2][2]=0.5*a/4.0;
	M[4][0][1][2][3][0]=0.5*a/4.0;
	M[4][0][1][2][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][1][2][3][2]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][1][2][4][1]=0.5*(1-a)/4.0;
	M[4][0][1][2][4][2]=0.5*a/4.0;
	M[4][0][1][3][1][0]=0.5*a/4.0;
	M[4][0][1][3][1][2]=0.5*(1-a)/4.0;
	M[4][0][1][3][2][0]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][1][3][2][1]=0.5*(1-a)/4.0;
	M[4][0][1][3][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][1][3][2][3]=0.5*a/4.0;
	M[4][0][1][3][3][0]=0.5*a/4.0;
	M[4][0][1][3][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][1][3][3][2]=0.5*(1-a)/4.0;
	M[4][0][1][3][3][3]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][1][3][4][1]=0.5*(1-a)/4.0;
	M[4][0][1][3][4][3]=0.5*a/4.0;
	M[4][0][1][4][1][2]=0.5*0.5/4.0;
	M[4][0][1][4][1][3]=0.5*0.5/4.0;
	M[4][0][1][4][2][1]=0.5*0.5/4.0;
	M[4][0][1][4][2][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][1][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][1][4][3][1]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][1][4][3][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][1][4][3][3]=0.5*0.5/4.0;
	M[4][0][1][4][4][1]=0.5*0.5/4.0;
	M[4][0][1][4][4][2]=0.5*0.5/4.0;
	M[4][0][2][0][1][0]=0.5*a/4.0;
	M[4][0][2][0][2][0]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][2][0][3][0]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][2][0][4][0]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][2][1][1][0]=0.5*a/4.0;
	M[4][0][2][1][2][0]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][2][1][2][1]=0.5*(1-a)/4.0;
	M[4][0][2][1][3][0]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][2][1][3][1]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][2][1][4][0]=0.5*(1-a)/4.0;
	M[4][0][2][1][4][1]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][2][2][1][0]=0.5*a/4.0;
	M[4][0][2][2][2][0]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][2][2][2][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][2][2][3][0]=0.5*a/4.0;
	M[4][0][2][2][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][2][2][3][2]=0.5*a/4.0;
	M[4][0][2][2][4][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][2][2][4][2]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][2][3][1][0]=0.5*a/4.0;
	M[4][0][2][3][2][0]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][2][3][2][1]=0.5*(1-a)/4.0;
	M[4][0][2][3][2][2]=0.5*(1-a)/4.0;
	M[4][0][2][3][3][0]=0.5*a/4.0;
	M[4][0][2][3][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][2][3][3][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][2][3][3][3]=0.5*a/4.0;
	M[4][0][2][3][4][1]=0.5*(1-a)/4.0;
	M[4][0][2][3][4][2]=0.5*(1-a)/4.0;
	M[4][0][2][3][4][3]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][2][4][1][2]=0.5*a/4.0;
	M[4][0][2][4][2][1]=0.5*(0.5-0.5*a)/4.0;
	M[4][0][2][4][2][2]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][2][4][2][3]=0.5*(0.5-0.5*a)/4.0;
	M[4][0][2][4][3][1]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][0][2][4][3][2]=0.5*a/4.0+0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][0][2][4][3][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][0][2][4][4][1]=0.5*(0.5-0.5*a)/4.0;
	M[4][0][2][4][4][2]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][2][4][4][3]=0.5*(0.5-0.5*a)/4.0;
	M[4][0][3][0][1][0]=0.5*a/4.0;
	M[4][0][3][0][2][0]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][3][0][3][0]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][3][0][4][0]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][3][1][1][0]=0.5*a/4.0;
	M[4][0][3][1][2][0]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][3][1][2][1]=0.5*(1-a)/4.0;
	M[4][0][3][1][3][0]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][0][3][1][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][3][1][4][0]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][3][1][4][1]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][3][2][1][0]=0.5*a/4.0;
	M[4][0][3][2][2][0]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][3][2][2][1]=0.5*(1-a)/4.0;
	M[4][0][3][2][3][0]=0.5*a/4.0;
	M[4][0][3][2][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][3][2][4][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][3][2][4][2]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][3][3][1][0]=0.5*a/4.0;
	M[4][0][3][3][2][0]=0.5*a/4.0+0.5*a/4.0;
	M[4][0][3][3][2][1]=0.5*(1-a)/4.0;
	M[4][0][3][3][3][0]=0.5*a/4.0;
	M[4][0][3][3][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][3][3][3][2]=0.5*(1-a)/4.0;
	M[4][0][3][3][4][1]=0.5*(1-a)/4.0;
	M[4][0][3][3][4][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][0][3][3][4][3]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][3][4][2][1]=0.5*0.5/4.0;
	M[4][0][3][4][2][2]=0.5*0.5/4.0;
	M[4][0][3][4][3][1]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][3][4][3][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][3][4][3][3]=0.5*0.5/4.0;
	M[4][0][3][4][4][1]=0.5*0.5/4.0;
	M[4][0][3][4][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][3][4][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][0][2][0]=0.5*0.5/4.0;
	M[4][0][4][0][3][0]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][0][4][0]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][1][2][1]=0.5*0.5/4.0;
	M[4][0][4][1][3][0]=0.5*0.5/4.0;
	M[4][0][4][1][3][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][1][4][0]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][1][4][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][2][2][1]=0.5*(0.5-0.5*a)/4.0;
	M[4][0][4][2][3][0]=0.5*a/4.0;
	M[4][0][4][2][3][1]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][0][4][2][3][2]=0.5*a/4.0;
	M[4][0][4][2][4][0]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][4][2][4][1]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][0][4][2][4][2]=0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][0][4][3][2][1]=0.5*0.5/4.0;
	M[4][0][4][3][3][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][3][3][2]=0.5*0.5/4.0;
	M[4][0][4][3][4][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][3][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][4][2][1]=0.5*0.5/4.0;
	M[4][0][4][4][3][1]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][4][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][4][4][1]=0.5*0.5/4.0;
	M[4][0][4][4][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][0][4][4][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][0][0][1][1]=0.5*1/4.0+0.5*1/4.0;
	M[4][1][0][0][2][0]=0.5*1/4.0+0.5*1/4.0;
	M[4][1][0][0][2][1]=0.5*1/4.0+0.5*1/4.0;
	M[4][1][0][0][3][0]=0.5*1/4.0+0.5*1/4.0;
	M[4][1][0][1][1][1]=0.5*1/4.0;
	M[4][1][0][1][1][2]=0.5*1/4.0;
	M[4][1][0][1][2][0]=0.5*1/4.0;
	M[4][1][0][1][2][1]=0.5*1/4.0+0.5*1/4.0;
	M[4][1][0][1][2][2]=0.5*1/4.0;
	M[4][1][0][1][3][0]=0.5*1/4.0;
	M[4][1][0][1][3][1]=0.5*1/4.0;
	M[4][1][0][2][1][1]=0.5*a/4.0;
	M[4][1][0][2][1][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][0][2][1][3]=0.5*a/4.0;
	M[4][1][0][2][2][0]=0.5*a/4.0;
	M[4][1][0][2][2][1]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][0][2][2][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][0][2][2][3]=0.5*a/4.0;
	M[4][1][0][2][3][0]=0.5*a/4.0;
	M[4][1][0][2][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][0][2][3][2]=0.5*a/4.0;
	M[4][1][0][3][1][1]=0.5*a/4.0;
	M[4][1][0][3][1][2]=0.5*(1-a)/4.0;
	M[4][1][0][3][1][3]=0.5*(1-a)/4.0;
	M[4][1][0][3][1][4]=0.5*a/4.0;
	M[4][1][0][3][2][0]=0.5*a/4.0;
	M[4][1][0][3][2][1]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][0][3][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][0][3][2][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][0][3][2][4]=0.5*a/4.0;
	M[4][1][0][3][3][0]=0.5*a/4.0;
	M[4][1][0][3][3][1]=0.5*(1-a)/4.0;
	M[4][1][0][3][3][2]=0.5*(1-a)/4.0;
	M[4][1][0][3][3][3]=0.5*a/4.0;
	M[4][1][0][4][1][2]=0.5*0.5/4.0;
	M[4][1][0][4][1][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][0][4][1][4]=0.5*0.5/4.0;
	M[4][1][0][4][2][1]=0.5*0.5/4.0;
	M[4][1][0][4][2][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][0][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][0][4][2][4]=0.5*0.5/4.0;
	M[4][1][0][4][3][1]=0.5*0.5/4.0;
	M[4][1][0][4][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][0][4][3][3]=0.5*0.5/4.0;
	M[4][1][1][0][1][1]=0.5*1/4.0;
	M[4][1][1][0][2][0]=0.5*1/4.0;
	M[4][1][1][0][2][1]=0.5*1/4.0+0.5*1/4.0;
	M[4][1][1][0][3][0]=0.5*1/4.0+0.5*1/4.0;
	M[4][1][1][0][3][1]=0.5*1/4.0;
	M[4][1][1][0][4][0]=0.5*1/4.0;
	M[4][1][1][1][1][1]=0.5*1/4.0;
	M[4][1][1][1][2][0]=0.5*1/4.0;
	M[4][1][1][1][2][1]=0.5*1/4.0;
	M[4][1][1][1][2][2]=0.5*1/4.0;
	M[4][1][1][1][3][0]=0.5*1/4.0;
	M[4][1][1][1][3][1]=0.5*1/4.0;
	M[4][1][1][1][3][2]=0.5*1/4.0;
	M[4][1][1][1][4][1]=0.5*1/4.0;
	M[4][1][1][2][1][1]=0.5*a/4.0;
	M[4][1][1][2][1][2]=0.5*(1-a)/4.0;
	M[4][1][1][2][2][0]=0.5*a/4.0;
	M[4][1][1][2][2][1]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][1][2][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][1][2][2][3]=0.5*a/4.0;
	M[4][1][1][2][3][0]=0.5*a/4.0;
	M[4][1][1][2][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][1][2][3][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][1][2][3][3]=0.5*a/4.0;
	M[4][1][1][2][4][1]=0.5*(1-a)/4.0;
	M[4][1][1][2][4][2]=0.5*a/4.0;
	M[4][1][1][3][1][1]=0.5*a/4.0;
	M[4][1][1][3][1][3]=0.5*(1-a)/4.0;
	M[4][1][1][3][2][0]=0.5*a/4.0;
	M[4][1][1][3][2][1]=0.5*a/4.0;
	M[4][1][1][3][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][1][3][2][3]=0.5*(1-a)/4.0;
	M[4][1][1][3][2][4]=0.5*a/4.0;
	M[4][1][1][3][3][0]=0.5*a/4.0;
	M[4][1][1][3][3][1]=0.5*(1-a)/4.0;
	M[4][1][1][3][3][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][1][3][3][3]=0.5*a/4.0;
	M[4][1][1][3][3][4]=0.5*a/4.0;
	M[4][1][1][3][4][1]=0.5*(1-a)/4.0;
	M[4][1][1][3][4][3]=0.5*a/4.0;
	M[4][1][1][4][1][3]=0.5*0.5/4.0;
	M[4][1][1][4][1][4]=0.5*0.5/4.0;
	M[4][1][1][4][2][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][1][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][1][4][2][4]=0.5*0.5/4.0;
	M[4][1][1][4][3][1]=0.5*0.5/4.0;
	M[4][1][1][4][3][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][1][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][1][4][4][1]=0.5*0.5/4.0;
	M[4][1][1][4][4][2]=0.5*0.5/4.0;
	M[4][1][2][0][1][1]=0.5*a/4.0;
	M[4][1][2][0][2][0]=0.5*a/4.0;
	M[4][1][2][0][2][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][1][2][0][3][0]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][2][0][3][1]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][2][0][4][0]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][1][2][0][4][1]=0.5*a/4.0;
	M[4][1][2][1][1][1]=0.5*a/4.0;
	M[4][1][2][1][2][0]=0.5*a/4.0;
	M[4][1][2][1][2][1]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][1][2][1][2][2]=0.5*(1-a)/4.0;
	M[4][1][2][1][3][0]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][2][1][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][2][1][3][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][2][1][4][0]=0.5*(1-a)/4.0;
	M[4][1][2][1][4][1]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][1][2][1][4][2]=0.5*a/4.0;
	M[4][1][2][2][1][1]=0.5*a/4.0;
	M[4][1][2][2][2][0]=0.5*a/4.0;
	M[4][1][2][2][2][1]=0.5*a/4.0;
	M[4][1][2][2][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][2][2][3][0]=0.5*a/4.0;
	M[4][1][2][2][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][2][2][3][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][2][2][3][3]=0.5*a/4.0;
	M[4][1][2][2][4][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][2][2][4][2]=0.5*a/4.0+0.5*a/4.0;
	M[4][1][2][2][4][3]=0.5*a/4.0;
	M[4][1][2][3][1][1]=0.5*a/4.0;
	M[4][1][2][3][2][0]=0.5*a/4.0;
	M[4][1][2][3][2][1]=0.5*a/4.0;
	M[4][1][2][3][2][2]=0.5*(1-a)/4.0;
	M[4][1][2][3][2][3]=0.5*(1-a)/4.0;
	M[4][1][2][3][3][0]=0.5*a/4.0;
	M[4][1][2][3][3][1]=0.5*(1-a)/4.0;
	M[4][1][2][3][3][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][2][3][3][3]=0.5*(1-a)/4.0;
	M[4][1][2][3][3][4]=0.5*a/4.0;
	M[4][1][2][3][4][1]=0.5*(1-a)/4.0;
	M[4][1][2][3][4][2]=0.5*(1-a)/4.0;
	M[4][1][2][3][4][3]=0.5*a/4.0+0.5*a/4.0;
	M[4][1][2][3][4][4]=0.5*a/4.0;
	M[4][1][2][4][1][3]=0.5*a/4.0;
	M[4][1][2][4][2][2]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][1][2][4][2][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][1][2][4][2][4]=0.5*(0.5-0.5*a)/4.0;
	M[4][1][2][4][3][1]=0.5*(0.5-0.5*a)/4.0;
	M[4][1][2][4][3][2]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][1][2][4][3][3]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][1][2][4][3][4]=0.5*(0.5-0.5*a)/4.0;
	M[4][1][2][4][4][1]=0.5*(0.5-0.5*a)/4.0;
	M[4][1][2][4][4][2]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][1][2][4][4][3]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][1][3][0][1][1]=0.5*a/4.0;
	M[4][1][3][0][2][0]=0.5*a/4.0;
	M[4][1][3][0][2][1]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][1][3][0][3][0]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][3][0][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][3][0][4][0]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][3][0][4][1]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][3][1][1][1]=0.5*a/4.0;
	M[4][1][3][1][2][0]=0.5*a/4.0;
	M[4][1][3][1][2][1]=0.5*a/4.0;
	M[4][1][3][1][2][2]=0.5*(1-a)/4.0;
	M[4][1][3][1][3][0]=0.5*a/4.0;
	M[4][1][3][1][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][3][1][3][2]=0.5*(1-a)/4.0;
	M[4][1][3][1][4][0]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][3][1][4][1]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][3][1][4][2]=0.5*a/4.0+0.5*a/4.0;
	M[4][1][3][2][1][1]=0.5*a/4.0;
	M[4][1][3][2][2][0]=0.5*a/4.0;
	M[4][1][3][2][2][1]=0.5*a/4.0;
	M[4][1][3][2][2][2]=0.5*(1-a)/4.0;
	M[4][1][3][2][3][0]=0.5*a/4.0;
	M[4][1][3][2][3][1]=0.5*(1-a)/4.0;
	M[4][1][3][2][3][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][3][2][4][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][3][2][4][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][1][3][2][4][3]=0.5*a/4.0+0.5*a/4.0;
	M[4][1][3][3][1][1]=0.5*a/4.0;
	M[4][1][3][3][2][0]=0.5*a/4.0;
	M[4][1][3][3][2][1]=0.5*a/4.0;
	M[4][1][3][3][2][2]=0.5*(1-a)/4.0;
	M[4][1][3][3][3][0]=0.5*a/4.0;
	M[4][1][3][3][3][1]=0.5*(1-a)/4.0;
	M[4][1][3][3][3][2]=0.5*(1-a)/4.0;
	M[4][1][3][3][3][3]=0.5*(1-a)/4.0;
	M[4][1][3][3][4][1]=0.5*(1-a)/4.0;
	M[4][1][3][3][4][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][1][3][3][4][3]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][1][3][3][4][4]=0.5*a/4.0+0.5*a/4.0;
	M[4][1][3][4][2][2]=0.5*0.5/4.0;
	M[4][1][3][4][2][3]=0.5*0.5/4.0;
	M[4][1][3][4][3][1]=0.5*0.5/4.0;
	M[4][1][3][4][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][3][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][3][4][3][4]=0.5*0.5/4.0;
	M[4][1][3][4][4][1]=0.5*0.5/4.0;
	M[4][1][3][4][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][3][4][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][3][4][4][4]=0.5*0.5/4.0;
	M[4][1][4][0][2][1]=0.5*0.5/4.0;
	M[4][1][4][0][3][0]=0.5*0.5/4.0;
	M[4][1][4][0][3][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][0][4][0]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][0][4][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][1][2][2]=0.5*0.5/4.0;
	M[4][1][4][1][3][1]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][1][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][1][4][0]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][1][4][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][1][4][2]=0.5*0.5/4.0;
	M[4][1][4][2][2][2]=0.5*(0.5-0.5*a)/4.0;
	M[4][1][4][2][3][1]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][1][4][2][3][2]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][1][4][2][3][3]=0.5*a/4.0;
	M[4][1][4][2][4][0]=0.5*a/4.0+0.5*a/4.0;
	M[4][1][4][2][4][1]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][1][4][2][4][2]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][1][4][2][4][3]=0.5*a/4.0;
	M[4][1][4][3][2][2]=0.5*0.5/4.0;
	M[4][1][4][3][3][1]=0.5*0.5/4.0;
	M[4][1][4][3][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][3][3][3]=0.5*0.5/4.0;
	M[4][1][4][3][4][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][3][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][3][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][4][2][2]=0.5*0.5/4.0;
	M[4][1][4][4][3][1]=0.5*0.5/4.0;
	M[4][1][4][4][3][2]=0.5*0.5/4.0;
	M[4][1][4][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][4][4][1]=0.5*0.5/4.0;
	M[4][1][4][4][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][4][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][1][4][4][4][4]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][2][0][0][1][1]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[4][2][0][0][2][0]=a*1/4.0+a*1/4.0;
	M[4][2][0][0][2][1]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[4][2][0][0][2][2]=a*1/4.0+a*1/4.0;
	M[4][2][0][0][3][1]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[4][2][0][1][1][1]=(0.5-0.5*a)*1/4.0;
	M[4][2][0][1][1][2]=(0.5-0.5*a)*1/4.0;
	M[4][2][0][1][2][0]=a*1/4.0;
	M[4][2][0][1][2][1]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+a*1/4.0;
	M[4][2][0][1][2][2]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+a*1/4.0;
	M[4][2][0][1][2][3]=a*1/4.0;
	M[4][2][0][1][3][1]=(0.5-0.5*a)*1/4.0;
	M[4][2][0][1][3][2]=(0.5-0.5*a)*1/4.0;
	M[4][2][0][2][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][0][2][1][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][0][2][1][3]=(0.5-0.5*a)*a/4.0;
	M[4][2][0][2][2][0]=a*a/4.0;
	M[4][2][0][2][2][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][0][2][2][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0+a*a/4.0;
	M[4][2][0][2][2][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][0][2][2][4]=a*a/4.0;
	M[4][2][0][2][3][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][0][2][3][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][0][2][3][3]=(0.5-0.5*a)*a/4.0;
	M[4][2][0][3][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][0][3][1][2]=(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][0][3][1][3]=(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][0][3][1][4]=(0.5-0.5*a)*a/4.0;
	M[4][2][0][3][2][0]=a*a/4.0;
	M[4][2][0][3][2][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][0][3][2][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0+a*(1-a)/4.0;
	M[4][2][0][3][2][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0+a*(1-a)/4.0;
	M[4][2][0][3][2][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*a/4.0+a*(1-a)/4.0;
	M[4][2][0][3][3][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][0][3][3][2]=(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][0][3][3][3]=(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][0][3][3][4]=(0.5-0.5*a)*a/4.0;
	M[4][2][0][4][1][2]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][0][4][1][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[4][2][0][4][1][4]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][0][4][2][1]=a*0.5/4.0;
	M[4][2][0][4][2][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][2][0][4][2][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][2][0][4][2][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][2][0][4][3][2]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][0][4][3][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[4][2][0][4][3][4]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][1][0][1][1]=(0.5-0.5*a)*1/4.0;
	M[4][2][1][0][2][0]=a*1/4.0;
	M[4][2][1][0][2][1]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[4][2][1][0][2][2]=a*1/4.0;
	M[4][2][1][0][3][0]=a*1/4.0;
	M[4][2][1][0][3][1]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[4][2][1][0][3][2]=a*1/4.0;
	M[4][2][1][0][4][1]=(0.5-0.5*a)*1/4.0;
	M[4][2][1][1][1][1]=(0.5-0.5*a)*1/4.0;
	M[4][2][1][1][2][0]=a*1/4.0;
	M[4][2][1][1][2][1]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[4][2][1][1][2][2]=(0.5-0.5*a)*1/4.0+a*1/4.0;
	M[4][2][1][1][3][1]=(0.5-0.5*a)*1/4.0+a*1/4.0;
	M[4][2][1][1][3][2]=(0.5-0.5*a)*1/4.0+(0.5-0.5*a)*1/4.0;
	M[4][2][1][1][3][3]=a*1/4.0;
	M[4][2][1][1][4][2]=(0.5-0.5*a)*1/4.0;
	M[4][2][1][2][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][1][2][1][2]=(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][1][2][2][0]=a*a/4.0;
	M[4][2][1][2][2][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][1][2][2][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][1][2][2][3]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][1][2][3][1]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][1][2][3][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][1][2][3][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][1][2][3][4]=a*a/4.0;
	M[4][2][1][2][4][2]=(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][1][2][4][3]=(0.5-0.5*a)*a/4.0;
	M[4][2][1][3][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][1][3][1][3]=(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][1][3][2][0]=a*a/4.0;
	M[4][2][1][3][2][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][1][3][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0+a*(1-a)/4.0;
	M[4][2][1][3][2][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][1][3][2][4]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][1][3][3][1]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][1][3][3][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][1][3][3][3]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0+a*(1-a)/4.0;
	M[4][2][1][3][3][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*a/4.0;
	M[4][2][1][3][4][2]=(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][1][3][4][4]=(0.5-0.5*a)*a/4.0;
	M[4][2][1][4][1][3]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][1][4][1][4]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][1][4][2][2]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][1][4][2][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][1][4][2][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][2][1][4][3][1]=a*0.5/4.0;
	M[4][2][1][4][3][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][1][4][3][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][1][4][3][4]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][1][4][4][2]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][1][4][4][3]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][2][0][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][2][0][2][0]=a*a/4.0;
	M[4][2][2][0][2][1]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][2][0][2][2]=a*a/4.0;
	M[4][2][2][0][3][0]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][2][0][3][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][2][0][3][2]=a*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][2][0][4][0]=a*a/4.0;
	M[4][2][2][0][4][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][2][0][4][2]=a*a/4.0;
	M[4][2][2][1][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][2][1][2][0]=a*a/4.0;
	M[4][2][2][1][2][1]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][2][1][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][2][1][3][0]=a*(1-a)/4.0;
	M[4][2][2][1][3][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][2][1][3][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][2][1][3][3]=a*(1-a)/4.0;
	M[4][2][2][1][4][1]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][2][1][4][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][2][1][4][3]=a*a/4.0;
	M[4][2][2][2][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][2][2][2][0]=a*a/4.0;
	M[4][2][2][2][2][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][2][2][2][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][2][2][3][1]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][2][2][3][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][2][2][3][3]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][2][2][4][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][2][2][4][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][2][2][4][4]=a*a/4.0;
	M[4][2][2][3][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][2][3][2][0]=a*a/4.0;
	M[4][2][2][3][2][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][2][3][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][2][3][2][3]=(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][2][3][3][1]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][2][3][3][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][2][3][3][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][2][3][3][4]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][2][3][4][2]=(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][2][3][4][3]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][2][3][4][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*a/4.0;
	M[4][2][2][4][1][3]=(0.5-0.5*a)*a/4.0;
	M[4][2][2][4][2][2]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[4][2][2][4][2][3]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][2][4][2][4]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[4][2][2][4][3][1]=a*(0.5-0.5*a)/4.0;
	M[4][2][2][4][3][2]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[4][2][2][4][3][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[4][2][2][4][3][4]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[4][2][2][4][4][2]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[4][2][2][4][4][3]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][2][4][4][4]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*a/4.0;
	M[4][2][3][0][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][3][0][2][0]=a*a/4.0;
	M[4][2][3][0][2][1]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][3][0][2][2]=a*a/4.0;
	M[4][2][3][0][3][0]=a*(1-a)/4.0;
	M[4][2][3][0][3][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][3][0][3][2]=a*(1-a)/4.0;
	M[4][2][3][0][4][0]=a*a/4.0+a*(1-a)/4.0;
	M[4][2][3][0][4][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][3][0][4][2]=a*a/4.0+a*(1-a)/4.0;
	M[4][2][3][1][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][3][1][2][0]=a*a/4.0;
	M[4][2][3][1][2][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][3][1][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][3][1][3][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][3][1][3][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][3][1][3][3]=a*(1-a)/4.0;
	M[4][2][3][1][4][0]=a*(1-a)/4.0;
	M[4][2][3][1][4][1]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][3][1][4][2]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][3][1][4][3]=a*a/4.0;
	M[4][2][3][2][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][3][2][2][0]=a*a/4.0;
	M[4][2][3][2][2][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][3][2][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][3][2][3][1]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][3][2][3][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][3][2][3][3]=a*(1-a)/4.0;
	M[4][2][3][2][4][1]=a*(1-a)/4.0;
	M[4][2][3][2][4][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][3][2][4][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][3][2][4][4]=a*a/4.0;
	M[4][2][3][3][1][1]=(0.5-0.5*a)*a/4.0;
	M[4][2][3][3][2][0]=a*a/4.0;
	M[4][2][3][3][2][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0;
	M[4][2][3][3][2][2]=(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][3][3][3][1]=(0.5-0.5*a)*a/4.0+a*(1-a)/4.0;
	M[4][2][3][3][3][2]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0;
	M[4][2][3][3][3][3]=(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][3][3][4][2]=(0.5-0.5*a)*(1-a)/4.0+a*(1-a)/4.0;
	M[4][2][3][3][4][3]=(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+(0.5-0.5*a)*(1-a)/4.0+a*a/4.0;
	M[4][2][3][3][4][4]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*a/4.0+a*(1-a)/4.0;
	M[4][2][3][4][2][2]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][3][4][2][3]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][3][4][3][1]=a*0.5/4.0;
	M[4][2][3][4][3][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][3][4][3][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][3][4][3][4]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][3][4][4][2]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][3][4][4][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][3][4][4][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][2][4][0][2][1]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][4][0][3][0]=a*0.5/4.0;
	M[4][2][4][0][3][1]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[4][2][4][0][3][2]=a*0.5/4.0;
	M[4][2][4][0][4][0]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][2][4][0][4][1]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[4][2][4][0][4][2]=a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][2][4][1][2][2]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][4][1][3][1]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][4][1][3][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[4][2][4][1][3][3]=a*0.5/4.0;
	M[4][2][4][1][4][0]=a*0.5/4.0+a*0.5/4.0;
	M[4][2][4][1][4][1]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][4][1][4][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][2][4][1][4][3]=a*0.5/4.0;
	M[4][2][4][2][2][2]=(0.5-0.5*a)*(0.5-0.5*a)/4.0;
	M[4][2][4][2][3][1]=(0.5-0.5*a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[4][2][4][2][3][2]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0;
	M[4][2][4][2][3][3]=(0.5-0.5*a)*a/4.0+a*(0.5-0.5*a)/4.0;
	M[4][2][4][2][4][0]=a*a/4.0;
	M[4][2][4][2][4][1]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[4][2][4][2][4][2]=(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+(0.5-0.5*a)*(0.5-0.5*a)/4.0+a*a/4.0+a*a/4.0;
	M[4][2][4][2][4][3]=(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+(0.5-0.5*a)*a/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0+a*(0.5-0.5*a)/4.0;
	M[4][2][4][2][4][4]=a*a/4.0;
	M[4][2][4][3][2][2]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][4][3][3][1]=a*0.5/4.0;
	M[4][2][4][3][3][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[4][2][4][3][3][3]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][4][3][4][1]=a*0.5/4.0;
	M[4][2][4][3][4][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][2][4][3][4][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][4][3][4][4]=a*0.5/4.0+a*0.5/4.0;
	M[4][2][4][4][2][2]=(0.5-0.5*a)*0.5/4.0;
	M[4][2][4][4][3][1]=a*0.5/4.0;
	M[4][2][4][4][3][2]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0;
	M[4][2][4][4][3][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][4][4][4][2]=(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][2][4][4][4][3]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0;
	M[4][2][4][4][4][4]=(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+(0.5-0.5*a)*0.5/4.0+a*0.5/4.0+a*0.5/4.0+a*0.5/4.0;
	M[4][3][0][0][1][1]=0.5*1/4.0+0.5*1/4.0;
	M[4][3][0][0][2][1]=0.5*1/4.0+0.5*1/4.0;
	M[4][3][0][0][2][2]=0.5*1/4.0+0.5*1/4.0;
	M[4][3][0][0][3][2]=0.5*1/4.0+0.5*1/4.0;
	M[4][3][0][1][1][1]=0.5*1/4.0;
	M[4][3][0][1][1][2]=0.5*1/4.0;
	M[4][3][0][1][2][1]=0.5*1/4.0;
	M[4][3][0][1][2][2]=0.5*1/4.0+0.5*1/4.0;
	M[4][3][0][1][2][3]=0.5*1/4.0;
	M[4][3][0][1][3][2]=0.5*1/4.0;
	M[4][3][0][1][3][3]=0.5*1/4.0;
	M[4][3][0][2][1][1]=0.5*a/4.0;
	M[4][3][0][2][1][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][0][2][1][3]=0.5*a/4.0;
	M[4][3][0][2][2][1]=0.5*a/4.0;
	M[4][3][0][2][2][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][0][2][2][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][0][2][2][4]=0.5*a/4.0;
	M[4][3][0][2][3][2]=0.5*a/4.0;
	M[4][3][0][2][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][0][2][3][4]=0.5*a/4.0;
	M[4][3][0][3][1][1]=0.5*a/4.0;
	M[4][3][0][3][1][2]=0.5*(1-a)/4.0;
	M[4][3][0][3][1][3]=0.5*(1-a)/4.0;
	M[4][3][0][3][1][4]=0.5*a/4.0;
	M[4][3][0][3][2][1]=0.5*a/4.0;
	M[4][3][0][3][2][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][0][3][2][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][0][3][2][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][0][3][3][2]=0.5*a/4.0;
	M[4][3][0][3][3][3]=0.5*(1-a)/4.0;
	M[4][3][0][3][3][4]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][0][4][1][2]=0.5*0.5/4.0;
	M[4][3][0][4][1][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][0][4][1][4]=0.5*0.5/4.0;
	M[4][3][0][4][2][2]=0.5*0.5/4.0;
	M[4][3][0][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][0][4][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][0][4][3][3]=0.5*0.5/4.0;
	M[4][3][0][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][1][0][1][1]=0.5*1/4.0;
	M[4][3][1][0][2][1]=0.5*1/4.0+0.5*1/4.0;
	M[4][3][1][0][2][2]=0.5*1/4.0;
	M[4][3][1][0][3][1]=0.5*1/4.0;
	M[4][3][1][0][3][2]=0.5*1/4.0+0.5*1/4.0;
	M[4][3][1][0][4][2]=0.5*1/4.0;
	M[4][3][1][1][1][1]=0.5*1/4.0;
	M[4][3][1][1][2][1]=0.5*1/4.0;
	M[4][3][1][1][2][2]=0.5*1/4.0+0.5*1/4.0;
	M[4][3][1][1][3][2]=0.5*1/4.0+0.5*1/4.0;
	M[4][3][1][1][3][3]=0.5*1/4.0;
	M[4][3][1][1][4][3]=0.5*1/4.0;
	M[4][3][1][2][1][1]=0.5*a/4.0;
	M[4][3][1][2][1][2]=0.5*(1-a)/4.0;
	M[4][3][1][2][2][1]=0.5*a/4.0;
	M[4][3][1][2][2][2]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][1][2][2][3]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][1][2][3][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][1][2][3][3]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][1][2][3][4]=0.5*a/4.0;
	M[4][3][1][2][4][3]=0.5*(1-a)/4.0;
	M[4][3][1][2][4][4]=0.5*a/4.0;
	M[4][3][1][3][1][1]=0.5*a/4.0;
	M[4][3][1][3][1][3]=0.5*(1-a)/4.0;
	M[4][3][1][3][2][1]=0.5*a/4.0;
	M[4][3][1][3][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][1][3][2][3]=0.5*(1-a)/4.0;
	M[4][3][1][3][2][4]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][1][3][3][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][1][3][3][3]=0.5*(1-a)/4.0;
	M[4][3][1][3][3][4]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][3][1][3][4][3]=0.5*(1-a)/4.0;
	M[4][3][1][3][4][4]=0.5*a/4.0;
	M[4][3][1][4][1][3]=0.5*0.5/4.0;
	M[4][3][1][4][1][4]=0.5*0.5/4.0;
	M[4][3][1][4][2][2]=0.5*0.5/4.0;
	M[4][3][1][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][1][4][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][1][4][3][2]=0.5*0.5/4.0;
	M[4][3][1][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][1][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][1][4][4][3]=0.5*0.5/4.0;
	M[4][3][1][4][4][4]=0.5*0.5/4.0;
	M[4][3][2][0][1][1]=0.5*a/4.0;
	M[4][3][2][0][2][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][2][0][2][2]=0.5*a/4.0;
	M[4][3][2][0][3][1]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][2][0][3][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][2][0][4][1]=0.5*a/4.0;
	M[4][3][2][0][4][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][2][1][1][1]=0.5*a/4.0;
	M[4][3][2][1][2][1]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][2][1][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][2][1][3][1]=0.5*(1-a)/4.0;
	M[4][3][2][1][3][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][2][1][3][3]=0.5*(1-a)/4.0;
	M[4][3][2][1][4][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][2][1][4][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][2][2][1][1]=0.5*a/4.0;
	M[4][3][2][2][2][1]=0.5*a/4.0;
	M[4][3][2][2][2][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][2][2][3][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][2][2][3][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][2][2][4][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][2][2][4][4]=0.5*a/4.0+0.5*a/4.0;
	M[4][3][2][3][1][1]=0.5*a/4.0;
	M[4][3][2][3][2][1]=0.5*a/4.0;
	M[4][3][2][3][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][2][3][2][3]=0.5*(1-a)/4.0;
	M[4][3][2][3][3][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][2][3][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][2][3][3][4]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][2][3][4][3]=0.5*(1-a)/4.0;
	M[4][3][2][3][4][4]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][3][2][4][1][3]=0.5*a/4.0;
	M[4][3][2][4][2][2]=0.5*(0.5-0.5*a)/4.0;
	M[4][3][2][4][2][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][3][2][4][2][4]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][3][2][4][3][2]=0.5*(0.5-0.5*a)/4.0;
	M[4][3][2][4][3][3]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][3][2][4][3][4]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][3][2][4][4][3]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][3][2][4][4][4]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][3][3][0][1][1]=0.5*a/4.0;
	M[4][3][3][0][2][1]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][3][0][2][2]=0.5*a/4.0;
	M[4][3][3][0][3][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][3][0][3][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][3][0][4][1]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][3][0][4][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][3][1][1][1]=0.5*a/4.0;
	M[4][3][3][1][2][1]=0.5*a/4.0;
	M[4][3][3][1][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][3][1][3][1]=0.5*(1-a)/4.0;
	M[4][3][3][1][3][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][3][1][3][3]=0.5*(1-a)/4.0;
	M[4][3][3][1][4][1]=0.5*(1-a)/4.0;
	M[4][3][3][1][4][2]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][3][1][4][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][3][2][1][1]=0.5*a/4.0;
	M[4][3][3][2][2][1]=0.5*a/4.0;
	M[4][3][3][2][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][3][2][3][2]=0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][3][2][3][3]=0.5*(1-a)/4.0;
	M[4][3][3][2][4][2]=0.5*(1-a)/4.0;
	M[4][3][3][2][4][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][3][2][4][4]=0.5*a/4.0+0.5*a/4.0;
	M[4][3][3][3][1][1]=0.5*a/4.0;
	M[4][3][3][3][2][1]=0.5*a/4.0;
	M[4][3][3][3][2][2]=0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][3][3][3][3][2]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][3][3][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][3][3][4][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][3][3][3][4][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][3][3][4][2][2]=0.5*0.5/4.0;
	M[4][3][3][4][2][3]=0.5*0.5/4.0;
	M[4][3][3][4][3][2]=0.5*0.5/4.0;
	M[4][3][3][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][3][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][3][4][4][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][3][4][4][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][0][2][1]=0.5*0.5/4.0;
	M[4][3][4][0][3][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][0][3][2]=0.5*0.5/4.0;
	M[4][3][4][0][4][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][0][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][1][2][2]=0.5*0.5/4.0;
	M[4][3][4][1][3][1]=0.5*0.5/4.0;
	M[4][3][4][1][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][1][3][3]=0.5*0.5/4.0;
	M[4][3][4][1][4][1]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][1][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][1][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][2][2][2]=0.5*(0.5-0.5*a)/4.0;
	M[4][3][4][2][3][1]=0.5*a/4.0;
	M[4][3][4][2][3][2]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][3][4][2][3][3]=0.5*a/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][3][4][2][4][1]=0.5*a/4.0;
	M[4][3][4][2][4][2]=0.5*(0.5-0.5*a)/4.0+0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][3][4][2][4][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][3][4][2][4][4]=0.5*a/4.0+0.5*a/4.0;
	M[4][3][4][3][2][2]=0.5*0.5/4.0;
	M[4][3][4][3][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][3][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][3][4][2]=0.5*0.5/4.0;
	M[4][3][4][3][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][3][4][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][4][2][2]=0.5*0.5/4.0;
	M[4][3][4][4][3][2]=0.5*0.5/4.0;
	M[4][3][4][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][4][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][3][4][4][4][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][0][0][1][1]=0.5*1/4.0+0.5*1/4.0;
	M[4][4][0][0][2][2]=0.5*1/4.0+0.5*1/4.0+0.5*1/4.0+0.5*1/4.0;
	M[4][4][0][0][3][3]=0.5*1/4.0+0.5*1/4.0;
	M[4][4][0][1][1][1]=0.5*1/4.0;
	M[4][4][0][1][1][2]=0.5*1/4.0;
	M[4][4][0][1][2][2]=0.5*1/4.0+0.5*1/4.0;
	M[4][4][0][1][2][3]=0.5*1/4.0+0.5*1/4.0;
	M[4][4][0][1][3][3]=0.5*1/4.0;
	M[4][4][0][1][3][4]=0.5*1/4.0;
	M[4][4][0][2][1][1]=0.5*a/4.0;
	M[4][4][0][2][1][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][0][2][1][3]=0.5*a/4.0;
	M[4][4][0][2][2][2]=0.5*a/4.0+0.5*a/4.0;
	M[4][4][0][2][2][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][0][2][2][4]=0.5*a/4.0+0.5*a/4.0;
	M[4][4][0][2][3][3]=0.5*a/4.0;
	M[4][4][0][2][3][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][0][3][1][1]=0.5*a/4.0;
	M[4][4][0][3][1][2]=0.5*(1-a)/4.0;
	M[4][4][0][3][1][3]=0.5*(1-a)/4.0;
	M[4][4][0][3][1][4]=0.5*a/4.0;
	M[4][4][0][3][2][2]=0.5*a/4.0+0.5*a/4.0;
	M[4][4][0][3][2][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][0][3][2][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][0][3][3][3]=0.5*a/4.0;
	M[4][4][0][3][3][4]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][0][4][1][2]=0.5*0.5/4.0;
	M[4][4][0][4][1][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][0][4][1][4]=0.5*0.5/4.0;
	M[4][4][0][4][2][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][0][4][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][0][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][1][0][1][1]=0.5*1/4.0;
	M[4][4][1][0][2][1]=0.5*1/4.0;
	M[4][4][1][0][2][2]=0.5*1/4.0+0.5*1/4.0;
	M[4][4][1][0][3][2]=0.5*1/4.0+0.5*1/4.0;
	M[4][4][1][0][3][3]=0.5*1/4.0;
	M[4][4][1][0][4][3]=0.5*1/4.0;
	M[4][4][1][1][1][1]=0.5*1/4.0;
	M[4][4][1][1][2][2]=0.5*1/4.0+0.5*1/4.0+0.5*1/4.0;
	M[4][4][1][1][3][3]=0.5*1/4.0+0.5*1/4.0+0.5*1/4.0;
	M[4][4][1][1][4][4]=0.5*1/4.0;
	M[4][4][1][2][1][1]=0.5*a/4.0;
	M[4][4][1][2][1][2]=0.5*(1-a)/4.0;
	M[4][4][1][2][2][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][1][2][2][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][1][2][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][1][2][3][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][1][2][4][4]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][1][3][1][1]=0.5*a/4.0;
	M[4][4][1][3][1][3]=0.5*(1-a)/4.0;
	M[4][4][1][3][2][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][1][3][2][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][1][3][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][1][3][3][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][1][3][4][4]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][1][4][1][3]=0.5*0.5/4.0;
	M[4][4][1][4][1][4]=0.5*0.5/4.0;
	M[4][4][1][4][2][2]=0.5*0.5/4.0;
	M[4][4][1][4][2][3]=0.5*0.5/4.0;
	M[4][4][1][4][2][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][1][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][1][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][1][4][4][4]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][2][0][1][1]=0.5*a/4.0;
	M[4][4][2][0][2][1]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][2][0][2][2]=0.5*a/4.0+0.5*a/4.0;
	M[4][4][2][0][3][1]=0.5*a/4.0;
	M[4][4][2][0][3][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][2][0][3][3]=0.5*a/4.0;
	M[4][4][2][0][4][2]=0.5*a/4.0+0.5*a/4.0;
	M[4][4][2][0][4][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][2][1][1][1]=0.5*a/4.0;
	M[4][4][2][1][2][1]=0.5*(1-a)/4.0;
	M[4][4][2][1][2][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][2][1][3][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][2][1][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][2][1][4][3]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][2][1][4][4]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][2][2][1][1]=0.5*a/4.0;
	M[4][4][2][2][2][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][2][2][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0;
	M[4][4][2][2][4][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][2][3][1][1]=0.5*a/4.0;
	M[4][4][2][3][2][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][2][3][2][3]=0.5*(1-a)/4.0;
	M[4][4][2][3][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][2][3][3][4]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][2][3][4][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][2][4][1][3]=0.5*a/4.0;
	M[4][4][2][4][2][2]=0.5*(0.5-0.5*a)/4.0;
	M[4][4][2][4][2][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][4][2][4][2][4]=0.5*a/4.0+0.5*a/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][4][2][4][3][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][4][2][4][3][4]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][4][2][4][4][4]=0.5*a/4.0+0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][4][3][0][1][1]=0.5*a/4.0;
	M[4][4][3][0][2][1]=0.5*(1-a)/4.0;
	M[4][4][3][0][2][2]=0.5*a/4.0+0.5*a/4.0;
	M[4][4][3][0][3][1]=0.5*(1-a)/4.0;
	M[4][4][3][0][3][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][3][0][3][3]=0.5*a/4.0;
	M[4][4][3][0][4][1]=0.5*a/4.0;
	M[4][4][3][0][4][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][3][0][4][3]=0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][3][1][1][1]=0.5*a/4.0;
	M[4][4][3][1][2][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][3][1][3][1]=0.5*(1-a)/4.0;
	M[4][4][3][1][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][3][1][4][2]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][3][1][4][3]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][3][1][4][4]=0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][3][2][1][1]=0.5*a/4.0;
	M[4][4][3][2][2][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][3][2][3][2]=0.5*(1-a)/4.0;
	M[4][4][3][2][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][3][2][4][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][3][2][4][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0;
	M[4][4][3][3][1][1]=0.5*a/4.0;
	M[4][4][3][3][2][2]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][3][3][3][3]=0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*(1-a)/4.0;
	M[4][4][3][3][4][4]=0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0+0.5*a/4.0+0.5*a/4.0+0.5*(1-a)/4.0+0.5*(1-a)/4.0;
	M[4][4][3][4][2][2]=0.5*0.5/4.0;
	M[4][4][3][4][2][3]=0.5*0.5/4.0;
	M[4][4][3][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][3][4][3][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][3][4][4][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][0][2][1]=0.5*0.5/4.0;
	M[4][4][4][0][3][1]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][0][3][2]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][0][4][1]=0.5*0.5/4.0;
	M[4][4][4][0][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][0][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][1][2][2]=0.5*0.5/4.0;
	M[4][4][4][1][3][1]=0.5*0.5/4.0;
	M[4][4][4][1][3][2]=0.5*0.5/4.0;
	M[4][4][4][1][3][3]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][1][4][1]=0.5*0.5/4.0;
	M[4][4][4][1][4][2]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][1][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][1][4][4]=0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][2][2][2]=0.5*(0.5-0.5*a)/4.0;
	M[4][4][4][2][3][1]=0.5*a/4.0;
	M[4][4][4][2][3][2]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][4][4][2][3][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][4][4][2][4][2]=0.5*a/4.0+0.5*a/4.0+0.5*(0.5-0.5*a)/4.0;
	M[4][4][4][2][4][3]=0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][4][4][2][4][4]=0.5*a/4.0+0.5*a/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*(0.5-0.5*a)/4.0+0.5*a/4.0;
	M[4][4][4][3][2][2]=0.5*0.5/4.0;
	M[4][4][4][3][3][2]=0.5*0.5/4.0;
	M[4][4][4][3][3][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][3][4][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][3][4][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][4][2][2]=0.5*0.5/4.0;
	M[4][4][4][4][3][3]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
	M[4][4][4][4][4][4]=0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0+0.5*0.5/4.0;
}


void calculateCHIT_male_x (double a, double M[5][5][5][5][5][5])
{
	int f1, f2, m1, m2, o1, o2;
	for (f1=0; f1<=4; f1++) {
		for (f2=0; f2<=4; f2++) {
			for (m1=0; m1<=4; m1++) {
				for (m2=0; m2<=4; m2++) {
					for (o1=0; o1<=4; o1++) {
						for (o2=0; o2<=4; o2++) {
							M[f1][f2][m1][m2][o1][o2] = 0;
						}
					}
				}
			}
		}
	}
	M[0][0][0][0][0][0]=1/2.0+1/2.0;
	M[0][0][0][1][0][0]=1/2.0;
	M[0][0][0][1][0][1]=1/2.0;
	M[0][0][0][2][0][0]=a/2.0;
	M[0][0][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][0][0][2][0][2]=a/2.0;
	M[0][0][0][3][0][0]=a/2.0;
	M[0][0][0][3][0][1]=(1-a)/2.0;
	M[0][0][0][3][0][2]=(1-a)/2.0;
	M[0][0][0][3][0][3]=a/2.0;
	M[0][0][0][4][0][1]=0.5/2.0;
	M[0][0][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[0][0][0][4][0][3]=0.5/2.0;
	M[0][0][1][0][0][0]=1/2.0;
	M[0][0][1][0][1][0]=1/2.0;
	M[0][0][1][1][0][0]=1/2.0;
	M[0][0][1][1][1][1]=1/2.0;
	M[0][0][1][2][0][0]=a/2.0;
	M[0][0][1][2][0][1]=(1-a)/2.0;
	M[0][0][1][2][1][1]=(1-a)/2.0;
	M[0][0][1][2][1][2]=a/2.0;
	M[0][0][1][3][0][0]=a/2.0;
	M[0][0][1][3][0][2]=(1-a)/2.0;
	M[0][0][1][3][1][1]=(1-a)/2.0;
	M[0][0][1][3][1][3]=a/2.0;
	M[0][0][1][4][0][2]=0.5/2.0;
	M[0][0][1][4][0][3]=0.5/2.0;
	M[0][0][1][4][1][1]=0.5/2.0;
	M[0][0][1][4][1][2]=0.5/2.0;
	M[0][0][2][0][0][0]=a/2.0;
	M[0][0][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[0][0][2][0][2][0]=a/2.0;
	M[0][0][2][1][0][0]=a/2.0;
	M[0][0][2][1][1][0]=(1-a)/2.0;
	M[0][0][2][1][1][1]=(1-a)/2.0;
	M[0][0][2][1][2][1]=a/2.0;
	M[0][0][2][2][0][0]=a/2.0;
	M[0][0][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][0][2][2][2][2]=a/2.0;
	M[0][0][2][3][0][0]=a/2.0;
	M[0][0][2][3][1][1]=(1-a)/2.0;
	M[0][0][2][3][1][2]=(1-a)/2.0;
	M[0][0][2][3][2][3]=a/2.0;
	M[0][0][2][4][0][2]=a/2.0;
	M[0][0][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[0][0][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][0][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[0][0][2][4][2][2]=a/2.0;
	M[0][0][3][0][0][0]=a/2.0;
	M[0][0][3][0][1][0]=(1-a)/2.0;
	M[0][0][3][0][2][0]=(1-a)/2.0;
	M[0][0][3][0][3][0]=a/2.0;
	M[0][0][3][1][0][0]=a/2.0;
	M[0][0][3][1][1][1]=(1-a)/2.0;
	M[0][0][3][1][2][0]=(1-a)/2.0;
	M[0][0][3][1][3][1]=a/2.0;
	M[0][0][3][2][0][0]=a/2.0;
	M[0][0][3][2][1][1]=(1-a)/2.0;
	M[0][0][3][2][2][1]=(1-a)/2.0;
	M[0][0][3][2][3][2]=a/2.0;
	M[0][0][3][3][0][0]=a/2.0;
	M[0][0][3][3][1][1]=(1-a)/2.0;
	M[0][0][3][3][2][2]=(1-a)/2.0;
	M[0][0][3][3][3][3]=a/2.0;
	M[0][0][3][4][1][1]=0.5/2.0;
	M[0][0][3][4][1][2]=0.5/2.0;
	M[0][0][3][4][2][2]=0.5/2.0;
	M[0][0][3][4][2][3]=0.5/2.0;
	M[0][0][4][0][1][0]=0.5/2.0;
	M[0][0][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[0][0][4][0][3][0]=0.5/2.0;
	M[0][0][4][1][1][1]=0.5/2.0;
	M[0][0][4][1][2][0]=0.5/2.0;
	M[0][0][4][1][2][1]=0.5/2.0;
	M[0][0][4][1][3][0]=0.5/2.0;
	M[0][0][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[0][0][4][2][2][0]=a/2.0;
	M[0][0][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][0][4][2][2][2]=a/2.0;
	M[0][0][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[0][0][4][3][1][1]=0.5/2.0;
	M[0][0][4][3][2][1]=0.5/2.0;
	M[0][0][4][3][2][2]=0.5/2.0;
	M[0][0][4][3][3][2]=0.5/2.0;
	M[0][0][4][4][1][1]=0.5/2.0;
	M[0][0][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[0][0][4][4][3][3]=0.5/2.0;
	M[0][1][0][0][0][0]=1/2.0+1/2.0;
	M[0][1][0][1][0][0]=1/2.0;
	M[0][1][0][1][0][1]=1/2.0;
	M[0][1][0][2][0][0]=a/2.0;
	M[0][1][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][1][0][2][0][2]=a/2.0;
	M[0][1][0][3][0][0]=a/2.0;
	M[0][1][0][3][0][1]=(1-a)/2.0;
	M[0][1][0][3][0][2]=(1-a)/2.0;
	M[0][1][0][3][0][3]=a/2.0;
	M[0][1][0][4][0][1]=0.5/2.0;
	M[0][1][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[0][1][0][4][0][3]=0.5/2.0;
	M[0][1][1][0][0][0]=1/2.0;
	M[0][1][1][0][1][0]=1/2.0;
	M[0][1][1][1][0][0]=1/2.0;
	M[0][1][1][1][1][1]=1/2.0;
	M[0][1][1][2][0][0]=a/2.0;
	M[0][1][1][2][0][1]=(1-a)/2.0;
	M[0][1][1][2][1][1]=(1-a)/2.0;
	M[0][1][1][2][1][2]=a/2.0;
	M[0][1][1][3][0][0]=a/2.0;
	M[0][1][1][3][0][2]=(1-a)/2.0;
	M[0][1][1][3][1][1]=(1-a)/2.0;
	M[0][1][1][3][1][3]=a/2.0;
	M[0][1][1][4][0][2]=0.5/2.0;
	M[0][1][1][4][0][3]=0.5/2.0;
	M[0][1][1][4][1][1]=0.5/2.0;
	M[0][1][1][4][1][2]=0.5/2.0;
	M[0][1][2][0][0][0]=a/2.0;
	M[0][1][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[0][1][2][0][2][0]=a/2.0;
	M[0][1][2][1][0][0]=a/2.0;
	M[0][1][2][1][1][0]=(1-a)/2.0;
	M[0][1][2][1][1][1]=(1-a)/2.0;
	M[0][1][2][1][2][1]=a/2.0;
	M[0][1][2][2][0][0]=a/2.0;
	M[0][1][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][1][2][2][2][2]=a/2.0;
	M[0][1][2][3][0][0]=a/2.0;
	M[0][1][2][3][1][1]=(1-a)/2.0;
	M[0][1][2][3][1][2]=(1-a)/2.0;
	M[0][1][2][3][2][3]=a/2.0;
	M[0][1][2][4][0][2]=a/2.0;
	M[0][1][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[0][1][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][1][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[0][1][2][4][2][2]=a/2.0;
	M[0][1][3][0][0][0]=a/2.0;
	M[0][1][3][0][1][0]=(1-a)/2.0;
	M[0][1][3][0][2][0]=(1-a)/2.0;
	M[0][1][3][0][3][0]=a/2.0;
	M[0][1][3][1][0][0]=a/2.0;
	M[0][1][3][1][1][1]=(1-a)/2.0;
	M[0][1][3][1][2][0]=(1-a)/2.0;
	M[0][1][3][1][3][1]=a/2.0;
	M[0][1][3][2][0][0]=a/2.0;
	M[0][1][3][2][1][1]=(1-a)/2.0;
	M[0][1][3][2][2][1]=(1-a)/2.0;
	M[0][1][3][2][3][2]=a/2.0;
	M[0][1][3][3][0][0]=a/2.0;
	M[0][1][3][3][1][1]=(1-a)/2.0;
	M[0][1][3][3][2][2]=(1-a)/2.0;
	M[0][1][3][3][3][3]=a/2.0;
	M[0][1][3][4][1][1]=0.5/2.0;
	M[0][1][3][4][1][2]=0.5/2.0;
	M[0][1][3][4][2][2]=0.5/2.0;
	M[0][1][3][4][2][3]=0.5/2.0;
	M[0][1][4][0][1][0]=0.5/2.0;
	M[0][1][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[0][1][4][0][3][0]=0.5/2.0;
	M[0][1][4][1][1][1]=0.5/2.0;
	M[0][1][4][1][2][0]=0.5/2.0;
	M[0][1][4][1][2][1]=0.5/2.0;
	M[0][1][4][1][3][0]=0.5/2.0;
	M[0][1][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[0][1][4][2][2][0]=a/2.0;
	M[0][1][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][1][4][2][2][2]=a/2.0;
	M[0][1][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[0][1][4][3][1][1]=0.5/2.0;
	M[0][1][4][3][2][1]=0.5/2.0;
	M[0][1][4][3][2][2]=0.5/2.0;
	M[0][1][4][3][3][2]=0.5/2.0;
	M[0][1][4][4][1][1]=0.5/2.0;
	M[0][1][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[0][1][4][4][3][3]=0.5/2.0;
	M[0][2][0][0][0][0]=1/2.0+1/2.0;
	M[0][2][0][1][0][0]=1/2.0;
	M[0][2][0][1][0][1]=1/2.0;
	M[0][2][0][2][0][0]=a/2.0;
	M[0][2][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][2][0][2][0][2]=a/2.0;
	M[0][2][0][3][0][0]=a/2.0;
	M[0][2][0][3][0][1]=(1-a)/2.0;
	M[0][2][0][3][0][2]=(1-a)/2.0;
	M[0][2][0][3][0][3]=a/2.0;
	M[0][2][0][4][0][1]=0.5/2.0;
	M[0][2][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[0][2][0][4][0][3]=0.5/2.0;
	M[0][2][1][0][0][0]=1/2.0;
	M[0][2][1][0][1][0]=1/2.0;
	M[0][2][1][1][0][0]=1/2.0;
	M[0][2][1][1][1][1]=1/2.0;
	M[0][2][1][2][0][0]=a/2.0;
	M[0][2][1][2][0][1]=(1-a)/2.0;
	M[0][2][1][2][1][1]=(1-a)/2.0;
	M[0][2][1][2][1][2]=a/2.0;
	M[0][2][1][3][0][0]=a/2.0;
	M[0][2][1][3][0][2]=(1-a)/2.0;
	M[0][2][1][3][1][1]=(1-a)/2.0;
	M[0][2][1][3][1][3]=a/2.0;
	M[0][2][1][4][0][2]=0.5/2.0;
	M[0][2][1][4][0][3]=0.5/2.0;
	M[0][2][1][4][1][1]=0.5/2.0;
	M[0][2][1][4][1][2]=0.5/2.0;
	M[0][2][2][0][0][0]=a/2.0;
	M[0][2][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[0][2][2][0][2][0]=a/2.0;
	M[0][2][2][1][0][0]=a/2.0;
	M[0][2][2][1][1][0]=(1-a)/2.0;
	M[0][2][2][1][1][1]=(1-a)/2.0;
	M[0][2][2][1][2][1]=a/2.0;
	M[0][2][2][2][0][0]=a/2.0;
	M[0][2][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][2][2][2][2][2]=a/2.0;
	M[0][2][2][3][0][0]=a/2.0;
	M[0][2][2][3][1][1]=(1-a)/2.0;
	M[0][2][2][3][1][2]=(1-a)/2.0;
	M[0][2][2][3][2][3]=a/2.0;
	M[0][2][2][4][0][2]=a/2.0;
	M[0][2][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[0][2][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][2][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[0][2][2][4][2][2]=a/2.0;
	M[0][2][3][0][0][0]=a/2.0;
	M[0][2][3][0][1][0]=(1-a)/2.0;
	M[0][2][3][0][2][0]=(1-a)/2.0;
	M[0][2][3][0][3][0]=a/2.0;
	M[0][2][3][1][0][0]=a/2.0;
	M[0][2][3][1][1][1]=(1-a)/2.0;
	M[0][2][3][1][2][0]=(1-a)/2.0;
	M[0][2][3][1][3][1]=a/2.0;
	M[0][2][3][2][0][0]=a/2.0;
	M[0][2][3][2][1][1]=(1-a)/2.0;
	M[0][2][3][2][2][1]=(1-a)/2.0;
	M[0][2][3][2][3][2]=a/2.0;
	M[0][2][3][3][0][0]=a/2.0;
	M[0][2][3][3][1][1]=(1-a)/2.0;
	M[0][2][3][3][2][2]=(1-a)/2.0;
	M[0][2][3][3][3][3]=a/2.0;
	M[0][2][3][4][1][1]=0.5/2.0;
	M[0][2][3][4][1][2]=0.5/2.0;
	M[0][2][3][4][2][2]=0.5/2.0;
	M[0][2][3][4][2][3]=0.5/2.0;
	M[0][2][4][0][1][0]=0.5/2.0;
	M[0][2][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[0][2][4][0][3][0]=0.5/2.0;
	M[0][2][4][1][1][1]=0.5/2.0;
	M[0][2][4][1][2][0]=0.5/2.0;
	M[0][2][4][1][2][1]=0.5/2.0;
	M[0][2][4][1][3][0]=0.5/2.0;
	M[0][2][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[0][2][4][2][2][0]=a/2.0;
	M[0][2][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][2][4][2][2][2]=a/2.0;
	M[0][2][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[0][2][4][3][1][1]=0.5/2.0;
	M[0][2][4][3][2][1]=0.5/2.0;
	M[0][2][4][3][2][2]=0.5/2.0;
	M[0][2][4][3][3][2]=0.5/2.0;
	M[0][2][4][4][1][1]=0.5/2.0;
	M[0][2][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[0][2][4][4][3][3]=0.5/2.0;
	M[0][3][0][0][0][0]=1/2.0+1/2.0;
	M[0][3][0][1][0][0]=1/2.0;
	M[0][3][0][1][0][1]=1/2.0;
	M[0][3][0][2][0][0]=a/2.0;
	M[0][3][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][3][0][2][0][2]=a/2.0;
	M[0][3][0][3][0][0]=a/2.0;
	M[0][3][0][3][0][1]=(1-a)/2.0;
	M[0][3][0][3][0][2]=(1-a)/2.0;
	M[0][3][0][3][0][3]=a/2.0;
	M[0][3][0][4][0][1]=0.5/2.0;
	M[0][3][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[0][3][0][4][0][3]=0.5/2.0;
	M[0][3][1][0][0][0]=1/2.0;
	M[0][3][1][0][1][0]=1/2.0;
	M[0][3][1][1][0][0]=1/2.0;
	M[0][3][1][1][1][1]=1/2.0;
	M[0][3][1][2][0][0]=a/2.0;
	M[0][3][1][2][0][1]=(1-a)/2.0;
	M[0][3][1][2][1][1]=(1-a)/2.0;
	M[0][3][1][2][1][2]=a/2.0;
	M[0][3][1][3][0][0]=a/2.0;
	M[0][3][1][3][0][2]=(1-a)/2.0;
	M[0][3][1][3][1][1]=(1-a)/2.0;
	M[0][3][1][3][1][3]=a/2.0;
	M[0][3][1][4][0][2]=0.5/2.0;
	M[0][3][1][4][0][3]=0.5/2.0;
	M[0][3][1][4][1][1]=0.5/2.0;
	M[0][3][1][4][1][2]=0.5/2.0;
	M[0][3][2][0][0][0]=a/2.0;
	M[0][3][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[0][3][2][0][2][0]=a/2.0;
	M[0][3][2][1][0][0]=a/2.0;
	M[0][3][2][1][1][0]=(1-a)/2.0;
	M[0][3][2][1][1][1]=(1-a)/2.0;
	M[0][3][2][1][2][1]=a/2.0;
	M[0][3][2][2][0][0]=a/2.0;
	M[0][3][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][3][2][2][2][2]=a/2.0;
	M[0][3][2][3][0][0]=a/2.0;
	M[0][3][2][3][1][1]=(1-a)/2.0;
	M[0][3][2][3][1][2]=(1-a)/2.0;
	M[0][3][2][3][2][3]=a/2.0;
	M[0][3][2][4][0][2]=a/2.0;
	M[0][3][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[0][3][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][3][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[0][3][2][4][2][2]=a/2.0;
	M[0][3][3][0][0][0]=a/2.0;
	M[0][3][3][0][1][0]=(1-a)/2.0;
	M[0][3][3][0][2][0]=(1-a)/2.0;
	M[0][3][3][0][3][0]=a/2.0;
	M[0][3][3][1][0][0]=a/2.0;
	M[0][3][3][1][1][1]=(1-a)/2.0;
	M[0][3][3][1][2][0]=(1-a)/2.0;
	M[0][3][3][1][3][1]=a/2.0;
	M[0][3][3][2][0][0]=a/2.0;
	M[0][3][3][2][1][1]=(1-a)/2.0;
	M[0][3][3][2][2][1]=(1-a)/2.0;
	M[0][3][3][2][3][2]=a/2.0;
	M[0][3][3][3][0][0]=a/2.0;
	M[0][3][3][3][1][1]=(1-a)/2.0;
	M[0][3][3][3][2][2]=(1-a)/2.0;
	M[0][3][3][3][3][3]=a/2.0;
	M[0][3][3][4][1][1]=0.5/2.0;
	M[0][3][3][4][1][2]=0.5/2.0;
	M[0][3][3][4][2][2]=0.5/2.0;
	M[0][3][3][4][2][3]=0.5/2.0;
	M[0][3][4][0][1][0]=0.5/2.0;
	M[0][3][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[0][3][4][0][3][0]=0.5/2.0;
	M[0][3][4][1][1][1]=0.5/2.0;
	M[0][3][4][1][2][0]=0.5/2.0;
	M[0][3][4][1][2][1]=0.5/2.0;
	M[0][3][4][1][3][0]=0.5/2.0;
	M[0][3][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[0][3][4][2][2][0]=a/2.0;
	M[0][3][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][3][4][2][2][2]=a/2.0;
	M[0][3][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[0][3][4][3][1][1]=0.5/2.0;
	M[0][3][4][3][2][1]=0.5/2.0;
	M[0][3][4][3][2][2]=0.5/2.0;
	M[0][3][4][3][3][2]=0.5/2.0;
	M[0][3][4][4][1][1]=0.5/2.0;
	M[0][3][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[0][3][4][4][3][3]=0.5/2.0;
	M[0][4][0][0][0][0]=1/2.0+1/2.0;
	M[0][4][0][1][0][0]=1/2.0;
	M[0][4][0][1][0][1]=1/2.0;
	M[0][4][0][2][0][0]=a/2.0;
	M[0][4][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][4][0][2][0][2]=a/2.0;
	M[0][4][0][3][0][0]=a/2.0;
	M[0][4][0][3][0][1]=(1-a)/2.0;
	M[0][4][0][3][0][2]=(1-a)/2.0;
	M[0][4][0][3][0][3]=a/2.0;
	M[0][4][0][4][0][1]=0.5/2.0;
	M[0][4][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[0][4][0][4][0][3]=0.5/2.0;
	M[0][4][1][0][0][0]=1/2.0;
	M[0][4][1][0][1][0]=1/2.0;
	M[0][4][1][1][0][0]=1/2.0;
	M[0][4][1][1][1][1]=1/2.0;
	M[0][4][1][2][0][0]=a/2.0;
	M[0][4][1][2][0][1]=(1-a)/2.0;
	M[0][4][1][2][1][1]=(1-a)/2.0;
	M[0][4][1][2][1][2]=a/2.0;
	M[0][4][1][3][0][0]=a/2.0;
	M[0][4][1][3][0][2]=(1-a)/2.0;
	M[0][4][1][3][1][1]=(1-a)/2.0;
	M[0][4][1][3][1][3]=a/2.0;
	M[0][4][1][4][0][2]=0.5/2.0;
	M[0][4][1][4][0][3]=0.5/2.0;
	M[0][4][1][4][1][1]=0.5/2.0;
	M[0][4][1][4][1][2]=0.5/2.0;
	M[0][4][2][0][0][0]=a/2.0;
	M[0][4][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[0][4][2][0][2][0]=a/2.0;
	M[0][4][2][1][0][0]=a/2.0;
	M[0][4][2][1][1][0]=(1-a)/2.0;
	M[0][4][2][1][1][1]=(1-a)/2.0;
	M[0][4][2][1][2][1]=a/2.0;
	M[0][4][2][2][0][0]=a/2.0;
	M[0][4][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][4][2][2][2][2]=a/2.0;
	M[0][4][2][3][0][0]=a/2.0;
	M[0][4][2][3][1][1]=(1-a)/2.0;
	M[0][4][2][3][1][2]=(1-a)/2.0;
	M[0][4][2][3][2][3]=a/2.0;
	M[0][4][2][4][0][2]=a/2.0;
	M[0][4][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[0][4][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][4][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[0][4][2][4][2][2]=a/2.0;
	M[0][4][3][0][0][0]=a/2.0;
	M[0][4][3][0][1][0]=(1-a)/2.0;
	M[0][4][3][0][2][0]=(1-a)/2.0;
	M[0][4][3][0][3][0]=a/2.0;
	M[0][4][3][1][0][0]=a/2.0;
	M[0][4][3][1][1][1]=(1-a)/2.0;
	M[0][4][3][1][2][0]=(1-a)/2.0;
	M[0][4][3][1][3][1]=a/2.0;
	M[0][4][3][2][0][0]=a/2.0;
	M[0][4][3][2][1][1]=(1-a)/2.0;
	M[0][4][3][2][2][1]=(1-a)/2.0;
	M[0][4][3][2][3][2]=a/2.0;
	M[0][4][3][3][0][0]=a/2.0;
	M[0][4][3][3][1][1]=(1-a)/2.0;
	M[0][4][3][3][2][2]=(1-a)/2.0;
	M[0][4][3][3][3][3]=a/2.0;
	M[0][4][3][4][1][1]=0.5/2.0;
	M[0][4][3][4][1][2]=0.5/2.0;
	M[0][4][3][4][2][2]=0.5/2.0;
	M[0][4][3][4][2][3]=0.5/2.0;
	M[0][4][4][0][1][0]=0.5/2.0;
	M[0][4][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[0][4][4][0][3][0]=0.5/2.0;
	M[0][4][4][1][1][1]=0.5/2.0;
	M[0][4][4][1][2][0]=0.5/2.0;
	M[0][4][4][1][2][1]=0.5/2.0;
	M[0][4][4][1][3][0]=0.5/2.0;
	M[0][4][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[0][4][4][2][2][0]=a/2.0;
	M[0][4][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][4][4][2][2][2]=a/2.0;
	M[0][4][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[0][4][4][3][1][1]=0.5/2.0;
	M[0][4][4][3][2][1]=0.5/2.0;
	M[0][4][4][3][2][2]=0.5/2.0;
	M[0][4][4][3][3][2]=0.5/2.0;
	M[0][4][4][4][1][1]=0.5/2.0;
	M[0][4][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[0][4][4][4][3][3]=0.5/2.0;
	M[1][0][0][0][0][0]=1/2.0+1/2.0;
	M[1][0][0][1][0][0]=1/2.0;
	M[1][0][0][1][0][1]=1/2.0;
	M[1][0][0][2][0][0]=a/2.0;
	M[1][0][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][0][0][2][0][2]=a/2.0;
	M[1][0][0][3][0][0]=a/2.0;
	M[1][0][0][3][0][1]=(1-a)/2.0;
	M[1][0][0][3][0][2]=(1-a)/2.0;
	M[1][0][0][3][0][3]=a/2.0;
	M[1][0][0][4][0][1]=0.5/2.0;
	M[1][0][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[1][0][0][4][0][3]=0.5/2.0;
	M[1][0][1][0][0][0]=1/2.0;
	M[1][0][1][0][1][0]=1/2.0;
	M[1][0][1][1][0][0]=1/2.0;
	M[1][0][1][1][1][1]=1/2.0;
	M[1][0][1][2][0][0]=a/2.0;
	M[1][0][1][2][0][1]=(1-a)/2.0;
	M[1][0][1][2][1][1]=(1-a)/2.0;
	M[1][0][1][2][1][2]=a/2.0;
	M[1][0][1][3][0][0]=a/2.0;
	M[1][0][1][3][0][2]=(1-a)/2.0;
	M[1][0][1][3][1][1]=(1-a)/2.0;
	M[1][0][1][3][1][3]=a/2.0;
	M[1][0][1][4][0][2]=0.5/2.0;
	M[1][0][1][4][0][3]=0.5/2.0;
	M[1][0][1][4][1][1]=0.5/2.0;
	M[1][0][1][4][1][2]=0.5/2.0;
	M[1][0][2][0][0][0]=a/2.0;
	M[1][0][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[1][0][2][0][2][0]=a/2.0;
	M[1][0][2][1][0][0]=a/2.0;
	M[1][0][2][1][1][0]=(1-a)/2.0;
	M[1][0][2][1][1][1]=(1-a)/2.0;
	M[1][0][2][1][2][1]=a/2.0;
	M[1][0][2][2][0][0]=a/2.0;
	M[1][0][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][0][2][2][2][2]=a/2.0;
	M[1][0][2][3][0][0]=a/2.0;
	M[1][0][2][3][1][1]=(1-a)/2.0;
	M[1][0][2][3][1][2]=(1-a)/2.0;
	M[1][0][2][3][2][3]=a/2.0;
	M[1][0][2][4][0][2]=a/2.0;
	M[1][0][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[1][0][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][0][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[1][0][2][4][2][2]=a/2.0;
	M[1][0][3][0][0][0]=a/2.0;
	M[1][0][3][0][1][0]=(1-a)/2.0;
	M[1][0][3][0][2][0]=(1-a)/2.0;
	M[1][0][3][0][3][0]=a/2.0;
	M[1][0][3][1][0][0]=a/2.0;
	M[1][0][3][1][1][1]=(1-a)/2.0;
	M[1][0][3][1][2][0]=(1-a)/2.0;
	M[1][0][3][1][3][1]=a/2.0;
	M[1][0][3][2][0][0]=a/2.0;
	M[1][0][3][2][1][1]=(1-a)/2.0;
	M[1][0][3][2][2][1]=(1-a)/2.0;
	M[1][0][3][2][3][2]=a/2.0;
	M[1][0][3][3][0][0]=a/2.0;
	M[1][0][3][3][1][1]=(1-a)/2.0;
	M[1][0][3][3][2][2]=(1-a)/2.0;
	M[1][0][3][3][3][3]=a/2.0;
	M[1][0][3][4][1][1]=0.5/2.0;
	M[1][0][3][4][1][2]=0.5/2.0;
	M[1][0][3][4][2][2]=0.5/2.0;
	M[1][0][3][4][2][3]=0.5/2.0;
	M[1][0][4][0][1][0]=0.5/2.0;
	M[1][0][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[1][0][4][0][3][0]=0.5/2.0;
	M[1][0][4][1][1][1]=0.5/2.0;
	M[1][0][4][1][2][0]=0.5/2.0;
	M[1][0][4][1][2][1]=0.5/2.0;
	M[1][0][4][1][3][0]=0.5/2.0;
	M[1][0][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[1][0][4][2][2][0]=a/2.0;
	M[1][0][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][0][4][2][2][2]=a/2.0;
	M[1][0][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[1][0][4][3][1][1]=0.5/2.0;
	M[1][0][4][3][2][1]=0.5/2.0;
	M[1][0][4][3][2][2]=0.5/2.0;
	M[1][0][4][3][3][2]=0.5/2.0;
	M[1][0][4][4][1][1]=0.5/2.0;
	M[1][0][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[1][0][4][4][3][3]=0.5/2.0;
	M[1][1][0][0][0][0]=1/2.0+1/2.0;
	M[1][1][0][1][0][0]=1/2.0;
	M[1][1][0][1][0][1]=1/2.0;
	M[1][1][0][2][0][0]=a/2.0;
	M[1][1][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][1][0][2][0][2]=a/2.0;
	M[1][1][0][3][0][0]=a/2.0;
	M[1][1][0][3][0][1]=(1-a)/2.0;
	M[1][1][0][3][0][2]=(1-a)/2.0;
	M[1][1][0][3][0][3]=a/2.0;
	M[1][1][0][4][0][1]=0.5/2.0;
	M[1][1][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[1][1][0][4][0][3]=0.5/2.0;
	M[1][1][1][0][0][0]=1/2.0;
	M[1][1][1][0][1][0]=1/2.0;
	M[1][1][1][1][0][0]=1/2.0;
	M[1][1][1][1][1][1]=1/2.0;
	M[1][1][1][2][0][0]=a/2.0;
	M[1][1][1][2][0][1]=(1-a)/2.0;
	M[1][1][1][2][1][1]=(1-a)/2.0;
	M[1][1][1][2][1][2]=a/2.0;
	M[1][1][1][3][0][0]=a/2.0;
	M[1][1][1][3][0][2]=(1-a)/2.0;
	M[1][1][1][3][1][1]=(1-a)/2.0;
	M[1][1][1][3][1][3]=a/2.0;
	M[1][1][1][4][0][2]=0.5/2.0;
	M[1][1][1][4][0][3]=0.5/2.0;
	M[1][1][1][4][1][1]=0.5/2.0;
	M[1][1][1][4][1][2]=0.5/2.0;
	M[1][1][2][0][0][0]=a/2.0;
	M[1][1][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[1][1][2][0][2][0]=a/2.0;
	M[1][1][2][1][0][0]=a/2.0;
	M[1][1][2][1][1][0]=(1-a)/2.0;
	M[1][1][2][1][1][1]=(1-a)/2.0;
	M[1][1][2][1][2][1]=a/2.0;
	M[1][1][2][2][0][0]=a/2.0;
	M[1][1][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][1][2][2][2][2]=a/2.0;
	M[1][1][2][3][0][0]=a/2.0;
	M[1][1][2][3][1][1]=(1-a)/2.0;
	M[1][1][2][3][1][2]=(1-a)/2.0;
	M[1][1][2][3][2][3]=a/2.0;
	M[1][1][2][4][0][2]=a/2.0;
	M[1][1][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[1][1][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][1][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[1][1][2][4][2][2]=a/2.0;
	M[1][1][3][0][0][0]=a/2.0;
	M[1][1][3][0][1][0]=(1-a)/2.0;
	M[1][1][3][0][2][0]=(1-a)/2.0;
	M[1][1][3][0][3][0]=a/2.0;
	M[1][1][3][1][0][0]=a/2.0;
	M[1][1][3][1][1][1]=(1-a)/2.0;
	M[1][1][3][1][2][0]=(1-a)/2.0;
	M[1][1][3][1][3][1]=a/2.0;
	M[1][1][3][2][0][0]=a/2.0;
	M[1][1][3][2][1][1]=(1-a)/2.0;
	M[1][1][3][2][2][1]=(1-a)/2.0;
	M[1][1][3][2][3][2]=a/2.0;
	M[1][1][3][3][0][0]=a/2.0;
	M[1][1][3][3][1][1]=(1-a)/2.0;
	M[1][1][3][3][2][2]=(1-a)/2.0;
	M[1][1][3][3][3][3]=a/2.0;
	M[1][1][3][4][1][1]=0.5/2.0;
	M[1][1][3][4][1][2]=0.5/2.0;
	M[1][1][3][4][2][2]=0.5/2.0;
	M[1][1][3][4][2][3]=0.5/2.0;
	M[1][1][4][0][1][0]=0.5/2.0;
	M[1][1][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[1][1][4][0][3][0]=0.5/2.0;
	M[1][1][4][1][1][1]=0.5/2.0;
	M[1][1][4][1][2][0]=0.5/2.0;
	M[1][1][4][1][2][1]=0.5/2.0;
	M[1][1][4][1][3][0]=0.5/2.0;
	M[1][1][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[1][1][4][2][2][0]=a/2.0;
	M[1][1][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][1][4][2][2][2]=a/2.0;
	M[1][1][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[1][1][4][3][1][1]=0.5/2.0;
	M[1][1][4][3][2][1]=0.5/2.0;
	M[1][1][4][3][2][2]=0.5/2.0;
	M[1][1][4][3][3][2]=0.5/2.0;
	M[1][1][4][4][1][1]=0.5/2.0;
	M[1][1][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[1][1][4][4][3][3]=0.5/2.0;
	M[1][2][0][0][0][0]=1/2.0+1/2.0;
	M[1][2][0][1][0][0]=1/2.0;
	M[1][2][0][1][0][1]=1/2.0;
	M[1][2][0][2][0][0]=a/2.0;
	M[1][2][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][2][0][2][0][2]=a/2.0;
	M[1][2][0][3][0][0]=a/2.0;
	M[1][2][0][3][0][1]=(1-a)/2.0;
	M[1][2][0][3][0][2]=(1-a)/2.0;
	M[1][2][0][3][0][3]=a/2.0;
	M[1][2][0][4][0][1]=0.5/2.0;
	M[1][2][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[1][2][0][4][0][3]=0.5/2.0;
	M[1][2][1][0][0][0]=1/2.0;
	M[1][2][1][0][1][0]=1/2.0;
	M[1][2][1][1][0][0]=1/2.0;
	M[1][2][1][1][1][1]=1/2.0;
	M[1][2][1][2][0][0]=a/2.0;
	M[1][2][1][2][0][1]=(1-a)/2.0;
	M[1][2][1][2][1][1]=(1-a)/2.0;
	M[1][2][1][2][1][2]=a/2.0;
	M[1][2][1][3][0][0]=a/2.0;
	M[1][2][1][3][0][2]=(1-a)/2.0;
	M[1][2][1][3][1][1]=(1-a)/2.0;
	M[1][2][1][3][1][3]=a/2.0;
	M[1][2][1][4][0][2]=0.5/2.0;
	M[1][2][1][4][0][3]=0.5/2.0;
	M[1][2][1][4][1][1]=0.5/2.0;
	M[1][2][1][4][1][2]=0.5/2.0;
	M[1][2][2][0][0][0]=a/2.0;
	M[1][2][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[1][2][2][0][2][0]=a/2.0;
	M[1][2][2][1][0][0]=a/2.0;
	M[1][2][2][1][1][0]=(1-a)/2.0;
	M[1][2][2][1][1][1]=(1-a)/2.0;
	M[1][2][2][1][2][1]=a/2.0;
	M[1][2][2][2][0][0]=a/2.0;
	M[1][2][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][2][2][2][2][2]=a/2.0;
	M[1][2][2][3][0][0]=a/2.0;
	M[1][2][2][3][1][1]=(1-a)/2.0;
	M[1][2][2][3][1][2]=(1-a)/2.0;
	M[1][2][2][3][2][3]=a/2.0;
	M[1][2][2][4][0][2]=a/2.0;
	M[1][2][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[1][2][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][2][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[1][2][2][4][2][2]=a/2.0;
	M[1][2][3][0][0][0]=a/2.0;
	M[1][2][3][0][1][0]=(1-a)/2.0;
	M[1][2][3][0][2][0]=(1-a)/2.0;
	M[1][2][3][0][3][0]=a/2.0;
	M[1][2][3][1][0][0]=a/2.0;
	M[1][2][3][1][1][1]=(1-a)/2.0;
	M[1][2][3][1][2][0]=(1-a)/2.0;
	M[1][2][3][1][3][1]=a/2.0;
	M[1][2][3][2][0][0]=a/2.0;
	M[1][2][3][2][1][1]=(1-a)/2.0;
	M[1][2][3][2][2][1]=(1-a)/2.0;
	M[1][2][3][2][3][2]=a/2.0;
	M[1][2][3][3][0][0]=a/2.0;
	M[1][2][3][3][1][1]=(1-a)/2.0;
	M[1][2][3][3][2][2]=(1-a)/2.0;
	M[1][2][3][3][3][3]=a/2.0;
	M[1][2][3][4][1][1]=0.5/2.0;
	M[1][2][3][4][1][2]=0.5/2.0;
	M[1][2][3][4][2][2]=0.5/2.0;
	M[1][2][3][4][2][3]=0.5/2.0;
	M[1][2][4][0][1][0]=0.5/2.0;
	M[1][2][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[1][2][4][0][3][0]=0.5/2.0;
	M[1][2][4][1][1][1]=0.5/2.0;
	M[1][2][4][1][2][0]=0.5/2.0;
	M[1][2][4][1][2][1]=0.5/2.0;
	M[1][2][4][1][3][0]=0.5/2.0;
	M[1][2][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[1][2][4][2][2][0]=a/2.0;
	M[1][2][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][2][4][2][2][2]=a/2.0;
	M[1][2][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[1][2][4][3][1][1]=0.5/2.0;
	M[1][2][4][3][2][1]=0.5/2.0;
	M[1][2][4][3][2][2]=0.5/2.0;
	M[1][2][4][3][3][2]=0.5/2.0;
	M[1][2][4][4][1][1]=0.5/2.0;
	M[1][2][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[1][2][4][4][3][3]=0.5/2.0;
	M[1][3][0][0][0][0]=1/2.0+1/2.0;
	M[1][3][0][1][0][0]=1/2.0;
	M[1][3][0][1][0][1]=1/2.0;
	M[1][3][0][2][0][0]=a/2.0;
	M[1][3][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][3][0][2][0][2]=a/2.0;
	M[1][3][0][3][0][0]=a/2.0;
	M[1][3][0][3][0][1]=(1-a)/2.0;
	M[1][3][0][3][0][2]=(1-a)/2.0;
	M[1][3][0][3][0][3]=a/2.0;
	M[1][3][0][4][0][1]=0.5/2.0;
	M[1][3][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[1][3][0][4][0][3]=0.5/2.0;
	M[1][3][1][0][0][0]=1/2.0;
	M[1][3][1][0][1][0]=1/2.0;
	M[1][3][1][1][0][0]=1/2.0;
	M[1][3][1][1][1][1]=1/2.0;
	M[1][3][1][2][0][0]=a/2.0;
	M[1][3][1][2][0][1]=(1-a)/2.0;
	M[1][3][1][2][1][1]=(1-a)/2.0;
	M[1][3][1][2][1][2]=a/2.0;
	M[1][3][1][3][0][0]=a/2.0;
	M[1][3][1][3][0][2]=(1-a)/2.0;
	M[1][3][1][3][1][1]=(1-a)/2.0;
	M[1][3][1][3][1][3]=a/2.0;
	M[1][3][1][4][0][2]=0.5/2.0;
	M[1][3][1][4][0][3]=0.5/2.0;
	M[1][3][1][4][1][1]=0.5/2.0;
	M[1][3][1][4][1][2]=0.5/2.0;
	M[1][3][2][0][0][0]=a/2.0;
	M[1][3][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[1][3][2][0][2][0]=a/2.0;
	M[1][3][2][1][0][0]=a/2.0;
	M[1][3][2][1][1][0]=(1-a)/2.0;
	M[1][3][2][1][1][1]=(1-a)/2.0;
	M[1][3][2][1][2][1]=a/2.0;
	M[1][3][2][2][0][0]=a/2.0;
	M[1][3][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][3][2][2][2][2]=a/2.0;
	M[1][3][2][3][0][0]=a/2.0;
	M[1][3][2][3][1][1]=(1-a)/2.0;
	M[1][3][2][3][1][2]=(1-a)/2.0;
	M[1][3][2][3][2][3]=a/2.0;
	M[1][3][2][4][0][2]=a/2.0;
	M[1][3][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[1][3][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][3][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[1][3][2][4][2][2]=a/2.0;
	M[1][3][3][0][0][0]=a/2.0;
	M[1][3][3][0][1][0]=(1-a)/2.0;
	M[1][3][3][0][2][0]=(1-a)/2.0;
	M[1][3][3][0][3][0]=a/2.0;
	M[1][3][3][1][0][0]=a/2.0;
	M[1][3][3][1][1][1]=(1-a)/2.0;
	M[1][3][3][1][2][0]=(1-a)/2.0;
	M[1][3][3][1][3][1]=a/2.0;
	M[1][3][3][2][0][0]=a/2.0;
	M[1][3][3][2][1][1]=(1-a)/2.0;
	M[1][3][3][2][2][1]=(1-a)/2.0;
	M[1][3][3][2][3][2]=a/2.0;
	M[1][3][3][3][0][0]=a/2.0;
	M[1][3][3][3][1][1]=(1-a)/2.0;
	M[1][3][3][3][2][2]=(1-a)/2.0;
	M[1][3][3][3][3][3]=a/2.0;
	M[1][3][3][4][1][1]=0.5/2.0;
	M[1][3][3][4][1][2]=0.5/2.0;
	M[1][3][3][4][2][2]=0.5/2.0;
	M[1][3][3][4][2][3]=0.5/2.0;
	M[1][3][4][0][1][0]=0.5/2.0;
	M[1][3][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[1][3][4][0][3][0]=0.5/2.0;
	M[1][3][4][1][1][1]=0.5/2.0;
	M[1][3][4][1][2][0]=0.5/2.0;
	M[1][3][4][1][2][1]=0.5/2.0;
	M[1][3][4][1][3][0]=0.5/2.0;
	M[1][3][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[1][3][4][2][2][0]=a/2.0;
	M[1][3][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][3][4][2][2][2]=a/2.0;
	M[1][3][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[1][3][4][3][1][1]=0.5/2.0;
	M[1][3][4][3][2][1]=0.5/2.0;
	M[1][3][4][3][2][2]=0.5/2.0;
	M[1][3][4][3][3][2]=0.5/2.0;
	M[1][3][4][4][1][1]=0.5/2.0;
	M[1][3][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[1][3][4][4][3][3]=0.5/2.0;
	M[1][4][0][0][0][0]=1/2.0+1/2.0;
	M[1][4][0][1][0][0]=1/2.0;
	M[1][4][0][1][0][1]=1/2.0;
	M[1][4][0][2][0][0]=a/2.0;
	M[1][4][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][4][0][2][0][2]=a/2.0;
	M[1][4][0][3][0][0]=a/2.0;
	M[1][4][0][3][0][1]=(1-a)/2.0;
	M[1][4][0][3][0][2]=(1-a)/2.0;
	M[1][4][0][3][0][3]=a/2.0;
	M[1][4][0][4][0][1]=0.5/2.0;
	M[1][4][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[1][4][0][4][0][3]=0.5/2.0;
	M[1][4][1][0][0][0]=1/2.0;
	M[1][4][1][0][1][0]=1/2.0;
	M[1][4][1][1][0][0]=1/2.0;
	M[1][4][1][1][1][1]=1/2.0;
	M[1][4][1][2][0][0]=a/2.0;
	M[1][4][1][2][0][1]=(1-a)/2.0;
	M[1][4][1][2][1][1]=(1-a)/2.0;
	M[1][4][1][2][1][2]=a/2.0;
	M[1][4][1][3][0][0]=a/2.0;
	M[1][4][1][3][0][2]=(1-a)/2.0;
	M[1][4][1][3][1][1]=(1-a)/2.0;
	M[1][4][1][3][1][3]=a/2.0;
	M[1][4][1][4][0][2]=0.5/2.0;
	M[1][4][1][4][0][3]=0.5/2.0;
	M[1][4][1][4][1][1]=0.5/2.0;
	M[1][4][1][4][1][2]=0.5/2.0;
	M[1][4][2][0][0][0]=a/2.0;
	M[1][4][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[1][4][2][0][2][0]=a/2.0;
	M[1][4][2][1][0][0]=a/2.0;
	M[1][4][2][1][1][0]=(1-a)/2.0;
	M[1][4][2][1][1][1]=(1-a)/2.0;
	M[1][4][2][1][2][1]=a/2.0;
	M[1][4][2][2][0][0]=a/2.0;
	M[1][4][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][4][2][2][2][2]=a/2.0;
	M[1][4][2][3][0][0]=a/2.0;
	M[1][4][2][3][1][1]=(1-a)/2.0;
	M[1][4][2][3][1][2]=(1-a)/2.0;
	M[1][4][2][3][2][3]=a/2.0;
	M[1][4][2][4][0][2]=a/2.0;
	M[1][4][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[1][4][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][4][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[1][4][2][4][2][2]=a/2.0;
	M[1][4][3][0][0][0]=a/2.0;
	M[1][4][3][0][1][0]=(1-a)/2.0;
	M[1][4][3][0][2][0]=(1-a)/2.0;
	M[1][4][3][0][3][0]=a/2.0;
	M[1][4][3][1][0][0]=a/2.0;
	M[1][4][3][1][1][1]=(1-a)/2.0;
	M[1][4][3][1][2][0]=(1-a)/2.0;
	M[1][4][3][1][3][1]=a/2.0;
	M[1][4][3][2][0][0]=a/2.0;
	M[1][4][3][2][1][1]=(1-a)/2.0;
	M[1][4][3][2][2][1]=(1-a)/2.0;
	M[1][4][3][2][3][2]=a/2.0;
	M[1][4][3][3][0][0]=a/2.0;
	M[1][4][3][3][1][1]=(1-a)/2.0;
	M[1][4][3][3][2][2]=(1-a)/2.0;
	M[1][4][3][3][3][3]=a/2.0;
	M[1][4][3][4][1][1]=0.5/2.0;
	M[1][4][3][4][1][2]=0.5/2.0;
	M[1][4][3][4][2][2]=0.5/2.0;
	M[1][4][3][4][2][3]=0.5/2.0;
	M[1][4][4][0][1][0]=0.5/2.0;
	M[1][4][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[1][4][4][0][3][0]=0.5/2.0;
	M[1][4][4][1][1][1]=0.5/2.0;
	M[1][4][4][1][2][0]=0.5/2.0;
	M[1][4][4][1][2][1]=0.5/2.0;
	M[1][4][4][1][3][0]=0.5/2.0;
	M[1][4][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[1][4][4][2][2][0]=a/2.0;
	M[1][4][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][4][4][2][2][2]=a/2.0;
	M[1][4][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[1][4][4][3][1][1]=0.5/2.0;
	M[1][4][4][3][2][1]=0.5/2.0;
	M[1][4][4][3][2][2]=0.5/2.0;
	M[1][4][4][3][3][2]=0.5/2.0;
	M[1][4][4][4][1][1]=0.5/2.0;
	M[1][4][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[1][4][4][4][3][3]=0.5/2.0;
	M[2][0][0][0][0][0]=1/2.0+1/2.0;
	M[2][0][0][1][0][0]=1/2.0;
	M[2][0][0][1][0][1]=1/2.0;
	M[2][0][0][2][0][0]=a/2.0;
	M[2][0][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][0][0][2][0][2]=a/2.0;
	M[2][0][0][3][0][0]=a/2.0;
	M[2][0][0][3][0][1]=(1-a)/2.0;
	M[2][0][0][3][0][2]=(1-a)/2.0;
	M[2][0][0][3][0][3]=a/2.0;
	M[2][0][0][4][0][1]=0.5/2.0;
	M[2][0][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[2][0][0][4][0][3]=0.5/2.0;
	M[2][0][1][0][0][0]=1/2.0;
	M[2][0][1][0][1][0]=1/2.0;
	M[2][0][1][1][0][0]=1/2.0;
	M[2][0][1][1][1][1]=1/2.0;
	M[2][0][1][2][0][0]=a/2.0;
	M[2][0][1][2][0][1]=(1-a)/2.0;
	M[2][0][1][2][1][1]=(1-a)/2.0;
	M[2][0][1][2][1][2]=a/2.0;
	M[2][0][1][3][0][0]=a/2.0;
	M[2][0][1][3][0][2]=(1-a)/2.0;
	M[2][0][1][3][1][1]=(1-a)/2.0;
	M[2][0][1][3][1][3]=a/2.0;
	M[2][0][1][4][0][2]=0.5/2.0;
	M[2][0][1][4][0][3]=0.5/2.0;
	M[2][0][1][4][1][1]=0.5/2.0;
	M[2][0][1][4][1][2]=0.5/2.0;
	M[2][0][2][0][0][0]=a/2.0;
	M[2][0][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[2][0][2][0][2][0]=a/2.0;
	M[2][0][2][1][0][0]=a/2.0;
	M[2][0][2][1][1][0]=(1-a)/2.0;
	M[2][0][2][1][1][1]=(1-a)/2.0;
	M[2][0][2][1][2][1]=a/2.0;
	M[2][0][2][2][0][0]=a/2.0;
	M[2][0][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][0][2][2][2][2]=a/2.0;
	M[2][0][2][3][0][0]=a/2.0;
	M[2][0][2][3][1][1]=(1-a)/2.0;
	M[2][0][2][3][1][2]=(1-a)/2.0;
	M[2][0][2][3][2][3]=a/2.0;
	M[2][0][2][4][0][2]=a/2.0;
	M[2][0][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[2][0][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][0][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[2][0][2][4][2][2]=a/2.0;
	M[2][0][3][0][0][0]=a/2.0;
	M[2][0][3][0][1][0]=(1-a)/2.0;
	M[2][0][3][0][2][0]=(1-a)/2.0;
	M[2][0][3][0][3][0]=a/2.0;
	M[2][0][3][1][0][0]=a/2.0;
	M[2][0][3][1][1][1]=(1-a)/2.0;
	M[2][0][3][1][2][0]=(1-a)/2.0;
	M[2][0][3][1][3][1]=a/2.0;
	M[2][0][3][2][0][0]=a/2.0;
	M[2][0][3][2][1][1]=(1-a)/2.0;
	M[2][0][3][2][2][1]=(1-a)/2.0;
	M[2][0][3][2][3][2]=a/2.0;
	M[2][0][3][3][0][0]=a/2.0;
	M[2][0][3][3][1][1]=(1-a)/2.0;
	M[2][0][3][3][2][2]=(1-a)/2.0;
	M[2][0][3][3][3][3]=a/2.0;
	M[2][0][3][4][1][1]=0.5/2.0;
	M[2][0][3][4][1][2]=0.5/2.0;
	M[2][0][3][4][2][2]=0.5/2.0;
	M[2][0][3][4][2][3]=0.5/2.0;
	M[2][0][4][0][1][0]=0.5/2.0;
	M[2][0][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[2][0][4][0][3][0]=0.5/2.0;
	M[2][0][4][1][1][1]=0.5/2.0;
	M[2][0][4][1][2][0]=0.5/2.0;
	M[2][0][4][1][2][1]=0.5/2.0;
	M[2][0][4][1][3][0]=0.5/2.0;
	M[2][0][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[2][0][4][2][2][0]=a/2.0;
	M[2][0][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][0][4][2][2][2]=a/2.0;
	M[2][0][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[2][0][4][3][1][1]=0.5/2.0;
	M[2][0][4][3][2][1]=0.5/2.0;
	M[2][0][4][3][2][2]=0.5/2.0;
	M[2][0][4][3][3][2]=0.5/2.0;
	M[2][0][4][4][1][1]=0.5/2.0;
	M[2][0][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[2][0][4][4][3][3]=0.5/2.0;
	M[2][1][0][0][0][0]=1/2.0+1/2.0;
	M[2][1][0][1][0][0]=1/2.0;
	M[2][1][0][1][0][1]=1/2.0;
	M[2][1][0][2][0][0]=a/2.0;
	M[2][1][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][1][0][2][0][2]=a/2.0;
	M[2][1][0][3][0][0]=a/2.0;
	M[2][1][0][3][0][1]=(1-a)/2.0;
	M[2][1][0][3][0][2]=(1-a)/2.0;
	M[2][1][0][3][0][3]=a/2.0;
	M[2][1][0][4][0][1]=0.5/2.0;
	M[2][1][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[2][1][0][4][0][3]=0.5/2.0;
	M[2][1][1][0][0][0]=1/2.0;
	M[2][1][1][0][1][0]=1/2.0;
	M[2][1][1][1][0][0]=1/2.0;
	M[2][1][1][1][1][1]=1/2.0;
	M[2][1][1][2][0][0]=a/2.0;
	M[2][1][1][2][0][1]=(1-a)/2.0;
	M[2][1][1][2][1][1]=(1-a)/2.0;
	M[2][1][1][2][1][2]=a/2.0;
	M[2][1][1][3][0][0]=a/2.0;
	M[2][1][1][3][0][2]=(1-a)/2.0;
	M[2][1][1][3][1][1]=(1-a)/2.0;
	M[2][1][1][3][1][3]=a/2.0;
	M[2][1][1][4][0][2]=0.5/2.0;
	M[2][1][1][4][0][3]=0.5/2.0;
	M[2][1][1][4][1][1]=0.5/2.0;
	M[2][1][1][4][1][2]=0.5/2.0;
	M[2][1][2][0][0][0]=a/2.0;
	M[2][1][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[2][1][2][0][2][0]=a/2.0;
	M[2][1][2][1][0][0]=a/2.0;
	M[2][1][2][1][1][0]=(1-a)/2.0;
	M[2][1][2][1][1][1]=(1-a)/2.0;
	M[2][1][2][1][2][1]=a/2.0;
	M[2][1][2][2][0][0]=a/2.0;
	M[2][1][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][1][2][2][2][2]=a/2.0;
	M[2][1][2][3][0][0]=a/2.0;
	M[2][1][2][3][1][1]=(1-a)/2.0;
	M[2][1][2][3][1][2]=(1-a)/2.0;
	M[2][1][2][3][2][3]=a/2.0;
	M[2][1][2][4][0][2]=a/2.0;
	M[2][1][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[2][1][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][1][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[2][1][2][4][2][2]=a/2.0;
	M[2][1][3][0][0][0]=a/2.0;
	M[2][1][3][0][1][0]=(1-a)/2.0;
	M[2][1][3][0][2][0]=(1-a)/2.0;
	M[2][1][3][0][3][0]=a/2.0;
	M[2][1][3][1][0][0]=a/2.0;
	M[2][1][3][1][1][1]=(1-a)/2.0;
	M[2][1][3][1][2][0]=(1-a)/2.0;
	M[2][1][3][1][3][1]=a/2.0;
	M[2][1][3][2][0][0]=a/2.0;
	M[2][1][3][2][1][1]=(1-a)/2.0;
	M[2][1][3][2][2][1]=(1-a)/2.0;
	M[2][1][3][2][3][2]=a/2.0;
	M[2][1][3][3][0][0]=a/2.0;
	M[2][1][3][3][1][1]=(1-a)/2.0;
	M[2][1][3][3][2][2]=(1-a)/2.0;
	M[2][1][3][3][3][3]=a/2.0;
	M[2][1][3][4][1][1]=0.5/2.0;
	M[2][1][3][4][1][2]=0.5/2.0;
	M[2][1][3][4][2][2]=0.5/2.0;
	M[2][1][3][4][2][3]=0.5/2.0;
	M[2][1][4][0][1][0]=0.5/2.0;
	M[2][1][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[2][1][4][0][3][0]=0.5/2.0;
	M[2][1][4][1][1][1]=0.5/2.0;
	M[2][1][4][1][2][0]=0.5/2.0;
	M[2][1][4][1][2][1]=0.5/2.0;
	M[2][1][4][1][3][0]=0.5/2.0;
	M[2][1][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[2][1][4][2][2][0]=a/2.0;
	M[2][1][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][1][4][2][2][2]=a/2.0;
	M[2][1][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[2][1][4][3][1][1]=0.5/2.0;
	M[2][1][4][3][2][1]=0.5/2.0;
	M[2][1][4][3][2][2]=0.5/2.0;
	M[2][1][4][3][3][2]=0.5/2.0;
	M[2][1][4][4][1][1]=0.5/2.0;
	M[2][1][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[2][1][4][4][3][3]=0.5/2.0;
	M[2][2][0][0][0][0]=1/2.0+1/2.0;
	M[2][2][0][1][0][0]=1/2.0;
	M[2][2][0][1][0][1]=1/2.0;
	M[2][2][0][2][0][0]=a/2.0;
	M[2][2][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][2][0][2][0][2]=a/2.0;
	M[2][2][0][3][0][0]=a/2.0;
	M[2][2][0][3][0][1]=(1-a)/2.0;
	M[2][2][0][3][0][2]=(1-a)/2.0;
	M[2][2][0][3][0][3]=a/2.0;
	M[2][2][0][4][0][1]=0.5/2.0;
	M[2][2][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[2][2][0][4][0][3]=0.5/2.0;
	M[2][2][1][0][0][0]=1/2.0;
	M[2][2][1][0][1][0]=1/2.0;
	M[2][2][1][1][0][0]=1/2.0;
	M[2][2][1][1][1][1]=1/2.0;
	M[2][2][1][2][0][0]=a/2.0;
	M[2][2][1][2][0][1]=(1-a)/2.0;
	M[2][2][1][2][1][1]=(1-a)/2.0;
	M[2][2][1][2][1][2]=a/2.0;
	M[2][2][1][3][0][0]=a/2.0;
	M[2][2][1][3][0][2]=(1-a)/2.0;
	M[2][2][1][3][1][1]=(1-a)/2.0;
	M[2][2][1][3][1][3]=a/2.0;
	M[2][2][1][4][0][2]=0.5/2.0;
	M[2][2][1][4][0][3]=0.5/2.0;
	M[2][2][1][4][1][1]=0.5/2.0;
	M[2][2][1][4][1][2]=0.5/2.0;
	M[2][2][2][0][0][0]=a/2.0;
	M[2][2][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[2][2][2][0][2][0]=a/2.0;
	M[2][2][2][1][0][0]=a/2.0;
	M[2][2][2][1][1][0]=(1-a)/2.0;
	M[2][2][2][1][1][1]=(1-a)/2.0;
	M[2][2][2][1][2][1]=a/2.0;
	M[2][2][2][2][0][0]=a/2.0;
	M[2][2][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][2][2][2][2][2]=a/2.0;
	M[2][2][2][3][0][0]=a/2.0;
	M[2][2][2][3][1][1]=(1-a)/2.0;
	M[2][2][2][3][1][2]=(1-a)/2.0;
	M[2][2][2][3][2][3]=a/2.0;
	M[2][2][2][4][0][2]=a/2.0;
	M[2][2][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[2][2][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][2][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[2][2][2][4][2][2]=a/2.0;
	M[2][2][3][0][0][0]=a/2.0;
	M[2][2][3][0][1][0]=(1-a)/2.0;
	M[2][2][3][0][2][0]=(1-a)/2.0;
	M[2][2][3][0][3][0]=a/2.0;
	M[2][2][3][1][0][0]=a/2.0;
	M[2][2][3][1][1][1]=(1-a)/2.0;
	M[2][2][3][1][2][0]=(1-a)/2.0;
	M[2][2][3][1][3][1]=a/2.0;
	M[2][2][3][2][0][0]=a/2.0;
	M[2][2][3][2][1][1]=(1-a)/2.0;
	M[2][2][3][2][2][1]=(1-a)/2.0;
	M[2][2][3][2][3][2]=a/2.0;
	M[2][2][3][3][0][0]=a/2.0;
	M[2][2][3][3][1][1]=(1-a)/2.0;
	M[2][2][3][3][2][2]=(1-a)/2.0;
	M[2][2][3][3][3][3]=a/2.0;
	M[2][2][3][4][1][1]=0.5/2.0;
	M[2][2][3][4][1][2]=0.5/2.0;
	M[2][2][3][4][2][2]=0.5/2.0;
	M[2][2][3][4][2][3]=0.5/2.0;
	M[2][2][4][0][1][0]=0.5/2.0;
	M[2][2][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[2][2][4][0][3][0]=0.5/2.0;
	M[2][2][4][1][1][1]=0.5/2.0;
	M[2][2][4][1][2][0]=0.5/2.0;
	M[2][2][4][1][2][1]=0.5/2.0;
	M[2][2][4][1][3][0]=0.5/2.0;
	M[2][2][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[2][2][4][2][2][0]=a/2.0;
	M[2][2][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][2][4][2][2][2]=a/2.0;
	M[2][2][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[2][2][4][3][1][1]=0.5/2.0;
	M[2][2][4][3][2][1]=0.5/2.0;
	M[2][2][4][3][2][2]=0.5/2.0;
	M[2][2][4][3][3][2]=0.5/2.0;
	M[2][2][4][4][1][1]=0.5/2.0;
	M[2][2][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[2][2][4][4][3][3]=0.5/2.0;
	M[2][3][0][0][0][0]=1/2.0+1/2.0;
	M[2][3][0][1][0][0]=1/2.0;
	M[2][3][0][1][0][1]=1/2.0;
	M[2][3][0][2][0][0]=a/2.0;
	M[2][3][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][3][0][2][0][2]=a/2.0;
	M[2][3][0][3][0][0]=a/2.0;
	M[2][3][0][3][0][1]=(1-a)/2.0;
	M[2][3][0][3][0][2]=(1-a)/2.0;
	M[2][3][0][3][0][3]=a/2.0;
	M[2][3][0][4][0][1]=0.5/2.0;
	M[2][3][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[2][3][0][4][0][3]=0.5/2.0;
	M[2][3][1][0][0][0]=1/2.0;
	M[2][3][1][0][1][0]=1/2.0;
	M[2][3][1][1][0][0]=1/2.0;
	M[2][3][1][1][1][1]=1/2.0;
	M[2][3][1][2][0][0]=a/2.0;
	M[2][3][1][2][0][1]=(1-a)/2.0;
	M[2][3][1][2][1][1]=(1-a)/2.0;
	M[2][3][1][2][1][2]=a/2.0;
	M[2][3][1][3][0][0]=a/2.0;
	M[2][3][1][3][0][2]=(1-a)/2.0;
	M[2][3][1][3][1][1]=(1-a)/2.0;
	M[2][3][1][3][1][3]=a/2.0;
	M[2][3][1][4][0][2]=0.5/2.0;
	M[2][3][1][4][0][3]=0.5/2.0;
	M[2][3][1][4][1][1]=0.5/2.0;
	M[2][3][1][4][1][2]=0.5/2.0;
	M[2][3][2][0][0][0]=a/2.0;
	M[2][3][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[2][3][2][0][2][0]=a/2.0;
	M[2][3][2][1][0][0]=a/2.0;
	M[2][3][2][1][1][0]=(1-a)/2.0;
	M[2][3][2][1][1][1]=(1-a)/2.0;
	M[2][3][2][1][2][1]=a/2.0;
	M[2][3][2][2][0][0]=a/2.0;
	M[2][3][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][3][2][2][2][2]=a/2.0;
	M[2][3][2][3][0][0]=a/2.0;
	M[2][3][2][3][1][1]=(1-a)/2.0;
	M[2][3][2][3][1][2]=(1-a)/2.0;
	M[2][3][2][3][2][3]=a/2.0;
	M[2][3][2][4][0][2]=a/2.0;
	M[2][3][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[2][3][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][3][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[2][3][2][4][2][2]=a/2.0;
	M[2][3][3][0][0][0]=a/2.0;
	M[2][3][3][0][1][0]=(1-a)/2.0;
	M[2][3][3][0][2][0]=(1-a)/2.0;
	M[2][3][3][0][3][0]=a/2.0;
	M[2][3][3][1][0][0]=a/2.0;
	M[2][3][3][1][1][1]=(1-a)/2.0;
	M[2][3][3][1][2][0]=(1-a)/2.0;
	M[2][3][3][1][3][1]=a/2.0;
	M[2][3][3][2][0][0]=a/2.0;
	M[2][3][3][2][1][1]=(1-a)/2.0;
	M[2][3][3][2][2][1]=(1-a)/2.0;
	M[2][3][3][2][3][2]=a/2.0;
	M[2][3][3][3][0][0]=a/2.0;
	M[2][3][3][3][1][1]=(1-a)/2.0;
	M[2][3][3][3][2][2]=(1-a)/2.0;
	M[2][3][3][3][3][3]=a/2.0;
	M[2][3][3][4][1][1]=0.5/2.0;
	M[2][3][3][4][1][2]=0.5/2.0;
	M[2][3][3][4][2][2]=0.5/2.0;
	M[2][3][3][4][2][3]=0.5/2.0;
	M[2][3][4][0][1][0]=0.5/2.0;
	M[2][3][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[2][3][4][0][3][0]=0.5/2.0;
	M[2][3][4][1][1][1]=0.5/2.0;
	M[2][3][4][1][2][0]=0.5/2.0;
	M[2][3][4][1][2][1]=0.5/2.0;
	M[2][3][4][1][3][0]=0.5/2.0;
	M[2][3][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[2][3][4][2][2][0]=a/2.0;
	M[2][3][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][3][4][2][2][2]=a/2.0;
	M[2][3][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[2][3][4][3][1][1]=0.5/2.0;
	M[2][3][4][3][2][1]=0.5/2.0;
	M[2][3][4][3][2][2]=0.5/2.0;
	M[2][3][4][3][3][2]=0.5/2.0;
	M[2][3][4][4][1][1]=0.5/2.0;
	M[2][3][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[2][3][4][4][3][3]=0.5/2.0;
	M[2][4][0][0][0][0]=1/2.0+1/2.0;
	M[2][4][0][1][0][0]=1/2.0;
	M[2][4][0][1][0][1]=1/2.0;
	M[2][4][0][2][0][0]=a/2.0;
	M[2][4][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][4][0][2][0][2]=a/2.0;
	M[2][4][0][3][0][0]=a/2.0;
	M[2][4][0][3][0][1]=(1-a)/2.0;
	M[2][4][0][3][0][2]=(1-a)/2.0;
	M[2][4][0][3][0][3]=a/2.0;
	M[2][4][0][4][0][1]=0.5/2.0;
	M[2][4][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[2][4][0][4][0][3]=0.5/2.0;
	M[2][4][1][0][0][0]=1/2.0;
	M[2][4][1][0][1][0]=1/2.0;
	M[2][4][1][1][0][0]=1/2.0;
	M[2][4][1][1][1][1]=1/2.0;
	M[2][4][1][2][0][0]=a/2.0;
	M[2][4][1][2][0][1]=(1-a)/2.0;
	M[2][4][1][2][1][1]=(1-a)/2.0;
	M[2][4][1][2][1][2]=a/2.0;
	M[2][4][1][3][0][0]=a/2.0;
	M[2][4][1][3][0][2]=(1-a)/2.0;
	M[2][4][1][3][1][1]=(1-a)/2.0;
	M[2][4][1][3][1][3]=a/2.0;
	M[2][4][1][4][0][2]=0.5/2.0;
	M[2][4][1][4][0][3]=0.5/2.0;
	M[2][4][1][4][1][1]=0.5/2.0;
	M[2][4][1][4][1][2]=0.5/2.0;
	M[2][4][2][0][0][0]=a/2.0;
	M[2][4][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[2][4][2][0][2][0]=a/2.0;
	M[2][4][2][1][0][0]=a/2.0;
	M[2][4][2][1][1][0]=(1-a)/2.0;
	M[2][4][2][1][1][1]=(1-a)/2.0;
	M[2][4][2][1][2][1]=a/2.0;
	M[2][4][2][2][0][0]=a/2.0;
	M[2][4][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][4][2][2][2][2]=a/2.0;
	M[2][4][2][3][0][0]=a/2.0;
	M[2][4][2][3][1][1]=(1-a)/2.0;
	M[2][4][2][3][1][2]=(1-a)/2.0;
	M[2][4][2][3][2][3]=a/2.0;
	M[2][4][2][4][0][2]=a/2.0;
	M[2][4][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[2][4][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][4][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[2][4][2][4][2][2]=a/2.0;
	M[2][4][3][0][0][0]=a/2.0;
	M[2][4][3][0][1][0]=(1-a)/2.0;
	M[2][4][3][0][2][0]=(1-a)/2.0;
	M[2][4][3][0][3][0]=a/2.0;
	M[2][4][3][1][0][0]=a/2.0;
	M[2][4][3][1][1][1]=(1-a)/2.0;
	M[2][4][3][1][2][0]=(1-a)/2.0;
	M[2][4][3][1][3][1]=a/2.0;
	M[2][4][3][2][0][0]=a/2.0;
	M[2][4][3][2][1][1]=(1-a)/2.0;
	M[2][4][3][2][2][1]=(1-a)/2.0;
	M[2][4][3][2][3][2]=a/2.0;
	M[2][4][3][3][0][0]=a/2.0;
	M[2][4][3][3][1][1]=(1-a)/2.0;
	M[2][4][3][3][2][2]=(1-a)/2.0;
	M[2][4][3][3][3][3]=a/2.0;
	M[2][4][3][4][1][1]=0.5/2.0;
	M[2][4][3][4][1][2]=0.5/2.0;
	M[2][4][3][4][2][2]=0.5/2.0;
	M[2][4][3][4][2][3]=0.5/2.0;
	M[2][4][4][0][1][0]=0.5/2.0;
	M[2][4][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[2][4][4][0][3][0]=0.5/2.0;
	M[2][4][4][1][1][1]=0.5/2.0;
	M[2][4][4][1][2][0]=0.5/2.0;
	M[2][4][4][1][2][1]=0.5/2.0;
	M[2][4][4][1][3][0]=0.5/2.0;
	M[2][4][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[2][4][4][2][2][0]=a/2.0;
	M[2][4][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][4][4][2][2][2]=a/2.0;
	M[2][4][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[2][4][4][3][1][1]=0.5/2.0;
	M[2][4][4][3][2][1]=0.5/2.0;
	M[2][4][4][3][2][2]=0.5/2.0;
	M[2][4][4][3][3][2]=0.5/2.0;
	M[2][4][4][4][1][1]=0.5/2.0;
	M[2][4][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[2][4][4][4][3][3]=0.5/2.0;
	M[3][0][0][0][0][0]=1/2.0+1/2.0;
	M[3][0][0][1][0][0]=1/2.0;
	M[3][0][0][1][0][1]=1/2.0;
	M[3][0][0][2][0][0]=a/2.0;
	M[3][0][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][0][0][2][0][2]=a/2.0;
	M[3][0][0][3][0][0]=a/2.0;
	M[3][0][0][3][0][1]=(1-a)/2.0;
	M[3][0][0][3][0][2]=(1-a)/2.0;
	M[3][0][0][3][0][3]=a/2.0;
	M[3][0][0][4][0][1]=0.5/2.0;
	M[3][0][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[3][0][0][4][0][3]=0.5/2.0;
	M[3][0][1][0][0][0]=1/2.0;
	M[3][0][1][0][1][0]=1/2.0;
	M[3][0][1][1][0][0]=1/2.0;
	M[3][0][1][1][1][1]=1/2.0;
	M[3][0][1][2][0][0]=a/2.0;
	M[3][0][1][2][0][1]=(1-a)/2.0;
	M[3][0][1][2][1][1]=(1-a)/2.0;
	M[3][0][1][2][1][2]=a/2.0;
	M[3][0][1][3][0][0]=a/2.0;
	M[3][0][1][3][0][2]=(1-a)/2.0;
	M[3][0][1][3][1][1]=(1-a)/2.0;
	M[3][0][1][3][1][3]=a/2.0;
	M[3][0][1][4][0][2]=0.5/2.0;
	M[3][0][1][4][0][3]=0.5/2.0;
	M[3][0][1][4][1][1]=0.5/2.0;
	M[3][0][1][4][1][2]=0.5/2.0;
	M[3][0][2][0][0][0]=a/2.0;
	M[3][0][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[3][0][2][0][2][0]=a/2.0;
	M[3][0][2][1][0][0]=a/2.0;
	M[3][0][2][1][1][0]=(1-a)/2.0;
	M[3][0][2][1][1][1]=(1-a)/2.0;
	M[3][0][2][1][2][1]=a/2.0;
	M[3][0][2][2][0][0]=a/2.0;
	M[3][0][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][0][2][2][2][2]=a/2.0;
	M[3][0][2][3][0][0]=a/2.0;
	M[3][0][2][3][1][1]=(1-a)/2.0;
	M[3][0][2][3][1][2]=(1-a)/2.0;
	M[3][0][2][3][2][3]=a/2.0;
	M[3][0][2][4][0][2]=a/2.0;
	M[3][0][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[3][0][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][0][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[3][0][2][4][2][2]=a/2.0;
	M[3][0][3][0][0][0]=a/2.0;
	M[3][0][3][0][1][0]=(1-a)/2.0;
	M[3][0][3][0][2][0]=(1-a)/2.0;
	M[3][0][3][0][3][0]=a/2.0;
	M[3][0][3][1][0][0]=a/2.0;
	M[3][0][3][1][1][1]=(1-a)/2.0;
	M[3][0][3][1][2][0]=(1-a)/2.0;
	M[3][0][3][1][3][1]=a/2.0;
	M[3][0][3][2][0][0]=a/2.0;
	M[3][0][3][2][1][1]=(1-a)/2.0;
	M[3][0][3][2][2][1]=(1-a)/2.0;
	M[3][0][3][2][3][2]=a/2.0;
	M[3][0][3][3][0][0]=a/2.0;
	M[3][0][3][3][1][1]=(1-a)/2.0;
	M[3][0][3][3][2][2]=(1-a)/2.0;
	M[3][0][3][3][3][3]=a/2.0;
	M[3][0][3][4][1][1]=0.5/2.0;
	M[3][0][3][4][1][2]=0.5/2.0;
	M[3][0][3][4][2][2]=0.5/2.0;
	M[3][0][3][4][2][3]=0.5/2.0;
	M[3][0][4][0][1][0]=0.5/2.0;
	M[3][0][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[3][0][4][0][3][0]=0.5/2.0;
	M[3][0][4][1][1][1]=0.5/2.0;
	M[3][0][4][1][2][0]=0.5/2.0;
	M[3][0][4][1][2][1]=0.5/2.0;
	M[3][0][4][1][3][0]=0.5/2.0;
	M[3][0][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[3][0][4][2][2][0]=a/2.0;
	M[3][0][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][0][4][2][2][2]=a/2.0;
	M[3][0][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[3][0][4][3][1][1]=0.5/2.0;
	M[3][0][4][3][2][1]=0.5/2.0;
	M[3][0][4][3][2][2]=0.5/2.0;
	M[3][0][4][3][3][2]=0.5/2.0;
	M[3][0][4][4][1][1]=0.5/2.0;
	M[3][0][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[3][0][4][4][3][3]=0.5/2.0;
	M[3][1][0][0][0][0]=1/2.0+1/2.0;
	M[3][1][0][1][0][0]=1/2.0;
	M[3][1][0][1][0][1]=1/2.0;
	M[3][1][0][2][0][0]=a/2.0;
	M[3][1][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][1][0][2][0][2]=a/2.0;
	M[3][1][0][3][0][0]=a/2.0;
	M[3][1][0][3][0][1]=(1-a)/2.0;
	M[3][1][0][3][0][2]=(1-a)/2.0;
	M[3][1][0][3][0][3]=a/2.0;
	M[3][1][0][4][0][1]=0.5/2.0;
	M[3][1][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[3][1][0][4][0][3]=0.5/2.0;
	M[3][1][1][0][0][0]=1/2.0;
	M[3][1][1][0][1][0]=1/2.0;
	M[3][1][1][1][0][0]=1/2.0;
	M[3][1][1][1][1][1]=1/2.0;
	M[3][1][1][2][0][0]=a/2.0;
	M[3][1][1][2][0][1]=(1-a)/2.0;
	M[3][1][1][2][1][1]=(1-a)/2.0;
	M[3][1][1][2][1][2]=a/2.0;
	M[3][1][1][3][0][0]=a/2.0;
	M[3][1][1][3][0][2]=(1-a)/2.0;
	M[3][1][1][3][1][1]=(1-a)/2.0;
	M[3][1][1][3][1][3]=a/2.0;
	M[3][1][1][4][0][2]=0.5/2.0;
	M[3][1][1][4][0][3]=0.5/2.0;
	M[3][1][1][4][1][1]=0.5/2.0;
	M[3][1][1][4][1][2]=0.5/2.0;
	M[3][1][2][0][0][0]=a/2.0;
	M[3][1][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[3][1][2][0][2][0]=a/2.0;
	M[3][1][2][1][0][0]=a/2.0;
	M[3][1][2][1][1][0]=(1-a)/2.0;
	M[3][1][2][1][1][1]=(1-a)/2.0;
	M[3][1][2][1][2][1]=a/2.0;
	M[3][1][2][2][0][0]=a/2.0;
	M[3][1][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][1][2][2][2][2]=a/2.0;
	M[3][1][2][3][0][0]=a/2.0;
	M[3][1][2][3][1][1]=(1-a)/2.0;
	M[3][1][2][3][1][2]=(1-a)/2.0;
	M[3][1][2][3][2][3]=a/2.0;
	M[3][1][2][4][0][2]=a/2.0;
	M[3][1][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[3][1][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][1][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[3][1][2][4][2][2]=a/2.0;
	M[3][1][3][0][0][0]=a/2.0;
	M[3][1][3][0][1][0]=(1-a)/2.0;
	M[3][1][3][0][2][0]=(1-a)/2.0;
	M[3][1][3][0][3][0]=a/2.0;
	M[3][1][3][1][0][0]=a/2.0;
	M[3][1][3][1][1][1]=(1-a)/2.0;
	M[3][1][3][1][2][0]=(1-a)/2.0;
	M[3][1][3][1][3][1]=a/2.0;
	M[3][1][3][2][0][0]=a/2.0;
	M[3][1][3][2][1][1]=(1-a)/2.0;
	M[3][1][3][2][2][1]=(1-a)/2.0;
	M[3][1][3][2][3][2]=a/2.0;
	M[3][1][3][3][0][0]=a/2.0;
	M[3][1][3][3][1][1]=(1-a)/2.0;
	M[3][1][3][3][2][2]=(1-a)/2.0;
	M[3][1][3][3][3][3]=a/2.0;
	M[3][1][3][4][1][1]=0.5/2.0;
	M[3][1][3][4][1][2]=0.5/2.0;
	M[3][1][3][4][2][2]=0.5/2.0;
	M[3][1][3][4][2][3]=0.5/2.0;
	M[3][1][4][0][1][0]=0.5/2.0;
	M[3][1][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[3][1][4][0][3][0]=0.5/2.0;
	M[3][1][4][1][1][1]=0.5/2.0;
	M[3][1][4][1][2][0]=0.5/2.0;
	M[3][1][4][1][2][1]=0.5/2.0;
	M[3][1][4][1][3][0]=0.5/2.0;
	M[3][1][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[3][1][4][2][2][0]=a/2.0;
	M[3][1][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][1][4][2][2][2]=a/2.0;
	M[3][1][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[3][1][4][3][1][1]=0.5/2.0;
	M[3][1][4][3][2][1]=0.5/2.0;
	M[3][1][4][3][2][2]=0.5/2.0;
	M[3][1][4][3][3][2]=0.5/2.0;
	M[3][1][4][4][1][1]=0.5/2.0;
	M[3][1][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[3][1][4][4][3][3]=0.5/2.0;
	M[3][2][0][0][0][0]=1/2.0+1/2.0;
	M[3][2][0][1][0][0]=1/2.0;
	M[3][2][0][1][0][1]=1/2.0;
	M[3][2][0][2][0][0]=a/2.0;
	M[3][2][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][2][0][2][0][2]=a/2.0;
	M[3][2][0][3][0][0]=a/2.0;
	M[3][2][0][3][0][1]=(1-a)/2.0;
	M[3][2][0][3][0][2]=(1-a)/2.0;
	M[3][2][0][3][0][3]=a/2.0;
	M[3][2][0][4][0][1]=0.5/2.0;
	M[3][2][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[3][2][0][4][0][3]=0.5/2.0;
	M[3][2][1][0][0][0]=1/2.0;
	M[3][2][1][0][1][0]=1/2.0;
	M[3][2][1][1][0][0]=1/2.0;
	M[3][2][1][1][1][1]=1/2.0;
	M[3][2][1][2][0][0]=a/2.0;
	M[3][2][1][2][0][1]=(1-a)/2.0;
	M[3][2][1][2][1][1]=(1-a)/2.0;
	M[3][2][1][2][1][2]=a/2.0;
	M[3][2][1][3][0][0]=a/2.0;
	M[3][2][1][3][0][2]=(1-a)/2.0;
	M[3][2][1][3][1][1]=(1-a)/2.0;
	M[3][2][1][3][1][3]=a/2.0;
	M[3][2][1][4][0][2]=0.5/2.0;
	M[3][2][1][4][0][3]=0.5/2.0;
	M[3][2][1][4][1][1]=0.5/2.0;
	M[3][2][1][4][1][2]=0.5/2.0;
	M[3][2][2][0][0][0]=a/2.0;
	M[3][2][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[3][2][2][0][2][0]=a/2.0;
	M[3][2][2][1][0][0]=a/2.0;
	M[3][2][2][1][1][0]=(1-a)/2.0;
	M[3][2][2][1][1][1]=(1-a)/2.0;
	M[3][2][2][1][2][1]=a/2.0;
	M[3][2][2][2][0][0]=a/2.0;
	M[3][2][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][2][2][2][2][2]=a/2.0;
	M[3][2][2][3][0][0]=a/2.0;
	M[3][2][2][3][1][1]=(1-a)/2.0;
	M[3][2][2][3][1][2]=(1-a)/2.0;
	M[3][2][2][3][2][3]=a/2.0;
	M[3][2][2][4][0][2]=a/2.0;
	M[3][2][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[3][2][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][2][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[3][2][2][4][2][2]=a/2.0;
	M[3][2][3][0][0][0]=a/2.0;
	M[3][2][3][0][1][0]=(1-a)/2.0;
	M[3][2][3][0][2][0]=(1-a)/2.0;
	M[3][2][3][0][3][0]=a/2.0;
	M[3][2][3][1][0][0]=a/2.0;
	M[3][2][3][1][1][1]=(1-a)/2.0;
	M[3][2][3][1][2][0]=(1-a)/2.0;
	M[3][2][3][1][3][1]=a/2.0;
	M[3][2][3][2][0][0]=a/2.0;
	M[3][2][3][2][1][1]=(1-a)/2.0;
	M[3][2][3][2][2][1]=(1-a)/2.0;
	M[3][2][3][2][3][2]=a/2.0;
	M[3][2][3][3][0][0]=a/2.0;
	M[3][2][3][3][1][1]=(1-a)/2.0;
	M[3][2][3][3][2][2]=(1-a)/2.0;
	M[3][2][3][3][3][3]=a/2.0;
	M[3][2][3][4][1][1]=0.5/2.0;
	M[3][2][3][4][1][2]=0.5/2.0;
	M[3][2][3][4][2][2]=0.5/2.0;
	M[3][2][3][4][2][3]=0.5/2.0;
	M[3][2][4][0][1][0]=0.5/2.0;
	M[3][2][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[3][2][4][0][3][0]=0.5/2.0;
	M[3][2][4][1][1][1]=0.5/2.0;
	M[3][2][4][1][2][0]=0.5/2.0;
	M[3][2][4][1][2][1]=0.5/2.0;
	M[3][2][4][1][3][0]=0.5/2.0;
	M[3][2][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[3][2][4][2][2][0]=a/2.0;
	M[3][2][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][2][4][2][2][2]=a/2.0;
	M[3][2][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[3][2][4][3][1][1]=0.5/2.0;
	M[3][2][4][3][2][1]=0.5/2.0;
	M[3][2][4][3][2][2]=0.5/2.0;
	M[3][2][4][3][3][2]=0.5/2.0;
	M[3][2][4][4][1][1]=0.5/2.0;
	M[3][2][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[3][2][4][4][3][3]=0.5/2.0;
	M[3][3][0][0][0][0]=1/2.0+1/2.0;
	M[3][3][0][1][0][0]=1/2.0;
	M[3][3][0][1][0][1]=1/2.0;
	M[3][3][0][2][0][0]=a/2.0;
	M[3][3][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][3][0][2][0][2]=a/2.0;
	M[3][3][0][3][0][0]=a/2.0;
	M[3][3][0][3][0][1]=(1-a)/2.0;
	M[3][3][0][3][0][2]=(1-a)/2.0;
	M[3][3][0][3][0][3]=a/2.0;
	M[3][3][0][4][0][1]=0.5/2.0;
	M[3][3][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[3][3][0][4][0][3]=0.5/2.0;
	M[3][3][1][0][0][0]=1/2.0;
	M[3][3][1][0][1][0]=1/2.0;
	M[3][3][1][1][0][0]=1/2.0;
	M[3][3][1][1][1][1]=1/2.0;
	M[3][3][1][2][0][0]=a/2.0;
	M[3][3][1][2][0][1]=(1-a)/2.0;
	M[3][3][1][2][1][1]=(1-a)/2.0;
	M[3][3][1][2][1][2]=a/2.0;
	M[3][3][1][3][0][0]=a/2.0;
	M[3][3][1][3][0][2]=(1-a)/2.0;
	M[3][3][1][3][1][1]=(1-a)/2.0;
	M[3][3][1][3][1][3]=a/2.0;
	M[3][3][1][4][0][2]=0.5/2.0;
	M[3][3][1][4][0][3]=0.5/2.0;
	M[3][3][1][4][1][1]=0.5/2.0;
	M[3][3][1][4][1][2]=0.5/2.0;
	M[3][3][2][0][0][0]=a/2.0;
	M[3][3][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[3][3][2][0][2][0]=a/2.0;
	M[3][3][2][1][0][0]=a/2.0;
	M[3][3][2][1][1][0]=(1-a)/2.0;
	M[3][3][2][1][1][1]=(1-a)/2.0;
	M[3][3][2][1][2][1]=a/2.0;
	M[3][3][2][2][0][0]=a/2.0;
	M[3][3][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][3][2][2][2][2]=a/2.0;
	M[3][3][2][3][0][0]=a/2.0;
	M[3][3][2][3][1][1]=(1-a)/2.0;
	M[3][3][2][3][1][2]=(1-a)/2.0;
	M[3][3][2][3][2][3]=a/2.0;
	M[3][3][2][4][0][2]=a/2.0;
	M[3][3][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[3][3][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][3][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[3][3][2][4][2][2]=a/2.0;
	M[3][3][3][0][0][0]=a/2.0;
	M[3][3][3][0][1][0]=(1-a)/2.0;
	M[3][3][3][0][2][0]=(1-a)/2.0;
	M[3][3][3][0][3][0]=a/2.0;
	M[3][3][3][1][0][0]=a/2.0;
	M[3][3][3][1][1][1]=(1-a)/2.0;
	M[3][3][3][1][2][0]=(1-a)/2.0;
	M[3][3][3][1][3][1]=a/2.0;
	M[3][3][3][2][0][0]=a/2.0;
	M[3][3][3][2][1][1]=(1-a)/2.0;
	M[3][3][3][2][2][1]=(1-a)/2.0;
	M[3][3][3][2][3][2]=a/2.0;
	M[3][3][3][3][0][0]=a/2.0;
	M[3][3][3][3][1][1]=(1-a)/2.0;
	M[3][3][3][3][2][2]=(1-a)/2.0;
	M[3][3][3][3][3][3]=a/2.0;
	M[3][3][3][4][1][1]=0.5/2.0;
	M[3][3][3][4][1][2]=0.5/2.0;
	M[3][3][3][4][2][2]=0.5/2.0;
	M[3][3][3][4][2][3]=0.5/2.0;
	M[3][3][4][0][1][0]=0.5/2.0;
	M[3][3][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[3][3][4][0][3][0]=0.5/2.0;
	M[3][3][4][1][1][1]=0.5/2.0;
	M[3][3][4][1][2][0]=0.5/2.0;
	M[3][3][4][1][2][1]=0.5/2.0;
	M[3][3][4][1][3][0]=0.5/2.0;
	M[3][3][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[3][3][4][2][2][0]=a/2.0;
	M[3][3][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][3][4][2][2][2]=a/2.0;
	M[3][3][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[3][3][4][3][1][1]=0.5/2.0;
	M[3][3][4][3][2][1]=0.5/2.0;
	M[3][3][4][3][2][2]=0.5/2.0;
	M[3][3][4][3][3][2]=0.5/2.0;
	M[3][3][4][4][1][1]=0.5/2.0;
	M[3][3][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[3][3][4][4][3][3]=0.5/2.0;
	M[3][4][0][0][0][0]=1/2.0+1/2.0;
	M[3][4][0][1][0][0]=1/2.0;
	M[3][4][0][1][0][1]=1/2.0;
	M[3][4][0][2][0][0]=a/2.0;
	M[3][4][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][4][0][2][0][2]=a/2.0;
	M[3][4][0][3][0][0]=a/2.0;
	M[3][4][0][3][0][1]=(1-a)/2.0;
	M[3][4][0][3][0][2]=(1-a)/2.0;
	M[3][4][0][3][0][3]=a/2.0;
	M[3][4][0][4][0][1]=0.5/2.0;
	M[3][4][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[3][4][0][4][0][3]=0.5/2.0;
	M[3][4][1][0][0][0]=1/2.0;
	M[3][4][1][0][1][0]=1/2.0;
	M[3][4][1][1][0][0]=1/2.0;
	M[3][4][1][1][1][1]=1/2.0;
	M[3][4][1][2][0][0]=a/2.0;
	M[3][4][1][2][0][1]=(1-a)/2.0;
	M[3][4][1][2][1][1]=(1-a)/2.0;
	M[3][4][1][2][1][2]=a/2.0;
	M[3][4][1][3][0][0]=a/2.0;
	M[3][4][1][3][0][2]=(1-a)/2.0;
	M[3][4][1][3][1][1]=(1-a)/2.0;
	M[3][4][1][3][1][3]=a/2.0;
	M[3][4][1][4][0][2]=0.5/2.0;
	M[3][4][1][4][0][3]=0.5/2.0;
	M[3][4][1][4][1][1]=0.5/2.0;
	M[3][4][1][4][1][2]=0.5/2.0;
	M[3][4][2][0][0][0]=a/2.0;
	M[3][4][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[3][4][2][0][2][0]=a/2.0;
	M[3][4][2][1][0][0]=a/2.0;
	M[3][4][2][1][1][0]=(1-a)/2.0;
	M[3][4][2][1][1][1]=(1-a)/2.0;
	M[3][4][2][1][2][1]=a/2.0;
	M[3][4][2][2][0][0]=a/2.0;
	M[3][4][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][4][2][2][2][2]=a/2.0;
	M[3][4][2][3][0][0]=a/2.0;
	M[3][4][2][3][1][1]=(1-a)/2.0;
	M[3][4][2][3][1][2]=(1-a)/2.0;
	M[3][4][2][3][2][3]=a/2.0;
	M[3][4][2][4][0][2]=a/2.0;
	M[3][4][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[3][4][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][4][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[3][4][2][4][2][2]=a/2.0;
	M[3][4][3][0][0][0]=a/2.0;
	M[3][4][3][0][1][0]=(1-a)/2.0;
	M[3][4][3][0][2][0]=(1-a)/2.0;
	M[3][4][3][0][3][0]=a/2.0;
	M[3][4][3][1][0][0]=a/2.0;
	M[3][4][3][1][1][1]=(1-a)/2.0;
	M[3][4][3][1][2][0]=(1-a)/2.0;
	M[3][4][3][1][3][1]=a/2.0;
	M[3][4][3][2][0][0]=a/2.0;
	M[3][4][3][2][1][1]=(1-a)/2.0;
	M[3][4][3][2][2][1]=(1-a)/2.0;
	M[3][4][3][2][3][2]=a/2.0;
	M[3][4][3][3][0][0]=a/2.0;
	M[3][4][3][3][1][1]=(1-a)/2.0;
	M[3][4][3][3][2][2]=(1-a)/2.0;
	M[3][4][3][3][3][3]=a/2.0;
	M[3][4][3][4][1][1]=0.5/2.0;
	M[3][4][3][4][1][2]=0.5/2.0;
	M[3][4][3][4][2][2]=0.5/2.0;
	M[3][4][3][4][2][3]=0.5/2.0;
	M[3][4][4][0][1][0]=0.5/2.0;
	M[3][4][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[3][4][4][0][3][0]=0.5/2.0;
	M[3][4][4][1][1][1]=0.5/2.0;
	M[3][4][4][1][2][0]=0.5/2.0;
	M[3][4][4][1][2][1]=0.5/2.0;
	M[3][4][4][1][3][0]=0.5/2.0;
	M[3][4][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[3][4][4][2][2][0]=a/2.0;
	M[3][4][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][4][4][2][2][2]=a/2.0;
	M[3][4][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[3][4][4][3][1][1]=0.5/2.0;
	M[3][4][4][3][2][1]=0.5/2.0;
	M[3][4][4][3][2][2]=0.5/2.0;
	M[3][4][4][3][3][2]=0.5/2.0;
	M[3][4][4][4][1][1]=0.5/2.0;
	M[3][4][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[3][4][4][4][3][3]=0.5/2.0;
	M[4][0][0][0][0][0]=1/2.0+1/2.0;
	M[4][0][0][1][0][0]=1/2.0;
	M[4][0][0][1][0][1]=1/2.0;
	M[4][0][0][2][0][0]=a/2.0;
	M[4][0][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][0][0][2][0][2]=a/2.0;
	M[4][0][0][3][0][0]=a/2.0;
	M[4][0][0][3][0][1]=(1-a)/2.0;
	M[4][0][0][3][0][2]=(1-a)/2.0;
	M[4][0][0][3][0][3]=a/2.0;
	M[4][0][0][4][0][1]=0.5/2.0;
	M[4][0][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[4][0][0][4][0][3]=0.5/2.0;
	M[4][0][1][0][0][0]=1/2.0;
	M[4][0][1][0][1][0]=1/2.0;
	M[4][0][1][1][0][0]=1/2.0;
	M[4][0][1][1][1][1]=1/2.0;
	M[4][0][1][2][0][0]=a/2.0;
	M[4][0][1][2][0][1]=(1-a)/2.0;
	M[4][0][1][2][1][1]=(1-a)/2.0;
	M[4][0][1][2][1][2]=a/2.0;
	M[4][0][1][3][0][0]=a/2.0;
	M[4][0][1][3][0][2]=(1-a)/2.0;
	M[4][0][1][3][1][1]=(1-a)/2.0;
	M[4][0][1][3][1][3]=a/2.0;
	M[4][0][1][4][0][2]=0.5/2.0;
	M[4][0][1][4][0][3]=0.5/2.0;
	M[4][0][1][4][1][1]=0.5/2.0;
	M[4][0][1][4][1][2]=0.5/2.0;
	M[4][0][2][0][0][0]=a/2.0;
	M[4][0][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[4][0][2][0][2][0]=a/2.0;
	M[4][0][2][1][0][0]=a/2.0;
	M[4][0][2][1][1][0]=(1-a)/2.0;
	M[4][0][2][1][1][1]=(1-a)/2.0;
	M[4][0][2][1][2][1]=a/2.0;
	M[4][0][2][2][0][0]=a/2.0;
	M[4][0][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][0][2][2][2][2]=a/2.0;
	M[4][0][2][3][0][0]=a/2.0;
	M[4][0][2][3][1][1]=(1-a)/2.0;
	M[4][0][2][3][1][2]=(1-a)/2.0;
	M[4][0][2][3][2][3]=a/2.0;
	M[4][0][2][4][0][2]=a/2.0;
	M[4][0][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[4][0][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][0][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[4][0][2][4][2][2]=a/2.0;
	M[4][0][3][0][0][0]=a/2.0;
	M[4][0][3][0][1][0]=(1-a)/2.0;
	M[4][0][3][0][2][0]=(1-a)/2.0;
	M[4][0][3][0][3][0]=a/2.0;
	M[4][0][3][1][0][0]=a/2.0;
	M[4][0][3][1][1][1]=(1-a)/2.0;
	M[4][0][3][1][2][0]=(1-a)/2.0;
	M[4][0][3][1][3][1]=a/2.0;
	M[4][0][3][2][0][0]=a/2.0;
	M[4][0][3][2][1][1]=(1-a)/2.0;
	M[4][0][3][2][2][1]=(1-a)/2.0;
	M[4][0][3][2][3][2]=a/2.0;
	M[4][0][3][3][0][0]=a/2.0;
	M[4][0][3][3][1][1]=(1-a)/2.0;
	M[4][0][3][3][2][2]=(1-a)/2.0;
	M[4][0][3][3][3][3]=a/2.0;
	M[4][0][3][4][1][1]=0.5/2.0;
	M[4][0][3][4][1][2]=0.5/2.0;
	M[4][0][3][4][2][2]=0.5/2.0;
	M[4][0][3][4][2][3]=0.5/2.0;
	M[4][0][4][0][1][0]=0.5/2.0;
	M[4][0][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[4][0][4][0][3][0]=0.5/2.0;
	M[4][0][4][1][1][1]=0.5/2.0;
	M[4][0][4][1][2][0]=0.5/2.0;
	M[4][0][4][1][2][1]=0.5/2.0;
	M[4][0][4][1][3][0]=0.5/2.0;
	M[4][0][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[4][0][4][2][2][0]=a/2.0;
	M[4][0][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][0][4][2][2][2]=a/2.0;
	M[4][0][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[4][0][4][3][1][1]=0.5/2.0;
	M[4][0][4][3][2][1]=0.5/2.0;
	M[4][0][4][3][2][2]=0.5/2.0;
	M[4][0][4][3][3][2]=0.5/2.0;
	M[4][0][4][4][1][1]=0.5/2.0;
	M[4][0][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[4][0][4][4][3][3]=0.5/2.0;
	M[4][1][0][0][0][0]=1/2.0+1/2.0;
	M[4][1][0][1][0][0]=1/2.0;
	M[4][1][0][1][0][1]=1/2.0;
	M[4][1][0][2][0][0]=a/2.0;
	M[4][1][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][1][0][2][0][2]=a/2.0;
	M[4][1][0][3][0][0]=a/2.0;
	M[4][1][0][3][0][1]=(1-a)/2.0;
	M[4][1][0][3][0][2]=(1-a)/2.0;
	M[4][1][0][3][0][3]=a/2.0;
	M[4][1][0][4][0][1]=0.5/2.0;
	M[4][1][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[4][1][0][4][0][3]=0.5/2.0;
	M[4][1][1][0][0][0]=1/2.0;
	M[4][1][1][0][1][0]=1/2.0;
	M[4][1][1][1][0][0]=1/2.0;
	M[4][1][1][1][1][1]=1/2.0;
	M[4][1][1][2][0][0]=a/2.0;
	M[4][1][1][2][0][1]=(1-a)/2.0;
	M[4][1][1][2][1][1]=(1-a)/2.0;
	M[4][1][1][2][1][2]=a/2.0;
	M[4][1][1][3][0][0]=a/2.0;
	M[4][1][1][3][0][2]=(1-a)/2.0;
	M[4][1][1][3][1][1]=(1-a)/2.0;
	M[4][1][1][3][1][3]=a/2.0;
	M[4][1][1][4][0][2]=0.5/2.0;
	M[4][1][1][4][0][3]=0.5/2.0;
	M[4][1][1][4][1][1]=0.5/2.0;
	M[4][1][1][4][1][2]=0.5/2.0;
	M[4][1][2][0][0][0]=a/2.0;
	M[4][1][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[4][1][2][0][2][0]=a/2.0;
	M[4][1][2][1][0][0]=a/2.0;
	M[4][1][2][1][1][0]=(1-a)/2.0;
	M[4][1][2][1][1][1]=(1-a)/2.0;
	M[4][1][2][1][2][1]=a/2.0;
	M[4][1][2][2][0][0]=a/2.0;
	M[4][1][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][1][2][2][2][2]=a/2.0;
	M[4][1][2][3][0][0]=a/2.0;
	M[4][1][2][3][1][1]=(1-a)/2.0;
	M[4][1][2][3][1][2]=(1-a)/2.0;
	M[4][1][2][3][2][3]=a/2.0;
	M[4][1][2][4][0][2]=a/2.0;
	M[4][1][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[4][1][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][1][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[4][1][2][4][2][2]=a/2.0;
	M[4][1][3][0][0][0]=a/2.0;
	M[4][1][3][0][1][0]=(1-a)/2.0;
	M[4][1][3][0][2][0]=(1-a)/2.0;
	M[4][1][3][0][3][0]=a/2.0;
	M[4][1][3][1][0][0]=a/2.0;
	M[4][1][3][1][1][1]=(1-a)/2.0;
	M[4][1][3][1][2][0]=(1-a)/2.0;
	M[4][1][3][1][3][1]=a/2.0;
	M[4][1][3][2][0][0]=a/2.0;
	M[4][1][3][2][1][1]=(1-a)/2.0;
	M[4][1][3][2][2][1]=(1-a)/2.0;
	M[4][1][3][2][3][2]=a/2.0;
	M[4][1][3][3][0][0]=a/2.0;
	M[4][1][3][3][1][1]=(1-a)/2.0;
	M[4][1][3][3][2][2]=(1-a)/2.0;
	M[4][1][3][3][3][3]=a/2.0;
	M[4][1][3][4][1][1]=0.5/2.0;
	M[4][1][3][4][1][2]=0.5/2.0;
	M[4][1][3][4][2][2]=0.5/2.0;
	M[4][1][3][4][2][3]=0.5/2.0;
	M[4][1][4][0][1][0]=0.5/2.0;
	M[4][1][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[4][1][4][0][3][0]=0.5/2.0;
	M[4][1][4][1][1][1]=0.5/2.0;
	M[4][1][4][1][2][0]=0.5/2.0;
	M[4][1][4][1][2][1]=0.5/2.0;
	M[4][1][4][1][3][0]=0.5/2.0;
	M[4][1][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[4][1][4][2][2][0]=a/2.0;
	M[4][1][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][1][4][2][2][2]=a/2.0;
	M[4][1][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[4][1][4][3][1][1]=0.5/2.0;
	M[4][1][4][3][2][1]=0.5/2.0;
	M[4][1][4][3][2][2]=0.5/2.0;
	M[4][1][4][3][3][2]=0.5/2.0;
	M[4][1][4][4][1][1]=0.5/2.0;
	M[4][1][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[4][1][4][4][3][3]=0.5/2.0;
	M[4][2][0][0][0][0]=1/2.0+1/2.0;
	M[4][2][0][1][0][0]=1/2.0;
	M[4][2][0][1][0][1]=1/2.0;
	M[4][2][0][2][0][0]=a/2.0;
	M[4][2][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][2][0][2][0][2]=a/2.0;
	M[4][2][0][3][0][0]=a/2.0;
	M[4][2][0][3][0][1]=(1-a)/2.0;
	M[4][2][0][3][0][2]=(1-a)/2.0;
	M[4][2][0][3][0][3]=a/2.0;
	M[4][2][0][4][0][1]=0.5/2.0;
	M[4][2][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[4][2][0][4][0][3]=0.5/2.0;
	M[4][2][1][0][0][0]=1/2.0;
	M[4][2][1][0][1][0]=1/2.0;
	M[4][2][1][1][0][0]=1/2.0;
	M[4][2][1][1][1][1]=1/2.0;
	M[4][2][1][2][0][0]=a/2.0;
	M[4][2][1][2][0][1]=(1-a)/2.0;
	M[4][2][1][2][1][1]=(1-a)/2.0;
	M[4][2][1][2][1][2]=a/2.0;
	M[4][2][1][3][0][0]=a/2.0;
	M[4][2][1][3][0][2]=(1-a)/2.0;
	M[4][2][1][3][1][1]=(1-a)/2.0;
	M[4][2][1][3][1][3]=a/2.0;
	M[4][2][1][4][0][2]=0.5/2.0;
	M[4][2][1][4][0][3]=0.5/2.0;
	M[4][2][1][4][1][1]=0.5/2.0;
	M[4][2][1][4][1][2]=0.5/2.0;
	M[4][2][2][0][0][0]=a/2.0;
	M[4][2][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[4][2][2][0][2][0]=a/2.0;
	M[4][2][2][1][0][0]=a/2.0;
	M[4][2][2][1][1][0]=(1-a)/2.0;
	M[4][2][2][1][1][1]=(1-a)/2.0;
	M[4][2][2][1][2][1]=a/2.0;
	M[4][2][2][2][0][0]=a/2.0;
	M[4][2][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][2][2][2][2][2]=a/2.0;
	M[4][2][2][3][0][0]=a/2.0;
	M[4][2][2][3][1][1]=(1-a)/2.0;
	M[4][2][2][3][1][2]=(1-a)/2.0;
	M[4][2][2][3][2][3]=a/2.0;
	M[4][2][2][4][0][2]=a/2.0;
	M[4][2][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[4][2][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][2][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[4][2][2][4][2][2]=a/2.0;
	M[4][2][3][0][0][0]=a/2.0;
	M[4][2][3][0][1][0]=(1-a)/2.0;
	M[4][2][3][0][2][0]=(1-a)/2.0;
	M[4][2][3][0][3][0]=a/2.0;
	M[4][2][3][1][0][0]=a/2.0;
	M[4][2][3][1][1][1]=(1-a)/2.0;
	M[4][2][3][1][2][0]=(1-a)/2.0;
	M[4][2][3][1][3][1]=a/2.0;
	M[4][2][3][2][0][0]=a/2.0;
	M[4][2][3][2][1][1]=(1-a)/2.0;
	M[4][2][3][2][2][1]=(1-a)/2.0;
	M[4][2][3][2][3][2]=a/2.0;
	M[4][2][3][3][0][0]=a/2.0;
	M[4][2][3][3][1][1]=(1-a)/2.0;
	M[4][2][3][3][2][2]=(1-a)/2.0;
	M[4][2][3][3][3][3]=a/2.0;
	M[4][2][3][4][1][1]=0.5/2.0;
	M[4][2][3][4][1][2]=0.5/2.0;
	M[4][2][3][4][2][2]=0.5/2.0;
	M[4][2][3][4][2][3]=0.5/2.0;
	M[4][2][4][0][1][0]=0.5/2.0;
	M[4][2][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[4][2][4][0][3][0]=0.5/2.0;
	M[4][2][4][1][1][1]=0.5/2.0;
	M[4][2][4][1][2][0]=0.5/2.0;
	M[4][2][4][1][2][1]=0.5/2.0;
	M[4][2][4][1][3][0]=0.5/2.0;
	M[4][2][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[4][2][4][2][2][0]=a/2.0;
	M[4][2][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][2][4][2][2][2]=a/2.0;
	M[4][2][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[4][2][4][3][1][1]=0.5/2.0;
	M[4][2][4][3][2][1]=0.5/2.0;
	M[4][2][4][3][2][2]=0.5/2.0;
	M[4][2][4][3][3][2]=0.5/2.0;
	M[4][2][4][4][1][1]=0.5/2.0;
	M[4][2][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[4][2][4][4][3][3]=0.5/2.0;
	M[4][3][0][0][0][0]=1/2.0+1/2.0;
	M[4][3][0][1][0][0]=1/2.0;
	M[4][3][0][1][0][1]=1/2.0;
	M[4][3][0][2][0][0]=a/2.0;
	M[4][3][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][3][0][2][0][2]=a/2.0;
	M[4][3][0][3][0][0]=a/2.0;
	M[4][3][0][3][0][1]=(1-a)/2.0;
	M[4][3][0][3][0][2]=(1-a)/2.0;
	M[4][3][0][3][0][3]=a/2.0;
	M[4][3][0][4][0][1]=0.5/2.0;
	M[4][3][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[4][3][0][4][0][3]=0.5/2.0;
	M[4][3][1][0][0][0]=1/2.0;
	M[4][3][1][0][1][0]=1/2.0;
	M[4][3][1][1][0][0]=1/2.0;
	M[4][3][1][1][1][1]=1/2.0;
	M[4][3][1][2][0][0]=a/2.0;
	M[4][3][1][2][0][1]=(1-a)/2.0;
	M[4][3][1][2][1][1]=(1-a)/2.0;
	M[4][3][1][2][1][2]=a/2.0;
	M[4][3][1][3][0][0]=a/2.0;
	M[4][3][1][3][0][2]=(1-a)/2.0;
	M[4][3][1][3][1][1]=(1-a)/2.0;
	M[4][3][1][3][1][3]=a/2.0;
	M[4][3][1][4][0][2]=0.5/2.0;
	M[4][3][1][4][0][3]=0.5/2.0;
	M[4][3][1][4][1][1]=0.5/2.0;
	M[4][3][1][4][1][2]=0.5/2.0;
	M[4][3][2][0][0][0]=a/2.0;
	M[4][3][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[4][3][2][0][2][0]=a/2.0;
	M[4][3][2][1][0][0]=a/2.0;
	M[4][3][2][1][1][0]=(1-a)/2.0;
	M[4][3][2][1][1][1]=(1-a)/2.0;
	M[4][3][2][1][2][1]=a/2.0;
	M[4][3][2][2][0][0]=a/2.0;
	M[4][3][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][3][2][2][2][2]=a/2.0;
	M[4][3][2][3][0][0]=a/2.0;
	M[4][3][2][3][1][1]=(1-a)/2.0;
	M[4][3][2][3][1][2]=(1-a)/2.0;
	M[4][3][2][3][2][3]=a/2.0;
	M[4][3][2][4][0][2]=a/2.0;
	M[4][3][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[4][3][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][3][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[4][3][2][4][2][2]=a/2.0;
	M[4][3][3][0][0][0]=a/2.0;
	M[4][3][3][0][1][0]=(1-a)/2.0;
	M[4][3][3][0][2][0]=(1-a)/2.0;
	M[4][3][3][0][3][0]=a/2.0;
	M[4][3][3][1][0][0]=a/2.0;
	M[4][3][3][1][1][1]=(1-a)/2.0;
	M[4][3][3][1][2][0]=(1-a)/2.0;
	M[4][3][3][1][3][1]=a/2.0;
	M[4][3][3][2][0][0]=a/2.0;
	M[4][3][3][2][1][1]=(1-a)/2.0;
	M[4][3][3][2][2][1]=(1-a)/2.0;
	M[4][3][3][2][3][2]=a/2.0;
	M[4][3][3][3][0][0]=a/2.0;
	M[4][3][3][3][1][1]=(1-a)/2.0;
	M[4][3][3][3][2][2]=(1-a)/2.0;
	M[4][3][3][3][3][3]=a/2.0;
	M[4][3][3][4][1][1]=0.5/2.0;
	M[4][3][3][4][1][2]=0.5/2.0;
	M[4][3][3][4][2][2]=0.5/2.0;
	M[4][3][3][4][2][3]=0.5/2.0;
	M[4][3][4][0][1][0]=0.5/2.0;
	M[4][3][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[4][3][4][0][3][0]=0.5/2.0;
	M[4][3][4][1][1][1]=0.5/2.0;
	M[4][3][4][1][2][0]=0.5/2.0;
	M[4][3][4][1][2][1]=0.5/2.0;
	M[4][3][4][1][3][0]=0.5/2.0;
	M[4][3][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[4][3][4][2][2][0]=a/2.0;
	M[4][3][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][3][4][2][2][2]=a/2.0;
	M[4][3][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[4][3][4][3][1][1]=0.5/2.0;
	M[4][3][4][3][2][1]=0.5/2.0;
	M[4][3][4][3][2][2]=0.5/2.0;
	M[4][3][4][3][3][2]=0.5/2.0;
	M[4][3][4][4][1][1]=0.5/2.0;
	M[4][3][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[4][3][4][4][3][3]=0.5/2.0;
	M[4][4][0][0][0][0]=1/2.0+1/2.0;
	M[4][4][0][1][0][0]=1/2.0;
	M[4][4][0][1][0][1]=1/2.0;
	M[4][4][0][2][0][0]=a/2.0;
	M[4][4][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][4][0][2][0][2]=a/2.0;
	M[4][4][0][3][0][0]=a/2.0;
	M[4][4][0][3][0][1]=(1-a)/2.0;
	M[4][4][0][3][0][2]=(1-a)/2.0;
	M[4][4][0][3][0][3]=a/2.0;
	M[4][4][0][4][0][1]=0.5/2.0;
	M[4][4][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[4][4][0][4][0][3]=0.5/2.0;
	M[4][4][1][0][0][0]=1/2.0;
	M[4][4][1][0][1][0]=1/2.0;
	M[4][4][1][1][0][0]=1/2.0;
	M[4][4][1][1][1][1]=1/2.0;
	M[4][4][1][2][0][0]=a/2.0;
	M[4][4][1][2][0][1]=(1-a)/2.0;
	M[4][4][1][2][1][1]=(1-a)/2.0;
	M[4][4][1][2][1][2]=a/2.0;
	M[4][4][1][3][0][0]=a/2.0;
	M[4][4][1][3][0][2]=(1-a)/2.0;
	M[4][4][1][3][1][1]=(1-a)/2.0;
	M[4][4][1][3][1][3]=a/2.0;
	M[4][4][1][4][0][2]=0.5/2.0;
	M[4][4][1][4][0][3]=0.5/2.0;
	M[4][4][1][4][1][1]=0.5/2.0;
	M[4][4][1][4][1][2]=0.5/2.0;
	M[4][4][2][0][0][0]=a/2.0;
	M[4][4][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[4][4][2][0][2][0]=a/2.0;
	M[4][4][2][1][0][0]=a/2.0;
	M[4][4][2][1][1][0]=(1-a)/2.0;
	M[4][4][2][1][1][1]=(1-a)/2.0;
	M[4][4][2][1][2][1]=a/2.0;
	M[4][4][2][2][0][0]=a/2.0;
	M[4][4][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][4][2][2][2][2]=a/2.0;
	M[4][4][2][3][0][0]=a/2.0;
	M[4][4][2][3][1][1]=(1-a)/2.0;
	M[4][4][2][3][1][2]=(1-a)/2.0;
	M[4][4][2][3][2][3]=a/2.0;
	M[4][4][2][4][0][2]=a/2.0;
	M[4][4][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[4][4][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][4][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[4][4][2][4][2][2]=a/2.0;
	M[4][4][3][0][0][0]=a/2.0;
	M[4][4][3][0][1][0]=(1-a)/2.0;
	M[4][4][3][0][2][0]=(1-a)/2.0;
	M[4][4][3][0][3][0]=a/2.0;
	M[4][4][3][1][0][0]=a/2.0;
	M[4][4][3][1][1][1]=(1-a)/2.0;
	M[4][4][3][1][2][0]=(1-a)/2.0;
	M[4][4][3][1][3][1]=a/2.0;
	M[4][4][3][2][0][0]=a/2.0;
	M[4][4][3][2][1][1]=(1-a)/2.0;
	M[4][4][3][2][2][1]=(1-a)/2.0;
	M[4][4][3][2][3][2]=a/2.0;
	M[4][4][3][3][0][0]=a/2.0;
	M[4][4][3][3][1][1]=(1-a)/2.0;
	M[4][4][3][3][2][2]=(1-a)/2.0;
	M[4][4][3][3][3][3]=a/2.0;
	M[4][4][3][4][1][1]=0.5/2.0;
	M[4][4][3][4][1][2]=0.5/2.0;
	M[4][4][3][4][2][2]=0.5/2.0;
	M[4][4][3][4][2][3]=0.5/2.0;
	M[4][4][4][0][1][0]=0.5/2.0;
	M[4][4][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[4][4][4][0][3][0]=0.5/2.0;
	M[4][4][4][1][1][1]=0.5/2.0;
	M[4][4][4][1][2][0]=0.5/2.0;
	M[4][4][4][1][2][1]=0.5/2.0;
	M[4][4][4][1][3][0]=0.5/2.0;
	M[4][4][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[4][4][4][2][2][0]=a/2.0;
	M[4][4][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][4][4][2][2][2]=a/2.0;
	M[4][4][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[4][4][4][3][1][1]=0.5/2.0;
	M[4][4][4][3][2][1]=0.5/2.0;
	M[4][4][4][3][2][2]=0.5/2.0;
	M[4][4][4][3][3][2]=0.5/2.0;
	M[4][4][4][4][1][1]=0.5/2.0;
	M[4][4][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[4][4][4][4][3][3]=0.5/2.0;
}

void calculateCHIT_female_x (double a, double M[5][5][5][5][5][5])
{
	int f1, f2, m1, m2, o1, o2;
	for (f1=0; f1<=4; f1++) {
		for (f2=0; f2<=4; f2++) {
			for (m1=0; m1<=4; m1++) {
				for (m2=0; m2<=4; m2++) {
					for (o1=0; o1<=4; o1++) {
						for (o2=0; o2<=4; o2++) {
							M[f1][f2][m1][m2][o1][o2] = 0;
						}
					}
				}
			}
		}
	}
	M[0][0][0][0][0][0]=1/2.0+1/2.0;
	M[0][0][0][1][0][0]=1/2.0;
	M[0][0][0][1][0][1]=1/2.0;
	M[0][0][0][2][0][0]=a/2.0;
	M[0][0][0][2][0][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][0][0][2][0][2]=a/2.0;
	M[0][0][0][3][0][0]=a/2.0;
	M[0][0][0][3][0][1]=(1-a)/2.0;
	M[0][0][0][3][0][2]=(1-a)/2.0;
	M[0][0][0][3][0][3]=a/2.0;
	M[0][0][0][4][0][1]=0.5/2.0;
	M[0][0][0][4][0][2]=0.5/2.0+0.5/2.0;
	M[0][0][0][4][0][3]=0.5/2.0;
	M[0][0][1][0][0][0]=1/2.0;
	M[0][0][1][0][1][0]=1/2.0;
	M[0][0][1][1][0][0]=1/2.0;
	M[0][0][1][1][1][1]=1/2.0;
	M[0][0][1][2][0][0]=a/2.0;
	M[0][0][1][2][0][1]=(1-a)/2.0;
	M[0][0][1][2][1][1]=(1-a)/2.0;
	M[0][0][1][2][1][2]=a/2.0;
	M[0][0][1][3][0][0]=a/2.0;
	M[0][0][1][3][0][2]=(1-a)/2.0;
	M[0][0][1][3][1][1]=(1-a)/2.0;
	M[0][0][1][3][1][3]=a/2.0;
	M[0][0][1][4][0][2]=0.5/2.0;
	M[0][0][1][4][0][3]=0.5/2.0;
	M[0][0][1][4][1][1]=0.5/2.0;
	M[0][0][1][4][1][2]=0.5/2.0;
	M[0][0][2][0][0][0]=a/2.0;
	M[0][0][2][0][1][0]=(1-a)/2.0+(1-a)/2.0;
	M[0][0][2][0][2][0]=a/2.0;
	M[0][0][2][1][0][0]=a/2.0;
	M[0][0][2][1][1][0]=(1-a)/2.0;
	M[0][0][2][1][1][1]=(1-a)/2.0;
	M[0][0][2][1][2][1]=a/2.0;
	M[0][0][2][2][0][0]=a/2.0;
	M[0][0][2][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][0][2][2][2][2]=a/2.0;
	M[0][0][2][3][0][0]=a/2.0;
	M[0][0][2][3][1][1]=(1-a)/2.0;
	M[0][0][2][3][1][2]=(1-a)/2.0;
	M[0][0][2][3][2][3]=a/2.0;
	M[0][0][2][4][0][2]=a/2.0;
	M[0][0][2][4][1][1]=(0.5-0.5*a)/2.0;
	M[0][0][2][4][1][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][0][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[0][0][2][4][2][2]=a/2.0;
	M[0][0][3][0][0][0]=a/2.0;
	M[0][0][3][0][1][0]=(1-a)/2.0;
	M[0][0][3][0][2][0]=(1-a)/2.0;
	M[0][0][3][0][3][0]=a/2.0;
	M[0][0][3][1][0][0]=a/2.0;
	M[0][0][3][1][1][1]=(1-a)/2.0;
	M[0][0][3][1][2][0]=(1-a)/2.0;
	M[0][0][3][1][3][1]=a/2.0;
	M[0][0][3][2][0][0]=a/2.0;
	M[0][0][3][2][1][1]=(1-a)/2.0;
	M[0][0][3][2][2][1]=(1-a)/2.0;
	M[0][0][3][2][3][2]=a/2.0;
	M[0][0][3][3][0][0]=a/2.0;
	M[0][0][3][3][1][1]=(1-a)/2.0;
	M[0][0][3][3][2][2]=(1-a)/2.0;
	M[0][0][3][3][3][3]=a/2.0;
	M[0][0][3][4][1][1]=0.5/2.0;
	M[0][0][3][4][1][2]=0.5/2.0;
	M[0][0][3][4][2][2]=0.5/2.0;
	M[0][0][3][4][2][3]=0.5/2.0;
	M[0][0][4][0][1][0]=0.5/2.0;
	M[0][0][4][0][2][0]=0.5/2.0+0.5/2.0;
	M[0][0][4][0][3][0]=0.5/2.0;
	M[0][0][4][1][1][1]=0.5/2.0;
	M[0][0][4][1][2][0]=0.5/2.0;
	M[0][0][4][1][2][1]=0.5/2.0;
	M[0][0][4][1][3][0]=0.5/2.0;
	M[0][0][4][2][1][1]=(0.5-0.5*a)/2.0;
	M[0][0][4][2][2][0]=a/2.0;
	M[0][0][4][2][2][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][0][4][2][2][2]=a/2.0;
	M[0][0][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[0][0][4][3][1][1]=0.5/2.0;
	M[0][0][4][3][2][1]=0.5/2.0;
	M[0][0][4][3][2][2]=0.5/2.0;
	M[0][0][4][3][3][2]=0.5/2.0;
	M[0][0][4][4][1][1]=0.5/2.0;
	M[0][0][4][4][2][2]=0.5/2.0+0.5/2.0;
	M[0][0][4][4][3][3]=0.5/2.0;
	M[0][1][0][0][0][1]=1/2.0+1/2.0;
	M[0][1][0][1][0][1]=1/2.0;
	M[0][1][0][1][0][2]=1/2.0;
	M[0][1][0][2][0][1]=a/2.0;
	M[0][1][0][2][0][2]=(1-a)/2.0+(1-a)/2.0;
	M[0][1][0][2][0][3]=a/2.0;
	M[0][1][0][3][0][1]=a/2.0;
	M[0][1][0][3][0][2]=(1-a)/2.0;
	M[0][1][0][3][0][3]=(1-a)/2.0;
	M[0][1][0][3][0][4]=a/2.0;
	M[0][1][0][4][0][2]=0.5/2.0;
	M[0][1][0][4][0][3]=0.5/2.0+0.5/2.0;
	M[0][1][0][4][0][4]=0.5/2.0;
	M[0][1][1][0][0][1]=1/2.0;
	M[0][1][1][0][1][1]=1/2.0;
	M[0][1][1][1][0][1]=1/2.0;
	M[0][1][1][1][1][2]=1/2.0;
	M[0][1][1][2][0][1]=a/2.0;
	M[0][1][1][2][0][2]=(1-a)/2.0;
	M[0][1][1][2][1][2]=(1-a)/2.0;
	M[0][1][1][2][1][3]=a/2.0;
	M[0][1][1][3][0][1]=a/2.0;
	M[0][1][1][3][0][3]=(1-a)/2.0;
	M[0][1][1][3][1][2]=(1-a)/2.0;
	M[0][1][1][3][1][4]=a/2.0;
	M[0][1][1][4][0][3]=0.5/2.0;
	M[0][1][1][4][0][4]=0.5/2.0;
	M[0][1][1][4][1][2]=0.5/2.0;
	M[0][1][1][4][1][3]=0.5/2.0;
	M[0][1][2][0][0][1]=a/2.0;
	M[0][1][2][0][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[0][1][2][0][2][1]=a/2.0;
	M[0][1][2][1][0][1]=a/2.0;
	M[0][1][2][1][1][1]=(1-a)/2.0;
	M[0][1][2][1][1][2]=(1-a)/2.0;
	M[0][1][2][1][2][2]=a/2.0;
	M[0][1][2][2][0][1]=a/2.0;
	M[0][1][2][2][1][2]=(1-a)/2.0+(1-a)/2.0;
	M[0][1][2][2][2][3]=a/2.0;
	M[0][1][2][3][0][1]=a/2.0;
	M[0][1][2][3][1][2]=(1-a)/2.0;
	M[0][1][2][3][1][3]=(1-a)/2.0;
	M[0][1][2][3][2][4]=a/2.0;
	M[0][1][2][4][0][3]=a/2.0;
	M[0][1][2][4][1][2]=(0.5-0.5*a)/2.0;
	M[0][1][2][4][1][3]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][1][2][4][1][4]=(0.5-0.5*a)/2.0;
	M[0][1][2][4][2][3]=a/2.0;
	M[0][1][3][0][0][1]=a/2.0;
	M[0][1][3][0][1][1]=(1-a)/2.0;
	M[0][1][3][0][2][1]=(1-a)/2.0;
	M[0][1][3][0][3][1]=a/2.0;
	M[0][1][3][1][0][1]=a/2.0;
	M[0][1][3][1][1][2]=(1-a)/2.0;
	M[0][1][3][1][2][1]=(1-a)/2.0;
	M[0][1][3][1][3][2]=a/2.0;
	M[0][1][3][2][0][1]=a/2.0;
	M[0][1][3][2][1][2]=(1-a)/2.0;
	M[0][1][3][2][2][2]=(1-a)/2.0;
	M[0][1][3][2][3][3]=a/2.0;
	M[0][1][3][3][0][1]=a/2.0;
	M[0][1][3][3][1][2]=(1-a)/2.0;
	M[0][1][3][3][2][3]=(1-a)/2.0;
	M[0][1][3][3][3][4]=a/2.0;
	M[0][1][3][4][1][2]=0.5/2.0;
	M[0][1][3][4][1][3]=0.5/2.0;
	M[0][1][3][4][2][3]=0.5/2.0;
	M[0][1][3][4][2][4]=0.5/2.0;
	M[0][1][4][0][1][1]=0.5/2.0;
	M[0][1][4][0][2][1]=0.5/2.0+0.5/2.0;
	M[0][1][4][0][3][1]=0.5/2.0;
	M[0][1][4][1][1][2]=0.5/2.0;
	M[0][1][4][1][2][1]=0.5/2.0;
	M[0][1][4][1][2][2]=0.5/2.0;
	M[0][1][4][1][3][1]=0.5/2.0;
	M[0][1][4][2][1][2]=(0.5-0.5*a)/2.0;
	M[0][1][4][2][2][1]=a/2.0;
	M[0][1][4][2][2][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][1][4][2][2][3]=a/2.0;
	M[0][1][4][2][3][2]=(0.5-0.5*a)/2.0;
	M[0][1][4][3][1][2]=0.5/2.0;
	M[0][1][4][3][2][2]=0.5/2.0;
	M[0][1][4][3][2][3]=0.5/2.0;
	M[0][1][4][3][3][3]=0.5/2.0;
	M[0][1][4][4][1][2]=0.5/2.0;
	M[0][1][4][4][2][3]=0.5/2.0+0.5/2.0;
	M[0][1][4][4][3][4]=0.5/2.0;
	M[0][2][0][0][0][2]=1/2.0+1/2.0;
	M[0][2][0][1][0][2]=1/2.0;
	M[0][2][0][1][0][3]=1/2.0;
	M[0][2][0][2][0][2]=a/2.0;
	M[0][2][0][2][0][3]=(1-a)/2.0+(1-a)/2.0;
	M[0][2][0][2][0][4]=a/2.0;
	M[0][2][0][3][0][2]=a/2.0;
	M[0][2][0][3][0][3]=(1-a)/2.0;
	M[0][2][0][3][0][4]=a/2.0+(1-a)/2.0;
	M[0][2][0][4][0][3]=0.5/2.0;
	M[0][2][0][4][0][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[0][2][1][0][0][2]=1/2.0;
	M[0][2][1][0][1][2]=1/2.0;
	M[0][2][1][1][0][2]=1/2.0;
	M[0][2][1][1][1][3]=1/2.0;
	M[0][2][1][2][0][2]=a/2.0;
	M[0][2][1][2][0][3]=(1-a)/2.0;
	M[0][2][1][2][1][3]=(1-a)/2.0;
	M[0][2][1][2][1][4]=a/2.0;
	M[0][2][1][3][0][2]=a/2.0;
	M[0][2][1][3][0][4]=(1-a)/2.0;
	M[0][2][1][3][1][3]=(1-a)/2.0;
	M[0][2][1][3][1][4]=a/2.0;
	M[0][2][1][4][0][4]=0.5/2.0+0.5/2.0;
	M[0][2][1][4][1][3]=0.5/2.0;
	M[0][2][1][4][1][4]=0.5/2.0;
	M[0][2][2][0][0][2]=a/2.0;
	M[0][2][2][0][1][2]=(1-a)/2.0+(1-a)/2.0;
	M[0][2][2][0][2][2]=a/2.0;
	M[0][2][2][1][0][2]=a/2.0;
	M[0][2][2][1][1][2]=(1-a)/2.0;
	M[0][2][2][1][1][3]=(1-a)/2.0;
	M[0][2][2][1][2][3]=a/2.0;
	M[0][2][2][2][0][2]=a/2.0;
	M[0][2][2][2][1][3]=(1-a)/2.0+(1-a)/2.0;
	M[0][2][2][2][2][4]=a/2.0;
	M[0][2][2][3][0][2]=a/2.0;
	M[0][2][2][3][1][3]=(1-a)/2.0;
	M[0][2][2][3][1][4]=(1-a)/2.0;
	M[0][2][2][3][2][4]=a/2.0;
	M[0][2][2][4][0][4]=a/2.0;
	M[0][2][2][4][1][3]=(0.5-0.5*a)/2.0;
	M[0][2][2][4][1][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][2][2][4][2][4]=a/2.0;
	M[0][2][3][0][0][2]=a/2.0;
	M[0][2][3][0][1][2]=(1-a)/2.0;
	M[0][2][3][0][2][2]=(1-a)/2.0;
	M[0][2][3][0][3][2]=a/2.0;
	M[0][2][3][1][0][2]=a/2.0;
	M[0][2][3][1][1][3]=(1-a)/2.0;
	M[0][2][3][1][2][2]=(1-a)/2.0;
	M[0][2][3][1][3][3]=a/2.0;
	M[0][2][3][2][0][2]=a/2.0;
	M[0][2][3][2][1][3]=(1-a)/2.0;
	M[0][2][3][2][2][3]=(1-a)/2.0;
	M[0][2][3][2][3][4]=a/2.0;
	M[0][2][3][3][0][2]=a/2.0;
	M[0][2][3][3][1][3]=(1-a)/2.0;
	M[0][2][3][3][2][4]=(1-a)/2.0;
	M[0][2][3][3][3][4]=a/2.0;
	M[0][2][3][4][1][3]=0.5/2.0;
	M[0][2][3][4][1][4]=0.5/2.0;
	M[0][2][3][4][2][4]=0.5/2.0+0.5/2.0;
	M[0][2][4][0][1][2]=0.5/2.0;
	M[0][2][4][0][2][2]=0.5/2.0+0.5/2.0;
	M[0][2][4][0][3][2]=0.5/2.0;
	M[0][2][4][1][1][3]=0.5/2.0;
	M[0][2][4][1][2][2]=0.5/2.0;
	M[0][2][4][1][2][3]=0.5/2.0;
	M[0][2][4][1][3][2]=0.5/2.0;
	M[0][2][4][2][1][3]=(0.5-0.5*a)/2.0;
	M[0][2][4][2][2][2]=a/2.0;
	M[0][2][4][2][2][3]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][2][4][2][2][4]=a/2.0;
	M[0][2][4][2][3][3]=(0.5-0.5*a)/2.0;
	M[0][2][4][3][1][3]=0.5/2.0;
	M[0][2][4][3][2][3]=0.5/2.0;
	M[0][2][4][3][2][4]=0.5/2.0;
	M[0][2][4][3][3][4]=0.5/2.0;
	M[0][2][4][4][1][3]=0.5/2.0;
	M[0][2][4][4][2][4]=0.5/2.0+0.5/2.0;
	M[0][2][4][4][3][4]=0.5/2.0;
	M[0][3][0][0][0][3]=1/2.0+1/2.0;
	M[0][3][0][1][0][3]=1/2.0;
	M[0][3][0][1][0][4]=1/2.0;
	M[0][3][0][2][0][3]=a/2.0;
	M[0][3][0][2][0][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[0][3][0][3][0][3]=a/2.0;
	M[0][3][0][3][0][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[0][3][0][4][0][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[0][3][1][0][0][3]=1/2.0;
	M[0][3][1][0][1][3]=1/2.0;
	M[0][3][1][1][0][3]=1/2.0;
	M[0][3][1][1][1][4]=1/2.0;
	M[0][3][1][2][0][3]=a/2.0;
	M[0][3][1][2][0][4]=(1-a)/2.0;
	M[0][3][1][2][1][4]=a/2.0+(1-a)/2.0;
	M[0][3][1][3][0][3]=a/2.0;
	M[0][3][1][3][0][4]=(1-a)/2.0;
	M[0][3][1][3][1][4]=a/2.0+(1-a)/2.0;
	M[0][3][1][4][0][4]=0.5/2.0+0.5/2.0;
	M[0][3][1][4][1][4]=0.5/2.0+0.5/2.0;
	M[0][3][2][0][0][3]=a/2.0;
	M[0][3][2][0][1][3]=(1-a)/2.0+(1-a)/2.0;
	M[0][3][2][0][2][3]=a/2.0;
	M[0][3][2][1][0][3]=a/2.0;
	M[0][3][2][1][1][3]=(1-a)/2.0;
	M[0][3][2][1][1][4]=(1-a)/2.0;
	M[0][3][2][1][2][4]=a/2.0;
	M[0][3][2][2][0][3]=a/2.0;
	M[0][3][2][2][1][4]=(1-a)/2.0+(1-a)/2.0;
	M[0][3][2][2][2][4]=a/2.0;
	M[0][3][2][3][0][3]=a/2.0;
	M[0][3][2][3][1][4]=(1-a)/2.0+(1-a)/2.0;
	M[0][3][2][3][2][4]=a/2.0;
	M[0][3][2][4][0][4]=a/2.0;
	M[0][3][2][4][1][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][3][2][4][2][4]=a/2.0;
	M[0][3][3][0][0][3]=a/2.0;
	M[0][3][3][0][1][3]=(1-a)/2.0;
	M[0][3][3][0][2][3]=(1-a)/2.0;
	M[0][3][3][0][3][3]=a/2.0;
	M[0][3][3][1][0][3]=a/2.0;
	M[0][3][3][1][1][4]=(1-a)/2.0;
	M[0][3][3][1][2][3]=(1-a)/2.0;
	M[0][3][3][1][3][4]=a/2.0;
	M[0][3][3][2][0][3]=a/2.0;
	M[0][3][3][2][1][4]=(1-a)/2.0;
	M[0][3][3][2][2][4]=(1-a)/2.0;
	M[0][3][3][2][3][4]=a/2.0;
	M[0][3][3][3][0][3]=a/2.0;
	M[0][3][3][3][1][4]=(1-a)/2.0;
	M[0][3][3][3][2][4]=(1-a)/2.0;
	M[0][3][3][3][3][4]=a/2.0;
	M[0][3][3][4][1][4]=0.5/2.0+0.5/2.0;
	M[0][3][3][4][2][4]=0.5/2.0+0.5/2.0;
	M[0][3][4][0][1][3]=0.5/2.0;
	M[0][3][4][0][2][3]=0.5/2.0+0.5/2.0;
	M[0][3][4][0][3][3]=0.5/2.0;
	M[0][3][4][1][1][4]=0.5/2.0;
	M[0][3][4][1][2][3]=0.5/2.0;
	M[0][3][4][1][2][4]=0.5/2.0;
	M[0][3][4][1][3][3]=0.5/2.0;
	M[0][3][4][2][1][4]=(0.5-0.5*a)/2.0;
	M[0][3][4][2][2][3]=a/2.0;
	M[0][3][4][2][2][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0;
	M[0][3][4][2][3][4]=(0.5-0.5*a)/2.0;
	M[0][3][4][3][1][4]=0.5/2.0;
	M[0][3][4][3][2][4]=0.5/2.0+0.5/2.0;
	M[0][3][4][3][3][4]=0.5/2.0;
	M[0][3][4][4][1][4]=0.5/2.0;
	M[0][3][4][4][2][4]=0.5/2.0+0.5/2.0;
	M[0][3][4][4][3][4]=0.5/2.0;
	M[0][4][0][0][0][4]=1/2.0+1/2.0;
	M[0][4][0][1][0][4]=1/2.0+1/2.0;
	M[0][4][0][2][0][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[0][4][0][3][0][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[0][4][0][4][0][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[0][4][1][0][0][4]=1/2.0;
	M[0][4][1][0][1][4]=1/2.0;
	M[0][4][1][1][0][4]=1/2.0;
	M[0][4][1][1][1][4]=1/2.0;
	M[0][4][1][2][0][4]=a/2.0+(1-a)/2.0;
	M[0][4][1][2][1][4]=a/2.0+(1-a)/2.0;
	M[0][4][1][3][0][4]=a/2.0+(1-a)/2.0;
	M[0][4][1][3][1][4]=a/2.0+(1-a)/2.0;
	M[0][4][1][4][0][4]=0.5/2.0+0.5/2.0;
	M[0][4][1][4][1][4]=0.5/2.0+0.5/2.0;
	M[0][4][2][0][0][4]=a/2.0;
	M[0][4][2][0][1][4]=(1-a)/2.0+(1-a)/2.0;
	M[0][4][2][0][2][4]=a/2.0;
	M[0][4][2][1][0][4]=a/2.0;
	M[0][4][2][1][1][4]=(1-a)/2.0+(1-a)/2.0;
	M[0][4][2][1][2][4]=a/2.0;
	M[0][4][2][2][0][4]=a/2.0;
	M[0][4][2][2][1][4]=(1-a)/2.0+(1-a)/2.0;
	M[0][4][2][2][2][4]=a/2.0;
	M[0][4][2][3][0][4]=a/2.0;
	M[0][4][2][3][1][4]=(1-a)/2.0+(1-a)/2.0;
	M[0][4][2][3][2][4]=a/2.0;
	M[0][4][2][4][0][4]=a/2.0;
	M[0][4][2][4][1][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[0][4][2][4][2][4]=a/2.0;
	M[0][4][3][0][0][4]=a/2.0;
	M[0][4][3][0][1][4]=(1-a)/2.0;
	M[0][4][3][0][2][4]=(1-a)/2.0;
	M[0][4][3][0][3][4]=a/2.0;
	M[0][4][3][1][0][4]=a/2.0;
	M[0][4][3][1][1][4]=(1-a)/2.0;
	M[0][4][3][1][2][4]=(1-a)/2.0;
	M[0][4][3][1][3][4]=a/2.0;
	M[0][4][3][2][0][4]=a/2.0;
	M[0][4][3][2][1][4]=(1-a)/2.0;
	M[0][4][3][2][2][4]=(1-a)/2.0;
	M[0][4][3][2][3][4]=a/2.0;
	M[0][4][3][3][0][4]=a/2.0;
	M[0][4][3][3][1][4]=(1-a)/2.0;
	M[0][4][3][3][2][4]=(1-a)/2.0;
	M[0][4][3][3][3][4]=a/2.0;
	M[0][4][3][4][1][4]=0.5/2.0+0.5/2.0;
	M[0][4][3][4][2][4]=0.5/2.0+0.5/2.0;
	M[0][4][4][0][1][4]=0.5/2.0;
	M[0][4][4][0][2][4]=0.5/2.0+0.5/2.0;
	M[0][4][4][0][3][4]=0.5/2.0;
	M[0][4][4][1][1][4]=0.5/2.0;
	M[0][4][4][1][2][4]=0.5/2.0+0.5/2.0;
	M[0][4][4][1][3][4]=0.5/2.0;
	M[0][4][4][2][1][4]=(0.5-0.5*a)/2.0;
	M[0][4][4][2][2][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0+a/2.0;
	M[0][4][4][2][3][4]=(0.5-0.5*a)/2.0;
	M[0][4][4][3][1][4]=0.5/2.0;
	M[0][4][4][3][2][4]=0.5/2.0+0.5/2.0;
	M[0][4][4][3][3][4]=0.5/2.0;
	M[0][4][4][4][1][4]=0.5/2.0;
	M[0][4][4][4][2][4]=0.5/2.0+0.5/2.0;
	M[0][4][4][4][3][4]=0.5/2.0;
	M[1][0][0][0][1][0]=1/2.0+1/2.0;
	M[1][0][0][1][1][0]=1/2.0;
	M[1][0][0][1][1][1]=1/2.0;
	M[1][0][0][2][1][0]=a/2.0;
	M[1][0][0][2][1][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][0][0][2][1][2]=a/2.0;
	M[1][0][0][3][1][0]=a/2.0;
	M[1][0][0][3][1][1]=(1-a)/2.0;
	M[1][0][0][3][1][2]=(1-a)/2.0;
	M[1][0][0][3][1][3]=a/2.0;
	M[1][0][0][4][1][1]=0.5/2.0;
	M[1][0][0][4][1][2]=0.5/2.0+0.5/2.0;
	M[1][0][0][4][1][3]=0.5/2.0;
	M[1][0][1][0][1][0]=1/2.0;
	M[1][0][1][0][2][0]=1/2.0;
	M[1][0][1][1][1][0]=1/2.0;
	M[1][0][1][1][2][1]=1/2.0;
	M[1][0][1][2][1][0]=a/2.0;
	M[1][0][1][2][1][1]=(1-a)/2.0;
	M[1][0][1][2][2][1]=(1-a)/2.0;
	M[1][0][1][2][2][2]=a/2.0;
	M[1][0][1][3][1][0]=a/2.0;
	M[1][0][1][3][1][2]=(1-a)/2.0;
	M[1][0][1][3][2][1]=(1-a)/2.0;
	M[1][0][1][3][2][3]=a/2.0;
	M[1][0][1][4][1][2]=0.5/2.0;
	M[1][0][1][4][1][3]=0.5/2.0;
	M[1][0][1][4][2][1]=0.5/2.0;
	M[1][0][1][4][2][2]=0.5/2.0;
	M[1][0][2][0][1][0]=a/2.0;
	M[1][0][2][0][2][0]=(1-a)/2.0+(1-a)/2.0;
	M[1][0][2][0][3][0]=a/2.0;
	M[1][0][2][1][1][0]=a/2.0;
	M[1][0][2][1][2][0]=(1-a)/2.0;
	M[1][0][2][1][2][1]=(1-a)/2.0;
	M[1][0][2][1][3][1]=a/2.0;
	M[1][0][2][2][1][0]=a/2.0;
	M[1][0][2][2][2][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][0][2][2][3][2]=a/2.0;
	M[1][0][2][3][1][0]=a/2.0;
	M[1][0][2][3][2][1]=(1-a)/2.0;
	M[1][0][2][3][2][2]=(1-a)/2.0;
	M[1][0][2][3][3][3]=a/2.0;
	M[1][0][2][4][1][2]=a/2.0;
	M[1][0][2][4][2][1]=(0.5-0.5*a)/2.0;
	M[1][0][2][4][2][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][0][2][4][2][3]=(0.5-0.5*a)/2.0;
	M[1][0][2][4][3][2]=a/2.0;
	M[1][0][3][0][1][0]=a/2.0;
	M[1][0][3][0][2][0]=(1-a)/2.0;
	M[1][0][3][0][3][0]=(1-a)/2.0;
	M[1][0][3][0][4][0]=a/2.0;
	M[1][0][3][1][1][0]=a/2.0;
	M[1][0][3][1][2][1]=(1-a)/2.0;
	M[1][0][3][1][3][0]=(1-a)/2.0;
	M[1][0][3][1][4][1]=a/2.0;
	M[1][0][3][2][1][0]=a/2.0;
	M[1][0][3][2][2][1]=(1-a)/2.0;
	M[1][0][3][2][3][1]=(1-a)/2.0;
	M[1][0][3][2][4][2]=a/2.0;
	M[1][0][3][3][1][0]=a/2.0;
	M[1][0][3][3][2][1]=(1-a)/2.0;
	M[1][0][3][3][3][2]=(1-a)/2.0;
	M[1][0][3][3][4][3]=a/2.0;
	M[1][0][3][4][2][1]=0.5/2.0;
	M[1][0][3][4][2][2]=0.5/2.0;
	M[1][0][3][4][3][2]=0.5/2.0;
	M[1][0][3][4][3][3]=0.5/2.0;
	M[1][0][4][0][2][0]=0.5/2.0;
	M[1][0][4][0][3][0]=0.5/2.0+0.5/2.0;
	M[1][0][4][0][4][0]=0.5/2.0;
	M[1][0][4][1][2][1]=0.5/2.0;
	M[1][0][4][1][3][0]=0.5/2.0;
	M[1][0][4][1][3][1]=0.5/2.0;
	M[1][0][4][1][4][0]=0.5/2.0;
	M[1][0][4][2][2][1]=(0.5-0.5*a)/2.0;
	M[1][0][4][2][3][0]=a/2.0;
	M[1][0][4][2][3][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][0][4][2][3][2]=a/2.0;
	M[1][0][4][2][4][1]=(0.5-0.5*a)/2.0;
	M[1][0][4][3][2][1]=0.5/2.0;
	M[1][0][4][3][3][1]=0.5/2.0;
	M[1][0][4][3][3][2]=0.5/2.0;
	M[1][0][4][3][4][2]=0.5/2.0;
	M[1][0][4][4][2][1]=0.5/2.0;
	M[1][0][4][4][3][2]=0.5/2.0+0.5/2.0;
	M[1][0][4][4][4][3]=0.5/2.0;
	M[1][1][0][0][1][1]=1/2.0+1/2.0;
	M[1][1][0][1][1][1]=1/2.0;
	M[1][1][0][1][1][2]=1/2.0;
	M[1][1][0][2][1][1]=a/2.0;
	M[1][1][0][2][1][2]=(1-a)/2.0+(1-a)/2.0;
	M[1][1][0][2][1][3]=a/2.0;
	M[1][1][0][3][1][1]=a/2.0;
	M[1][1][0][3][1][2]=(1-a)/2.0;
	M[1][1][0][3][1][3]=(1-a)/2.0;
	M[1][1][0][3][1][4]=a/2.0;
	M[1][1][0][4][1][2]=0.5/2.0;
	M[1][1][0][4][1][3]=0.5/2.0+0.5/2.0;
	M[1][1][0][4][1][4]=0.5/2.0;
	M[1][1][1][0][1][1]=1/2.0;
	M[1][1][1][0][2][1]=1/2.0;
	M[1][1][1][1][1][1]=1/2.0;
	M[1][1][1][1][2][2]=1/2.0;
	M[1][1][1][2][1][1]=a/2.0;
	M[1][1][1][2][1][2]=(1-a)/2.0;
	M[1][1][1][2][2][2]=(1-a)/2.0;
	M[1][1][1][2][2][3]=a/2.0;
	M[1][1][1][3][1][1]=a/2.0;
	M[1][1][1][3][1][3]=(1-a)/2.0;
	M[1][1][1][3][2][2]=(1-a)/2.0;
	M[1][1][1][3][2][4]=a/2.0;
	M[1][1][1][4][1][3]=0.5/2.0;
	M[1][1][1][4][1][4]=0.5/2.0;
	M[1][1][1][4][2][2]=0.5/2.0;
	M[1][1][1][4][2][3]=0.5/2.0;
	M[1][1][2][0][1][1]=a/2.0;
	M[1][1][2][0][2][1]=(1-a)/2.0+(1-a)/2.0;
	M[1][1][2][0][3][1]=a/2.0;
	M[1][1][2][1][1][1]=a/2.0;
	M[1][1][2][1][2][1]=(1-a)/2.0;
	M[1][1][2][1][2][2]=(1-a)/2.0;
	M[1][1][2][1][3][2]=a/2.0;
	M[1][1][2][2][1][1]=a/2.0;
	M[1][1][2][2][2][2]=(1-a)/2.0+(1-a)/2.0;
	M[1][1][2][2][3][3]=a/2.0;
	M[1][1][2][3][1][1]=a/2.0;
	M[1][1][2][3][2][2]=(1-a)/2.0;
	M[1][1][2][3][2][3]=(1-a)/2.0;
	M[1][1][2][3][3][4]=a/2.0;
	M[1][1][2][4][1][3]=a/2.0;
	M[1][1][2][4][2][2]=(0.5-0.5*a)/2.0;
	M[1][1][2][4][2][3]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][1][2][4][2][4]=(0.5-0.5*a)/2.0;
	M[1][1][2][4][3][3]=a/2.0;
	M[1][1][3][0][1][1]=a/2.0;
	M[1][1][3][0][2][1]=(1-a)/2.0;
	M[1][1][3][0][3][1]=(1-a)/2.0;
	M[1][1][3][0][4][1]=a/2.0;
	M[1][1][3][1][1][1]=a/2.0;
	M[1][1][3][1][2][2]=(1-a)/2.0;
	M[1][1][3][1][3][1]=(1-a)/2.0;
	M[1][1][3][1][4][2]=a/2.0;
	M[1][1][3][2][1][1]=a/2.0;
	M[1][1][3][2][2][2]=(1-a)/2.0;
	M[1][1][3][2][3][2]=(1-a)/2.0;
	M[1][1][3][2][4][3]=a/2.0;
	M[1][1][3][3][1][1]=a/2.0;
	M[1][1][3][3][2][2]=(1-a)/2.0;
	M[1][1][3][3][3][3]=(1-a)/2.0;
	M[1][1][3][3][4][4]=a/2.0;
	M[1][1][3][4][2][2]=0.5/2.0;
	M[1][1][3][4][2][3]=0.5/2.0;
	M[1][1][3][4][3][3]=0.5/2.0;
	M[1][1][3][4][3][4]=0.5/2.0;
	M[1][1][4][0][2][1]=0.5/2.0;
	M[1][1][4][0][3][1]=0.5/2.0+0.5/2.0;
	M[1][1][4][0][4][1]=0.5/2.0;
	M[1][1][4][1][2][2]=0.5/2.0;
	M[1][1][4][1][3][1]=0.5/2.0;
	M[1][1][4][1][3][2]=0.5/2.0;
	M[1][1][4][1][4][1]=0.5/2.0;
	M[1][1][4][2][2][2]=(0.5-0.5*a)/2.0;
	M[1][1][4][2][3][1]=a/2.0;
	M[1][1][4][2][3][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][1][4][2][3][3]=a/2.0;
	M[1][1][4][2][4][2]=(0.5-0.5*a)/2.0;
	M[1][1][4][3][2][2]=0.5/2.0;
	M[1][1][4][3][3][2]=0.5/2.0;
	M[1][1][4][3][3][3]=0.5/2.0;
	M[1][1][4][3][4][3]=0.5/2.0;
	M[1][1][4][4][2][2]=0.5/2.0;
	M[1][1][4][4][3][3]=0.5/2.0+0.5/2.0;
	M[1][1][4][4][4][4]=0.5/2.0;
	M[1][2][0][0][1][2]=1/2.0+1/2.0;
	M[1][2][0][1][1][2]=1/2.0;
	M[1][2][0][1][1][3]=1/2.0;
	M[1][2][0][2][1][2]=a/2.0;
	M[1][2][0][2][1][3]=(1-a)/2.0+(1-a)/2.0;
	M[1][2][0][2][1][4]=a/2.0;
	M[1][2][0][3][1][2]=a/2.0;
	M[1][2][0][3][1][3]=(1-a)/2.0;
	M[1][2][0][3][1][4]=a/2.0+(1-a)/2.0;
	M[1][2][0][4][1][3]=0.5/2.0;
	M[1][2][0][4][1][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[1][2][1][0][1][2]=1/2.0;
	M[1][2][1][0][2][2]=1/2.0;
	M[1][2][1][1][1][2]=1/2.0;
	M[1][2][1][1][2][3]=1/2.0;
	M[1][2][1][2][1][2]=a/2.0;
	M[1][2][1][2][1][3]=(1-a)/2.0;
	M[1][2][1][2][2][3]=(1-a)/2.0;
	M[1][2][1][2][2][4]=a/2.0;
	M[1][2][1][3][1][2]=a/2.0;
	M[1][2][1][3][1][4]=(1-a)/2.0;
	M[1][2][1][3][2][3]=(1-a)/2.0;
	M[1][2][1][3][2][4]=a/2.0;
	M[1][2][1][4][1][4]=0.5/2.0+0.5/2.0;
	M[1][2][1][4][2][3]=0.5/2.0;
	M[1][2][1][4][2][4]=0.5/2.0;
	M[1][2][2][0][1][2]=a/2.0;
	M[1][2][2][0][2][2]=(1-a)/2.0+(1-a)/2.0;
	M[1][2][2][0][3][2]=a/2.0;
	M[1][2][2][1][1][2]=a/2.0;
	M[1][2][2][1][2][2]=(1-a)/2.0;
	M[1][2][2][1][2][3]=(1-a)/2.0;
	M[1][2][2][1][3][3]=a/2.0;
	M[1][2][2][2][1][2]=a/2.0;
	M[1][2][2][2][2][3]=(1-a)/2.0+(1-a)/2.0;
	M[1][2][2][2][3][4]=a/2.0;
	M[1][2][2][3][1][2]=a/2.0;
	M[1][2][2][3][2][3]=(1-a)/2.0;
	M[1][2][2][3][2][4]=(1-a)/2.0;
	M[1][2][2][3][3][4]=a/2.0;
	M[1][2][2][4][1][4]=a/2.0;
	M[1][2][2][4][2][3]=(0.5-0.5*a)/2.0;
	M[1][2][2][4][2][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][2][2][4][3][4]=a/2.0;
	M[1][2][3][0][1][2]=a/2.0;
	M[1][2][3][0][2][2]=(1-a)/2.0;
	M[1][2][3][0][3][2]=(1-a)/2.0;
	M[1][2][3][0][4][2]=a/2.0;
	M[1][2][3][1][1][2]=a/2.0;
	M[1][2][3][1][2][3]=(1-a)/2.0;
	M[1][2][3][1][3][2]=(1-a)/2.0;
	M[1][2][3][1][4][3]=a/2.0;
	M[1][2][3][2][1][2]=a/2.0;
	M[1][2][3][2][2][3]=(1-a)/2.0;
	M[1][2][3][2][3][3]=(1-a)/2.0;
	M[1][2][3][2][4][4]=a/2.0;
	M[1][2][3][3][1][2]=a/2.0;
	M[1][2][3][3][2][3]=(1-a)/2.0;
	M[1][2][3][3][3][4]=(1-a)/2.0;
	M[1][2][3][3][4][4]=a/2.0;
	M[1][2][3][4][2][3]=0.5/2.0;
	M[1][2][3][4][2][4]=0.5/2.0;
	M[1][2][3][4][3][4]=0.5/2.0+0.5/2.0;
	M[1][2][4][0][2][2]=0.5/2.0;
	M[1][2][4][0][3][2]=0.5/2.0+0.5/2.0;
	M[1][2][4][0][4][2]=0.5/2.0;
	M[1][2][4][1][2][3]=0.5/2.0;
	M[1][2][4][1][3][2]=0.5/2.0;
	M[1][2][4][1][3][3]=0.5/2.0;
	M[1][2][4][1][4][2]=0.5/2.0;
	M[1][2][4][2][2][3]=(0.5-0.5*a)/2.0;
	M[1][2][4][2][3][2]=a/2.0;
	M[1][2][4][2][3][3]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][2][4][2][3][4]=a/2.0;
	M[1][2][4][2][4][3]=(0.5-0.5*a)/2.0;
	M[1][2][4][3][2][3]=0.5/2.0;
	M[1][2][4][3][3][3]=0.5/2.0;
	M[1][2][4][3][3][4]=0.5/2.0;
	M[1][2][4][3][4][4]=0.5/2.0;
	M[1][2][4][4][2][3]=0.5/2.0;
	M[1][2][4][4][3][4]=0.5/2.0+0.5/2.0;
	M[1][2][4][4][4][4]=0.5/2.0;
	M[1][3][0][0][1][3]=1/2.0+1/2.0;
	M[1][3][0][1][1][3]=1/2.0;
	M[1][3][0][1][1][4]=1/2.0;
	M[1][3][0][2][1][3]=a/2.0;
	M[1][3][0][2][1][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[1][3][0][3][1][3]=a/2.0;
	M[1][3][0][3][1][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[1][3][0][4][1][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[1][3][1][0][1][3]=1/2.0;
	M[1][3][1][0][2][3]=1/2.0;
	M[1][3][1][1][1][3]=1/2.0;
	M[1][3][1][1][2][4]=1/2.0;
	M[1][3][1][2][1][3]=a/2.0;
	M[1][3][1][2][1][4]=(1-a)/2.0;
	M[1][3][1][2][2][4]=a/2.0+(1-a)/2.0;
	M[1][3][1][3][1][3]=a/2.0;
	M[1][3][1][3][1][4]=(1-a)/2.0;
	M[1][3][1][3][2][4]=a/2.0+(1-a)/2.0;
	M[1][3][1][4][1][4]=0.5/2.0+0.5/2.0;
	M[1][3][1][4][2][4]=0.5/2.0+0.5/2.0;
	M[1][3][2][0][1][3]=a/2.0;
	M[1][3][2][0][2][3]=(1-a)/2.0+(1-a)/2.0;
	M[1][3][2][0][3][3]=a/2.0;
	M[1][3][2][1][1][3]=a/2.0;
	M[1][3][2][1][2][3]=(1-a)/2.0;
	M[1][3][2][1][2][4]=(1-a)/2.0;
	M[1][3][2][1][3][4]=a/2.0;
	M[1][3][2][2][1][3]=a/2.0;
	M[1][3][2][2][2][4]=(1-a)/2.0+(1-a)/2.0;
	M[1][3][2][2][3][4]=a/2.0;
	M[1][3][2][3][1][3]=a/2.0;
	M[1][3][2][3][2][4]=(1-a)/2.0+(1-a)/2.0;
	M[1][3][2][3][3][4]=a/2.0;
	M[1][3][2][4][1][4]=a/2.0;
	M[1][3][2][4][2][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][3][2][4][3][4]=a/2.0;
	M[1][3][3][0][1][3]=a/2.0;
	M[1][3][3][0][2][3]=(1-a)/2.0;
	M[1][3][3][0][3][3]=(1-a)/2.0;
	M[1][3][3][0][4][3]=a/2.0;
	M[1][3][3][1][1][3]=a/2.0;
	M[1][3][3][1][2][4]=(1-a)/2.0;
	M[1][3][3][1][3][3]=(1-a)/2.0;
	M[1][3][3][1][4][4]=a/2.0;
	M[1][3][3][2][1][3]=a/2.0;
	M[1][3][3][2][2][4]=(1-a)/2.0;
	M[1][3][3][2][3][4]=(1-a)/2.0;
	M[1][3][3][2][4][4]=a/2.0;
	M[1][3][3][3][1][3]=a/2.0;
	M[1][3][3][3][2][4]=(1-a)/2.0;
	M[1][3][3][3][3][4]=(1-a)/2.0;
	M[1][3][3][3][4][4]=a/2.0;
	M[1][3][3][4][2][4]=0.5/2.0+0.5/2.0;
	M[1][3][3][4][3][4]=0.5/2.0+0.5/2.0;
	M[1][3][4][0][2][3]=0.5/2.0;
	M[1][3][4][0][3][3]=0.5/2.0+0.5/2.0;
	M[1][3][4][0][4][3]=0.5/2.0;
	M[1][3][4][1][2][4]=0.5/2.0;
	M[1][3][4][1][3][3]=0.5/2.0;
	M[1][3][4][1][3][4]=0.5/2.0;
	M[1][3][4][1][4][3]=0.5/2.0;
	M[1][3][4][2][2][4]=(0.5-0.5*a)/2.0;
	M[1][3][4][2][3][3]=a/2.0;
	M[1][3][4][2][3][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0;
	M[1][3][4][2][4][4]=(0.5-0.5*a)/2.0;
	M[1][3][4][3][2][4]=0.5/2.0;
	M[1][3][4][3][3][4]=0.5/2.0+0.5/2.0;
	M[1][3][4][3][4][4]=0.5/2.0;
	M[1][3][4][4][2][4]=0.5/2.0;
	M[1][3][4][4][3][4]=0.5/2.0+0.5/2.0;
	M[1][3][4][4][4][4]=0.5/2.0;
	M[1][4][0][0][1][4]=1/2.0+1/2.0;
	M[1][4][0][1][1][4]=1/2.0+1/2.0;
	M[1][4][0][2][1][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[1][4][0][3][1][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[1][4][0][4][1][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[1][4][1][0][1][4]=1/2.0;
	M[1][4][1][0][2][4]=1/2.0;
	M[1][4][1][1][1][4]=1/2.0;
	M[1][4][1][1][2][4]=1/2.0;
	M[1][4][1][2][1][4]=a/2.0+(1-a)/2.0;
	M[1][4][1][2][2][4]=a/2.0+(1-a)/2.0;
	M[1][4][1][3][1][4]=a/2.0+(1-a)/2.0;
	M[1][4][1][3][2][4]=a/2.0+(1-a)/2.0;
	M[1][4][1][4][1][4]=0.5/2.0+0.5/2.0;
	M[1][4][1][4][2][4]=0.5/2.0+0.5/2.0;
	M[1][4][2][0][1][4]=a/2.0;
	M[1][4][2][0][2][4]=(1-a)/2.0+(1-a)/2.0;
	M[1][4][2][0][3][4]=a/2.0;
	M[1][4][2][1][1][4]=a/2.0;
	M[1][4][2][1][2][4]=(1-a)/2.0+(1-a)/2.0;
	M[1][4][2][1][3][4]=a/2.0;
	M[1][4][2][2][1][4]=a/2.0;
	M[1][4][2][2][2][4]=(1-a)/2.0+(1-a)/2.0;
	M[1][4][2][2][3][4]=a/2.0;
	M[1][4][2][3][1][4]=a/2.0;
	M[1][4][2][3][2][4]=(1-a)/2.0+(1-a)/2.0;
	M[1][4][2][3][3][4]=a/2.0;
	M[1][4][2][4][1][4]=a/2.0;
	M[1][4][2][4][2][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[1][4][2][4][3][4]=a/2.0;
	M[1][4][3][0][1][4]=a/2.0;
	M[1][4][3][0][2][4]=(1-a)/2.0;
	M[1][4][3][0][3][4]=(1-a)/2.0;
	M[1][4][3][0][4][4]=a/2.0;
	M[1][4][3][1][1][4]=a/2.0;
	M[1][4][3][1][2][4]=(1-a)/2.0;
	M[1][4][3][1][3][4]=(1-a)/2.0;
	M[1][4][3][1][4][4]=a/2.0;
	M[1][4][3][2][1][4]=a/2.0;
	M[1][4][3][2][2][4]=(1-a)/2.0;
	M[1][4][3][2][3][4]=(1-a)/2.0;
	M[1][4][3][2][4][4]=a/2.0;
	M[1][4][3][3][1][4]=a/2.0;
	M[1][4][3][3][2][4]=(1-a)/2.0;
	M[1][4][3][3][3][4]=(1-a)/2.0;
	M[1][4][3][3][4][4]=a/2.0;
	M[1][4][3][4][2][4]=0.5/2.0+0.5/2.0;
	M[1][4][3][4][3][4]=0.5/2.0+0.5/2.0;
	M[1][4][4][0][2][4]=0.5/2.0;
	M[1][4][4][0][3][4]=0.5/2.0+0.5/2.0;
	M[1][4][4][0][4][4]=0.5/2.0;
	M[1][4][4][1][2][4]=0.5/2.0;
	M[1][4][4][1][3][4]=0.5/2.0+0.5/2.0;
	M[1][4][4][1][4][4]=0.5/2.0;
	M[1][4][4][2][2][4]=(0.5-0.5*a)/2.0;
	M[1][4][4][2][3][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0+a/2.0;
	M[1][4][4][2][4][4]=(0.5-0.5*a)/2.0;
	M[1][4][4][3][2][4]=0.5/2.0;
	M[1][4][4][3][3][4]=0.5/2.0+0.5/2.0;
	M[1][4][4][3][4][4]=0.5/2.0;
	M[1][4][4][4][2][4]=0.5/2.0;
	M[1][4][4][4][3][4]=0.5/2.0+0.5/2.0;
	M[1][4][4][4][4][4]=0.5/2.0;
	M[2][0][0][0][2][0]=1/2.0+1/2.0;
	M[2][0][0][1][2][0]=1/2.0;
	M[2][0][0][1][2][1]=1/2.0;
	M[2][0][0][2][2][0]=a/2.0;
	M[2][0][0][2][2][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][0][0][2][2][2]=a/2.0;
	M[2][0][0][3][2][0]=a/2.0;
	M[2][0][0][3][2][1]=(1-a)/2.0;
	M[2][0][0][3][2][2]=(1-a)/2.0;
	M[2][0][0][3][2][3]=a/2.0;
	M[2][0][0][4][2][1]=0.5/2.0;
	M[2][0][0][4][2][2]=0.5/2.0+0.5/2.0;
	M[2][0][0][4][2][3]=0.5/2.0;
	M[2][0][1][0][2][0]=1/2.0;
	M[2][0][1][0][3][0]=1/2.0;
	M[2][0][1][1][2][0]=1/2.0;
	M[2][0][1][1][3][1]=1/2.0;
	M[2][0][1][2][2][0]=a/2.0;
	M[2][0][1][2][2][1]=(1-a)/2.0;
	M[2][0][1][2][3][1]=(1-a)/2.0;
	M[2][0][1][2][3][2]=a/2.0;
	M[2][0][1][3][2][0]=a/2.0;
	M[2][0][1][3][2][2]=(1-a)/2.0;
	M[2][0][1][3][3][1]=(1-a)/2.0;
	M[2][0][1][3][3][3]=a/2.0;
	M[2][0][1][4][2][2]=0.5/2.0;
	M[2][0][1][4][2][3]=0.5/2.0;
	M[2][0][1][4][3][1]=0.5/2.0;
	M[2][0][1][4][3][2]=0.5/2.0;
	M[2][0][2][0][2][0]=a/2.0;
	M[2][0][2][0][3][0]=(1-a)/2.0+(1-a)/2.0;
	M[2][0][2][0][4][0]=a/2.0;
	M[2][0][2][1][2][0]=a/2.0;
	M[2][0][2][1][3][0]=(1-a)/2.0;
	M[2][0][2][1][3][1]=(1-a)/2.0;
	M[2][0][2][1][4][1]=a/2.0;
	M[2][0][2][2][2][0]=a/2.0;
	M[2][0][2][2][3][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][0][2][2][4][2]=a/2.0;
	M[2][0][2][3][2][0]=a/2.0;
	M[2][0][2][3][3][1]=(1-a)/2.0;
	M[2][0][2][3][3][2]=(1-a)/2.0;
	M[2][0][2][3][4][3]=a/2.0;
	M[2][0][2][4][2][2]=a/2.0;
	M[2][0][2][4][3][1]=(0.5-0.5*a)/2.0;
	M[2][0][2][4][3][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][0][2][4][3][3]=(0.5-0.5*a)/2.0;
	M[2][0][2][4][4][2]=a/2.0;
	M[2][0][3][0][2][0]=a/2.0;
	M[2][0][3][0][3][0]=(1-a)/2.0;
	M[2][0][3][0][4][0]=a/2.0+(1-a)/2.0;
	M[2][0][3][1][2][0]=a/2.0;
	M[2][0][3][1][3][1]=(1-a)/2.0;
	M[2][0][3][1][4][0]=(1-a)/2.0;
	M[2][0][3][1][4][1]=a/2.0;
	M[2][0][3][2][2][0]=a/2.0;
	M[2][0][3][2][3][1]=(1-a)/2.0;
	M[2][0][3][2][4][1]=(1-a)/2.0;
	M[2][0][3][2][4][2]=a/2.0;
	M[2][0][3][3][2][0]=a/2.0;
	M[2][0][3][3][3][1]=(1-a)/2.0;
	M[2][0][3][3][4][2]=(1-a)/2.0;
	M[2][0][3][3][4][3]=a/2.0;
	M[2][0][3][4][3][1]=0.5/2.0;
	M[2][0][3][4][3][2]=0.5/2.0;
	M[2][0][3][4][4][2]=0.5/2.0;
	M[2][0][3][4][4][3]=0.5/2.0;
	M[2][0][4][0][3][0]=0.5/2.0;
	M[2][0][4][0][4][0]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][0][4][1][3][1]=0.5/2.0;
	M[2][0][4][1][4][0]=0.5/2.0+0.5/2.0;
	M[2][0][4][1][4][1]=0.5/2.0;
	M[2][0][4][2][3][1]=(0.5-0.5*a)/2.0;
	M[2][0][4][2][4][0]=a/2.0;
	M[2][0][4][2][4][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][0][4][2][4][2]=a/2.0;
	M[2][0][4][3][3][1]=0.5/2.0;
	M[2][0][4][3][4][1]=0.5/2.0;
	M[2][0][4][3][4][2]=0.5/2.0+0.5/2.0;
	M[2][0][4][4][3][1]=0.5/2.0;
	M[2][0][4][4][4][2]=0.5/2.0+0.5/2.0;
	M[2][0][4][4][4][3]=0.5/2.0;
	M[2][1][0][0][2][1]=1/2.0+1/2.0;
	M[2][1][0][1][2][1]=1/2.0;
	M[2][1][0][1][2][2]=1/2.0;
	M[2][1][0][2][2][1]=a/2.0;
	M[2][1][0][2][2][2]=(1-a)/2.0+(1-a)/2.0;
	M[2][1][0][2][2][3]=a/2.0;
	M[2][1][0][3][2][1]=a/2.0;
	M[2][1][0][3][2][2]=(1-a)/2.0;
	M[2][1][0][3][2][3]=(1-a)/2.0;
	M[2][1][0][3][2][4]=a/2.0;
	M[2][1][0][4][2][2]=0.5/2.0;
	M[2][1][0][4][2][3]=0.5/2.0+0.5/2.0;
	M[2][1][0][4][2][4]=0.5/2.0;
	M[2][1][1][0][2][1]=1/2.0;
	M[2][1][1][0][3][1]=1/2.0;
	M[2][1][1][1][2][1]=1/2.0;
	M[2][1][1][1][3][2]=1/2.0;
	M[2][1][1][2][2][1]=a/2.0;
	M[2][1][1][2][2][2]=(1-a)/2.0;
	M[2][1][1][2][3][2]=(1-a)/2.0;
	M[2][1][1][2][3][3]=a/2.0;
	M[2][1][1][3][2][1]=a/2.0;
	M[2][1][1][3][2][3]=(1-a)/2.0;
	M[2][1][1][3][3][2]=(1-a)/2.0;
	M[2][1][1][3][3][4]=a/2.0;
	M[2][1][1][4][2][3]=0.5/2.0;
	M[2][1][1][4][2][4]=0.5/2.0;
	M[2][1][1][4][3][2]=0.5/2.0;
	M[2][1][1][4][3][3]=0.5/2.0;
	M[2][1][2][0][2][1]=a/2.0;
	M[2][1][2][0][3][1]=(1-a)/2.0+(1-a)/2.0;
	M[2][1][2][0][4][1]=a/2.0;
	M[2][1][2][1][2][1]=a/2.0;
	M[2][1][2][1][3][1]=(1-a)/2.0;
	M[2][1][2][1][3][2]=(1-a)/2.0;
	M[2][1][2][1][4][2]=a/2.0;
	M[2][1][2][2][2][1]=a/2.0;
	M[2][1][2][2][3][2]=(1-a)/2.0+(1-a)/2.0;
	M[2][1][2][2][4][3]=a/2.0;
	M[2][1][2][3][2][1]=a/2.0;
	M[2][1][2][3][3][2]=(1-a)/2.0;
	M[2][1][2][3][3][3]=(1-a)/2.0;
	M[2][1][2][3][4][4]=a/2.0;
	M[2][1][2][4][2][3]=a/2.0;
	M[2][1][2][4][3][2]=(0.5-0.5*a)/2.0;
	M[2][1][2][4][3][3]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][1][2][4][3][4]=(0.5-0.5*a)/2.0;
	M[2][1][2][4][4][3]=a/2.0;
	M[2][1][3][0][2][1]=a/2.0;
	M[2][1][3][0][3][1]=(1-a)/2.0;
	M[2][1][3][0][4][1]=a/2.0+(1-a)/2.0;
	M[2][1][3][1][2][1]=a/2.0;
	M[2][1][3][1][3][2]=(1-a)/2.0;
	M[2][1][3][1][4][1]=(1-a)/2.0;
	M[2][1][3][1][4][2]=a/2.0;
	M[2][1][3][2][2][1]=a/2.0;
	M[2][1][3][2][3][2]=(1-a)/2.0;
	M[2][1][3][2][4][2]=(1-a)/2.0;
	M[2][1][3][2][4][3]=a/2.0;
	M[2][1][3][3][2][1]=a/2.0;
	M[2][1][3][3][3][2]=(1-a)/2.0;
	M[2][1][3][3][4][3]=(1-a)/2.0;
	M[2][1][3][3][4][4]=a/2.0;
	M[2][1][3][4][3][2]=0.5/2.0;
	M[2][1][3][4][3][3]=0.5/2.0;
	M[2][1][3][4][4][3]=0.5/2.0;
	M[2][1][3][4][4][4]=0.5/2.0;
	M[2][1][4][0][3][1]=0.5/2.0;
	M[2][1][4][0][4][1]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][1][4][1][3][2]=0.5/2.0;
	M[2][1][4][1][4][1]=0.5/2.0+0.5/2.0;
	M[2][1][4][1][4][2]=0.5/2.0;
	M[2][1][4][2][3][2]=(0.5-0.5*a)/2.0;
	M[2][1][4][2][4][1]=a/2.0;
	M[2][1][4][2][4][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][1][4][2][4][3]=a/2.0;
	M[2][1][4][3][3][2]=0.5/2.0;
	M[2][1][4][3][4][2]=0.5/2.0;
	M[2][1][4][3][4][3]=0.5/2.0+0.5/2.0;
	M[2][1][4][4][3][2]=0.5/2.0;
	M[2][1][4][4][4][3]=0.5/2.0+0.5/2.0;
	M[2][1][4][4][4][4]=0.5/2.0;
	M[2][2][0][0][2][2]=1/2.0+1/2.0;
	M[2][2][0][1][2][2]=1/2.0;
	M[2][2][0][1][2][3]=1/2.0;
	M[2][2][0][2][2][2]=a/2.0;
	M[2][2][0][2][2][3]=(1-a)/2.0+(1-a)/2.0;
	M[2][2][0][2][2][4]=a/2.0;
	M[2][2][0][3][2][2]=a/2.0;
	M[2][2][0][3][2][3]=(1-a)/2.0;
	M[2][2][0][3][2][4]=a/2.0+(1-a)/2.0;
	M[2][2][0][4][2][3]=0.5/2.0;
	M[2][2][0][4][2][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][2][1][0][2][2]=1/2.0;
	M[2][2][1][0][3][2]=1/2.0;
	M[2][2][1][1][2][2]=1/2.0;
	M[2][2][1][1][3][3]=1/2.0;
	M[2][2][1][2][2][2]=a/2.0;
	M[2][2][1][2][2][3]=(1-a)/2.0;
	M[2][2][1][2][3][3]=(1-a)/2.0;
	M[2][2][1][2][3][4]=a/2.0;
	M[2][2][1][3][2][2]=a/2.0;
	M[2][2][1][3][2][4]=(1-a)/2.0;
	M[2][2][1][3][3][3]=(1-a)/2.0;
	M[2][2][1][3][3][4]=a/2.0;
	M[2][2][1][4][2][4]=0.5/2.0+0.5/2.0;
	M[2][2][1][4][3][3]=0.5/2.0;
	M[2][2][1][4][3][4]=0.5/2.0;
	M[2][2][2][0][2][2]=a/2.0;
	M[2][2][2][0][3][2]=(1-a)/2.0+(1-a)/2.0;
	M[2][2][2][0][4][2]=a/2.0;
	M[2][2][2][1][2][2]=a/2.0;
	M[2][2][2][1][3][2]=(1-a)/2.0;
	M[2][2][2][1][3][3]=(1-a)/2.0;
	M[2][2][2][1][4][3]=a/2.0;
	M[2][2][2][2][2][2]=a/2.0;
	M[2][2][2][2][3][3]=(1-a)/2.0+(1-a)/2.0;
	M[2][2][2][2][4][4]=a/2.0;
	M[2][2][2][3][2][2]=a/2.0;
	M[2][2][2][3][3][3]=(1-a)/2.0;
	M[2][2][2][3][3][4]=(1-a)/2.0;
	M[2][2][2][3][4][4]=a/2.0;
	M[2][2][2][4][2][4]=a/2.0;
	M[2][2][2][4][3][3]=(0.5-0.5*a)/2.0;
	M[2][2][2][4][3][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][2][2][4][4][4]=a/2.0;
	M[2][2][3][0][2][2]=a/2.0;
	M[2][2][3][0][3][2]=(1-a)/2.0;
	M[2][2][3][0][4][2]=a/2.0+(1-a)/2.0;
	M[2][2][3][1][2][2]=a/2.0;
	M[2][2][3][1][3][3]=(1-a)/2.0;
	M[2][2][3][1][4][2]=(1-a)/2.0;
	M[2][2][3][1][4][3]=a/2.0;
	M[2][2][3][2][2][2]=a/2.0;
	M[2][2][3][2][3][3]=(1-a)/2.0;
	M[2][2][3][2][4][3]=(1-a)/2.0;
	M[2][2][3][2][4][4]=a/2.0;
	M[2][2][3][3][2][2]=a/2.0;
	M[2][2][3][3][3][3]=(1-a)/2.0;
	M[2][2][3][3][4][4]=a/2.0+(1-a)/2.0;
	M[2][2][3][4][3][3]=0.5/2.0;
	M[2][2][3][4][3][4]=0.5/2.0;
	M[2][2][3][4][4][4]=0.5/2.0+0.5/2.0;
	M[2][2][4][0][3][2]=0.5/2.0;
	M[2][2][4][0][4][2]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][2][4][1][3][3]=0.5/2.0;
	M[2][2][4][1][4][2]=0.5/2.0+0.5/2.0;
	M[2][2][4][1][4][3]=0.5/2.0;
	M[2][2][4][2][3][3]=(0.5-0.5*a)/2.0;
	M[2][2][4][2][4][2]=a/2.0;
	M[2][2][4][2][4][3]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][2][4][2][4][4]=a/2.0;
	M[2][2][4][3][3][3]=0.5/2.0;
	M[2][2][4][3][4][3]=0.5/2.0;
	M[2][2][4][3][4][4]=0.5/2.0+0.5/2.0;
	M[2][2][4][4][3][3]=0.5/2.0;
	M[2][2][4][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][3][0][0][2][3]=1/2.0+1/2.0;
	M[2][3][0][1][2][3]=1/2.0;
	M[2][3][0][1][2][4]=1/2.0;
	M[2][3][0][2][2][3]=a/2.0;
	M[2][3][0][2][2][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[2][3][0][3][2][3]=a/2.0;
	M[2][3][0][3][2][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[2][3][0][4][2][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][3][1][0][2][3]=1/2.0;
	M[2][3][1][0][3][3]=1/2.0;
	M[2][3][1][1][2][3]=1/2.0;
	M[2][3][1][1][3][4]=1/2.0;
	M[2][3][1][2][2][3]=a/2.0;
	M[2][3][1][2][2][4]=(1-a)/2.0;
	M[2][3][1][2][3][4]=a/2.0+(1-a)/2.0;
	M[2][3][1][3][2][3]=a/2.0;
	M[2][3][1][3][2][4]=(1-a)/2.0;
	M[2][3][1][3][3][4]=a/2.0+(1-a)/2.0;
	M[2][3][1][4][2][4]=0.5/2.0+0.5/2.0;
	M[2][3][1][4][3][4]=0.5/2.0+0.5/2.0;
	M[2][3][2][0][2][3]=a/2.0;
	M[2][3][2][0][3][3]=(1-a)/2.0+(1-a)/2.0;
	M[2][3][2][0][4][3]=a/2.0;
	M[2][3][2][1][2][3]=a/2.0;
	M[2][3][2][1][3][3]=(1-a)/2.0;
	M[2][3][2][1][3][4]=(1-a)/2.0;
	M[2][3][2][1][4][4]=a/2.0;
	M[2][3][2][2][2][3]=a/2.0;
	M[2][3][2][2][3][4]=(1-a)/2.0+(1-a)/2.0;
	M[2][3][2][2][4][4]=a/2.0;
	M[2][3][2][3][2][3]=a/2.0;
	M[2][3][2][3][3][4]=(1-a)/2.0+(1-a)/2.0;
	M[2][3][2][3][4][4]=a/2.0;
	M[2][3][2][4][2][4]=a/2.0;
	M[2][3][2][4][3][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][3][2][4][4][4]=a/2.0;
	M[2][3][3][0][2][3]=a/2.0;
	M[2][3][3][0][3][3]=(1-a)/2.0;
	M[2][3][3][0][4][3]=a/2.0+(1-a)/2.0;
	M[2][3][3][1][2][3]=a/2.0;
	M[2][3][3][1][3][4]=(1-a)/2.0;
	M[2][3][3][1][4][3]=(1-a)/2.0;
	M[2][3][3][1][4][4]=a/2.0;
	M[2][3][3][2][2][3]=a/2.0;
	M[2][3][3][2][3][4]=(1-a)/2.0;
	M[2][3][3][2][4][4]=(1-a)/2.0+a/2.0;
	M[2][3][3][3][2][3]=a/2.0;
	M[2][3][3][3][3][4]=(1-a)/2.0;
	M[2][3][3][3][4][4]=a/2.0+(1-a)/2.0;
	M[2][3][3][4][3][4]=0.5/2.0+0.5/2.0;
	M[2][3][3][4][4][4]=0.5/2.0+0.5/2.0;
	M[2][3][4][0][3][3]=0.5/2.0;
	M[2][3][4][0][4][3]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][3][4][1][3][4]=0.5/2.0;
	M[2][3][4][1][4][3]=0.5/2.0+0.5/2.0;
	M[2][3][4][1][4][4]=0.5/2.0;
	M[2][3][4][2][3][4]=(0.5-0.5*a)/2.0;
	M[2][3][4][2][4][3]=a/2.0;
	M[2][3][4][2][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0;
	M[2][3][4][3][3][4]=0.5/2.0;
	M[2][3][4][3][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][3][4][4][3][4]=0.5/2.0;
	M[2][3][4][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][4][0][0][2][4]=1/2.0+1/2.0;
	M[2][4][0][1][2][4]=1/2.0+1/2.0;
	M[2][4][0][2][2][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[2][4][0][3][2][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[2][4][0][4][2][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][4][1][0][2][4]=1/2.0;
	M[2][4][1][0][3][4]=1/2.0;
	M[2][4][1][1][2][4]=1/2.0;
	M[2][4][1][1][3][4]=1/2.0;
	M[2][4][1][2][2][4]=a/2.0+(1-a)/2.0;
	M[2][4][1][2][3][4]=a/2.0+(1-a)/2.0;
	M[2][4][1][3][2][4]=a/2.0+(1-a)/2.0;
	M[2][4][1][3][3][4]=a/2.0+(1-a)/2.0;
	M[2][4][1][4][2][4]=0.5/2.0+0.5/2.0;
	M[2][4][1][4][3][4]=0.5/2.0+0.5/2.0;
	M[2][4][2][0][2][4]=a/2.0;
	M[2][4][2][0][3][4]=(1-a)/2.0+(1-a)/2.0;
	M[2][4][2][0][4][4]=a/2.0;
	M[2][4][2][1][2][4]=a/2.0;
	M[2][4][2][1][3][4]=(1-a)/2.0+(1-a)/2.0;
	M[2][4][2][1][4][4]=a/2.0;
	M[2][4][2][2][2][4]=a/2.0;
	M[2][4][2][2][3][4]=(1-a)/2.0+(1-a)/2.0;
	M[2][4][2][2][4][4]=a/2.0;
	M[2][4][2][3][2][4]=a/2.0;
	M[2][4][2][3][3][4]=(1-a)/2.0+(1-a)/2.0;
	M[2][4][2][3][4][4]=a/2.0;
	M[2][4][2][4][2][4]=a/2.0;
	M[2][4][2][4][3][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[2][4][2][4][4][4]=a/2.0;
	M[2][4][3][0][2][4]=a/2.0;
	M[2][4][3][0][3][4]=(1-a)/2.0;
	M[2][4][3][0][4][4]=a/2.0+(1-a)/2.0;
	M[2][4][3][1][2][4]=a/2.0;
	M[2][4][3][1][3][4]=(1-a)/2.0;
	M[2][4][3][1][4][4]=a/2.0+(1-a)/2.0;
	M[2][4][3][2][2][4]=a/2.0;
	M[2][4][3][2][3][4]=(1-a)/2.0;
	M[2][4][3][2][4][4]=(1-a)/2.0+a/2.0;
	M[2][4][3][3][2][4]=a/2.0;
	M[2][4][3][3][3][4]=(1-a)/2.0;
	M[2][4][3][3][4][4]=a/2.0+(1-a)/2.0;
	M[2][4][3][4][3][4]=0.5/2.0+0.5/2.0;
	M[2][4][3][4][4][4]=0.5/2.0+0.5/2.0;
	M[2][4][4][0][3][4]=0.5/2.0;
	M[2][4][4][0][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][4][4][1][3][4]=0.5/2.0;
	M[2][4][4][1][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][4][4][2][3][4]=(0.5-0.5*a)/2.0;
	M[2][4][4][2][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0+a/2.0;
	M[2][4][4][3][3][4]=0.5/2.0;
	M[2][4][4][3][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[2][4][4][4][3][4]=0.5/2.0;
	M[2][4][4][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][0][0][0][3][0]=1/2.0+1/2.0;
	M[3][0][0][1][3][0]=1/2.0;
	M[3][0][0][1][3][1]=1/2.0;
	M[3][0][0][2][3][0]=a/2.0;
	M[3][0][0][2][3][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][0][0][2][3][2]=a/2.0;
	M[3][0][0][3][3][0]=a/2.0;
	M[3][0][0][3][3][1]=(1-a)/2.0;
	M[3][0][0][3][3][2]=(1-a)/2.0;
	M[3][0][0][3][3][3]=a/2.0;
	M[3][0][0][4][3][1]=0.5/2.0;
	M[3][0][0][4][3][2]=0.5/2.0+0.5/2.0;
	M[3][0][0][4][3][3]=0.5/2.0;
	M[3][0][1][0][3][0]=1/2.0;
	M[3][0][1][0][4][0]=1/2.0;
	M[3][0][1][1][3][0]=1/2.0;
	M[3][0][1][1][4][1]=1/2.0;
	M[3][0][1][2][3][0]=a/2.0;
	M[3][0][1][2][3][1]=(1-a)/2.0;
	M[3][0][1][2][4][1]=(1-a)/2.0;
	M[3][0][1][2][4][2]=a/2.0;
	M[3][0][1][3][3][0]=a/2.0;
	M[3][0][1][3][3][2]=(1-a)/2.0;
	M[3][0][1][3][4][1]=(1-a)/2.0;
	M[3][0][1][3][4][3]=a/2.0;
	M[3][0][1][4][3][2]=0.5/2.0;
	M[3][0][1][4][3][3]=0.5/2.0;
	M[3][0][1][4][4][1]=0.5/2.0;
	M[3][0][1][4][4][2]=0.5/2.0;
	M[3][0][2][0][3][0]=a/2.0;
	M[3][0][2][0][4][0]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][0][2][1][3][0]=a/2.0;
	M[3][0][2][1][4][0]=(1-a)/2.0;
	M[3][0][2][1][4][1]=a/2.0+(1-a)/2.0;
	M[3][0][2][2][3][0]=a/2.0;
	M[3][0][2][2][4][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][0][2][2][4][2]=a/2.0;
	M[3][0][2][3][3][0]=a/2.0;
	M[3][0][2][3][4][1]=(1-a)/2.0;
	M[3][0][2][3][4][2]=(1-a)/2.0;
	M[3][0][2][3][4][3]=a/2.0;
	M[3][0][2][4][3][2]=a/2.0;
	M[3][0][2][4][4][1]=(0.5-0.5*a)/2.0;
	M[3][0][2][4][4][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0;
	M[3][0][2][4][4][3]=(0.5-0.5*a)/2.0;
	M[3][0][3][0][3][0]=a/2.0;
	M[3][0][3][0][4][0]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][0][3][1][3][0]=a/2.0;
	M[3][0][3][1][4][0]=(1-a)/2.0;
	M[3][0][3][1][4][1]=a/2.0+(1-a)/2.0;
	M[3][0][3][2][3][0]=a/2.0;
	M[3][0][3][2][4][1]=(1-a)/2.0+(1-a)/2.0;
	M[3][0][3][2][4][2]=a/2.0;
	M[3][0][3][3][3][0]=a/2.0;
	M[3][0][3][3][4][1]=(1-a)/2.0;
	M[3][0][3][3][4][2]=(1-a)/2.0;
	M[3][0][3][3][4][3]=a/2.0;
	M[3][0][3][4][4][1]=0.5/2.0;
	M[3][0][3][4][4][2]=0.5/2.0+0.5/2.0;
	M[3][0][3][4][4][3]=0.5/2.0;
	M[3][0][4][0][4][0]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][0][4][1][4][0]=0.5/2.0+0.5/2.0;
	M[3][0][4][1][4][1]=0.5/2.0+0.5/2.0;
	M[3][0][4][2][4][0]=a/2.0;
	M[3][0][4][2][4][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][0][4][2][4][2]=a/2.0;
	M[3][0][4][3][4][1]=0.5/2.0+0.5/2.0;
	M[3][0][4][3][4][2]=0.5/2.0+0.5/2.0;
	M[3][0][4][4][4][1]=0.5/2.0;
	M[3][0][4][4][4][2]=0.5/2.0+0.5/2.0;
	M[3][0][4][4][4][3]=0.5/2.0;
	M[3][1][0][0][3][1]=1/2.0+1/2.0;
	M[3][1][0][1][3][1]=1/2.0;
	M[3][1][0][1][3][2]=1/2.0;
	M[3][1][0][2][3][1]=a/2.0;
	M[3][1][0][2][3][2]=(1-a)/2.0+(1-a)/2.0;
	M[3][1][0][2][3][3]=a/2.0;
	M[3][1][0][3][3][1]=a/2.0;
	M[3][1][0][3][3][2]=(1-a)/2.0;
	M[3][1][0][3][3][3]=(1-a)/2.0;
	M[3][1][0][3][3][4]=a/2.0;
	M[3][1][0][4][3][2]=0.5/2.0;
	M[3][1][0][4][3][3]=0.5/2.0+0.5/2.0;
	M[3][1][0][4][3][4]=0.5/2.0;
	M[3][1][1][0][3][1]=1/2.0;
	M[3][1][1][0][4][1]=1/2.0;
	M[3][1][1][1][3][1]=1/2.0;
	M[3][1][1][1][4][2]=1/2.0;
	M[3][1][1][2][3][1]=a/2.0;
	M[3][1][1][2][3][2]=(1-a)/2.0;
	M[3][1][1][2][4][2]=(1-a)/2.0;
	M[3][1][1][2][4][3]=a/2.0;
	M[3][1][1][3][3][1]=a/2.0;
	M[3][1][1][3][3][3]=(1-a)/2.0;
	M[3][1][1][3][4][2]=(1-a)/2.0;
	M[3][1][1][3][4][4]=a/2.0;
	M[3][1][1][4][3][3]=0.5/2.0;
	M[3][1][1][4][3][4]=0.5/2.0;
	M[3][1][1][4][4][2]=0.5/2.0;
	M[3][1][1][4][4][3]=0.5/2.0;
	M[3][1][2][0][3][1]=a/2.0;
	M[3][1][2][0][4][1]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][1][2][1][3][1]=a/2.0;
	M[3][1][2][1][4][1]=(1-a)/2.0;
	M[3][1][2][1][4][2]=a/2.0+(1-a)/2.0;
	M[3][1][2][2][3][1]=a/2.0;
	M[3][1][2][2][4][2]=(1-a)/2.0+(1-a)/2.0;
	M[3][1][2][2][4][3]=a/2.0;
	M[3][1][2][3][3][1]=a/2.0;
	M[3][1][2][3][4][2]=(1-a)/2.0;
	M[3][1][2][3][4][3]=(1-a)/2.0;
	M[3][1][2][3][4][4]=a/2.0;
	M[3][1][2][4][3][3]=a/2.0;
	M[3][1][2][4][4][2]=(0.5-0.5*a)/2.0;
	M[3][1][2][4][4][3]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0;
	M[3][1][2][4][4][4]=(0.5-0.5*a)/2.0;
	M[3][1][3][0][3][1]=a/2.0;
	M[3][1][3][0][4][1]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][1][3][1][3][1]=a/2.0;
	M[3][1][3][1][4][1]=(1-a)/2.0;
	M[3][1][3][1][4][2]=a/2.0+(1-a)/2.0;
	M[3][1][3][2][3][1]=a/2.0;
	M[3][1][3][2][4][2]=(1-a)/2.0+(1-a)/2.0;
	M[3][1][3][2][4][3]=a/2.0;
	M[3][1][3][3][3][1]=a/2.0;
	M[3][1][3][3][4][2]=(1-a)/2.0;
	M[3][1][3][3][4][3]=(1-a)/2.0;
	M[3][1][3][3][4][4]=a/2.0;
	M[3][1][3][4][4][2]=0.5/2.0;
	M[3][1][3][4][4][3]=0.5/2.0+0.5/2.0;
	M[3][1][3][4][4][4]=0.5/2.0;
	M[3][1][4][0][4][1]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][1][4][1][4][1]=0.5/2.0+0.5/2.0;
	M[3][1][4][1][4][2]=0.5/2.0+0.5/2.0;
	M[3][1][4][2][4][1]=a/2.0;
	M[3][1][4][2][4][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][1][4][2][4][3]=a/2.0;
	M[3][1][4][3][4][2]=0.5/2.0+0.5/2.0;
	M[3][1][4][3][4][3]=0.5/2.0+0.5/2.0;
	M[3][1][4][4][4][2]=0.5/2.0;
	M[3][1][4][4][4][3]=0.5/2.0+0.5/2.0;
	M[3][1][4][4][4][4]=0.5/2.0;
	M[3][2][0][0][3][2]=1/2.0+1/2.0;
	M[3][2][0][1][3][2]=1/2.0;
	M[3][2][0][1][3][3]=1/2.0;
	M[3][2][0][2][3][2]=a/2.0;
	M[3][2][0][2][3][3]=(1-a)/2.0+(1-a)/2.0;
	M[3][2][0][2][3][4]=a/2.0;
	M[3][2][0][3][3][2]=a/2.0;
	M[3][2][0][3][3][3]=(1-a)/2.0;
	M[3][2][0][3][3][4]=a/2.0+(1-a)/2.0;
	M[3][2][0][4][3][3]=0.5/2.0;
	M[3][2][0][4][3][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][2][1][0][3][2]=1/2.0;
	M[3][2][1][0][4][2]=1/2.0;
	M[3][2][1][1][3][2]=1/2.0;
	M[3][2][1][1][4][3]=1/2.0;
	M[3][2][1][2][3][2]=a/2.0;
	M[3][2][1][2][3][3]=(1-a)/2.0;
	M[3][2][1][2][4][3]=(1-a)/2.0;
	M[3][2][1][2][4][4]=a/2.0;
	M[3][2][1][3][3][2]=a/2.0;
	M[3][2][1][3][3][4]=(1-a)/2.0;
	M[3][2][1][3][4][3]=(1-a)/2.0;
	M[3][2][1][3][4][4]=a/2.0;
	M[3][2][1][4][3][4]=0.5/2.0+0.5/2.0;
	M[3][2][1][4][4][3]=0.5/2.0;
	M[3][2][1][4][4][4]=0.5/2.0;
	M[3][2][2][0][3][2]=a/2.0;
	M[3][2][2][0][4][2]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][2][2][1][3][2]=a/2.0;
	M[3][2][2][1][4][2]=(1-a)/2.0;
	M[3][2][2][1][4][3]=a/2.0+(1-a)/2.0;
	M[3][2][2][2][3][2]=a/2.0;
	M[3][2][2][2][4][3]=(1-a)/2.0+(1-a)/2.0;
	M[3][2][2][2][4][4]=a/2.0;
	M[3][2][2][3][3][2]=a/2.0;
	M[3][2][2][3][4][3]=(1-a)/2.0;
	M[3][2][2][3][4][4]=(1-a)/2.0+a/2.0;
	M[3][2][2][4][3][4]=a/2.0;
	M[3][2][2][4][4][3]=(0.5-0.5*a)/2.0;
	M[3][2][2][4][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0;
	M[3][2][3][0][3][2]=a/2.0;
	M[3][2][3][0][4][2]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][2][3][1][3][2]=a/2.0;
	M[3][2][3][1][4][2]=(1-a)/2.0;
	M[3][2][3][1][4][3]=a/2.0+(1-a)/2.0;
	M[3][2][3][2][3][2]=a/2.0;
	M[3][2][3][2][4][3]=(1-a)/2.0+(1-a)/2.0;
	M[3][2][3][2][4][4]=a/2.0;
	M[3][2][3][3][3][2]=a/2.0;
	M[3][2][3][3][4][3]=(1-a)/2.0;
	M[3][2][3][3][4][4]=a/2.0+(1-a)/2.0;
	M[3][2][3][4][4][3]=0.5/2.0;
	M[3][2][3][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][2][4][0][4][2]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][2][4][1][4][2]=0.5/2.0+0.5/2.0;
	M[3][2][4][1][4][3]=0.5/2.0+0.5/2.0;
	M[3][2][4][2][4][2]=a/2.0;
	M[3][2][4][2][4][3]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[3][2][4][2][4][4]=a/2.0;
	M[3][2][4][3][4][3]=0.5/2.0+0.5/2.0;
	M[3][2][4][3][4][4]=0.5/2.0+0.5/2.0;
	M[3][2][4][4][4][3]=0.5/2.0;
	M[3][2][4][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][3][0][0][3][3]=1/2.0+1/2.0;
	M[3][3][0][1][3][3]=1/2.0;
	M[3][3][0][1][3][4]=1/2.0;
	M[3][3][0][2][3][3]=a/2.0;
	M[3][3][0][2][3][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][3][0][3][3][3]=a/2.0;
	M[3][3][0][3][3][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][3][0][4][3][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][3][1][0][3][3]=1/2.0;
	M[3][3][1][0][4][3]=1/2.0;
	M[3][3][1][1][3][3]=1/2.0;
	M[3][3][1][1][4][4]=1/2.0;
	M[3][3][1][2][3][3]=a/2.0;
	M[3][3][1][2][3][4]=(1-a)/2.0;
	M[3][3][1][2][4][4]=a/2.0+(1-a)/2.0;
	M[3][3][1][3][3][3]=a/2.0;
	M[3][3][1][3][3][4]=(1-a)/2.0;
	M[3][3][1][3][4][4]=a/2.0+(1-a)/2.0;
	M[3][3][1][4][3][4]=0.5/2.0+0.5/2.0;
	M[3][3][1][4][4][4]=0.5/2.0+0.5/2.0;
	M[3][3][2][0][3][3]=a/2.0;
	M[3][3][2][0][4][3]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][3][2][1][3][3]=a/2.0;
	M[3][3][2][1][4][3]=(1-a)/2.0;
	M[3][3][2][1][4][4]=a/2.0+(1-a)/2.0;
	M[3][3][2][2][3][3]=a/2.0;
	M[3][3][2][2][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0;
	M[3][3][2][3][3][3]=a/2.0;
	M[3][3][2][3][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0;
	M[3][3][2][4][3][4]=a/2.0;
	M[3][3][2][4][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0;
	M[3][3][3][0][3][3]=a/2.0;
	M[3][3][3][0][4][3]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][3][3][1][3][3]=a/2.0;
	M[3][3][3][1][4][3]=(1-a)/2.0;
	M[3][3][3][1][4][4]=a/2.0+(1-a)/2.0;
	M[3][3][3][2][3][3]=a/2.0;
	M[3][3][3][2][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0;
	M[3][3][3][3][3][3]=a/2.0;
	M[3][3][3][3][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][3][3][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][3][4][0][4][3]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][3][4][1][4][3]=0.5/2.0+0.5/2.0;
	M[3][3][4][1][4][4]=0.5/2.0+0.5/2.0;
	M[3][3][4][2][4][3]=a/2.0;
	M[3][3][4][2][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0;
	M[3][3][4][3][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][3][4][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][4][0][0][3][4]=1/2.0+1/2.0;
	M[3][4][0][1][3][4]=1/2.0+1/2.0;
	M[3][4][0][2][3][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][4][0][3][3][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][4][0][4][3][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][4][1][0][3][4]=1/2.0;
	M[3][4][1][0][4][4]=1/2.0;
	M[3][4][1][1][3][4]=1/2.0;
	M[3][4][1][1][4][4]=1/2.0;
	M[3][4][1][2][3][4]=a/2.0+(1-a)/2.0;
	M[3][4][1][2][4][4]=a/2.0+(1-a)/2.0;
	M[3][4][1][3][3][4]=a/2.0+(1-a)/2.0;
	M[3][4][1][3][4][4]=a/2.0+(1-a)/2.0;
	M[3][4][1][4][3][4]=0.5/2.0+0.5/2.0;
	M[3][4][1][4][4][4]=0.5/2.0+0.5/2.0;
	M[3][4][2][0][3][4]=a/2.0;
	M[3][4][2][0][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][4][2][1][3][4]=a/2.0;
	M[3][4][2][1][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][4][2][2][3][4]=a/2.0;
	M[3][4][2][2][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0;
	M[3][4][2][3][3][4]=a/2.0;
	M[3][4][2][3][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0;
	M[3][4][2][4][3][4]=a/2.0;
	M[3][4][2][4][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0;
	M[3][4][3][0][3][4]=a/2.0;
	M[3][4][3][0][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][4][3][1][3][4]=a/2.0;
	M[3][4][3][1][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][4][3][2][3][4]=a/2.0;
	M[3][4][3][2][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0;
	M[3][4][3][3][3][4]=a/2.0;
	M[3][4][3][3][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[3][4][3][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][4][4][0][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][4][4][1][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][4][4][2][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0+a/2.0;
	M[3][4][4][3][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[3][4][4][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][0][0][0][4][0]=1/2.0+1/2.0;
	M[4][0][0][1][4][0]=1/2.0;
	M[4][0][0][1][4][1]=1/2.0;
	M[4][0][0][2][4][0]=a/2.0;
	M[4][0][0][2][4][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][0][0][2][4][2]=a/2.0;
	M[4][0][0][3][4][0]=a/2.0;
	M[4][0][0][3][4][1]=(1-a)/2.0;
	M[4][0][0][3][4][2]=(1-a)/2.0;
	M[4][0][0][3][4][3]=a/2.0;
	M[4][0][0][4][4][1]=0.5/2.0;
	M[4][0][0][4][4][2]=0.5/2.0+0.5/2.0;
	M[4][0][0][4][4][3]=0.5/2.0;
	M[4][0][1][0][4][0]=1/2.0+1/2.0;
	M[4][0][1][1][4][0]=1/2.0;
	M[4][0][1][1][4][1]=1/2.0;
	M[4][0][1][2][4][0]=a/2.0;
	M[4][0][1][2][4][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][0][1][2][4][2]=a/2.0;
	M[4][0][1][3][4][0]=a/2.0;
	M[4][0][1][3][4][1]=(1-a)/2.0;
	M[4][0][1][3][4][2]=(1-a)/2.0;
	M[4][0][1][3][4][3]=a/2.0;
	M[4][0][1][4][4][1]=0.5/2.0;
	M[4][0][1][4][4][2]=0.5/2.0+0.5/2.0;
	M[4][0][1][4][4][3]=0.5/2.0;
	M[4][0][2][0][4][0]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][0][2][1][4][0]=a/2.0+(1-a)/2.0;
	M[4][0][2][1][4][1]=a/2.0+(1-a)/2.0;
	M[4][0][2][2][4][0]=a/2.0;
	M[4][0][2][2][4][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][0][2][2][4][2]=a/2.0;
	M[4][0][2][3][4][0]=a/2.0;
	M[4][0][2][3][4][1]=(1-a)/2.0;
	M[4][0][2][3][4][2]=(1-a)/2.0;
	M[4][0][2][3][4][3]=a/2.0;
	M[4][0][2][4][4][1]=(0.5-0.5*a)/2.0;
	M[4][0][2][4][4][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0+a/2.0;
	M[4][0][2][4][4][3]=(0.5-0.5*a)/2.0;
	M[4][0][3][0][4][0]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][0][3][1][4][0]=a/2.0+(1-a)/2.0;
	M[4][0][3][1][4][1]=a/2.0+(1-a)/2.0;
	M[4][0][3][2][4][0]=a/2.0;
	M[4][0][3][2][4][1]=(1-a)/2.0+(1-a)/2.0;
	M[4][0][3][2][4][2]=a/2.0;
	M[4][0][3][3][4][0]=a/2.0;
	M[4][0][3][3][4][1]=(1-a)/2.0;
	M[4][0][3][3][4][2]=(1-a)/2.0;
	M[4][0][3][3][4][3]=a/2.0;
	M[4][0][3][4][4][1]=0.5/2.0;
	M[4][0][3][4][4][2]=0.5/2.0+0.5/2.0;
	M[4][0][3][4][4][3]=0.5/2.0;
	M[4][0][4][0][4][0]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][0][4][1][4][0]=0.5/2.0+0.5/2.0;
	M[4][0][4][1][4][1]=0.5/2.0+0.5/2.0;
	M[4][0][4][2][4][0]=a/2.0;
	M[4][0][4][2][4][1]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][0][4][2][4][2]=a/2.0;
	M[4][0][4][3][4][1]=0.5/2.0+0.5/2.0;
	M[4][0][4][3][4][2]=0.5/2.0+0.5/2.0;
	M[4][0][4][4][4][1]=0.5/2.0;
	M[4][0][4][4][4][2]=0.5/2.0+0.5/2.0;
	M[4][0][4][4][4][3]=0.5/2.0;
	M[4][1][0][0][4][1]=1/2.0+1/2.0;
	M[4][1][0][1][4][1]=1/2.0;
	M[4][1][0][1][4][2]=1/2.0;
	M[4][1][0][2][4][1]=a/2.0;
	M[4][1][0][2][4][2]=(1-a)/2.0+(1-a)/2.0;
	M[4][1][0][2][4][3]=a/2.0;
	M[4][1][0][3][4][1]=a/2.0;
	M[4][1][0][3][4][2]=(1-a)/2.0;
	M[4][1][0][3][4][3]=(1-a)/2.0;
	M[4][1][0][3][4][4]=a/2.0;
	M[4][1][0][4][4][2]=0.5/2.0;
	M[4][1][0][4][4][3]=0.5/2.0+0.5/2.0;
	M[4][1][0][4][4][4]=0.5/2.0;
	M[4][1][1][0][4][1]=1/2.0+1/2.0;
	M[4][1][1][1][4][1]=1/2.0;
	M[4][1][1][1][4][2]=1/2.0;
	M[4][1][1][2][4][1]=a/2.0;
	M[4][1][1][2][4][2]=(1-a)/2.0+(1-a)/2.0;
	M[4][1][1][2][4][3]=a/2.0;
	M[4][1][1][3][4][1]=a/2.0;
	M[4][1][1][3][4][2]=(1-a)/2.0;
	M[4][1][1][3][4][3]=(1-a)/2.0;
	M[4][1][1][3][4][4]=a/2.0;
	M[4][1][1][4][4][2]=0.5/2.0;
	M[4][1][1][4][4][3]=0.5/2.0+0.5/2.0;
	M[4][1][1][4][4][4]=0.5/2.0;
	M[4][1][2][0][4][1]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][1][2][1][4][1]=a/2.0+(1-a)/2.0;
	M[4][1][2][1][4][2]=a/2.0+(1-a)/2.0;
	M[4][1][2][2][4][1]=a/2.0;
	M[4][1][2][2][4][2]=(1-a)/2.0+(1-a)/2.0;
	M[4][1][2][2][4][3]=a/2.0;
	M[4][1][2][3][4][1]=a/2.0;
	M[4][1][2][3][4][2]=(1-a)/2.0;
	M[4][1][2][3][4][3]=(1-a)/2.0;
	M[4][1][2][3][4][4]=a/2.0;
	M[4][1][2][4][4][2]=(0.5-0.5*a)/2.0;
	M[4][1][2][4][4][3]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0+a/2.0;
	M[4][1][2][4][4][4]=(0.5-0.5*a)/2.0;
	M[4][1][3][0][4][1]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][1][3][1][4][1]=a/2.0+(1-a)/2.0;
	M[4][1][3][1][4][2]=a/2.0+(1-a)/2.0;
	M[4][1][3][2][4][1]=a/2.0;
	M[4][1][3][2][4][2]=(1-a)/2.0+(1-a)/2.0;
	M[4][1][3][2][4][3]=a/2.0;
	M[4][1][3][3][4][1]=a/2.0;
	M[4][1][3][3][4][2]=(1-a)/2.0;
	M[4][1][3][3][4][3]=(1-a)/2.0;
	M[4][1][3][3][4][4]=a/2.0;
	M[4][1][3][4][4][2]=0.5/2.0;
	M[4][1][3][4][4][3]=0.5/2.0+0.5/2.0;
	M[4][1][3][4][4][4]=0.5/2.0;
	M[4][1][4][0][4][1]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][1][4][1][4][1]=0.5/2.0+0.5/2.0;
	M[4][1][4][1][4][2]=0.5/2.0+0.5/2.0;
	M[4][1][4][2][4][1]=a/2.0;
	M[4][1][4][2][4][2]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][1][4][2][4][3]=a/2.0;
	M[4][1][4][3][4][2]=0.5/2.0+0.5/2.0;
	M[4][1][4][3][4][3]=0.5/2.0+0.5/2.0;
	M[4][1][4][4][4][2]=0.5/2.0;
	M[4][1][4][4][4][3]=0.5/2.0+0.5/2.0;
	M[4][1][4][4][4][4]=0.5/2.0;
	M[4][2][0][0][4][2]=1/2.0+1/2.0;
	M[4][2][0][1][4][2]=1/2.0;
	M[4][2][0][1][4][3]=1/2.0;
	M[4][2][0][2][4][2]=a/2.0;
	M[4][2][0][2][4][3]=(1-a)/2.0+(1-a)/2.0;
	M[4][2][0][2][4][4]=a/2.0;
	M[4][2][0][3][4][2]=a/2.0;
	M[4][2][0][3][4][3]=(1-a)/2.0;
	M[4][2][0][3][4][4]=a/2.0+(1-a)/2.0;
	M[4][2][0][4][4][3]=0.5/2.0;
	M[4][2][0][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][2][1][0][4][2]=1/2.0+1/2.0;
	M[4][2][1][1][4][2]=1/2.0;
	M[4][2][1][1][4][3]=1/2.0;
	M[4][2][1][2][4][2]=a/2.0;
	M[4][2][1][2][4][3]=(1-a)/2.0+(1-a)/2.0;
	M[4][2][1][2][4][4]=a/2.0;
	M[4][2][1][3][4][2]=a/2.0;
	M[4][2][1][3][4][3]=(1-a)/2.0;
	M[4][2][1][3][4][4]=a/2.0+(1-a)/2.0;
	M[4][2][1][4][4][3]=0.5/2.0;
	M[4][2][1][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][2][2][0][4][2]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][2][2][1][4][2]=a/2.0+(1-a)/2.0;
	M[4][2][2][1][4][3]=a/2.0+(1-a)/2.0;
	M[4][2][2][2][4][2]=a/2.0;
	M[4][2][2][2][4][3]=(1-a)/2.0+(1-a)/2.0;
	M[4][2][2][2][4][4]=a/2.0;
	M[4][2][2][3][4][2]=a/2.0;
	M[4][2][2][3][4][3]=(1-a)/2.0;
	M[4][2][2][3][4][4]=(1-a)/2.0+a/2.0;
	M[4][2][2][4][4][3]=(0.5-0.5*a)/2.0;
	M[4][2][2][4][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0+a/2.0;
	M[4][2][3][0][4][2]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][2][3][1][4][2]=a/2.0+(1-a)/2.0;
	M[4][2][3][1][4][3]=a/2.0+(1-a)/2.0;
	M[4][2][3][2][4][2]=a/2.0;
	M[4][2][3][2][4][3]=(1-a)/2.0+(1-a)/2.0;
	M[4][2][3][2][4][4]=a/2.0;
	M[4][2][3][3][4][2]=a/2.0;
	M[4][2][3][3][4][3]=(1-a)/2.0;
	M[4][2][3][3][4][4]=a/2.0+(1-a)/2.0;
	M[4][2][3][4][4][3]=0.5/2.0;
	M[4][2][3][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][2][4][0][4][2]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][2][4][1][4][2]=0.5/2.0+0.5/2.0;
	M[4][2][4][1][4][3]=0.5/2.0+0.5/2.0;
	M[4][2][4][2][4][2]=a/2.0;
	M[4][2][4][2][4][3]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0;
	M[4][2][4][2][4][4]=a/2.0;
	M[4][2][4][3][4][3]=0.5/2.0+0.5/2.0;
	M[4][2][4][3][4][4]=0.5/2.0+0.5/2.0;
	M[4][2][4][4][4][3]=0.5/2.0;
	M[4][2][4][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][3][0][0][4][3]=1/2.0+1/2.0;
	M[4][3][0][1][4][3]=1/2.0;
	M[4][3][0][1][4][4]=1/2.0;
	M[4][3][0][2][4][3]=a/2.0;
	M[4][3][0][2][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][3][0][3][4][3]=a/2.0;
	M[4][3][0][3][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][3][0][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][3][1][0][4][3]=1/2.0+1/2.0;
	M[4][3][1][1][4][3]=1/2.0;
	M[4][3][1][1][4][4]=1/2.0;
	M[4][3][1][2][4][3]=a/2.0;
	M[4][3][1][2][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][3][1][3][4][3]=a/2.0;
	M[4][3][1][3][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][3][1][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][3][2][0][4][3]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][3][2][1][4][3]=a/2.0+(1-a)/2.0;
	M[4][3][2][1][4][4]=a/2.0+(1-a)/2.0;
	M[4][3][2][2][4][3]=a/2.0;
	M[4][3][2][2][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0;
	M[4][3][2][3][4][3]=a/2.0;
	M[4][3][2][3][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0;
	M[4][3][2][4][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0+a/2.0;
	M[4][3][3][0][4][3]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][3][3][1][4][3]=a/2.0+(1-a)/2.0;
	M[4][3][3][1][4][4]=a/2.0+(1-a)/2.0;
	M[4][3][3][2][4][3]=a/2.0;
	M[4][3][3][2][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0;
	M[4][3][3][3][4][3]=a/2.0;
	M[4][3][3][3][4][4]=a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][3][3][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][3][4][0][4][3]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][3][4][1][4][3]=0.5/2.0+0.5/2.0;
	M[4][3][4][1][4][4]=0.5/2.0+0.5/2.0;
	M[4][3][4][2][4][3]=a/2.0;
	M[4][3][4][2][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0;
	M[4][3][4][3][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][3][4][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][4][0][0][4][4]=1/2.0+1/2.0;
	M[4][4][0][1][4][4]=1/2.0+1/2.0;
	M[4][4][0][2][4][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][4][0][3][4][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][4][0][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][4][1][0][4][4]=1/2.0+1/2.0;
	M[4][4][1][1][4][4]=1/2.0+1/2.0;
	M[4][4][1][2][4][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][4][1][3][4][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][4][1][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][4][2][0][4][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][4][2][1][4][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][4][2][2][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0+a/2.0;
	M[4][4][2][3][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0+a/2.0;
	M[4][4][2][4][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0+a/2.0;
	M[4][4][3][0][4][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][4][3][1][4][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][4][3][2][4][4]=(1-a)/2.0+(1-a)/2.0+a/2.0+a/2.0;
	M[4][4][3][3][4][4]=a/2.0+a/2.0+(1-a)/2.0+(1-a)/2.0;
	M[4][4][3][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][4][4][0][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][4][4][1][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][4][4][2][4][4]=(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+(0.5-0.5*a)/2.0+a/2.0+a/2.0;
	M[4][4][4][3][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
	M[4][4][4][4][4][4]=0.5/2.0+0.5/2.0+0.5/2.0+0.5/2.0;
}
